package devisor2.foundation;


import java.awt.*;
import java.io.*;
import java.util.Vector;
import java.awt.geom.*;

import devisor2.grid.options.*;
import devisor2.grid.GUI.framework.*;
import devisor2.grid.backend.*;

/**
 *  This is the abstract superclass for EVERYTHING that can be displayed,
 *  modified or messed with on the DrawingArea. It provides abstract
 *  method definitions which are common to all grid items: 
 *  All grid items must be able to draw themselves, perform an affine
 *  transformation on themselves, they must calculate their own perimeter
 *  etc etc.
 *
 * @author Dominik Goeddeke
 *
 */
public abstract class GridItem 
{

    /**
     *  superconstructor just sets references correctly
     */
    public GridItem ()
    {
	cc = ControlCenter.getMyself ();
	marked = false;
    }

    /****************************ATTRIBUTES*****************/

    /**
     *  the all-important ControlCenter reference
     */
    protected ControlCenter cc;

    /**
     *  flag to indicate if the item is currently selected
     */
    protected boolean marked;

    /**
     *  flag to indicate that in the currect transform step, this item has
     *  already been performed. 
     */
    protected boolean transformed;

    /**
     *  for better performance, the extremal coordinates of the grid
     *  item are stored as separate variables and are updated via the 
     *  <code>updateInnerData()<\code> method
     */
    protected int xmin, xmax, ymin, ymax;




    
    /***************************EASY ACCESS METHODS**********************/

    /**
     *  clears the "a transformation has already been performed"-flag
     */
    public void clearTransformedFlag ()
    {
	transformed = false;
    }

    /**
     *  marks the item as selected
     */
    public void setMarker()
    {
	marked = true;
    }
    
    /**
     *  marks the item as unselected
     */
    public void unsetMarker()
    {
	marked=false;
    }
    
    /**
     *  returns if the item is selected
     */
    public boolean isMarked()
    {
	return marked;
    }

    /**
     *  sets the marker variable directly
     */
    public void setMarker (boolean m)
    {
	marked = m;
    }


    /*******************************METHOD DEFINITIONS*************/

    /**
     *  in this method, the item paints itself to the given graphics
     *  context
     *
     *  @param g- the graphics context to paint to
     */
    public abstract void draw (Graphics g);

    /**
     *  Is the item inside a given rectangular area?
     *  All in WORLD COORDINATES!!!
     *
     *  @param topleft - the topleft corner of the rectangle
     *  @param bottomright - the bottom right corner of the rectangle
     *  @return true if the item is completely inside the rectangle
     */
    public abstract boolean isInside (int[] topleft, int[] bottomright);


    /**
     *  Determines if the item has been hit by a click on the grid
     *  ALL IN SCREEN COORDINATES to avoid the snap mechanism
     *
     *  @param x,y - coordinates of the event point where the click
     *         occurred
     *  @return true if the item has been hit
     */
    public abstract boolean isHit (int x, int y);

    /**
     *  returns all parameters of the griditem in an int array
     */
    public abstract double[] getParameters ();

    /**
     *  sets all parameters of the GridItem according to the given int array
     */
    public abstract void setParameters (double[] para);



    /**
     *  This method returns the perimeter of the item, that is the smallest
     *  bounding rectangle the item lies is completely inside of.
     *
     *  @return a Rectangle object defining the perimeter
     */
    public abstract Rectangle getPerimeter ();


    /**
     *  This method performs the given affine transformation (in homogenous
     *  world coordinates) on the grid item and updates its position
     *  and appearance
     *
     *  @param trafo - a GridTransform instance representing the transformation
     */
    public abstract void performTransformation (GridTransformation trafo);

    /**
     *  This method mirrors the grid item at the given line
     *  @param n - the normalized normal of the line (length 1)
     *  @param p - an arbitrary point on the line
     */
    public abstract void performMirror (double nx, double ny, double px, double py);



 
}
