package devisor2.foundation.base;

import devisor2.foundation.boundary.*;
import devisor2.foundation.elements.*;
import devisor2.grid.options.*;
import devisor2.grid.GUI.dialogs.*;
import devisor2.grid.GUI.framework.*;

//import java.awt.*;
//import java.awt.geom.*;
//import java.awt.event.*;
//import java.util.*;
//import javax.swing.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.*;
import java.util.*;




/**
 *  This class DomainBoundary describes the boundaries of the domain.
 *  A DomainBoundary contains several boundaries, each of which is composed 
 *  of the actual boundary segments. <br>
 *  Here, methods to modify the boundaries are provided.
 *
 *  @author Christian Becker, Dominik Goeddeke
 *  @version 0.9 almost done
 */	
public class DomainBoundary implements java.io.Serializable
{
    /**
     * the boundaries themselves
     */	
    protected Vector boundaries;
    
    /**
     * the domain which contains this DomainBoundary
     */	
    protected BasicDomain domain;
    
    protected ControlCenter cc = ControlCenter.getMyself();
    
    /*********************************************************/


    /**
     * creates a new domain boundary
     */	
    public DomainBoundary(BasicDomain d)
    {
	domain = d;
	boundaries = new Vector(16,8);
    }
    
    
    /*********************************************************/
    
    public Vector getBoundaries()
    {
	return boundaries;
    }
    
    /**
     * returns the number of boundaries in the domain
     * @return number of boundaries
     */
    public int getBoundaryCount()
    {
	return boundaries.size ();
    }

    /**
     *  @return true if the given Boundary is already part of the 
     *  DomainBoundary
     */
    public boolean isBoundary (Boundary b)
    {
	if (boundaries != null)
	    return boundaries.contains (b);
	return false;
    }


    /**
     *  @return the last boundary of the domain, null otherwise
     */
    public Boundary getLastBoundary ()
    {
	if (!boundaries.isEmpty())
	    return (Boundary)boundaries.lastElement ();
	else
	    return null;
    }

    
    /**
     *  adds a segment to the domain boundary, as last segment to the given 
     *  boundary
     *  @param boundary boundary the segment should be inserted in
     *  @param s segment to be inserted
     *  @see Boundary.addSegment 
     */	
    public void addSegment(Boundary boundary, Segment s)
    {
	if (isBoundary (boundary))
	{
	    boundary.addSegment (s);
	    s.setBoundary (boundary);
	    domain.changed=true;
	}
    }
    
    
    public void delBoundary(Boundary b)
    {
	if (boundaries.contains (b))
	{
	    boundaries.remove (b);
	    domain.changed = true;
	}
	else
	    System.out.println ("Trying to remove a nonexisting Boundary");
    }
    
    /**
     * returns the boundary object with the index boundarynumber (0..count-1)
     */
    public Boundary getBoundary(int boundarynumber)
    {
	if (boundarynumber < getBoundaryCount ())
	{
	    Boundary b = (Boundary)boundaries.elementAt(boundarynumber);
	    return b;
	}
	else
	{
	    System.out.println ("trying to get a nonexisting Boundary");
	    return null;
	}
    }


    /**
     *  deletes a segment of the domain boundary
     *  @param segment - the segment to delete
     *  @see Boundary.delSegment
     */
    public void delSegment(Segment s)
    {
	Boundary b = s.getBoundary ();
	b.delSegment (s);
	domain.changed = true;
    }
    
    
    /**
     * returns the coordinates of a parameter value on a boundary
     * @param boundary boundary where the parameter value should be located
     * @param tsearch parameter value
     * @return node representation of the coordinates, null if it doesnt fit
     */
    public Node getCartesianCoords(Boundary boundary, double tsearch)
    {
	// figure out correct segment
	int segno = (int)tsearch;
	if (segno == boundary.getParaMax())
	    segno = 0;
	Segment s = boundary.getSegment(segno);
	// calc correct local parvalue
	double localpara = tsearch - segno;
	if (localpara > 1.0)
	    localpara -= 1.0;
	// return correct coords
	return s.getCartesianCoords(localpara);
    }

    /**
     *  resets the boundary, segment and parameter value attributes of the 
     *  given BoundaryNode according to its cartesian coords 
     *  (used for copy and paste). returns null if it doesn't fit.
     */
    public BoundaryNode reconfigureBoundaryNode (BoundaryNode n)
    {
	Enumeration bounds = boundaries.elements ();
	while (bounds.hasMoreElements ())
	{
	    Boundary actbound = (Boundary)bounds.nextElement ();
	    Enumeration segs = actbound.getSegments().elements();
	    while (segs.hasMoreElements())
	    {
		Segment actseg = (Segment)segs.nextElement();
		if (actseg.isOnBoundary (n.getCoord(0), n.getCoord(1)))
		{
		    n.setBoundary (actbound);
		    double paravalue = actseg.getParameterValue (n);
		    paravalue += actseg.getSegmentNumber ();
		    if (paravalue >= (double)actseg.getBoundary().getSegmentCount())
		    {
			paravalue -=(double)actseg.getBoundary().getSegmentCount();
			actseg = actseg.getBoundary().getSegment (0);
		    }
		    n.setParaValue (paravalue);
		    n.setSegment (actseg);
		    domain.changed = true;
		    return n;
		}
	    }
	}
	return null;
    }
    
    
    /**
     *  adds a new boundary to the domain
     */
    public void addBoundary()
    {
	Boundary b = new Boundary();
	boundaries.addElement(b);
	domain.changed = true;
    }

    /**
     *  adds the given boundary to the domain
     */
    public void addBoundary (Boundary b)
    {
	boundaries.addElement (b);
	domain.changed = true;
    }
    
    
    /**
     *  Adds multiple Boundary Nodes to the specified boundary, evenly 
     *  distributed between the given start and end parameter values
     *  with the given increment
     *
     *  @param incr - increment
     *  @param startt - start parameter value
     *  @param endt - end parameter value
     *  @param boundary - the boundary to add nodes to
     */
    public void addMultipleBoundaryNodes (double incr, double startt, double endt, Boundary boundary)
    {
	if (incr>0)			
	    for (double t=startt; t<=endt; t+=incr)
	    {
		if (t < boundary.getParaMax())
		{
		    BoundaryNode bn = new BoundaryNode(t,boundary);
		    domain.adjustNode(bn);
		    //System.out.println ("added: "+bn);
		}
	    }
	else
	{
           //   BoundaryNode bns = new BoundaryNode(startt,boundary); 
//  	    domain.adjustNode(bns);
	        
//  	    double len;
//  	    len = getBoundaryLength (0,startt,endt);
//  	    boolean found = false;
//  	    double dts = getParaMax(0);
//  	    double dt = startt;
//  	    double zilen=len/(-incr);
//  	    double zlen=zilen;
//  	    for (int i=0; i<(-incr); i++)
//  	    {
//  		zlen=zilen;
//  		found = false;
//  		dts = getParaMax(0)-dt;
//  		while (!found)
//  		{
//  		    len = getBoundaryLength (0,dt,dt+dts/2);
//  		    if (len>zlen)
//  		    {
//  			dts=dts/2;
//  		    }
//  		    else
//  		    {
//  			dt=dt+dts/2;
//  			dts=dts/2;
//  			zlen=zlen-len;
//  		    }
//  		    found = (Math.abs(len-zlen)<0.00001);
//  		}
//  		BoundaryNode bn = new BoundaryNode(dt,boundary); 
//  		domain.adjustNode(bn);
//  	    }
	}
	domain.updateNodeNumbers();
	domain.changed = true;
    }
    
    /**
     * draws the whole domain boundary
     * @param g graphics context
     * @return none
     */	
    public void draw(Graphics g)
    {
	Enumeration bounds = boundaries.elements ();
	while (bounds.hasMoreElements ())
	{
	    Boundary actbound = (Boundary)bounds.nextElement ();
	    Enumeration segs = actbound.getSegments().elements();
	    while (segs.hasMoreElements())
	    {
		((Segment)segs.nextElement()).draw (g);
	    }
	}
    }
    
    
    public void updateNumbering()
    {
	domain.updateCellNumbers();
	domain.updateEdgeNumbers();
	domain.updateNodeNumbers();
	
	Enumeration enum = domain.getNodes().elements();
	while (enum.hasMoreElements())
	{
	    Node n = (Node)enum.nextElement();
	    if (n instanceof BoundaryNode)
		((BoundaryNode)n).updateBoundaryNode();
	}
	Enumeration bounds = boundaries.elements ();
    }


    /**
     *  swaps/inverts the orientation of the given boundary
     *
     *  @param index - number of the boundary to swap
     */
    public void swapOrientation(Boundary b)
    {

        StringBuffer ges = new StringBuffer();

  	ges.append(cc.wS("swapOrientation_startmessage")+"\n");
  	updateNumbering();
  	//continousBoundary();
  	for(int j=0; (j<b.getSegmentCount()); j++)
  	{
            ges.append(cc.wS("swapOrientation_procsegment")+(j+1)+"\n");
  	    Segment s1 = b.getSegment(j);
  	    s1.swapPerimeter();
  	}
	
  	int maxs=b.getSegmentCount();
  	for(int j=0; (j<maxs/2); j++)
  	{
            ges.append(cc.wS("swapOrientation_change1")+(j+1)
	      +cc.wS("swapOrientation_change2")+(maxs-j)+"\n");
  	    Segment temp1 = new SegmentLine();
  	    Segment temp2 = new SegmentLine();
  	    Segment cs1  = b.getSegment(j);
	
  	    if (cs1 instanceof SegmentLine)
  		temp1=(Segment)((SegmentLine) cs1).clone();
  	    if (cs1 instanceof SegmentCircle)
  		temp1=(Segment)((SegmentCircle) cs1).clone();
	
  	    Segment cs2  = b.getSegment((maxs-j-1));
	
  	    if (cs2 instanceof SegmentLine)
  		temp2=(Segment)((SegmentLine) cs2).clone();
  	    if (cs2 instanceof SegmentCircle)
  		temp2=(Segment)((SegmentCircle) cs2).clone();
	
  	    b.segments.setElementAt(temp2,j);
  	    b.segments.setElementAt(temp1,(maxs-j-1));
	
  	}
	
  	for(int j=0;j<domain.getNodeCount();j++)
  	    if (domain.getNode(j) instanceof BoundaryNode)
  	    {
                ges.append(cc.wS("swapOrientation_procnode")+(j+1)+"\n");
  		BoundaryNode bn = (BoundaryNode)domain.getNode(j);
  		if (bn.getBoundary()==b) bn.swap();
  	    }
	
  	updateNumbering();

        ges.append("---------------------\n");
 	
	ges.append(cc.wS("swapOrientation_ok")+"\n");


        TextFrame.initialize (cc.gc.mf, "swapOrientation", "waiting for the sun...");
	TextFrame.setText (ges.toString());
	TextFrame.showDialog ();
	
		  
	 cc.gc.mf.getDrawingArea().repaint();
	
    }
    
    
    
    public int linkBoundaries()
    {
	//  double[] tmp = {16,16};
	
//  	//    double[] fs = ((Transformation2D)domain.tf).rescale(tmp);
//  	double[] fs = {0,0};
	
//  	double eps = domain.getEps();
	
//  	//	System.out.println("link Boundaries: starting ...");
	
//  	for(int i=0; i<BoundaryCount; i++)
//  	{
//  	    Boundary b = getBoundary(i);
	    
//  	    //System.out.println("Test auf Circle am Ende/Anfang fehlt!");
	    
//  	    Boundary nb = new Boundary();
	    
//  	    int[] disc = new int[32];
//  	    int discidx = 0 ;
	    
//  	    for(int j=0; (j<b.SegmentCount); j++)
//  	    {
		
//  		//System.out.println("Processing Segment "+j);
		
//  		boolean cont = true;
		
//  		Segment s1 = new SegmentLine();
//  		Segment s2 = new SegmentLine();
		
//  		Node s11 = new Node();
//  		Node s20 = new Node();
//  		Node s21 = new Node();
		
//  		boolean found1,found2;
		
//  		found1=false;
//  		found2=false;
		
//  		s1 = getSegment(i,j);
		
//  		if ((j+1)!=b.SegmentCount)
//  		    s2 = getSegment(i,j+1);
//  		else
//  		    s2 = getSegment(i,0);
		
//  		s11 = s1.getCartesianCoords(1);
		
//  		s20 = s2.getCartesianCoords(0);
//  		s21 = s2.getCartesianCoords(1);
		
//  		found1 = s11.equals(s20,0.0000001);
//  		found2 = s11.equals(s21,0.0000001);
		
//  		if (!found1 && !found2) 
//  		{
//  		    disc[discidx++]=j+1;
//  		    //          System.out.println("Discountinuity found at "+j+"/"+(j+1));
//  		}
		
//  	    }
	    
//  	    //          System.out.println("\n\nTrying to resolve discountinuity\n\n");
	    
//  	    if (discidx!=3)
//  	    {
//  		//          box = new AlertBox(domain.root,this,300,150,"ERROR",
//  		//          	"Can't resolve discountinuity!","Cancel");
//  		//          box.show();
//  		System.out.println("\n\nCan't resolve discountinuity!\n\n");
		
//  		return 0;
//  	    }
//  	    else
//  	    {
		
//  		for(int j=1;j<=disc[0];j++) 
//  		{
//  		    //          System.out.println("TRF : nb["+j+"]=b["+j+"]");
//  		    nb.addSegment(getSegment(0,-1+j));
//  		}
		
//  		for(int j=disc[0]+1;j<=disc[0]+1+(disc[2]-disc[1]-1+1)-1;j++)
//  		{ 
//  		    //          System.out.println("TRF : nb["+j+"]=b["+(j-disc[0]-1+disc[1]+1)+"]");
//  		    nb.addSegment(getSegment(0,-1+(j-disc[0]-1+disc[1]+1)));
//  		}
		
		
//  		for(int j=disc[0]+1+(disc[2]-disc[1]-1+1)-1+1;
//  		    j<=b.getSegmentCount();j++)
//  		{ 
//  		    nb.addSegment(getSegment(0,
//  					     -1+(j-(disc[0]+1+(disc[2]-disc[1]-1+1)-1+1)+disc[0]+1)));
//  		}
		
//  		for(i=0;i<domain.getNodeCount();i++)
//  		{
//  		    Node n = domain.getNode(i);
//  		    if (n instanceof BoundaryNode)
//  		    {
//  			double t=((BoundaryNode)n).getParaValue();
			
//  			if ((t>=disc[1]-1) && (t<=b.getSegmentCount()))
//  			    ((BoundaryNode)n).setParaValue(t-(disc[1]-1)+(disc[0]-1),this);
			
//  			if ((t>=disc[0]) && (t<=disc[1]-1))
//  			    ((BoundaryNode)n).setParaValue(
//  							   t-(disc[0])+b.getSegmentCount()-(disc[1]-disc[0]),this);
			
//  		    }
//  		}
		
//  		boundaries.setElementAt(nb,0);
		
//  		updateNumbering();
		
//  	    }
	    
//  	}
	
	return 0;
	
    }
    
    
    public int continousBoundary()
    {
	int[] tmp = domain.getPerimeter();
	
        int xlen=tmp[2]-tmp[0];
        int ylen=tmp[3]-tmp[1];

	double fs = Math.min(xlen*0.05,ylen*0.05);     // Fehler trafoabhaengig machen
	fs=cc.gc.eps;
	
        System.out.println("EPS FK "+fs);

        StringBuffer ges = new StringBuffer();

  	ges.append(cc.wS("continousBoundary_startmessage")+"\n");
  	for(int i=0; i<getBoundaryCount(); i++)
  	{
  	    Boundary b = getBoundary(i);
  	    boolean fault = false;
  	    ges.append(cc.wS("continousBoundary_procboundary")+(i+1)+"\n");

  	    for(int j=0; (j<b.getSegmentCount()) && !fault; j++)
  	    {
  	        ges.append(cc.wS("continousBoundary_procsegment")+(i+1)+"\n");
  		boolean cont = true;
  		Segment s1 = new SegmentLine();
  		Segment s2 = new SegmentLine();
		
  		Node s11 = new Node();
  		Node s20 = new Node();
  		Node s21 = new Node();
		
  		boolean found1,found2;
		
  		found1=false;
  		found2=false;
  		for(;cont && !fault;)
  		{
  	            s1 = b.getSegment(j);
  	            if ((j+1)!=b.getSegmentCount())
  			s2 = b.getSegment(j+1);
  	            else
  			s2 = b.getSegment(0);
		    
  	            s11 = s1.getCartesianCoords(1);
  	            s20 = s2.getCartesianCoords(0);
  	            s21 = s2.getCartesianCoords(1);
		    
  	            found1 = s11.equals2(s20,fs);
  	            found2 = s11.equals2(s21,fs);
  	            if (!found1 && !found2)
  	            {
     	                ges.append(cc.wS("continousBoundary_bonderror")+"\n");
  			boolean found3=false;
  			int k=0;
  			for(k=j+2;(k<b.getSegmentCount()) && !found3;k++)
  			{
  			    Segment s3 = b.getSegment(k);
  			    Node s30 = s3.getCartesianCoords(0);
  			    Node s31 = s3.getCartesianCoords(1);
  			    boolean found31 = s11.equals2(s30,fs);
  			    boolean found32 = s11.equals2(s31,fs);
  			    found3=found31 || found32;
  			}
			
  			if (found3)
  			{
     	                  ges.append(cc.wS("continousBoundary_segfound")+"\n");
			    
  			    Segment temp1 = new SegmentLine();
  			    Segment temp2 = new SegmentLine();
  			    Segment cs1 = b.getSegment(k-1);
  			    if (cs1 instanceof SegmentLine)
  				temp1=(Segment)((SegmentLine) cs1).clone();
  			    if (cs1 instanceof SegmentCircle)
  				temp1=(Segment)((SegmentCircle) cs1).clone();
  			    Segment cs2  = b.getSegment(j+1);
  			    if (cs2 instanceof SegmentLine)
  				temp2=(Segment)((SegmentLine) cs2).clone();
  			    if (cs2 instanceof SegmentCircle)
  				temp2=(Segment)((SegmentCircle) cs2).clone();
			    
  			    b.segments.setElementAt(temp2,k-1);
  			    b.segments.setElementAt(temp1,j+1);
  			    cont=true;
			    
  			    adjustBoundaryNodes(b,k-1,j+1);
			    
  			}
  			else
  			{
			  ges.append("---------------------\n");
      	                  ges.append(cc.wS("continousBoundary_error")+"\n");

	  	
  			    fault=true;
  			    cont=false;
			    
			    
        TextFrame.initialize (cc.gc.mf, "ContinousBoundary", "waiting for the sun...");
	TextFrame.setText (ges.toString());
	TextFrame.showDialog ();
	
  			    return -1;
  			}
  	            }
  	            else
  			cont=false;						
  		}  
		
  		if (!fault)
  		{						
  	            if (found1)
  	            {
      	                  ges.append(cc.wS("continousBoundary_action1")+"\n");
  			s2.adjustPerimeter(s11,0);
  	            }
  	            else if (found2)
  	            {
      	                  ges.append(cc.wS("continousBoundary_action2")+"\n");
  			s2.swapPerimeter();
  			s2.adjustPerimeter(s11,0);
  	            }
  	            else
      	                  ges.append(cc.wS("continousBoundary_error")+"\n");
  		}
  	    }
  	}
				  ges.append("---------------------\n");

      	                  ges.append(cc.wS("continousBoundary_ok")+"\n");

	
        cc.gc.mf.getDrawingArea().repaint();
	  	
  	updateNumbering();
	
        TextFrame.initialize (cc.gc.mf, "ContinousBoundary", "waiting for the sun...");
	TextFrame.setText (ges.toString());
	TextFrame.showDialog ();
	    
	    
	return 0;
	
    }
    
    /**
     *  adjusts all boundary nodes on boundary bn
     */
    public void adjustBoundaryNodes(Boundary bn, int von, int bis)
    {
        for(int j=0;j<domain.getNodeCount();j++)
  	{
  	    Node temp = domain.getNode(j);
	    
  	    if (temp instanceof BoundaryNode)
  	    {
  		BoundaryNode btemp = (BoundaryNode)temp;
		
  		if (btemp.getBoundary()==bn)
  		{
		    
  		  // 		    System.out.println("Swapping Boundary Node ...");
		    
  		    double t = btemp.getParaValue();
		    
  		    if ((t>=(double)von) && (t<=(double)(von+1))) 
  			t=1.0-(t-(double)von)+(double)bis;
  		    else								
  			if ((t>=(double)bis) && (t<=(double)(bis+1))) 
  			    t=1.0-(t-(double)bis)+(double)von;
		    
  		    btemp.setParaValue(t);
		    
  		}
  	    }
  	}
    }
    
    /**
     * gets the perimeter of the domain boundary
     * @return double[4]  perimeter (xmin,ymin,xmax,ymax)
     */
    public int[] getPerimeter()
    {
	  int[] erg = new int[4];
	  	Rectangle temperg;
	  	int i,j;
	
	        temperg=new Rectangle();
		
	  	erg[0]=100000000;
	  	erg[1]=100000000;
	  	erg[2]=0;
	  	erg[3]=0;
	  	for(i=0; i<getBoundaryCount(); i++)
		{
		  Boundary b = getBoundary(i);
		  
	  	  for(j=0; j<b.getSegmentCount(); j++)
		{
	
	  		    if (b.getSegment(j) instanceof SegmentLine)				
	  			temperg=((SegmentLine)b.getSegment(j)).getPerimeter();
	          else if (b.getSegment(j) instanceof SegmentCircle)
	            temperg=((SegmentCircle)b.getSegment(j)).getPerimeter();
	  //        else if (getSegment(i,j) instanceof SegmentFile)
	  //          temperg=((SegmentFile)getSegment(i,j)).getPerimeter();
	          else
	          {
	            System.out.println("Yet not implementedA!");
	          }
	
	          if (temperg.x<erg[0]) erg[0]=temperg.x;
	          if (temperg.y<erg[1]) erg[1]=temperg.y;
	          if ((temperg.x+temperg.width)>erg[2]) 
		    erg[2]=(temperg.x+temperg.width);
	          if ((temperg.y+temperg.height)>erg[3]) 
		    erg[3]=(temperg.y+temperg.height);
	        }
	}
	      return erg;
    }
    
    
    /**
     * returns the possible intersection point with an line:
     * @param double x[0] x startpoint
     * @param double x[1] y startpoint
     * @param double x[2] x endpoint
     * @param double x[3] y endpoint
     * @return double[0]  flag, <0 intersection
     * @return double[1/2] intersection point
     * @return (int)double[3] boundary number
     * @return (int)double[4] segment number
     */
    public int[] getIntersection(int[] x)
    {
	//  int[] erg = new int[5];
//  	int[] temperg= new int[5];
//  	int i,j;
	
//  	for(i=0; i<BoundaryCount; i++)
//  	    for(j=0; j<((Boundary)boundaries.elementAt(i)).SegmentCount; j++)
//  	    {
//  		if (getSegment(i,j) instanceof SegmentLine)
//  		{
//  		    temperg=((SegmentLine)getSegment(i,j)).getIntersection(x);
		    
//  		    if ((temperg[0]<0.0) && 
//  			(domain.getNodeIndex(new Node(temperg[1],temperg[2]))==-1))
//  		    {
//  			temperg[3]=i;
//  			temperg[4]=j;
//  			return temperg;
//  		    }
//  		}				
//  		else
//  		{
//  		    //		    System.out.println("Yet not implementedB!");
//  		    temperg = new int[4];
//  		}
		
//  	    }
	
//  	return temperg;
	return null;
    }
    
    public double getBoundaryLength(Boundary b, double dt1, double dt2)
    {
	//  int ip1 = (int)Math.ceil(dt1);
//  	int ip2 = (int)Math.floor(dt2);
	
//  	double dx1=0.0; 
//  	double dy1=0.0;
	
//  	double dx2=0.0; 
//  	double dy2=0.0;       
	
//  	double dbl=0.0;
	
//  	if (ip2<ip1)
//  	{
	    
//  	    Node x1=getCartesianCoords(b,dt1);
//  	    Node x2=getCartesianCoords(b,dt2);
	    
//  	    dx1=x1.getCoord(0); 
//  	    dy1=x1.getCoord(1);
	    
//  	    dx2=x2.getCoord(0); 
//  	    dy2=x2.getCoord(1);       
	    
//  	    dbl=Math.sqrt((dx2-dx1)*(dx2-dx1)+(dy2-dy1)*(dy2-dy1));
//  	}
//  	else
//  	{
//  	    if (dt2<dt1)
//  	    {
//  		double dtmax=getParaMax(bidx);
		
//  		dbl=getBoundaryLength(bidx,dt1,dtmax);
//  		dbl=dbl+getBoundaryLength(bidx,dtmax,dt2);
//  	    }
//  	    else
//  	    {
		
//  		dx1=0.0; 
//  		dy1=0.0;
		
//  		dx2=0.0; 
//  		dy2=0.0;    
		
//  		if (((double)ip1)==dt1)
//  		{
//  		    Node x2=getCartesianCoords(bidx,dt1);
//  		    dbl=0.0;
		    
//  		    dx2=x2.getCoord(0); 
//  		    dy2=x2.getCoord(1);   
//  		}
//  		else
//  		{
//  		    Node x1=getCartesianCoords(bidx,dt1);
//  		    Node x2=getCartesianCoords(bidx,(double)ip1);
		    
//  		    dx1=x1.getCoord(0); 
//  		    dy1=x1.getCoord(1);
		    
//  		    dx2=x2.getCoord(0); 
//  		    dy2=x2.getCoord(1);    
//  		    dbl=Math.sqrt((dx2-dx1)*(dx2-dx1)+(dy2-dy1)*(dy2-dy1));
//  		}
		
//  		for(int i=ip1+1;i<=ip2;i++)
//  		{
//  		    dx1=dx2;
//  		    dy1=dy2;
		    
//  		    Node x2=getCartesianCoords(bidx,(double)i);
//  		    dx2=x2.getCoord(0); 
//  		    dy2=x2.getCoord(1);    
		    
//  		    dbl=dbl+Math.sqrt((dx2-dx1)*(dx2-dx1)+(dy2-dy1)*(dy2-dy1));
		    
//  		}
		
//  		if (((double)ip2)!=dt2)
//  		{
//  		    Node x2=getCartesianCoords(bidx,dt2);
//  		    dx1=x2.getCoord(0); 
//  		    dy1=x2.getCoord(1);    
		    
//  		    dbl=dbl+Math.sqrt((dx1-dx2)*(dx1-dx2)+(dy1-dy2)*(dy1-dy2));
//  		}
		
//  	    }
	    
//  	}
	
//  	return dbl;
	return 0;
    }

    /**
     *  calculates the crossproduct of the vectors in the given arrays
     *  at indices a1,a2 and a3
     *  @param
     *  @return the crossproduct of the two vectors 
     *          (nodes[a1]-nodes[a2]) and (nodes[a3]-nodes[a1])
     */
    public double crossProduct (int a1, int a2, int a3, Node[] store)
    {
	double[] v1 = {(double)store[a1].getCoord(0)-(double)store[a2].getCoord(0),(double)store[a1].getCoord(1)-(double)store[a2].getCoord(1)};
	double[] v2 = {(double)store[a3].getCoord(0)-(double)store[a2].getCoord(0),(double)store[a3].getCoord(1)-(double)store[a2].getCoord(1)};
	return v1[0]*v2[1]-v1[1]*v2[0];
    }

    /**
     *  prints out the DomainBoundary
     */
    public void printDomainBoundary ()
    {
	System.out.println ("DomainBoundary");
	Enumeration bounds = boundaries.elements ();
	while (bounds.hasMoreElements ())
	{
	    Boundary b = (Boundary)bounds.nextElement();
	    System.out.println ("Boundary "+b.getNumber()+" w/ "+b.getSegmentCount()+" Segments w/ "+b.getParaMax()+" tmax.");
	    Enumeration segs = b.getSegments().elements();
	    while (segs.hasMoreElements())
	    {
		Segment s = (Segment)segs.nextElement();
		System.out.println ("Segment "+s.getSegmentNumber()+" @Boundary "+s.getBoundaryNumber());
	    }
	    System.out.println();
	}
    }
    
}













