package devisor2.foundation.boundary;

import devisor2.foundation.base.*;
import devisor2.grid.options.*;

import java.util.Vector;
import java.io.*;
import java.awt.*;


/**
 *  This class represents one Boundary of the Domain. All Boundaries form the
 *  DomainBoundary (see there for details), and every Boundary contains
 *  several Segments. <br>
 *  A Boundary has a natural parametrisation: Every segment has parameter
 *  values from 0 inclusive to 1 exclusive, thus the boundary has parameter 
 *  values (or a total length if you prefer that term) from 0 to its number of 
 *  segments. <br>
 *  This class provides methods to add and remove Segments from the Boundary,
 *  to add and remove BondaryNodes, and to support copy and paste.
 *
 *  @author Christian Becker, Dominik Goeddeke
 *
 *  @version 0.9
 */
public class Boundary implements java.io.Serializable
{
    /**
     * the segments itselves
     */	
    public Vector segments;


    /**
     *  the ControlCenter reference
     */
    protected ControlCenter cc;
	
	
/*********************************************************/


    /**
     * Creates a new (empty) boundary
     */	
    public Boundary()
    {
	segments = new Vector(16,8);
	cc = ControlCenter.getMyself ();
    }
    
    
    /*********************************************************/
    
    /**
     *  @return the segments of this boundary
     */
    public Vector getSegments ()
    {
	return segments;
    }

    
    /**
     * returns the number of segments in a boundary
     * @return number of segments
     */	
    public int getSegmentCount()
    {
	return segments.size();
    }

    /**
     *  @return the boundary number, -1 if not part of the domainboundary
     */
    public int getNumber ()
    {
	if (cc.d.db.isBoundary (this))
	    return cc.d.db.getBoundaries().indexOf (this);
	else
	    return -1;
    }
    
    /**
     *  @return the maximal parameter value of this boundary
     */		
    public double getParaMax()
    {
	return segments.size ();
    }
    
    
    /**
     * returns the segmentnumber-th segment in a boundary
     * @param segmentnumber segment number from 0 to getSegmentCount()-1
     * @return null if not found
     */	
    public Segment getSegment(int segmentnumber)
    {
	if (segmentnumber < getSegmentCount ())
	    return (Segment)segments.elementAt(segmentnumber);
	else
	{
	    //	    System.out.println ("Boundary.getSegment: segment number "+ segmentnumber+" not found");
	    return null;
	}
    }
    
    /**
     * adds a segment as last segment to the Boundary
     * @param s segment to be inserted
     */	
    public void addSegment(Segment s)
    {
	if (s != null)
	    segments.add (s);
    }
    
    
    /**
     * deletes the given segment 
     * @param segment - the useless segment to be kicked out
     */	
    public void delSegment(Segment s)
    {
	if (segments.contains(s))
	    segments.remove (s);
	else
	    System.out.println ("Boundary.delsegment: trying to delete a nonexisting segment");
    }

    /**
     *  clones the Boundary shallowly
     */
    public Object clone ()
    {
	Boundary b = new Boundary ();
	b.segments = segments;
	return b;
    }

    /**
     *  @return a String representation of this boundary
     */
    public String toString ()
    {
	return "Boundary "+getNumber()+" w/ "+getSegmentCount()+" segments and length "+getParaMax();
    }
 


   
}















