package devisor2.foundation.boundary;

import devisor2.foundation.elements.Node;
import devisor2.foundation.base.*;
import devisor2.grid.backend.*;

import java.awt.*;
import java.awt.geom.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;


/**
 *  This class BoundaryNode represents a BoundaryNode of a grid, which is more
 *  special than a regular node, because its position is not controlled via 
 *  the coordinates, but via a parameter value on the boundary segment.
 *
 *  @author Christian Becker, Dominik Goeddeke
 *
 *  @version 0.9
 */
public class BoundaryNode extends Node implements java.io.Serializable
{

    /**
     *  parameter value of the node on the specific boundary
     *  (range: 0 to segmentcount of the specific boundary)
     */
    public double t;
    
    /**
     *  the boundary the node lies on
     */
    public Boundary boundary;

    /**
     *  the segment the node lies on
     */
    public Segment segment;
    
    
         
    /*********************************************************/
    
    /**
     * Creates a boundary node with parameter value 0
     */
    public BoundaryNode()
    {
	t=0;
	coord = new int[2];
	setCoord(0,0);
	parents = new Vector(4,4);
	Segment s = new SegmentLine();
	segment = s;
    }

    /**
     * Creates a boundary node with parameter value tt on boundary 
     * bn of the DomainBoundary
     * @param tt parameter value
     * @param bn boundary
     */	
    public BoundaryNode(double tt, Boundary bn)
    {
	int segno = (int)tt;
	if (segno == bn.getSegmentCount())
	    segno = 0;
	Segment s = bn.getSegment (segno);
	Node tempnode = cc.d.db.getCartesianCoords(bn,tt);
	setCoord(tempnode.getCoord(0),tempnode.getCoord(1));
	t=tt;
	boundary = bn;
	segment = s;
	parents=new Vector(4,4);
    }
    

    /**
     * Creates a boundary node with parameter value tt on boundary 
     * bn of the DomainBoundary
     * @param tt parameter value
     * @param bn boundary
     * @param s segment to be placed on
     */	
    public BoundaryNode(double tt, Boundary bn, Segment s)
    {
	Node tempnode = cc.d.db.getCartesianCoords(bn,tt);
	setCoord(tempnode.getCoord(0),tempnode.getCoord(1));
	t=tt;
	boundary = bn;
	segment = s;
	parents=new Vector(4,4);
    }

    /*********************************************************/
    
    
    /**
     *  returns the number of the defining boundary
     *  @return boundary number 
     */
    public int getBoundaryNumber()
    {
        return boundary.getNumber();
    }

    /**
     *  returns the parameter value of this node
     *  @return parameter value
     */
    public double getParaValue()
    {
	return t;
    }

    /**
     *  sets the parameter value of this boundary node
     *  and updates coordinates accordingly
     */
    public void setParaValue(double tt)
    {
	t=tt;
    }

    /**
     * returns the boundary of this node
     * @return boundary 
     */
    public Boundary getBoundary()
    {
	return boundary;
    }
        
    /**
     * sets the boundary of this node
     * @param new boundary
     */
    public void setBoundary(Boundary bn)
    {
	boundary = bn;
    }

    /**
     *  returns the segment the node lies on
     */
    public Segment getSegment ()
    {
	return segment;
    }

    /**
     *  sets the segment to the new value
     */
    public void setSegment (Segment s)
    {
	segment = s;
    }

    /**
     * sets the cartesian coordinates of the node according 
     * to the parameter value
     */
    public void updateBoundaryNode()
    {
	Node tempnode = cc.d.db.getCartesianCoords (boundary,t);
	setCoord(tempnode.getCoord(0),tempnode.getCoord(1));
	int segno = (int)t;
	if (segno == boundary.getSegmentCount())
	    segno = 0;
	segment = boundary.getSegment (segno);
    }
    
    
    /**
     *  swaps the parameter value of this node
     */
    public void swap()
    {
	double maxt = boundary.getParaMax();
	t=maxt-t;
	Node tempnode = cc.d.db.getCartesianCoords(boundary,t);
	setCoord(tempnode.getCoord(0),tempnode.getCoord(1));
	int segno = (int)t;
	if (segno == boundary.getSegmentCount())
	    segno = 0;
	segment = boundary.getSegment (segno);
    }

    /**
     *  @return a String representation of this node
     */
    public String toString ()
    {
	return "BN @boundary "+boundary.getNumber()+" @segment "+segment.getSegmentNumber()+" w/ par "+t;
    }

    /** 
     * returns a copy of this boundary node
     */
    public Object clone() 
    {
	BoundaryNode temp = new BoundaryNode();
	temp.setCoord(coord[0],coord[1]);
	if (value != null) 
	{
	    int l = value.length;
	    temp.value = new double[l];
	    for (int i=0; i<l; i++) 
	    {
		temp.setValue(i, getValue(i)); 
	    }
	}
	temp.setNumber(number);
	temp.setMarker(marker);
	temp.setMarker (this.isMarked ());
	temp.t=t;
	temp.boundary = boundary;
	temp.segment = segment;
	return temp;
    }
}






