package devisor2.foundation.boundary;

import devisor2.foundation.elements.*;
import devisor2.foundation.*;
import devisor2.grid.backend.*;
import java.awt.*;
import java.awt.geom.*;
import java.io.*;

/**
 * This class Segment builds the super class for the boundary segments.
 * A segment is an atomic unit of the boundary curve.
 *
 * @author Christian Becker, Dominik Goeddeke
 *
 */
public abstract class Segment extends GridItem implements java.io.Serializable
{

    /**
     * length of the segment
     */
    protected double tmax;
    

    /**
     *  first point (world coords) of the orientation marker triangle
     */
    protected int[] orientation_marker_1;

    /**
     *  second point (world coords) of the orientation marker triangle
     */
    protected int[] orientation_marker_2;

    /**
     *  third point (world coords) of the orientation marker triangle
     */
    protected int[] orientation_marker_3;

    /**
     * number of Segment. can be read by getSegmentNumber().
     */
    //  protected int segmentnumber;
    
    
    /**
     * link to boundary to which this segment belongs
     */
    protected Boundary boundary;
    
    /*********************************************************/
    
    
    /**
     * creates a segment with length stmax
     */
    public Segment(double stmax)
    {
	tmax=stmax;
    }
    

    /**
     *  default constructor
     */
    public Segment ()
    {
	super ();
    }
    
  
    /*********************************************************/
    
    /**
     *  sets the maximal parameter value to the given value
     */
    public void setTMax (double value)
    { 
	tmax = value;
    }

    /**
     *  @return the current max parameter value
     */
    public double getTMax ()
    {
	return tmax;
    }

    /**
     * return internally saved boundary number
     */
    public int getBoundaryNumber()
    {
	return boundary.getNumber();
    }


    /**
     *  @return the boundary to which this segment belongs
     */
    public Boundary getBoundary ()
    {
	return boundary;
    }

    /**
     * sets the boundary of this segment
     */
    public void setBoundary (Boundary b)
    {
	this.boundary = b;
    }
    
    /**
     * @return number of the segment in the boundary
     */
    public int getSegmentNumber()
    {
	return boundary.getSegments().indexOf (this);
    }
    
    /**
     *  gets the cartesian coordinates of the node according to parameter t
     *  @param t parameter value
     *  @return Node representation of the coordinates
     */
    public abstract Node getCartesianCoords(double t);
    
    
    /**
     * draws the segment 
     * @param g  - the graphics context to paint to
     */
    public abstract void draw(Graphics g);
    
    
    /**
     *  gets the perimeter of the segment
     */
    public abstract Rectangle getPerimeter();
    
    
    /**
     *  returns whether or not a point lies on a boundary segment
     *  @param x,y - world coord points to check
     *  @return true if the Point is on the segment, false otherwise
     */
    public abstract boolean isOnBoundary(int x, int y);
    
    /**
     *  calculate parameter value of node 'n' from its x/y
     *  coordinates (note: only the relative parameter value is calculated,
     *  the boundary-wide value is not considered)
     */
    public abstract double getParameterValue (Node n);
    
    /**
     * inverts orientation of this segment without changing
     * parameter values of related boundarynodes.
     */
    public abstract void swapPerimeter();
    
    /**
     * change coordinates therefor, that at indes 0 that
     * the beginning of this segment is at node 'n', else 
     * that the ending of this segment is at node 'n'.
     */
    public abstract void adjustPerimeter(Node n, int index);

    /**
     *  Is the item inside a given rectangular area?
     *  All in WORLD COORDINATES!!!
     *
     *  @param topleft - the topleft corner of the rectangle
     *  @param bottomright - the bottom right corner of the rectangle
     *  @return true if the item is completely inside the rectangle
     */
    public abstract boolean isInside (int[] topleft, int[] bottomright);


    /**
     *  Determines if the item has been hit by a click on the grid
     *  ALL IN WORLD COORDINATES to avoid the snap mechanism
     *
     *  @param x,y - coordinates of the event point where the click
     *         occurred
     *  @return true if the item has been hit
     */
    public abstract boolean isHit (int x, int y);

    /**
     *  updates the inner data used for hit detection etc. Needs to be called
     *  after every change to the Segment
     */
    public abstract void updateInnerData ();

    /**
     *  applies the Transformation to the Segment and calls updateInnerData
     */
    public abstract void performTransformation (GridTransformation trafo);

    /**
     *  This method mirrors the grid item at the given line
     *  @param n - the normalized normal of the line (length 1)
     *  @param p - an arbitrary point on the line
     */
    public abstract void performMirror (double nx, double ny, double px, double py);


    /**
     *  creates a shallow clone of this segment
     */
    public abstract Object clone ();
    
    /**
     *  sets the parameters of the segment
     */
    public abstract void setParameters(double ux[]);
    
    /**
     *  gets the parameters of the segment
     */
    public abstract double[] getParameters();

    /**
     *  sets the parameters of the segment
     */
    public abstract void setParams(int ux[]);
    
    /**
     *  gets the parameters of the segment
     */
    public abstract int[] getParams();

    /**
     *  draws the orientation marker triangle
     *  using the coordinates in the three orientation_marker local attributes.
     *  these attributes have to be set by the updateInnerData method.
     */
    public void drawOrientationMarker (Graphics g)
    {
	if (cc.dc.draw_orientationmarkers)
	{
	    int[] p1 = cc.gc.fromWorldToScreen (orientation_marker_1[0], orientation_marker_1[1]);
	    int[] p2 = cc.gc.fromWorldToScreen (orientation_marker_2[0], orientation_marker_2[1]);
	    int[] p3 = cc.gc.fromWorldToScreen (orientation_marker_3[0], orientation_marker_3[1]);
	    g.drawLine (p1[0], p1[1], p2[0], p2[1]);
	    g.drawLine (p2[0], p2[1], p3[0], p3[1]);
	    g.drawLine (p3[0], p3[1], p1[0], p1[1]);
	}
    }


}








