package devisor2.foundation.boundary;

import devisor2.foundation.elements.*;
import devisor2.grid.options.*;
import devisor2.grid.GUI.framework.*;
import devisor2.grid.backend.*;
import java.awt.*;
import java.awt.geom.*;
import javax.swing.*;

import java.io.*;
import java.util.*;


/**
 *  This class represents the Segment type CIRCLE. As always, coordinates are
 *  all given in world space (the big ints). A circle is completely defined
 *  by the midpoints, its radius, and a start- and end angle of the arc. 
 *  These angles are all measured in degrees against the three o'clock 
 *  position.
 *
 *  @author Christian Becker, Dominik Goeddeke
 *  @see Segment
 */
public class SegmentCircle extends Segment implements java.io.Serializable
{
  
    /**
     * middle point of the circle
     */
    private int x,y;
  
    /**
     * radius of the circle
     */
    private int r;
  
    /**
     * arcs of the circle (in degrees, NOT in radians!)
     */
    private double sa,sb;

    /**
     *  location of the numbering information
     */
    private int[] numberpos;
    
  
    /*********************************************************/
  
  
    /**
     *  creates the segment type circle
     * @param tx,ty middle point of the circle
     * @param tr radius
     * @param tsa,tsb start/stop arcs of the circle
     */
    public SegmentCircle (int tx, int ty, int tr, double tsa, double tsb)
    {
	super (1.0);
	x = tx;
	y = ty;
	r = tr;
	sa = tsa;
	sb = tsb;
	updateInnerData ();
    }
    
  
    /**
     * creates an empty SegmentCircle
     */	
    public SegmentCircle()
    {
	super (0);
	x = 0; y = 0;
	r = 0;
	sa = 0;
	sb = 360;
	updateInnerData ();
    }
    

    /*********************************************************/
    
 
     /**
     *  sets the parameters of the Circle
     */
    public void setParameters(double ux[])
    {
	x = (int)ux[0];
	y = (int)ux[1];
	r = (int)ux[2];
	sa = ux[3];
	sb = ux[4];
	updateInnerData ();
    }


     /**
     *  gets the parameters of the circle
     */
    public double[] getParameters()
    {
        double[] ux = new double[5];
	ux[0]=x;
	ux[1]=y;
	ux[2]=r;
	ux[3]=sa;
	ux[4]=sb;
	return ux;
    }
  /**
     *  sets the parameters of the Circle
     */
    public void setParams (int[] ux)
    {
	x = ux[0];
	y = ux[1];
	r = ux[2];
	sa = ux[3];
	sb = ux[4];
	updateInnerData ();
    }


     /**
     *  gets the parameters of the circle
     */
    public int[] getParams()
    {
        int[] ux = new int[5];
	ux[0]=x;
	ux[1]=y;
	ux[2]=r;
	ux[3]=(int)sa;
	ux[4]=(int)sb;
	return ux;
    }

          
    /**
     * returns the radius of the circle (WORLD COORDS)
     */
    public int getRadius ()
    {
	return r;
    }

    /**
     *  sets the radius to the given value (WORLD COORDS)
     */
    public void setRadius (int newr)
    {
	r = newr;
	updateInnerData ();
    }

    /**
     * returns the midpoint of the circle (WORLD COORDS)
     */
    public int[] getMidPoint ()
    {
	int[] a = new int[2];
	a[0] = x;
	a[1] = y;
	return a;
    }

    /**
     *  sets the midpoint to the given parameters (WORLD COORDS)
     */
    public void setMidPoint (int xx, int yy)
    {
	x = xx;
	y = yy;
	updateInnerData ();
    }

    /**
     *  returns the start angle (in degrees)
     */
    public double getStartAngle ()
    {
	return sa;
    }

    /**
     * sets the start angle (in degrees)
     */
    public void setStartAngle (double angle)
    {
	sa = angle;
	updateInnerData ();
    }

    /**
     *  returns the end angle (in degrees)
     */
    public double getEndAngle ()
    {
	return sb;
    }

    /**
     * sets the end angle (in degrees)
     */
    public void setEndAngle (double angle)
    {
	sb = angle;
	updateInnerData ();
    }


    /************************************************************/    
    

    /**
     * @see Segment.swapPerimeter
     */
    public void swapPerimeter()
    {
	double temp=sa;
	sa=sb;
	sb=temp;
	updateInnerData ();
    }
    
    
    /**
     * @see Segment.adjustPerimeter
     */
    public void adjustPerimeter(Node n, int index)
    {
	if (index==0)
	{
	    Node nn = getCartesianCoords(0);
	    x=x+n.getCoord(0)-nn.getCoord(0);
	    y=y+n.getCoord(1)-nn.getCoord(1);
	}
	else
        {
	    Node nn = getCartesianCoords(1);
	    x=x+n.getCoord(0)-nn.getCoord(0);
	    y=y+n.getCoord(1)-nn.getCoord(1);
	}
    }
    
    /**
     * @see Segment.getCartesianCoords
     */
    public Node getCartesianCoords(double t)
    {
	int tx,ty;
	double phi;
	
	phi=sa+t*(sb-sa);
	tx=(int)(x+r*Math.cos(Math.toRadians(phi)));
	ty=(int)(y+r*Math.sin(Math.toRadians(phi)));
	Node tmp = new Node(tx,ty);
	return tmp;
    }
    
    
    /**
     * @see Segment.isOnBoundary
     */
    public boolean isOnBoundary(int tx, int ty)
    {
	double tr=Math.sqrt((double)(tx-x)*(double)(tx-x)+(double)(ty-y)*(double)(ty-y));
	double erg = 0.0;
	if (Math.abs(r-tr) <= cc.dc.tolerance)
	{
	    double t=getunboundedParameterCoord(tx,ty);
	    if((t>=0.0)&&(t<=1.0))
	    {
	        return true;
	    }
	}
	return false;
    }
    
    
    /**
     * internal helper function for isOnBoundary() and 
     * getParameterValue()
     */
    private double getunboundedParameterCoord(double tx, double ty)
    {
	double arc=0.0;
	double t=0;
	double sa1 = Math.toRadians (sa);
	double sb1 = Math.toRadians (sb);
	if ((ty>=y))
	    arc = Math.PI/2.0+Math.atan((double)(x-tx)/(double)(ty-y));
	
	if ((ty<y))
	    arc = 3.0/2.0*Math.PI+Math.atan((double)(x-tx)/(double)(ty-y));
	if (sb1>sa1)
	    if (arc>=sa1)
		t=(arc-sa1)/(sb1-sa1);
	    else
	        t=(arc-sa1+2.0*Math.PI)/(sb1-sa1);
	else
	{			
	    if (arc<=sa1)
	        t=(sa1-arc)/(sa1-sb1);
	    else
	        t=(sa1-arc+2.0*Math.PI)/(sa1-sb1);
	}
	return t;
    }
    
    
    /**
     * @see Segment.getParameterValue
     */
    public double getParameterValue (Node n)
    {
	double tx = n.getCoord(0);
	double ty = n.getCoord(1);
	double t = getunboundedParameterCoord(tx,ty);
	if (t<0.0) t=0.0;
	if (t>1.0) t=1.0;
	return t;
    }
    
    
    /**
     * draws the segment
     */
    public void draw(Graphics g)
    {
	// set color
	if (marked)
	    g.setColor (cc.dc.color_selection);
	else
	    g.setColor(cc.dc.boundary_palette[boundary.getNumber()%16]);
	// transform manually to screen coords
	int[] midpoint = cc.gc.fromWorldToScreen(x,y);
	int radius = cc.gc.SingleFromWorldToScreen(r);
	if (marked)
	{
	    Graphics2D g2d = (Graphics2D)g;
	    g2d.setStroke (new BasicStroke (4));
	    g2d.drawArc (midpoint[0]-radius, midpoint[1]-radius, 2*radius, 2*radius, (int)sa, (int)(sb-sa));
	    g2d.setStroke (new BasicStroke (1));
	}
	else
	{
	    g.drawArc (midpoint[0]-radius, midpoint[1]-radius, 2*radius, 2*radius, (int)sa, (int)(sb-sa));
	}
	// numbering
	if (cc.dc.number_boundary)
	{
	    g.setColor (cc.dc.color_boundarynumber);
	    int[] num = cc.gc.fromWorldToScreen (numberpos[0], numberpos[1]);
	    g.drawString (((getSegmentNumber()+1)+","+(getBoundaryNumber()+1)), num[0],num[1]);
	}
	// orientation markers
	drawOrientationMarker (g);
    }

    /**
     *  Is the item inside a given rectangular area?
     *  All in WORLD COORDINATES!!!
     *
     *  @param topleft - the topleft corner of the rectangle
     *  @param bottomright - the bottom right corner of the rectangle
     *  @return true if the item is completely inside the rectangle
     */
    public boolean isInside (int[] topleft, int[] bottomright)
    {
	return (topleft[0] < xmin && bottomright[0] > xmax && topleft[1] > ymax && bottomright[1] < ymin);
    }

    /**
     *  Determines if the item has been hit by a click on the grid
     *  ALL IN WORLD COORDINATES to avoid the snap mechanism
     *
     *  @param x,y - coordinates of the event point where the click
     *         occurred
     *  @return true if the item has been hit
     */
    public boolean isHit (int xx, int yy)
    {
	return isOnBoundary (xx,yy);
    }

    /**
     *  updates the inner variables xmin, xmax, ymin, ymax and the polygonal
     *  representation of the circle object
     */
    public void updateInnerData ()
    {
	int[] para0 = new int[2];
  	Node tmpnode = getCartesianCoords(0.0);
  	para0[0] = tmpnode.getCoord(0);
  	para0[1] = tmpnode.getCoord(1);
	// calc bounding box
	Arc2D arc = new Arc2D.Double (x-r, y-r, 2*r, 2*r, sa, sb-sa, Arc2D.OPEN);
	Rectangle2D bb = arc.getBounds2D ();
	xmin = (int)bb.getMinX ();
	xmax = (int)bb.getMaxX ();
	ymin = (int)bb.getMinY ();
	ymax = (int)bb.getMaxY ();
	// update orientation marker variables
	int side = cc.gc.SingleFromScreenToWorld (5);
	Node tmp1 = getCartesianCoords (0.9);
	Node tmp2 = getCartesianCoords (0.85);
	double tmpdx = tmp2.getCoord(0) - tmp1.getCoord(0);
	double tmpdy = tmp2.getCoord(1) - tmp1.getCoord(1);
	double l = Math.sqrt (Math.pow (tmpdx,2)+ Math.pow (tmpdy,2));
	tmpdx /= l;
	tmpdy /= l;
	double[] n = {tmpdy, tmpdx};
	if (Math.abs (tmpdx) > Math.abs (tmpdy))
	    n[1] = -n[1];
	else
	    n[0] = -n[0];
	l = Math.sqrt (n[0]*n[0] + n[1]*n[1]);
	n[0] /= l;
	n[1] /= l;
	orientation_marker_1 = new int[2];
	orientation_marker_1[0] = tmp1.getCoord(0);
	orientation_marker_1[1] = tmp1.getCoord(1);
	orientation_marker_2 = new int[2];
	orientation_marker_2[0] = (int)(tmp1.getCoord(0) + tmpdx*side + n[0]*side);
	orientation_marker_2[1] = (int)(tmp1.getCoord(1) + tmpdy*side + n[1]*side);
	orientation_marker_3 = new int[2];
	orientation_marker_3[0] = (int)(tmp1.getCoord(0) + tmpdx*side - n[0]*side);
	orientation_marker_3[1] = (int)(tmp1.getCoord(1) + tmpdy*side - n[1]*side);
	// update numbering position
	numberpos = new int[2];
	numberpos[0] = para0[0] + 100;
	numberpos[1] = para0[1];

    }

   
    /**
     *  returns the perimeter of the Circle element, that is the smallest 
     *  bounding box completely containing the Circle
     */
    public Rectangle getPerimeter ()
    {
	return new Rectangle (xmin, ymin, xmax-xmin, ymax-ymin);
    }

    /**
     *  applies the given affine transformation to the circle
     */
    public void performTransformation (GridTransformation trafo)
    {
	if (!transformed)
  	{
	    transformed=true;
	    double angle = trafo.getAngle ();
	    AffineTransform trf = trafo.getMatrix ();
	    if (angle != 0)
	    { 
		double[] tmp = {x,y};
		double[] ntmp = new double[2];
		trf.transform (tmp,0,ntmp,0,1);
		x = (int)ntmp[0];
		y = (int)ntmp[1];
		sa += angle;
		sb += angle;
	    }
	    else
	    {
		double[] tmp = {x,y};
		double[] ntmp = new double[2];
		trf.transform (tmp,0,ntmp,0,1);
		x = (int)ntmp[0];
		y = (int)ntmp[1];
		r *= trafo.getScaleX();
	    }
	updateInnerData ();
	}
    }

   /**
     *  This method mirrors the circle at the given line
     *  @param n - the normalized normal of the line (length 1)
     *  @param p - an arbitrary point on the line
     */
    public void performMirror (double nx, double ny, double px, double py)
    {
	if (!transformed)
        {
	    transformed = true;
	    // vector v from line to node must point TOWARDS the node
	    double vx =  px - x;
	    double vy = py - y;
	    // scalarproduct of n and v is distance from node to line
	    double sp = nx*vx + ny*vy;
	    // perform translation of midpoint
	    if (sp > 0)
	    {
		x += 2*sp*nx;
		y += 2*sp*ny;
	    }
	    else
		if (sp < 0)
		{
		    x += 2*sp*nx;
		    y += 2*sp*ny;
		}
	    // update angles
	    double delta = Math.toDegrees(Math.acos (Math.abs (nx)));
	    double gamma = 90-delta;
	    double newsb = 2*gamma - sa;
	    double newsa = 2*gamma - sb;
	    sa = newsa;
	    sb = newsb;
	    updateInnerData ();
	}
    }

    /**
     *  creates a full clone of the SegmentCircle object, including boundary
     *  information
     */
    public Object clone ()
    {
	SegmentCircle circle = new SegmentCircle (x,y,r,sa,sb);
	circle.tmax = tmax;
	circle.setMarker (this.isMarked ());
	circle.boundary = boundary;
	return circle;
    }
  



}


