package devisor2.foundation.elements;


import java.util.*;
import java.awt.*;
import java.awt.geom.*;
import java.io.*;

import devisor2.grid.options.*;
import devisor2.foundation.*;
import devisor2.grid.backend.*;

/**
 *  The class Cell provides a superior ground class for
 *  the cells which defines the discrete domain. 
 *  This class is abstract and must be derived.
 *  Basically, a Cell is a container for some faces (or in 2D edges) which are
 *  defined via the cornering nodes. As always, there are additional properties
 *  like numbering, material constants etc.
 *
 *  @see Quad
 *  @see Tri
 *  @see Line
 *  @see Macro
 *
 * @author Dominik Goeddeke, Christian Becker
 */

public abstract class Cell extends GridItem implements java.io.Serializable
{

    /**
     * Number of the cell
     */
    protected int number;
    
    /**
     * Material number of the cell (UCD)
     */
    protected int material_number;
    
    /**
     * Number of the cell
     */
    protected int level;
    
    /**
     * Marker of the cell
     */
    protected int marker;
    
    
    /**
     * parameters of the cell
     */	
    protected int[] parameters;
    
    /**
     * Array for the neighbourhood, updated by updateCellNeighbours()
     * should contain cells.
     */
    protected Cell[] neighbours;
    
    /**
     * Array for the children,
     * should contain cells.
     */
    protected Cell[] children;
    
    /**
     * parent Cell
     */
    protected Cell parent;
    
    /**
     * Values stored in the cell.
     */
    protected double [] values;
    
    /**
     * parameters of the cell
     */	
    protected int parallelblock;	

/**
* parameters of the cell
*/
  protected int matrixblock;
      
    /**
     * nodes of this cell, updated by updateCellNodes() 
     */
    protected Node[] nodes;
    
    
    public void setParallelBlock(int i)
    {
	parallelblock=i;
    }
    
    
    public int getParallelBlock()
    {
	return parallelblock;
    }   
    

  public void setMatrixBlock(int i)
  {
    matrixblock=i;
  }


  public int getMatrixBlock()
  {
    return matrixblock;
  }  
  
      
    /**
     * Empty Constructor.
     * Sets the number to 0.
     */
    public Cell() 
    {
	super ();
	level = 0;
	number = 0;
	material_number = 0;
	marker=0;
        matrixblock=1;
        parallelblock=1;
	neighbours = null;
	parent = null;
	children = null;
    }

    
    /**
     * Sets the number of the cell to i.
     */
    Cell( int i ) 
    {
	super ();
	level = 0;
	number = i;
	material_number = 0;
        matrixblock=1;
        parallelblock=1;
	marker = 0;
	neighbours = null;
	parent = null;
	children = null;
    }
    
    
    /**
     * Defines the length of the array of values in the cell
     */
    public void defValue(int i) 
    {
	if (i>0)
	    values = new double[i];
    }
    
    
    /**
     * Sets the ith value to val.
     */
    public void setValue(int i, double val) 
    {
	values[i] = val;
    }
    
    
    /**
     * Returns the ith value.
     */
    public double getValue(int i) 
    {
	return values[i];
    }
    
    
    public boolean isRect()
    {
	
	double v1x=nodes[3].getCoord(0)-nodes[0].getCoord(0);
	double v1y=nodes[3].getCoord(1)-nodes[0].getCoord(1);
	
	double v2x=nodes[1].getCoord(0)-nodes[0].getCoord(0);
	double v2y=nodes[1].getCoord(1)-nodes[0].getCoord(1);
	
	double dskala=v1x*v2x+v1y*v2y;
	
	
	v1x=nodes[3].getCoord(0)-nodes[2].getCoord(0);
	v1y=nodes[3].getCoord(1)-nodes[2].getCoord(1);
	
	v2x=nodes[1].getCoord(0)-nodes[2].getCoord(0);
	v2y=nodes[1].getCoord(1)-nodes[2].getCoord(1);
	
	double dskalb=v1x*v2x+v1y*v2y;
	
	return ((Math.abs(dskala)<1E-12) && (Math.abs(dskalb)<1E-12));
	
    }
    
    
    /**
     * Returns the average of the ith value
     * of the nodes defining this cell.
     */
    public double getNodeAverageValue(int index) 
    {
	int npt = getNPT();
	double value = 0.;
	for (int i=0; i<npt; i++) 
	    {
		value += nodes[i].getValue(index);
	    }
	value /= (double)(npt);
	return value;
    }
    
    
    /**
     * Returns the marker of the cell.
     */
    public int getMarker()
    {
	return marker;
    }
    
    
    /**
     * sets the marker of the cell.
     * @param m marker value
     */
    public void setMarker(int m)
    {
	marker=m;
    }
    
    
    /**
     * Returns the level of the cell.
     */
    public int getLevel()
    {
	return level;
    }
    
    
    /**
     * sets the level of the cell.
     * @param m level value
     */
    public void setLevel(int m)
    {
	level=m;
    }
    
    
    /**
     * Sets the number of the cell to i.
     */
    public void setNumber( int i ) 
    {
	number = i;
    }

    public Cell getParent ()
    {
	return parent;
    }

    public Cell[] getNeighbours ()
    {
	return neighbours;
    }

    public Cell[] getChildren ()
    {
	return children;
    }

    public Node[] getNodes ()
    {
	return nodes;
    }
    
    
    /**
     * gets the number of the cell.
     */
    public int getNumber( ) 
    {
	return number ;
    }
    
    /**
     * updates the Node array of this cell
     * @return the nodes of this cell in order of faces/edges
     */
    public abstract Node[] updateCellNodes();
    
    
    /**
     * updates the neighbours of this cell
     */
    public abstract Cell[]  updateCellNeighbours();
    
    
    /**
     * Draws the perimeter of the cell.
     */
    public abstract void draw(Graphics g);
    
    public abstract int getNPT();
    
    
    /**
     * Draws the cell filled with its color.
     */
    public abstract void fill(Graphics g);
  
    
    /**
     *  Is the item inside a given rectangular area?
     *  All in WORLD COORDINATES!!!
     *
     *  @param topleft - the topleft corner of the rectangle
     *  @param bottomright - the bottom right corner of the rectangle
     *  @return true if the item is completely inside the rectangle
     */
    public abstract boolean isInside (int[] topleft, int[] bottomright);

    /**
     *  Determines if the item has been hit by a click on the grid
     *  ALL IN SCREEN COORDINATES to avoid the snap mechanism
     *
     *  @param x,y - coordinates of the event point where the click
     *         occurred
     *  @return true if the item has been hit
     */
    public abstract boolean isHit (int x, int y);
    
    public abstract Rectangle getPerimeter();

    public abstract void performTransformation (GridTransformation trafo);
    

  
}
