package devisor2.foundation.elements;

import java.util.*;
import java.awt.*;
import java.awt.geom.*;
import java.io.*;
import devisor2.foundation.*;
import devisor2.grid.backend.*;

/**
 * The class Cell2D provides a ground class for
 * 2D cell types which defines a discrete 2D domain. 
 * This class is abstract and must be derived.
 * @see Quad
 * @see Tri
 * @see Line
 * @see MacroQuad
 */

public abstract class Cell2D extends Cell implements java.io.Serializable
{

    /**
     *  the cell representation as a Polygon in user coordinates
     */
    protected Polygon poly_world;

  

    /**
     * Face of the cell
     * @see Face
     */
    protected Edge[] edges;
    
    /**
     * Empty Constructor.
     * Sets the number to 0.
     */
    public Cell2D() 
    {
	super();
    }
    
    
    /**
     *  creates a cell with number i
     *  @param i - the new number tof the Cell to be created
     */
    public Cell2D(int i)
    {
	super(i);
    }
    
    

    /**
     *  @param n - the new node to be placed at
     *  @param i - the index where to place the node
     */
    public void setNode (Node n, int index)
    {
	nodes[index] = null;
	nodes[index] = n;
	//	System.out.println ("new cell "+number+" uses node "+nodes[index]);
    }  

    /**
     * Defines the edges of the cell,
     * and set the number of the cell to i.
     * @see Face
     * @see Cell
     */
    public void setCell ( int i, Edge[] e ) 
    {
	try 
	{
	    if (edges != null) 
	    {
		for (int j=0; j<getNPT(); j++)
		    edges[j].removeParent(this);
	    }
	} catch (NullPointerException excep) {}
	edges = e;
	number = i;
	for (int j=0; j<getNPT(); j++)
	    edges[j].addParent(this);
	updateInnerData ();
    }

    /**
     * Defines the edges of the cell,
     * and set the number of the cell to 0.
     * @see Face
     * @see Cell
     */
    public void setCell (Edge[] e)
    {
	setCell(0,e);
    }

    /**
     * sets the i-th edge of the cell to e
     */
    public void setEdge( int i, Edge e ) 
    {
	int j = i%getNPT();
	if (edges[j]!=null)
	    edges[j].removeParent(this);
	edges[j] = e;
	edges[j].addParent(this);
	updateInnerData ();
    }

    public Edge[] getEdges ()
    {
	return edges;
    }

    public abstract void updateInnerData ();


    /**
     * returns the nodes of this cell with respect to
     * the edge order.
     */
    public Node[] updateCellNodes()
    {
	Node[] temp = new Node[getNPT()];
	
	if (getNPT()==3)
	{
	    Node a1 = getEdge(0).getNode(0);
	    Node a2 = getEdge(0).getNode(1);
	    
	    Node b1 = getEdge(1).getNode(0);
	    Node b2 = getEdge(1).getNode(1);
	    
	    temp[0]=a1;
	    temp[1]=a2;
	    
	    if (b1==a2)
		temp[2]=b2;
	    else
		temp[2]=b1;
	}
	else
	{
	    int k=0;
	    
	    Node a1 = getEdge(0).getNode(0);
	    Node a2 = getEdge(0).getNode(1);
	    
	    Node b1 = getEdge(3).getNode(0);
	    Node b2 = getEdge(3).getNode(1);
	    
	    if (a2==b2)
	    {
		temp[0]=a2;
		temp[1]=a1;
		temp[3]=b1;
	    }
	    else if (a1==b1)
	    {
		temp[0]=a1;
		temp[1]=a2;
		temp[3]=b2;
	    }
	    else if (b2==a1)
	    {
		temp[0]=a1;
		temp[1]=a2;
		temp[3]=b1;
	    }
	    else if (b1==a2)
	    {
		temp[0]=a2;
		temp[1]=a1;
		temp[3]=b2;
	    }
	    else
		System.out.println("Face orientation error");

	    Node c1 = getEdge(1).getNode(0);
	    Node c2 = getEdge(1).getNode(1);
	    
	    if (c1==temp[1])
		temp[2]=c2;
	    else
		temp[2]=c1;
	    
	}

	nodes = temp;
	return temp;
    }
    
    
    /**
     * update the neighbours array of this cell
     */
    public Cell[] updateCellNeighbours() 
    {
	Edge edge;
	Cell c;
	Vector arg = new Vector();
	for (int i=0; i<edges.length; i++) 
	{
	    edge = edges[i];
	    Enumeration e = edge.parents.elements();
	    while (e.hasMoreElements()) 
	    {
		c = (Cell)e.nextElement();
		if ( (c!=this) && (arg.indexOf(c, 0) < 0) ) 
		{
		    arg.addElement(c);
		}
	    }
	}
	if (arg.isEmpty()) 
	{
	    return null;
	}
	else 
	{
	    neighbours = new Cell[arg.size()];
	    arg.copyInto(neighbours);
	    return neighbours;
	}
    }
    
    
    /**
     * gives the ith node of the cell
     */
    public Node getNode(int i) 
    {
	if ( (nodes==null) || (i>=nodes.length) || (nodes[i]==null) )
	    return null;
	return nodes[i];
    }
  
  
    /**
     * gives the node array of the cell
     */
    public Node[] getNodes() 
    {
	return nodes;
    }
    
    
    /**
     * gives the ith Edge of the cell
     */
    abstract public Edge getEdge(int i);
    
    /**
     *  in this method, the item paints itself to the given graphics
     *  context
     *
     *  @param g- the graphics context to paint on
     */
    public abstract void draw (Graphics g);

    /**
     *  Is the item inside a given rectangular area?
     *  All in WORLD COORDINATES!!!
     *
     *  @param topleft - the topleft corner of the rectangle
     *  @param bottomright - the bottom right corner of the rectangle
     *  @return true if the item is completely inside the rectangle
     */
    public abstract boolean isInside (int[] topleft, int[] bottomright);


    /**
     *  Determines if the item has been hit by a click on the grid
     *  ALL IN SCREEN COORDINATES to avoid the snap mechanism
     *
     *  @param x,y - coordinates of the event point where the click
     *         occurred
     *  @return true if the item has been hit
     */
    public abstract boolean isHit (int x, int y);

    public abstract Rectangle getPerimeter ();

    public abstract void performTransformation (GridTransformation trafo);

    public abstract Object clone ();


    /**
     *  calculates and returns the smallest inner angle of the element.
     *  @return a double representing an angle in degrees!
     */
    public abstract double getSmallestInnerAngle ();


    /**
     *  calculates the aspect ratio of the element, that is the quotient of
     *  the shortest and the longest edge
     */
    public abstract double getAspectRatio ();

    /**
     *  calculates and returns the area (size) of the element
     */
    public abstract double getArea ();


    
}






