package devisor2.foundation.elements;

import java.awt.*;
import java.io.*;
import java.util.Vector;
import java.awt.geom.*;

import devisor2.grid.options.*;
import devisor2.grid.GUI.framework.*;
import devisor2.foundation.*;
import devisor2.foundation.boundary.*;
import devisor2.grid.backend.*;


/**
 *  This class represents the GridItem extension for edges.
 *  Edges are defined via two nodes and can carry additional properties
 *  such as edge stati etc. Edges form the top-level elements.
 *
 *
 *  @author Christian Becker, Dominik Goeddeke
 */
public class Edge extends GridItem implements java.io.Serializable
{


    /**
     * the nodes of this edge
     */
    protected Node[] p;
    
    /**
     * the number of this edge, updated by updateEdgeNumbers()
     */
    protected int number;
    
    /**
     * the hierarchical level of this edge (yet not implemented)
     */
    public int level;
    
    /**
     * marker of this edge
     */
    public int marker;
    
    /**
     * The Cell2D Objects which contain this edge
     */
    protected Vector parents;
    
    /** 
     * Status of the edge (use constants provided)
     */
    protected int edgestatus;

    /**
     *  Constant for edge status NONE
     */
    public static final int STATUS_NONE = 0;
    /**
     *  Constant for edge status INNER
     */
    public static final int STATUS_INNER = 2;
    /**
     *  Constant for edge status REAL
     */
    public static final int STATUS_REAL = 1;
  /**
     *  Constant for edge status MIRROR BOUNDARY
     */
    public static final int STATUS_MIRRORB = 3;
    
    /** 
     * boundary condition of the edge (use constants provided)
     */
    protected int edgebc;
    /**
     *  Constant for boundary condition NONE
     */
    public static final int BC_NONE = 0;
    /**
     *  Constant for boundary condition NEUMANN
     */
    public static final int BC_NEUMANN = 1;
    /**
     *  Constant for boundary condition DIRICHLET
     */
    public static final int BC_DIRICHLET = 0;
    
    /** 
     * Boundary Edge or not
     */
    protected boolean boundaryEdge;
    
    /** 
     * if boundary Edge then contains the boundary of the edge
     */
    protected Boundary boundary;

    /**
     *  a sort of bounding box or rectangular area for picking purposes
     */
    private Polygon poly;
    
    
    /**
     * creates an edge from node a to b
     */
    public Edge(Node n1, Node n2)
    {
	reinit(n1, n2, 0,0);
    }


    public Edge(Node n1, Node n2, int es)
    {
	reinit(n1, n2, es,0);
    }
    
    
    public Edge(Node n1, Node n2, int es, int bc)
    {
	reinit(n1, n2, es,bc);
    }

    ///////////////////////////////////////

    public double[] getParameters ()
    {
	double[] params = new double[4];
	params[0] = p[0].getParameters ()[0];
	params[1] = p[0].getParameters ()[1];
	params[2] = p[1].getParameters ()[0];
	params[3] = p[1].getParameters ()[1];
	return params;
    }
    public void setParameters (double[] params)
    {
	double[] paras = new double[2];
	paras[0] = params[0];
	paras[1] = params[1];
	p[0].setParameters (paras);
	paras[0] = params[2];
	paras[1] = params[3];
	p[1].setParameters (paras);
	updateInnerData ();
    }
    

    /**
     *  sets the i-th node of the edge to n
     */
    public void setNode (int i, Node n)
    {
	p[i] = n;
    }

   
    /**
     * Pseudo constructor
     */
    public void reinit(Node n1, Node n2, int es, int bc)
    {
	//level=0;
	//parent=null;
	
	p = new Node[2];
	p[0]=n1;
	p[1]=n2;
	n1.addParent(this);
	n2.addParent(this);
		
	if ( (n1 instanceof BoundaryNode) && (n2 instanceof BoundaryNode) )
	{
	    boundaryEdge = true;
	    BoundaryNode ba = (BoundaryNode)n1;
	    BoundaryNode bb = (BoundaryNode)n2;
	    if (ba.getBoundary() != bb.getBoundary()) 
	    {
		boundary = null;		    
	    }
	    else 
	    {
		boundary = ba.getBoundary();
	    }
	}
	else
	{
	    boundaryEdge = false;
	    boundary = null;
	}
	parents=new Vector(4,4);
	marker=0;
	edgestatus=es;
	edgebc=bc;
	updateInnerData ();
  }


    /**
     * changes the orientation of this edge 
     */
    public void swap()
    {
	Node temp;
	temp=p[0];
	p[0]=p[1];
	p[1]=temp;
	updateInnerData ();
    }
    
    /**
     *  returns the status of the edge
     */
    public int getStatus()
    { 
	return edgestatus;
    } 
    
    /**
     *  sets the status of the edge
     */
    public void setStatus(int newstatus)
    {
	edgestatus=newstatus;
    }
  
    /**
     *  returns the boundarycondition of the edge
     */
    public int getBC()
    {
	return edgebc;
    }
    
    /**
     *  sets the boundary condition
     */
    public void setBC(int newbc)
    {
	edgebc=newbc;
    }
    
    
    /**
     * sets the marker value to i
     */
    public void setMarker(int i)
    {
	marker=i;
    }
    
    
    /**
     * returns the marker value of this edge
     */
    public int getMarker()
    {
	return marker;
    }
    
    
    /**
     * returns the i-th Node (0 or 1) of this edge
     */
    public Node getNode(int i)
    {
	return p[i];
    }
    
    
    /**
     * defines the number of the edge
     */
    public void setNumber(int i)
    {
	number = i;
    }
    
    
    /**
     * returns the number of the edge
     */
    public int getNumber()
    {
	return number;
    }
    
    
    
    /**
     * addParent adds the given Cell2D c to the parent vector
     * @param Cell2D c parentCell
     */
    public void addParent(Cell2D c)
    {
	parents.addElement(c);
    }
  
    
    
    /**
     * removeParent removes the given Cell2D c from the parent vector
     * @param Cell2D c parentCell
     */
    public void removeParent(Cell2D c)
    {
	parents.removeElement(c);
    }
    
    
    public void removeParent(Cell c)
    {
	parents.removeElement(c);
    }
    
    
    
    
    
    /**
     * getParents() returns the parent Cells(2D) as array of Objects
     * @return parent Object
     */
    public Object[] getParents()
    {
	if (parents.size()==0)
	    return null;
	else
	{			
	    Cell[] temp = new Cell[parents.size()];
	    
	    for(int i=0;i<parents.size();i++)
		temp[i]=(Cell)parents.elementAt(i);
	    
	    return temp;
	    
	}
    }

    public Vector getFathers ()
    {
	return parents;
    } 
    
    /**
     * Returns the average of the ith value
     * of the nodes defining this cell.
     */
    public double getNodeAverageValue(int index) 
    {
	return 0.5* ( getNode(0).getValue(index) + getNode(1).getValue(index) );
    }

    /**
     *  updates the inner variables of the edge used for picking and
     *  selecting
     */
    public void updateInnerData ()
    {  
	int[] coord = {p[0].coord[0],p[0].coord[1],p[1].coord[0],p[1].coord[1]};
	// the maximal extensions (i.e. the corners of the bounding box)
        // used for selecting
	xmax = Math.max (coord[0], coord[2]);
	xmin = Math.min (coord[0], coord[2]);
	ymax = Math.max (coord[1], coord[3]);
	ymin = Math.min (coord[1], coord[3]);
	// get normal vector of this line and scale down to length 1
        int sx = p[1].coord[0] - p[0].coord[0];
        int sy = p[1].coord[1] - p[0].coord[1];
	// get normal vector of this line and scale down to length 1
	double[] n = {sy, sx};
	if (Math.abs (sx) > Math.abs (sy))
	    n[1] = -n[1];
	else
	    n[0] = -n[0];
	double l = Math.sqrt (n[0]*n[0] + n[1]*n[1]);
	n[0] /= l;
	n[1] /= l;
	// get scaled versions of sx and sy
	l = Math.sqrt ((double)(sx*sx+sy*sy));
	double[] s = {sx,sy};
	double ssx = (s[0] /= l);
	double ssy = (s[1] /= l);
	// create parallel polygon around line with distance click_tolerance
	poly = new Polygon ();
	poly.addPoint ((int)(coord[0] - cc.dc.tolerance*ssx+n[0]*cc.dc.tolerance), (int)(coord[1]-cc.dc.tolerance*ssy+n[1]*cc.dc.tolerance));
	poly.addPoint ((int)(coord[2]+cc.dc.tolerance*ssx+n[0]*cc.dc.tolerance), (int)(coord[3]+cc.dc.tolerance*ssy+n[1]*cc.dc.tolerance));
	poly.addPoint ((int)(coord[2]+cc.dc.tolerance*ssx-n[0]*cc.dc.tolerance), (int)(coord[3]+cc.dc.tolerance*ssy-n[1]*cc.dc.tolerance));
	poly.addPoint ((int)(coord[0]- cc.dc.tolerance*ssx-n[0]*cc.dc.tolerance), (int)(coord[1]-cc.dc.tolerance*ssy-n[1]*cc.dc.tolerance));

	//System.out.println ("Edge from "+coord[0]+","+coord[1]+" to "+coord[2]+","+coord[3]);
	//for (int i=0; i<4; i++)
	// System.out.println (poly.xpoints[i] + "," + poly.ypoints[i]);
    }
    
    
    /**
     * draw this edge with respect to transformation and
     * drawcontrol
     */
    public void draw(Graphics g)
    {
	g.setPaintMode();
	if (marked)
	    g.setColor (cc.dc.color_selection);
	else
	    g.setColor(cc.dc.color_edgebase);
	int[] p0 = cc.gc.fromWorldToScreen(p[0].getCoord(0),p[0].getCoord(1));
	int[] p1 = cc.gc.fromWorldToScreen(p[1].getCoord(0),p[1].getCoord(1));
	g.drawLine (p0[0],p0[1],p1[0],p1[1]);
	if (edgestatus!=0)
	{
	    g.drawLine(p0[0]-1, p0[1]-1, p1[0]-1, p1[1]-1);
	    g.drawLine(p0[0]+1, p0[1]+1, p1[0]+1, p1[1]+1);
	}
	if (edgestatus==1)
	{
	    g.drawLine(p0[0]-2, p0[1]-2, p1[0]-2, p1[1]-2);
	    g.drawLine(p0[0]+2, p0[1]+2, p1[0]+2, p1[1]+2);
	}
	g.setFont(cc.dc.font_numbering);
	g.setColor(cc.dc.color_edgenumber);
	if ((edgebc==0) && (edgestatus==1))
	    g.drawString("D",(p0[0]+p1[0])/2,(p0[1]+p1[1])/2);
	
	if ((edgebc==1) && (edgestatus==1))
	    g.drawString("N",(p0[0]+p1[0])/2,(p0[1]+p1[1])/2);
	
	if (edgestatus==2)
	    g.drawString("X",(p0[0]+p1[0])/2,(p0[1]+p1[1])/2);
	
	if (edgestatus==3)
	    g.drawString("M",(p0[0]+p1[0])/2,(p0[1]+p1[1])/2);
    }
	 
    /**
     * fill this edge with its color with respect to transformation and
     * drawcontrol
     */
    public void fill(Graphics g)
    {
	//    double[] x0 = tf.transform( p[0].getX() );
	//    double[] x1 = tf.transform( p[1].getX() );
	
	//    g.setColor(new Color (Integer.parseInt(cc.op.get(Options.color_edge))));
	//    g.drawLine2D((int)x0[0], (int)x0[1], 
	//      (int)x1[0], (int)x1[1]);
    }
    
    
    public boolean isOnEdge(Node n, double eps)
    {
	double tx = n.getCoord(0);
	double ty = n.getCoord(1);
	
	//    return ( (Math.sqrt((tx-p[0].getX(0))*(tx-p[0].getX(0))
	//      +(ty-p[0].getX(1))*(ty-p[0].getX(1))) +
	//      Math.sqrt((tx-p[1].getX(0))*(tx-p[1].getX(0))
	//      +(ty-p[1].getX(1))*(ty-p[1].getX(1))) -
	//      Math.sqrt((p[0].getX(0)-p[1].getX(0))*(p[0].getX(0)-p[1].getX(0))
	//      +(p[0].getX(1)-p[1].getX(1))*(p[0].getX(1)-p[1].getX(1)))) < eps);
	
	return false;
	
	
    }
 
    /**
     *  Is the item inside a given rectangular area?
     *  All in WORLD COORDINATES!!!
     *
     *  @param topleft - the topleft corner of the rectangle
     *  @param bottomright - the bottom right corner of the rectangle
     *  @return true if the item is completely inside the rectangle
     */
    public boolean isInside (int[] topleft, int[] bottomright)
    {
	return (topleft[0] < xmin && bottomright[0] > xmax && topleft[1] > ymax && bottomright[1] < ymin);
    }


    /**
     *  Determines if the item has been hit by a click on the grid
     *  ALL IN world COORDINATES to avoid the snap mechanism
     *
     *  @param x,y - coordinates of the event point where the click
     *         occurred
     *  @return true if the item has been hit
     */
    public boolean isHit (int cx, int cy)
    {
	return poly.contains (cx,cy);
    }

    /**
     *  returns the perimeter of the Edge element, that is the smallest 
     *  bounding box completely containing the Edge
     */
    public Rectangle getPerimeter ()
    {
	return new Rectangle (xmin, ymin, xmax-xmin, ymax-ymin);
    }
  

    /**
     *  applies the given AffineTransform instance on this edge
     *  @param trafo - the GridTransform instance representing the 
     *  transformation
     */
    public void performTransformation (GridTransformation trafo)
    {
	if (!transformed)
	{
	    transformed = true;
	    p[0].performTransformation (trafo);
	    p[1].performTransformation (trafo);
	    updateInnerData ();
	}
    }

   /**
     *  This method mirrors the edge at the given line (by mirroring its two
     *  nodes)
     *  @param n - the normalized normal of the line (length 1)
     *  @param q - an arbitrary point on the line
     */
    public void performMirror (double nx, double ny, double px, double py)
    {
	if (!transformed)
        {
	    transformed = true;
	    p[0].performMirror (nx,ny,px,py);
	    p[1].performMirror (nx,ny,px,py);
	    updateInnerData ();
	}
    }

    /**
     *  returns an exact copy of this edge, but not cloning recursively 
     *  (i.e. all nodes part of this edge are not cloned
     */
    public Object clone ()
    {
	int status = getStatus ();
	int boundary = getBC ();
	Edge temp = new Edge (p[0],p[1],status,boundary);
	temp.setMarker (this.isMarked());
	temp.updateInnerData ();
	return temp;
    }

    /**
     *  calculates the length of the edge (simple pythagoras)
     *  and returns it
     */
    public double getLength ()
    {
	return Math.sqrt (Math.pow((double)(p[0].getCoord(0)-p[1].getCoord(0)),2) + Math.pow((double)(p[0].getCoord(1)-p[1].getCoord(1)),2) );
    }
}





