
package devisor2.foundation.elements;

import java.awt.*;
import java.io.*;
import java.util.Vector;
import java.awt.geom.*;

import devisor2.grid.options.*;
import devisor2.grid.GUI.framework.*;
import devisor2.foundation.boundary.BoundaryNode;
import devisor2.foundation.*;
import devisor2.grid.backend.*;
/**
 * This class represents a node of a grid. As all foundation classes, 
 * a node can draw itself and provides convenience methods to support
 * the GridListener and the GridSelector.
 *
 * @author Christian Becker, Dominik Goeddeke
 *
 */
public class Node extends GridItem implements Serializable 
{

    /**
     * cartesian coordinates of the node
     */
    protected int[] coord;

    /**
     * The values of the node
     */
    protected double [] value;

    /**
     * The edges which contains this node
     */
    public Vector parents;
    
    /**
     * number of the node, updated by updateNodeNumbers()
     */
    protected int number;
    
    /**
     * index of the node,
     * read in mesh file
     */
    protected int index;
	
    /**
     * marker variable
     */
    protected int marker;

    
    /**
     * boundary information variable
     */
    protected int npr;
    
    /*********************************************************/

    /**
     * Creates a zero node
     */
    public Node() 
    {
	super ();
	coord = new int[2];
	setCoord(0,0);
	parents = new Vector(4,4);
	npr=0;
    }
    
    
    /**
     * Creates a node at (x,y) without values
     */
    public Node(int xx, int yy) 
    {  
	super ();
	coord = new int[2];
	setCoord(xx,yy);
	parents=new Vector(4,4);
	npr=0;
    }
    
    
    /**
     * Creates a node at (x,y) in an edge p
     */
    public Node(int xx, int yy, Edge p) 
    {  
	super ();
	coord = new int[2];
	setCoord(xx,yy);
	parents = new Vector(4,4);
	parents.addElement(p);
	npr=0;
    }
    
    
    /**
     * Creates a node at (x,y) with nbValues values
     */  
    public Node(int xx, int yy, int nbValues) 
    {
	super ();
	coord = new int[2];
	setCoord(xx, yy);
	parents = new Vector(4,4);
	npr=0;
	if (nbValues > 0)
	    value = new double[nbValues];
    }
    
    
    
    /*********************************************************/

    /**
     * setNPR sets the nodal property
     * @param unpr nodal property
     */
    public void setNPR(int unpr)
    {
      npr=unpr;
    }
    
    /**
     * setNPR returns the nodal property
     * @param return nodal property
     */
    public int getNPR()
    {
      return npr;
    }
            
    /**
     * getParents() returns the parent edges as array of edges
     * @param parent edges
     */
    public Edge[] getParents()
    {
	if (parents.size()==0)
	    return null;
	else
	{			
	    Edge[] temp = new Edge[parents.size()];
	    
	    for(int i=0;i<parents.size();i++)
		temp[i]=(Edge)parents.elementAt(i);
	    
	    return temp;
	}
    }

    public Vector getFathers ()
    {
	return parents;
    }
    
    
    /**
     * addParent adds the given edge p to the parent vector
     * @param Edge p parentedge
     */
    public void addParent(Edge p)
    {
	parents.addElement(p);
    }
    
    
    /**
     * delParent deletes the given edge p from the parent vector
     * @param Edge p edge to be deleted
     */
    public boolean removeParent(Edge p)
    {
	return parents.removeElement(p);
    }

    /**
     *  @return all parameters of the node
     */
    public double[] getParameters ()
    {
	double[] tmp = new double[2];
	tmp[0] = coord[0];
	tmp[1] = coord[1];
	return tmp;
    }

    public int[] getParams ()
    {
	return coord;
    }

    public void setParams (int[] prm)
    {
	coord = prm;
    }

    /**
     *  sets the parameters to the given ones
     */
    public void setParameters (double[] params)
    {
	setCoord ((int)params[0], (int)params[1]);
    }
    
    /*********************************************************/
    
    /**
     * Sets the number of the node to i.
     */
    public void setNumber( int i ) 
    {
	number = i;
    }
    
    
    /**
     * gets the number of the node.
     */
    public int getNumber( ) 
    {
	return number ;
    }
    
    /*********************************************************/
    
    /**
     * gets the marker of the node.
     */
    public int getMarker( ) 
    {
	return marker ;
    }
    
    
    /**
     * sets the marker of the node to n.
     */
    public void setMarker(int n ) 
    {
	marker=n;
    }
    
    /*********************************************************/
    
    /**
     * setCoords sets the coordinates of the node
     * @param double xx,yy coordinates 
     */
    public void setCoord(int xx, int yy) 
    {
	coord[0] = xx;
	coord[1] = yy;
    }
    
    
    /**
     * setCoord sets the coordinates of the node
     * @param double[] x and y coordinates
     */
    public void setCoord(int[] xx)
    {
	coord[0] = xx[0];
	coord[1] = xx[1];
    }
    
    /**
     * setCoord sets the coordinates of the node and the marker
     * @param double xx,yy coordinates 
     * @param int m marker value
     */	
    public void setCoord(int xx, int yy, int m) 
    {
	setCoord(xx,yy);
	setMarker(m);
    }
    
    
    /**
     * sets the coordinates of the actual node to the coordinates of Node n
     * @param n Node to be copied 
     * @return  none
     */
    public void setCoord( Node n ) 
    {
	setCoord(n.coord[0],n.coord[1]);
    }
    
    
    /**
     * gets the coordinate vector of the Node
     */
    public int[] getCoord() 
    {
	return coord;
    }
    
    
    /**
     * gets the coordinate i of the Node
     */
    public int getCoord(int i) 
    {
	return coord[i];
    }
    
    /*********************************************************/
    
    /**
     * Sets the values of the node
     *
     * @param i index of the value
     * @param value value
     * @return none
     */
    public void setValue(int i, double val) 
    {
	value[i] = val;
    }
    
    
    /**
     * gets an value of the node
     *
     * @param i index of the value
     * @return  selected value
     */
    public double getValue(int i) 
    {
	return value[i];
    }
    
    /*********************************************************/
    
    /**
     * creates a new Node with a copy of 
     * the data of this Node
     * @return Node
     */
    public Object clone() 
    {
	Node temp = new Node();
	temp.setCoord(this);
	if (value != null) 
	{
	    int l = value.length;
	    temp.value = new double[l];
	    for (int i=0; i<l; i++) 
	    {
		temp.setValue(i, getValue(i)); 
	    }
	}
	temp.setMarker (this.isMarked());
	temp.setNumber(number);
	return temp;
    }
    
    
    /**
     * prints the node's coordinates to the standard output
     *
     * @return  none
     */    
    public String toString()
    {
  	return ("Node: x:"+coord[0]+" y:"+coord[1]);
    }
    
    /**
     * draws the node to Graphics context g
     *
     */  
    public void draw(Graphics g)
    {
	int[] screencoord = cc.gc.fromWorldToScreen(coord[0],coord[1]);
	// color is depending on the marked variable
	if (marked)
	    g.setColor (cc.dc.color_selection);
	else
	    g.setColor(cc.dc.color_node);
	if (npr!=0) g.setColor(cc.dc.boundary_palette[(npr-1)%16]);
	// paint
	if (marked)
	{
	    g.fillRect (screencoord[0]-4, screencoord[1]-4,12,12);
	}
	else
	{	    
	    g.drawRect (screencoord[0]-2,screencoord[1]-2,4,4);
	    if (this instanceof BoundaryNode)
		g.drawRect (screencoord[0]-4,screencoord[1]-4,8,8);
	}
	// paint some text
	if (cc.dc.number_node)
	{
	    g.setColor(cc.dc.color_nodenumber);
	    g.setFont(cc.dc.font_numbering);
	    g.drawString((number+1)+" ",(int)(screencoord[0]+4),screencoord[1]);
	}
	
	/////////////////////////////////////
	
	// old code
	
	//  double scale = 1.0/g.getTransform().getScaleX();
	//  double scaley = Math.abs(1.0/g.getTransform().getScaleY());
	
	//  	  g.setColor(cc.dc.color_node);
	
	//        g.draw(new Rectangle((int)(coord[0]-2*scale),(int)(coord[1]-2*scaley),(int)(4*scale),(int)(4*scaley)));
	
	//        if (this instanceof BoundaryNode)
	//        g.draw(new Rectangle((int)(coord[0]-4*scale),(int)(coord[1]-4*scaley),(int)(8*scale),(int)(8*scaley)));
	
	//  //      if (Boolean.valueOf(cc.op.get (Options.draw_numbernode)).booleanValue())
	//        {
	//  	  g.setColor(cc.dc.color_nodenumber);
	//  	  g.setFont(cc.dc.font_numbering);
	//  	  DrawingArea.drawString(g,(number+1)+" ",(int)(coord[0]+4*scale),coord[1]);
	//        }
	
	
	//        if (isMarker())
	//        {
	//  //	  g.setColor(new Color (Integer.parseInt(cc.op.get(Options.color_selection))));
	//  //	  g.fillRect((int)xt[0]-1,(int)xt[1]-1,2,2);
	//        }  
    }
    
    
    /**
     * set an interpolated node between noeud1 and noeud2 
     * related to parameter coeff
     *
     * @param  noeud1 Node 1
     * @param  noeud2 Node 2
     * @param coeff Range
     * @return  none
     */	
    public void interpolate( Node noeud1, Node noeud2, double coeff ) 
    {
	coord[0] = (int)(coeff * noeud2.coord[0] + (1. - coeff) * (double)noeud1.coord[0]);
	coord[1] = (int)( coeff * noeud2.coord[1] + (1. - coeff) * (double)noeud1.coord[1]);
    }
    
    
    /**
     * determines if two nodes located at the same place
     * 
     * @param  n node 
     * @param  eps accuracy
     * @return  the same? yes/no
     */
    public boolean equals(Node n, double eps) 
    {
//System.out.println("NODEEQ "+(Math.abs((double)(n.coord[0] - coord[0])/coord[0])));
//System.out.println("NODEEQ "+(Math.abs((double)(n.coord[1] - coord[1])/coord[1])));

	return ((Math.abs((double)(n.coord[0] - coord[0])/coord[0])< eps) && 
		(Math.abs((double)(n.coord[1] - coord[1])/coord[1])< eps));
    }
    
    public boolean equals2(Node n, double eps) 
    {
//System.out.println("NODEEQ "+(Math.abs((double)(n.coord[0] - coord[0]))));
//System.out.println("NODEEQ "+(Math.abs((double)(n.coord[1] - coord[1]))));

	return ((Math.abs((double)(n.coord[0] - coord[0]))< eps) && 
		(Math.abs((double)(n.coord[1] - coord[1]))< eps));
    }    
    
    /**
     * Are we inside of a give 2D area ?
     * 
     * @param topleft, bottomright - the two corners of a rectangular area
     * @return is inside ? yes/no
     */
    public boolean isInside(int[] topleft, int[] bottomright)
    {
	return ((topleft[0] < coord[0]) && (topleft[1] > coord[1]) && (bottomright[0] > coord[0]) && (bottomright[1] < coord[1]));
    }


    /**
     * Has a node been hit by a click on the given coordinates? This method 
     * works in screen coordinates only because it is pure convenience for
     * the GridSelector class. On the other hand, the tolerance factor
     * is unspecified in world coordinates.
     *
     * @param x,y - coordinates of the clickpoint (world coords)
     */
    public boolean isHit (int x, int y)
    {
	int[] topleft = new int[2];
	topleft[0] = x - cc.dc.tolerance;
	topleft[1] = y - cc.dc.tolerance;
	int[] bottomright = new int[2];
	bottomright[0] = x + cc.dc.tolerance;
	bottomright[1] = y + cc.dc.tolerance;
	return ((coord[0]>topleft[0]) && (coord[1]>topleft[1]) &&
		(coord[0]<bottomright[0]) && (coord[1]<bottomright[1]));
    }
    
    
    /**
     * calculates the perimeter of this node. Thus a node is a Point,
     * the differences between calculated limits are 0
     *
     * @return Perimeter of this node
     */
    public Rectangle getPerimeter()
    {
	return new Rectangle (coord[0], coord[1],1,1);
    }

    /**
     *  applies the given affine transformation to the node
     *
     *  @param trf - homonenous world coords
     */
    public void performTransformation (GridTransformation trf)
    { 
  	if (!transformed)
  	{
	    AffineTransform trafo = trf.getMatrix ();
  	    transformed=true;
	    //System.out.println ("old coords : "+coord[0]+","+coord[1]);
	    double[] tmp1 = {coord[0],coord[1]};
	    double[] ntmp1 = new double[2];
	    trafo.transform(tmp1,0,ntmp1,0,1);
	    //System.out.println ("new coords: "+ntmp1[0]+","+ntmp1[1]);
  	    coord[0] = (int)ntmp1[0];
  	    coord[1] = (int)ntmp1[1];
 	}
    }

    /**
     *  This method mirrors the node at the given line
     *  @param n - the normalized normal of the line (length 1)
     *  @param p - an arbitrary point on the line
     */
    public void performMirror (double nx, double ny, double px, double py)
    {
	if (!transformed)
        {
	    transformed = true;
	    // vector v from line to node must point TOWARDS the node
	    double vx = px - coord[0];
	    double vy = py - coord[1];
	    // scalarproduct of n and v is distance from node to line
	    double sp =  nx*vx + ny*vy;
	    // perform translation
	    if (sp > 0)
	    {
		coord[0] += 2*sp*nx;
		coord[1] += 2*sp*ny;
	    }
	    else
		if (sp < 0)
		{
		    coord[0] += 2*sp*nx;
		    coord[1] += 2*sp*ny;
		}
	}
    }
    
}




