package devisor2.grid.GUI.dialogs;

import java.util.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import devisor2.grid.options.*;
import devisor2.grid.GUI.framework.*;


/**
 *  This class is the abstract superclass for our non-modal dockable dialog.
 *  <br> <br>
 *  When first displayed, instances of this class' subclasses appear at a 
 *  fixed position. They are attached to the MainFrame, meaning that whenever
 *  the MainFrame is moved or resized, they are moved accordingly so that 
 *  their relative position remains the same. If necessary, the dialog can 
 *  also be undocked when it is moved away from the MainFrame separately by 
 *  more than the margin specified in the dockMargin class variable. In
 *  undocked state, the dialog does not follow the MainFrame's movements but
 *  is completely independent. To dock the dialog again, it has to be moved
 *  towards the MainFrame, as soon as the margin is less than the dockMargin,
 *  its position is snapped towards the next docking position. The middle of
 *  the dialog's canvas is considered Event Point.<br>
 *  <br>
 *  The whole docking information is made persistent, that means every time 
 *  the dialog is shown, the last docking state is restored from the Options
 *  class. If the dialog has been docked before, it appears at its earlier
 *  position, if it was undocked, it appears at its initial position. <br>
 *  <br>
 *
 *  @author Dominik Goeddeke
 *  @version fully tested
 *  
 */
public abstract class DockableDialog extends JDialog
{
    /**
     *  reference to the MainFrame of the application
     */
    protected MainFrame parent;

    /**
     *  reference to the ControlCenter
     */
    protected ControlCenter cc;

    /**
     *  flag to determine the dialogs docking state
     */
    protected boolean docked = true;

    /**
     *  the docking margin (in pixels)
     */
    protected int dockMargin = 100;

    /**
     *  the region in which to dock
     */
    protected int region = 2;

    /**
     *  constant for the gap between docked dialog and MainFrame
     */
    public static final int GAP = 5;

    /**
     *  the location on screen
     */
    protected Point origin;

    /**
     *  the listener responsible for trailing the dialog's movements
     */
    protected DockableDialogMoveListener listener;

    /**  
     * removes the move listener
     */
    public void removeMoveListener ()
    {
	ComponentListener[] listeners = (ComponentListener[])(getListeners(ComponentListener.class));
	for (int i = 0; i < listeners.length; i++)
	    removeComponentListener (listeners[i]);
    }

    /**
     *  adds the move listener
     */
    public void addMoveListener ()
    {
	ComponentListener[] listeners = (ComponentListener[])(getListeners(ComponentListener.class));
	for (int i = 0; i < listeners.length; i++)
	    removeComponentListener (listeners[i]);
	addComponentListener (listener);
    }

    /**
     *  repositions the dialog
     */
    public void rePosition ()
    {
	setPositionOnScreen (calcPosition ());
    }

    /**
     *  returns if the dialog is docked
     */
    public boolean isDocked ()
    {
	return docked;
    }

    /**
     *  sets the docked variable
     */
    public void setDocked (boolean dock)
    {
	docked = dock;
    }

    /** 
     *  returns the initial Position of the dialog
     */
    public abstract Point getInitialPosition ();

    /**
     *  sets the dockMargin
     */
    public void setDockMargin (int delta)
    {
	dockMargin = delta;
    }

    /**
     *  returns the dockMargin
     */
    public int getDockMargin ()
    {
	return dockMargin;
    }

    /**
     *  sets the location on the screen
     */
    public void setPositionOnScreen (Point newPosition)
    {
	removeMoveListener ();
	setLocation (newPosition);
	origin = newPosition;
	addMoveListener ();
    }

    /**
     *  returns the position on the screen
     */
    public Point getPositionOnScreen ()
    {
	return origin;
    }

    /**
     *  this method can be used to update the GUI of the dialog dynamically.
     *  it is called by the DialogManager e.g. when some comboboxes containing
     *  boundary information need to be refreshed
     */
    public abstract void refreshGUI ();

    /** 
     *  creates a new Dialog
     *
     *  @param parent - the mainframe connects the dialog to the rest of the
     *                  application.
     */
    public DockableDialog (MainFrame parent)
    {
	// basics and attribute initialisation
	super (parent,false);
	this.parent = parent;
	cc = ControlCenter.getMyself();
	setResizable (false);
	setDefaultCloseOperation (DO_NOTHING_ON_CLOSE);
	listener = new DockableDialogMoveListener ();
    }

    /**
     *  calculates the dockableArea of the MainFrame and sets the docked flag
     *  - the dockable area is calculated by adding the dockMargin value to
     *    all four sides of the MainFrame, both on the inside and outside
     *  - this partitions the area around the MainFrame's edges into
     *    four regions, 1=LEFT, 2=RIGHT, 3=TOP. 4=BOTTOM
     *  - if the dialog is outside any of these regions, the docked flag
     *    is set to false, otherwise to true
     */
    protected void calcDockable ()
    {
	region = 0;
	// calculate MainFrame perimeter
	Point p_topleft = parent.getLocationOnScreen();
	Point p_bottomleft = new Point (p_topleft.x, p_topleft.y+parent.getHeight ());
	Point p_topright = new Point (p_topleft.x + parent.getWidth(), p_topleft.y);
	Point p_bottomright = new Point (p_topright.x, p_bottomleft.y);
	// calculate event point: dialog center
	Point eventPoint = new Point (origin.x + (getWidth()/2), origin.y + (getHeight()/2));
	// determine region
	// left region ??
	if ( (eventPoint.x < p_topleft.x+dockMargin) & (eventPoint.x > p_topleft.x - dockMargin) & (eventPoint.y > p_topleft.y - dockMargin) & (eventPoint.y < p_bottomleft.y + dockMargin) )
	{
	    region = 1;
	}
	else
	{
	    // right region
	    if ( (eventPoint.x < p_topright.x+dockMargin) & (eventPoint.x > p_topright.x - dockMargin) & (eventPoint.y > p_topright.y - dockMargin) & (eventPoint.y < p_bottomleft.y + dockMargin) )
	    {
		region = 2;
	    }
	    else
	    {
		// top region
		if ( (eventPoint.y < p_topleft.y+dockMargin) & (eventPoint.y > p_topleft.y - dockMargin) & (eventPoint.x < p_topright.x - dockMargin) & (eventPoint.x > p_topleft.x + dockMargin) )
		{
		    region = 3;
		}
		else
		{
		    // bottom region
		    if ( (eventPoint.y < p_bottomleft.y+dockMargin) & (eventPoint.y > p_bottomleft.y - dockMargin) & (eventPoint.x < p_bottomright.x - dockMargin) & (eventPoint.x > p_bottomleft.x + dockMargin))
		    {
			region = 4;
		    }
		}
	    }
	}
	// do we need to undock ??
	if (region == 0)
	    docked = false;
	else
	    docked = true;
    }

    /**
     *  This method repositions the dialog according to the following rules:
     *  if docked:
     *    - in the left region, the dialog's top right corner is 
     *      aligned with the parent's top left corner if the origin is above 
     *      the parent's top border, with the bottom left corner if below,
     *      otherwise the dialog is just snapped rightwards.
     *    - in the top region, the dialog's bottom edge is just snapped 
     *      downwards to the parent's top edge
     *    - in the right region, the dialog's top left corner is 
     *      aligned with the parent's top right corner if the origin is above 
     *      the parent's top border, with the bottom right corner if below,
     *      otherwise the dialog is just snapped leftwards.
     *    - in the bottom region, the dialog's top edge is just snapped 
     *      upwards to the parent's bottom edge
     *    - in any case, the dialog is moved ifwards if neccessary so 
     *      that it does not disappear over the screen edge.
     */
    protected Point calcPosition ()
    {
	Point neworigin = new Point (origin);
	if (docked)
	{
	    // get screen size
	    Dimension screen = getToolkit().getScreenSize();
	    // calculate MainFrame's corners
	    Point p_topleft = parent.getLocationOnScreen ();
	    Point p_bottomleft = new Point (p_topleft.x, p_topleft.y+parent.getHeight ());
	    Point p_topright = new Point (p_topleft.x + parent.getWidth(), p_topleft.y);
	    Point p_bottomright = new Point (p_topright.x, p_bottomleft.y);
	    // calculate dialogs origin
	    Point d_topleft = origin;
	    // translate dialog 
	    switch (region)
	    {
	    case 1 :
		{
		    // too high
		    if (d_topleft.y < p_topleft.y)
			neworigin.y = p_topleft.y;
		    // too low
		    if (d_topleft.y > p_bottomleft.y)
			neworigin.y = p_bottomleft.y - getHeight ();
		    // and horizontally
		    neworigin.x = p_topleft.x - GAP - getWidth ();
		    break;
		}
	    case 2 :
		{
		    if (d_topleft.y < p_topright.y)
			neworigin.y = p_topright.y;
		    if (d_topleft.y > p_bottomright.y)
			neworigin.y = p_bottomright.y - getHeight ();
		    neworigin.x = p_topright.x + GAP;
		    break;
		}
	    case 3 :
		{
		    // just snap downwards
		    neworigin.y = p_topleft.y - GAP - getHeight ();
		    break;
		}
	    case 4 :
		{
		    // just snap upwards
		    neworigin.y = p_bottomleft.y + GAP;
		    break;
		}
	    }
	    // fix some "over the edge problem"
	    // too high
	    if (neworigin.y <= 0)
		neworigin.y = GAP;
	    // too low
	    if (neworigin.y + getHeight() > (int)screen.getHeight())
		neworigin.y = (int)screen.getHeight() - getHeight() - GAP;
	    // too left
	    if (neworigin.x <= 0)
		neworigin.x = GAP;
	    // too right
	    if (neworigin.x + getWidth() > (int)screen.getWidth())
		neworigin.x = (int)screen.getWidth() - getWidth() - GAP;
	    origin = neworigin;
	}
	return neworigin;
    }

    /**
     *  the anonymous listener class
     */
    class DockableDialogMoveListener extends ComponentAdapter
    {
	public void componentMoved (ComponentEvent e)
	{
	    if (isVisible ())
	    {
		origin = getLocationOnScreen ();
		calcDockable ();
		setPositionOnScreen (calcPosition ());
	    }
	}
    }


    
}























