package devisor2.grid.GUI.dialogs;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.filechooser.*;
import java.io.*;

import devisor2.grid.options.*;

/** 
 *  This class is purely convenience: It just extends the default file chooser
 *  of the swing API to suit our needs. More precisely: Everything we wanted
 *  to be the same in all instances of the dialog was hard-coded, everything
 *  we wanted to change at times was put into the constructor's parameter
 *  list.  <br>
 *  <br>
 *  Usage Instructions: <br>
 *  All you have to do is determine what type of Filter you want to use,
 *  and look up the corresponding filter constant in the FilePickerFilter
 *  class, then think of a caption for the file and the ACCEPT button
 *  caption, the initial directory to be displayed, and whether files or 
 *  directories should be selectable. <br>
 *  After that, just call the constructor with these parameters. <br>
 *  <br>
 *  To show the dialog, call the <code>showDialog()<\code>  method which returns
 *  an integer as code for the button pressed. <br>
 *  After the dialog has been closed, you can get the selected file via the
 *  <code>getSelectedFile() <\code> method. It automatically checks if the 
 *  suffix (according to the filter) has been added, and adds it
 *  in case the user forgot.
 *
 *  @author Dominik Goeddeke
 *  @version fully tested
 */

public class FilePicker extends JFileChooser
{
    private Component parent;
    private ControlCenter cc;
  
    /**
     *  The construktor creates a new FilePicker instance with the given 
     *  parameters:
     *  @param parent - The component to center the dialog on
     *  @param caption - The caption for the dialog title
     *  @param button - the caption for the APPROVE BUTTON
     *  @param initialDirectory - The initial Directory to browse
     *  @param filter - a file filter to be used in this instance of the dialog
     *  @param selectFiles - if true, files only are selectable, otherwise directories only are selectable
     */
    public FilePicker (Component parent, String caption, String button, String initialDirectory, FilePickerFilter filter, boolean selectFiles)
    {
	super (initialDirectory);
	if (selectFiles)
	    this.setFileSelectionMode (FILES_ONLY);
	else
	    this.setFileSelectionMode (DIRECTORIES_ONLY);
	this.addChoosableFileFilter (filter);
	this.setFileHidingEnabled (false);
	this.setMultiSelectionEnabled (false);
	this.setDialogTitle (caption);
	this.setApproveButtonText (button);
	this.setAcceptAllFileFilterUsed(false);
	this.parent = parent;
	cc = ControlCenter.getMyself ();
    }

    /**
     *  The showDialog method offers an alternative parameter list
     *  to suit our needs.
     *  Just calling this method will use the parameters given in the constructor
     *
     *  @return APPROVE_OPTION if the user made a valid selection, CANCEL_OPTION or ERROR_OPTION otherwise (all constants from class JFileChooser)
     */
    public int showDialog ()
    {
	return super.showDialog (parent, null);
    }

    public String getPurePath()
    {
	File f = super.getSelectedFile ();
	return f.getPath();
    }
	    
    /**
     *  If the filter is set to INP, FEAST, the extension is 
     *  automatically added. If the filter is set to ALL, the 
     *  filename remains the same. If the filter is set to FEAT, the extension
     *  prm or tri is removed to suit our save methods.
     *  This means if the user didn't enter an extension in ALL mode,
     *  an error message will be displayed.
     *
     *  @return the selected file with a valid extension
     */
    public File getSelectedFile ()
    {
	File f = super.getSelectedFile ();
	if (f != null)
	{
	    // if directories are only selectable, return the file directly
	    if (getFileSelectionMode() == FILES_ONLY)
	    {
		int filterType = ((FilePickerFilter)getFileFilter()).getFilterType ();
		switch (filterType)
		    {
		    case FilePickerFilter.FILTER_PS :
			{
			    // check for extension and add it
			    String extension = getExtension (f);
			    if (!(extension.equals ("ps")))
				f = new File (f.getAbsolutePath()+ ".ps");
			    break;
			}		    
		   case FilePickerFilter.FILTER_UCD :
			{
			    // check for extension and add it
			    String extension = getExtension (f);
			    if (!(extension.equals ("inp")))
				f = new File (f.getAbsolutePath()+ ".inp");
			    break;
			}
		    case FilePickerFilter.FILTER_FEAST :
			{
			    // check for extension and add it
			    String extension = getExtension (f);
			    if (!(extension.equals ("feast")))
				f = new File (f.getAbsolutePath()+ ".feast");
			    break;
			}
		    case FilePickerFilter.FILTER_FEAT :
			{
			    // check for extension and remove it
			    String extension = getExtension (f);
			    if (extension.equals ("prm") || extension.equals("tri"))
				{
				    String newfile = f.getAbsolutePath();
				    int last = newfile.lastIndexOf (".");
				    newfile = newfile.substring (0,last);
				    f = new File (newfile);
				}
			}
		    }
	    }
	}
	return f;
    }
	
    public String getExtension(File f) 
    {
	if(f != null) 
	{
	    String filename = f.getName();
	    int i = filename.lastIndexOf('.');
	    if(i>0 && i<filename.length()-1) 
	    {
		return filename.substring(i+1).toLowerCase();
	    };
	}
	return "";
    }
}
