package devisor2.grid.GUI.dialogs;



import java.io.File;
import java.util.Hashtable;
import java.util.Enumeration;
import javax.swing.*;
import javax.swing.filechooser.*;

import devisor2.grid.options.*;

/**
 *
 *  This class represents the non-customizable file filter for the
 *  application. It provides (via several constructors) all the file filters
 *  needed.
 *  <br>
 *  To add a new extension type, do the following:<br>
 *  (1) add a new FILTER_??? constant <br>
 *  (2) add a corresponding case block to the constructor <br>
 *  (3) add a corresponding case block to the getExtensions method <br>
 *  (4) add a corresponding case block to the getSelectedFile method in the FilePicker class <br>
 *
 *  @author Dominik Goeddeke
 *  @version fully tested
 *  @see MainActionListener, OptionsActionListener for usage example
 */
public class FilePickerFilter extends FileFilter 
{
    /**
     *  Use this filter to accept only files with extension ".inp"
     */
    public static final int FILTER_UCD = 100000;
    /**
     *  Use this filter to accept only files with extension ".tri" or ".prm"
     */
    public static final int FILTER_FEAT = 100001;
    /**
     *  Use this filter to accept only files with extension ".feast"
     */
    public static final int FILTER_FEAST = 100002;
    /**
     *  Use this filter to accept all files
     */
    public static final int FILTER_ALL = 100003;
    /**
     *  Use this filter to accept only directories
     */
    public static final int FILTER_DIRECTORIES = 100004;
    /**
     *  Use this filter to accept all supported file types
     */
    public static final int FILTER_DEFAULT = 100005;
    /**
     *  Use this filter to accept binaries
     */
    public static final int FILTER_BIN = 100006;
    /**
     *  Use this filter to accept PostScript files
     */
    public static final int FILTER_PS = 100007;



    private ControlCenter cc;
    private Hashtable filters = null;
    private String description = null;
    private int filterType = FILTER_DEFAULT;

    /**
     *  creates a file filter which accepts *.*
     */
    public FilePickerFilter ()
    {
	super ();
    }

    /**
     * Creates a file filter that accepts only the given file types
     *
     *  @param filterType - one of the FILTER_??? constants declared in this class
     *
     */
    public FilePickerFilter (int filterType) 
    {
	cc = ControlCenter.getMyself ();
	this.filters = new Hashtable (5);
	this.filterType = filterType;
	switch (filterType)
	{
	case FILTER_UCD :
	    {
		this.setDescription ((String)cc.rb.getObject ("GUI_dialogs_FilePickerFilter_description_ucd"));
		this.addExtension ("inp");
		break;
	    }
	case FILTER_FEAT :
	    {
		this.setDescription ((String)cc.rb.getObject ("GUI_dialogs_FilePickerFilter_description_feat"));
		this.addExtension ("prm");
		this.addExtension ("tri");
		break;
	    }
	case FILTER_FEAST :
	    {
		this.setDescription ((String)cc.rb.getObject ("GUI_dialogs_FilePickerFilter_description_feast"));
		this.addExtension ("feast");
		break;
	    }
	case FILTER_ALL :
	    {
		this.setDescription ((String)cc.rb.getObject ("GUI_dialogs_FilePickerFilter_description_all"));
		this.addExtension ("*.*");
		break;
	    }
	case FILTER_DIRECTORIES :
	    {
		this.setDescription ((String)cc.rb.getObject ("GUI_dialogs_FilePickerFilter_description_directories"));
		break;
	    }
	case FILTER_BIN :
	    {
		this.setDescription ((String)cc.rb.getObject ("GUI_dialogs_FilePickerFilter_description_binaries"));
		break;
	    }
	case FILTER_PS :
	    {
		this.setDescription ((String)cc.rb.getObject ("GUI_dialogs_FilePickerFilter_description_ps"));
		this.addExtension ("ps");
		break;
	    }
        default :
	    {
		this.setDescription ((String)cc.rb.getObject ("GUI_dialogs_FilePickerFilter_description_default"));
		this.addExtension ("inp");
		this.addExtension ("prm");
		this.addExtension ("tri");
		this.addExtension ("feat");
		this.addExtension ("feast");
		filterType = FILTER_DEFAULT;
		break;
	    }
	}
    }

    /**
     *  returns the filter type of this instance
     *
     *   @return  FILTER_DEFAULT, FILTER_UCD, FILTER_FEAST, FILTER_FEAT etc
     */
    public int getFilterType ()
    {
	return this.filterType;
    }


    /**
     * Determines whether the given file should be diaplayed in the 
     * FilePicker's file list
     *
     * @param f - a file
     * @return true if the file complies with the filter, false otherwise
     *
     * @see FileFilter#accepts
     */
    public boolean accept(File f) 
    {
	if(f != null) 
	{
	    if(f.isDirectory()) 
	    {
		return true;
	    }
	    else
	    {
		if (filterType == FILTER_ALL)
		    return true;
		else
		{
		    String extension = getExtension(f);
		    if(extension != null && filters.get(getExtension(f)) != null) 
		    {
			return true;
		    };
		}
	    }
	}
	return false;
    }
    
    /**
     * Return the extension portion of the file's name .
     *
     * @see #getExtension
     * @see FileFilter#accept
     */
    private String getExtension(File f) 
    {
	if(f != null) 
	{
	    String filename = f.getName();
	    int i = filename.lastIndexOf('.');
	    if(i>0 && i<filename.length()-1) 
	    {
		return filename.substring(i+1).toLowerCase();
	    };
	}
	return null;
    }

    /**
     *  Adds a file type to the current filter.
     *  Note: dots are ignored
     */
    private void addExtension(String extension) 
    {
	filters.put(extension.toLowerCase(), this);
    }


    /**
     * Returns a textual description for the current filter.
     */
    public String getDescription() 
    {
	return description;
    }

    /**
     * Sets the description text.
     */
    private void setDescription(String description) 
    {
	this.description = description;
    }

    /**
     *  returns an array of accepted extensions
     */
    public String[] getExtensions ()
    {
	String[] ext;
	if (filterType == FILTER_UCD)
	{
	    ext = new String[1];
	    ext[0] =  "inp";
	    return ext;
	}
	else
	{
	    if (filterType ==  FILTER_FEAT)
	    {
		ext = new String[2];
		ext[0] =  "prm";
		ext[1] =  "tri";
		return ext;
	    }
	    else
	    {
		if (filterType == FILTER_FEAST)
		{
		    ext = new String[1];
		    ext[0] =  "feast";
		    return ext;
		}
		else
		{
		    if (filterType == FILTER_DIRECTORIES)
		    {
			return null;
		    }
		    else
		    {
			if (filterType == FILTER_PS)
			{
			    ext = new String[1];
			    ext[0] =  "ps";
			    return ext;
			}
			else
			{			
			  if (filterType == FILTER_ALL)
			  {
			      ext = new String[1];
			      ext[0] =  "*.*";
			      return ext;
			  }
			  else
			  {
			     ext = new String[4];
			     ext[0] =  "inp";
			     ext[1] =  "prm";
			     ext[2] =  "tri";
			     ext[3] =  "feast";
			     return ext;
			   }
			}
		    }
		}
	    }
	}
    }
    

}


