package devisor2.grid.GUI.dialogs;

import java.util.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;

import devisor2.grid.options.*;
import devisor2.grid.GUI.framework.*;
import devisor2.grid.GUI.event.*;

/** 
 *  This class represents the OptionsDialog for the DeViSoR2 application.
 *  <br>
 *  On various tabbed panes (tabsheets), the user can set all options
 *  for the application. All options are managed internally in the class
 *  Options in package options, which also defines the default values 
 *  which can be restored if the user wants to. <br>
 *  <br>
 *  Like every dialog in the application, it is controlled via two static
 *  methods: "initialize" has to be called once (e.g. in the startup class)
 *  and again every time the look&feel is changed. then, the dialog can be
 *  displayed whenever neccessary using the "showDialog" method. <br>
 *  <br>
 *  This is how the dialog works: <br>
 *  The user can change the options via TextInputFields, secondary dialogs, 
 *  tables and other GUI-only elements. This only affects the representation
 *  in the dialog (i.e. in the GUI), nothing is written back into the
 *  Options instance. <br>
 *  If the user clicks on CANCEL, all changes that have beed made are
 *  ignored and the dialog is closed. Accordingly, all changes are ignored
 *  if the user clicks the LOAD respectively RESTORE DEFAULTS buttons.
 *  Only if the user clicks on APPLY or SAVE, the Options instance is 
 *  updated before closing the dialog or writing the Options file to disk.
 *  Note that updating might take quite a while, especially when GUI elements
 *  such as the menu (for accelerators) or even the whole GUI (for look&feel)
 *  are reconfigured and repainted. <br>
 *  <br>
 *  A short description of the settings and how they are managed:<br>
 *  - The internal storage of Options is described in the Options class, see
 *    there for details. Note that only Strings are stored.<br>
 *  - everything that is represented by a String (such as paths etc) are 
 *    stored the way they are without further changes. <br>
 *  - Path input is implemented in two ways: the user can directly type
 *    the path into a textfield (followed by a short check by the program
 *    of course), or he can use the dialog provided to browse the directory
 *    tree to his destination path. <br>
 *  - The draw flags on the DRAW tab sheet are stored as String literals
 *    <code>true<\code> and <code>false<\code> respectively.<br>
 *  - To assign new colors to certain elements and displayed items, an
 *    instance of the default JColorChooser Dialog is displayed when the
 *    corresponding button is clicked. Colors are converted to an integer
 *    representing the color in the RGBA model. <br>
 *  - Accelerator input is implemented as follows: In a JTable, all available
 *    functions that accelerators can be assigned to are listed, allowing the
 *    user to pick the function he wants to reassign. Then, the modifier and
 *    key can be changed via radiobuttons and a combobox. The user has to
 *    click the APPLY button explicitly to notify the table (the data model
 *    of which holds the accelerators). Note that APPLY only updates the table
 *    and not the Options instance, consistent with the general behaviour of 
 *    this dialog. Assigning the NONE accelerator to a function disables
 *    keyboard support for the specified function. This can be accomplished 
 *    by clicking the REMOVE button.<br>
 *
 *
 *
 *  -  as soon as other options are added, i will update this description
 *     subsequently<br>
 *
 *
 *
 *  @author Dominik Goeddeke
 *  @version 0.9 (for Options declared so far, full GUI support available)
 *  @see options.Options
 */
public class OptionsDialog extends JDialog
{
    // a static self-reference used for the two public static methods
    private static OptionsDialog dialog;
    // the parent frame where the dialog is centered on
    private MainFrame parent;
    // the actionlistener object
    private OptionsActionListener actionlistener;
    // the ControlCenter reference
    private ControlCenter cc;
 
    /** 
     *  creates a new Dialog and sets up its gui.
     *  please don't use this constructor directly, use the two static
     *  methods provided.
     *
     *  @param parent - the mainframe connects the dialog to the rest of the
     *                  application.
     */
    public OptionsDialog (MainFrame parent)
    {
	// basics and attribute initialisation
	super (parent,true);
	this.parent = parent;
	actionlistener = new OptionsActionListener (parent,this);
	cc = ControlCenter.getMyself();
	// gui and event handling
	initDialog ();
	initGeneralPanel ();
	initDrawPanel ();
	initColorPanel ();
	initAcceleratorPanel ();
	initAddonPanel ();
	initOtherPanel ();
	// ok all done
	getContentPane().add(tabs,BorderLayout.CENTER);
	pack();
    }


    /**
     * creates basic layout, the south panel for the buttons, 
     * positioning etc. 
     */
    private void initDialog ()
    {
	tabs = new JTabbedPane ();
	getContentPane().setLayout (new BorderLayout());
	setTitle ((String)cc.rb.getObject ("options_OptionsDialog_general_title"));
	setDefaultCloseOperation (DO_NOTHING_ON_CLOSE);
	// south panel includes a couple of buttons
	JPanel buttons = new JPanel ();
	buttons.setLayout (new FlowLayout ());
	JButton accept = new JButton ((String)cc.rb.getObject ("options_OptionsDialog_acceptButton"));
	accept.setToolTipText ((String)cc.rb.getObject ("options_OptionsDialog_acceptButton_tooltip"));
	accept.setMnemonic (((Integer)cc.rb.getObject("options_OptionsDialog_acceptButton_mnemonic")).intValue());
	accept.addActionListener (actionlistener);
	accept.setActionCommand (OptionsActionListener.ACCEPT);
	buttons.add(accept);
	JButton cancel = new JButton((String)cc.rb.getObject ("options_OptionsDialog_cancelButton"));
	cancel.setToolTipText ((String)cc.rb.getObject ("options_OptionsDialog_cancelButton_tooltip"));
	cancel.setMnemonic (((Integer)cc.rb.getObject("options_OptionsDialog_cancelButton_mnemonic")).intValue());
	cancel.addActionListener (actionlistener);
	cancel.setActionCommand (OptionsActionListener.CANCEL);
	buttons.add(cancel);
	JButton load = new JButton((String)cc.rb.getObject ("options_OptionsDialog_loadButton"));
	load.setToolTipText ((String)cc.rb.getObject ("options_OptionsDialog_loadButton_tooltip"));
	load.setMnemonic (((Integer)cc.rb.getObject("options_OptionsDialog_loadButton_mnemonic")).intValue());
	load.addActionListener (actionlistener);
	load.setActionCommand (OptionsActionListener.LOAD);
	buttons.add(load);
	JButton save = new JButton((String)cc.rb.getObject ("options_OptionsDialog_saveButton"));
	save.setToolTipText ((String)cc.rb.getObject ("options_OptionsDialog_saveButton_tooltip"));
	save.setMnemonic (((Integer)cc.rb.getObject("options_OptionsDialog_saveButton_mnemonic")).intValue());
	save.addActionListener (actionlistener);
	save.setActionCommand (OptionsActionListener.SAVE);
	buttons.add(save);
	JButton reset = new JButton((String)cc.rb.getObject ("options_OptionsDialog_resetButton"));
	reset.setToolTipText ((String)cc.rb.getObject ("options_OptionsDialog_resetButton_tooltip"));
	reset.setMnemonic (((Integer)cc.rb.getObject("options_OptionsDialog_resetButton_mnemonic")).intValue());
	reset.addActionListener (actionlistener);
	reset.setActionCommand (OptionsActionListener.RESET);
	buttons.add(reset);
	JButton help = new JButton((String)cc.rb.getObject ("options_OptionsDialog_helpButton"));
	help.setToolTipText ((String)cc.rb.getObject ("options_OptionsDialog_helpButton_tooltip"));
	help.setMnemonic (((Integer)cc.rb.getObject("options_OptionsDialog_helpButton_mnemonic")).intValue());
	help.addActionListener (actionlistener);
	help.setActionCommand (OptionsActionListener.HELP);
	buttons.add(help);
	// add the buttons
	getContentPane().add (buttons,BorderLayout.SOUTH);
	// and make the accept button the default one
	getRootPane().setDefaultButton (accept);
    }

    /** creates the GUI for the GENERAL tab */
    private void initGeneralPanel ()
    {
	JLabel label;
	generalPanel = new JPanel ();
	generalPanel.setBorder (new EmptyBorder (10,10,10,10));
	GridBagLayout layout = new GridBagLayout ();
	GridBagConstraints constraints = new GridBagConstraints ();
	generalPanel.setLayout (layout);

	// DEVISORHOME selection
	label = new JLabel ((String)cc.rb.getObject ("options_OptionsDialog_generalPanel_devisorhome")+":");
	constraints.gridy = 0;
	constraints.gridx = 0;
	layout.setConstraints (label, constraints);
	generalPanel.add (label);
	devhomeTextField = new JTextField (30);
//	devhomeTextField.setMaximumSize (new Dimension (100,31));
	//devhomeTextField.setMinimumSize (new Dimension (100,31));
	//devhomeTextField.setPreferredSize (new Dimension (100,31));
	constraints.gridx = 1;
	layout.setConstraints (devhomeTextField, constraints);
	generalPanel.add (devhomeTextField);
	devhomeButton = new JButton ("...");
	devhomeButton.setActionCommand (OptionsActionListener.DEVHOME);
	devhomeButton.addActionListener (actionlistener);
	//  devhomeButton.setMaximumSize (new Dimension (31,31));
//  	devhomeButton.setMinimumSize (new Dimension (31,30));
//  	devhomeButton.setPreferredSize (new Dimension (31,30));
	devhomeButton.setToolTipText ((String)cc.rb.getObject ("options_OptionsDialog_generalPanel_devisorhome_tooltip"));
	constraints.gridx = 2;
	layout.setConstraints (devhomeButton, constraints);
	generalPanel.add (devhomeButton);
	// GENERAL PATH
	label = new JLabel ((String)cc.rb.getObject ("options_OptionsDialog_generalPanel_generalpath")+":");
	constraints.gridy = 1;
	constraints.gridx = 0;
	layout.setConstraints (label, constraints);
	generalPanel.add (label);
	generalpathTextField = new JTextField (30);
	//  generalpathTextField.setMaximumSize (new Dimension (100,31));
//  	generalpathTextField.setMinimumSize (new Dimension (100,31));
//  	generalpathTextField.setPreferredSize (new Dimension (100,31));
	constraints.gridx = 1;
	layout.setConstraints (generalpathTextField, constraints);
	generalPanel.add (generalpathTextField);
	generalpathButton = new JButton ("...");
	generalpathButton.setActionCommand (OptionsActionListener.PATH);
	generalpathButton.addActionListener (actionlistener);
	//  generalpathButton.setMaximumSize (new Dimension (31,31));
//  	generalpathButton.setMinimumSize (new Dimension (31,30));
//  	generalpathButton.setPreferredSize (new Dimension (31,30));
	generalpathButton.setToolTipText ((String)cc.rb.getObject ("options_OptionsDialog_generalPanel_generalpath_tooltip"));
	constraints.gridx = 2;
	layout.setConstraints (generalpathButton, constraints);
	generalPanel.add (generalpathButton);
	// Trigen Path
	label = new JLabel ((String)cc.rb.getObject ("options_OptionsDialog_generalPanel_trigen")+":");
	constraints.gridy = 2;
	constraints.gridx = 0;
	layout.setConstraints (label, constraints);
	generalPanel.add (label);
	trigenTextField = new JTextField (30);
	//  trigenTextField.setMaximumSize (new Dimension (100,31));
//  	trigenTextField.setMinimumSize (new Dimension (100,31));
//  	trigenTextField.setPreferredSize (new Dimension (100,31));
	constraints.gridx = 1;
	layout.setConstraints (trigenTextField, constraints);
	generalPanel.add (trigenTextField);
	trigenButton = new JButton ("...");
	trigenButton.setActionCommand (OptionsActionListener.TRIGEN);
	trigenButton.addActionListener (actionlistener);
	//  trigenButton.setMaximumSize (new Dimension (31,31));
//  	trigenButton.setMinimumSize (new Dimension (31,30));
//  	trigenButton.setPreferredSize (new Dimension (31,30));
	trigenButton.setToolTipText ((String)cc.rb.getObject ("options_OptionsDialog_generalPanel_trigen_tooltip"));
	constraints.gridx = 2;
	layout.setConstraints (trigenButton, constraints);
	generalPanel.add (trigenButton);
	// Feat Library Path
	label = new JLabel ((String)cc.rb.getObject ("options_OptionsDialog_generalPanel_featlib")+":");
	constraints.gridy = 3;
	constraints.gridx = 0;
	layout.setConstraints (label, constraints);
	generalPanel.add (label);
	featlibTextField = new JTextField (30);
	//  featlibTextField.setMaximumSize (new Dimension (100,31));
//  	featlibTextField.setMinimumSize (new Dimension (100,31));
//  	featlibTextField.setPreferredSize (new Dimension (100,31));
	constraints.gridx = 1;
	layout.setConstraints (featlibTextField, constraints);
	generalPanel.add (featlibTextField);
	featlibButton = new JButton ("...");
	featlibButton.setActionCommand (OptionsActionListener.FEATLIB);
	featlibButton.addActionListener (actionlistener);
	//  featlibButton.setMaximumSize (new Dimension (31,31));
//  	featlibButton.setMinimumSize (new Dimension (31,30));
//  	featlibButton.setPreferredSize (new Dimension (31,30));
	featlibButton.setToolTipText ((String)cc.rb.getObject ("options_OptionsDialog_generalPanel_featlib_tooltip"));
	constraints.gridx = 2;
	layout.setConstraints (featlibButton, constraints);
	generalPanel.add (featlibButton);	
	tabs.addTab ((String)cc.rb.getObject ("options_OptionsDialog_generalPanel_tabheader"),null,generalPanel,(String)cc.rb.getObject ("options_OptionsDialog_generalPanel_tabheader_tooltip"));
    }

  /** creates the GUI for the DRAW tab */
    private void initDrawPanel ()
    {
	drawPanel = new JPanel ();
	drawPanel.setBorder (new EmptyBorder (10,10,10,10));
	GridBagLayout layout = new GridBagLayout ();
	GridBagConstraints constraints = new GridBagConstraints ();
	drawPanel.setLayout (layout);
	constraints.fill = GridBagConstraints.HORIZONTAL;
	constraints.weightx = 1.0;
	// boundary
	JPanel boundary = new JPanel ();
	boundary.setLayout (new BoxLayout (boundary,BoxLayout.Y_AXIS));
	boundary.setBorder (BorderFactory.createTitledBorder((String)cc.rb.getObject ("options_OptionsDialog_drawPanel_boundary_header")));
	drawboundaryCheck = new JCheckBox ((String)cc.rb.getObject ("options_OptionsDialog_drawPanel_boundary_draw"));
	boundary.add (drawboundaryCheck);
	numberboundaryCheck = new JCheckBox ((String)cc.rb.getObject ("options_OptionsDialog_drawPanel_boundary_number"));
	boundary.add (numberboundaryCheck);
	// node
	JPanel node = new JPanel ();
	node.setLayout (new BoxLayout (node,BoxLayout.Y_AXIS));
	node.setBorder (BorderFactory.createTitledBorder((String)cc.rb.getObject ("options_OptionsDialog_drawPanel_node_header")));
	drawnodeCheck = new JCheckBox ((String)cc.rb.getObject ("options_OptionsDialog_drawPanel_node_draw"));
	node.add (drawnodeCheck);
	numbernodeCheck = new JCheckBox ((String)cc.rb.getObject ("options_OptionsDialog_drawPanel_node_number")); 
	node.add (numbernodeCheck);
	// edge
	JPanel edge = new JPanel ();
	edge.setLayout (new BoxLayout (edge,BoxLayout.Y_AXIS));
	edge.setBorder (BorderFactory.createTitledBorder((String)cc.rb.getObject ("options_OptionsDialog_drawPanel_edge_header")));
	drawedgeCheck = new JCheckBox ((String)cc.rb.getObject ("options_OptionsDialog_drawPanel_edge_draw"));
	edge.add (drawedgeCheck);
	numberedgeCheck = new JCheckBox ((String)cc.rb.getObject ("options_OptionsDialog_drawPanel_edge_number")); 
	edge.add (numberedgeCheck);
	// element
	JPanel element = new JPanel ();
	element.setLayout (new BoxLayout (element,BoxLayout.Y_AXIS));
	element.setBorder (BorderFactory.createTitledBorder((String)cc.rb.getObject ("options_OptionsDialog_drawPanel_element_header")));
	drawelementCheck = new JCheckBox ((String)cc.rb.getObject ("options_OptionsDialog_drawPanel_element_draw"));
	element.add (drawelementCheck);
	numberelementCheck = new JCheckBox ((String)cc.rb.getObject ("options_OptionsDialog_drawPanel_element_number")); 
	element.add (numberelementCheck);
	crosselementCheck = new JCheckBox ((String)cc.rb.getObject ("options_OptionsDialog_drawPanel_element_cross"));
	element.add (crosselementCheck);
	fillelementCheck = new JCheckBox ((String)cc.rb.getObject ("options_OptionsDialog_drawPanel_element_fill")); 
	element.add (fillelementCheck);
	// misc
	JPanel misc = new JPanel ();
	misc.setLayout (new BoxLayout (misc,BoxLayout.Y_AXIS));
	misc.setBorder (BorderFactory.createTitledBorder((String)cc.rb.getObject ("options_OptionsDialog_drawPanel_misc_header")));
	drawgridCheck = new JCheckBox ((String)cc.rb.getObject ("options_OptionsDialog_drawPanel_misc_grid"));
	misc.add (drawgridCheck);
	drawperimeterCheck = new JCheckBox ((String)cc.rb.getObject ("options_OptionsDialog_drawPanel_misc_perimeter"));
	misc.add (drawperimeterCheck);
	drawrectCheck = new JCheckBox ((String)cc.rb.getObject ("options_OptionsDialog_drawPanel_misc_rectangular"));
	misc.add (drawrectCheck);
	drawcoordsCheck = new JCheckBox ((String)cc.rb.getObject ("options_OptionsDialog_drawPanel_edit_drawcoords"));
	misc.add (drawcoordsCheck);
	draworimarkerCheck = new JCheckBox ((String)cc.rb.getObject ("options_OptionsDialog_drawPanel_edit_draworientationmarkers"));
	misc.add (draworimarkerCheck);
	// edit
	JPanel edit = new JPanel ();
	GridBagLayout editlayout = new GridBagLayout ();
	GridBagConstraints editconstraints = new GridBagConstraints ();
	edit.setLayout (editlayout);
	editconstraints.fill = GridBagConstraints.HORIZONTAL;
	editconstraints.weightx = 1.0;
	edit.setBorder (BorderFactory.createTitledBorder ((String)cc.rb.getObject("options_OptionsDialog_drawPanel_edit_header")));
	snapCheck = new JCheckBox ((String)cc.rb.getObject ("options_OptionsDialog_drawPanel_edit_snapon"));
	snapCheck.addActionListener (actionlistener);
	snapCheck.setActionCommand (OptionsActionListener.SNAP);
	editconstraints.gridy = 0;
	editconstraints.gridx = 0;
	editlayout.setConstraints(snapCheck, editconstraints);
	edit.add (snapCheck);
	JLabel snapx = new JLabel ((String)cc.rb.getObject ("options_OptionsDialog_drawPanel_edit_snapx"));
	editconstraints.gridy = 1;
	editconstraints.gridx = 1;
	editlayout.setConstraints(snapx, editconstraints);
	edit.add (snapx);
	snapXTF = new JTextField (5);
	editconstraints.gridx = 2;
	editlayout.setConstraints(snapXTF, editconstraints);
	edit.add (snapXTF);
	JLabel snapy = new JLabel ((String)cc.rb.getObject ("options_OptionsDialog_drawPanel_edit_snapy"));
	editconstraints.gridy = 2;
	editconstraints.gridx = 1;
	editlayout.setConstraints(snapy, editconstraints);
	edit.add (snapy);
	snapYTF = new JTextField (5);
	editconstraints.gridx = 2;
	editlayout.setConstraints(snapYTF, editconstraints);
	edit.add (snapYTF);
	JLabel tolLabel = new JLabel ((String)cc.rb.getObject ("options_OptionsDialog_drawPanel_edit_tolerance"));
	editconstraints.gridy = 3;
	editconstraints.gridx = 0;
	editlayout.setConstraints(tolLabel, editconstraints);
	edit.add (tolLabel);
	toleranceTF = new JTextField (5);
	editconstraints.gridx = 1;
	editlayout.setConstraints(toleranceTF, editconstraints);
	edit.add (toleranceTF);
	JLabel epsilonLabel = new JLabel ((String)cc.rb.getObject ("options_OptionsDialog_drawPanel_edit_epsilon"));
	editconstraints.gridy = 4;
	editconstraints.gridx = 0;
	editlayout.setConstraints(epsilonLabel, editconstraints);
	edit.add (epsilonLabel);
	epsilonSlider = new JSlider(JSlider.HORIZONTAL, 0, 100, 33);
	Hashtable sliderlabels = new Hashtable();
	sliderlabels.put( new Integer( 1 ), new JLabel((String)cc.rb.getObject("options_OptionsDialog_drawPanel_edit_epsilon_sliderlow") ));
	sliderlabels.put( new Integer( 50 ), new JLabel((String)cc.rb.getObject("options_OptionsDialog_drawPanel_edit_epsilon_slidermedium")) );
	sliderlabels.put( new Integer( 99 ), new JLabel((String)cc.rb.getObject("options_OptionsDialog_drawPanel_edit_epsilon_sliderhigh") ));
        epsilonSlider.setMajorTickSpacing(20);
        epsilonSlider.setMinorTickSpacing(10);
        epsilonSlider.setPaintTicks(true);
        epsilonSlider.setPaintLabels(true);
	epsilonSlider.setLabelTable (sliderlabels);
        epsilonSlider.setBorder(BorderFactory.createEmptyBorder(0,0,5,0));
	editconstraints.gridy = 5;
	editconstraints.gridx = 1;
	editconstraints.gridwidth = 3;
	editlayout.setConstraints(epsilonSlider, editconstraints);
	edit.add (epsilonSlider);
	// add all boxes in logical order
	constraints.gridy = 0;
	layout.setConstraints(boundary, constraints);
	drawPanel.add (boundary);
	constraints.gridx = 1;
	layout.setConstraints(node, constraints);
	drawPanel.add (node);
	constraints.gridy = 1;
	constraints.gridx = 0;
	layout.setConstraints(edge, constraints);
	drawPanel.add (edge);
	constraints.gridx = 1;
	layout.setConstraints(element, constraints);
	drawPanel.add (element);
	constraints.gridy = 2;
	constraints.gridx = 0;
	layout.setConstraints(misc, constraints);
	drawPanel.add (misc);
	constraints.gridx = 1;
	layout.setConstraints (edit, constraints);
	drawPanel.add (edit);
	tabs.addTab ((String)cc.rb.getObject ("options_OptionsDialog_drawPanel_tabheader"),null,drawPanel,(String)cc.rb.getObject ("options_OptionsDialog_drawPanel_tabheader_tooltip"));
    }

  /** creates the GUI for the COLOR tab */
    private void initColorPanel ()
    {
	colorPanel = new JPanel ();
	colorPanel.setBorder (new EmptyBorder (10,10,10,10));
	JLabel label;
	JButton button;
	// layoutmanager for the whole tab
	GridBagLayout layout = new GridBagLayout ();
	GridBagConstraints constraints = new GridBagConstraints ();
	colorPanel.setLayout (layout);
	constraints.fill = GridBagConstraints.BOTH;
	constraints.weightx = 1.0;
	constraints.weighty = 1.0;
	// layoutmanager for each panel
	GridBagLayout panellayout = new GridBagLayout ();
	GridBagConstraints panelconstraints = new GridBagConstraints ();
	panelconstraints.insets = new Insets (0,15,3,15);
	// objects - nodes
	JPanel objects = new JPanel ();
	objects.setLayout (panellayout);
	objects.setBorder (BorderFactory.createTitledBorder((String)cc.rb.getObject ("options_OptionsDialog_colorPanel_objects_header")));
	panelconstraints.gridy = 0;
	label = new JLabel ((String)cc.rb.getObject ("options_OptionsDialog_colorPanel_objects_node"));
	panelconstraints.gridx = 0;
	panellayout.setConstraints(label, panelconstraints);
	objects.add (label);
	colornodeLabel = new JLabel ();
	colornodeLabel.setPreferredSize (new Dimension (50,24));
	colornodeLabel.setOpaque (true);
	colornodeLabel.setBorder (BorderFactory.createLineBorder (Color.black));
	panelconstraints.gridx = 1;
	panellayout.setConstraints(colornodeLabel, panelconstraints);
	objects.add (colornodeLabel);
	button = new JButton ("...");
	button.addActionListener (actionlistener);
	button.setActionCommand (OptionsActionListener.COLORNODE);
	panelconstraints.gridx = 2;
	panellayout.setConstraints(button, panelconstraints);
	objects.add (button);
	// objects edges
	panelconstraints.gridy = 1;
	label = new JLabel ((String)cc.rb.getObject ("options_OptionsDialog_colorPanel_objects_edge"));
	panelconstraints.gridx = 0;
	panellayout.setConstraints(label, panelconstraints);
	objects.add (label);
	coloredgeLabel = new JLabel ();
	coloredgeLabel.setPreferredSize (new Dimension (50,24));
	coloredgeLabel.setOpaque (true);
	coloredgeLabel.setBorder (BorderFactory.createLineBorder (Color.black));
	panelconstraints.gridx = 1;
	panellayout.setConstraints(coloredgeLabel, panelconstraints);
	objects.add (coloredgeLabel);
	button = new JButton ("...");
	button.addActionListener (actionlistener);
	button.setActionCommand (OptionsActionListener.COLOREDGE);
	panelconstraints.gridx = 2;
	panellayout.setConstraints(button, panelconstraints);
	objects.add (button);
	// objects edgebase
	panelconstraints.gridy = 2;
	label = new JLabel ((String)cc.rb.getObject ("options_OptionsDialog_colorPanel_objects_edgebase"));
	panelconstraints.gridx = 0;
	panellayout.setConstraints(label, panelconstraints);
	objects.add (label);
	coloredgebaseLabel = new JLabel ();
	coloredgebaseLabel.setPreferredSize (new Dimension (50,24));
	coloredgebaseLabel.setOpaque (true);
	coloredgebaseLabel.setBorder (BorderFactory.createLineBorder (Color.black));
	panelconstraints.gridx = 1;
	panellayout.setConstraints(coloredgebaseLabel, panelconstraints);
	objects.add (coloredgebaseLabel);
	button = new JButton ("...");
	button.addActionListener (actionlistener);
	button.setActionCommand (OptionsActionListener.COLOREDGEBASE);
	panelconstraints.gridx = 2;
	panellayout.setConstraints(button, panelconstraints);
	objects.add (button);

	// objects - elements
	panelconstraints.gridy = 3;
	label = new JLabel ((String)cc.rb.getObject ("options_OptionsDialog_colorPanel_objects_element"));
	panelconstraints.gridx = 0;
	panellayout.setConstraints(label, panelconstraints);
	objects.add (label);
	colorelementLabel = new JLabel ();
	colorelementLabel.setPreferredSize (new Dimension (50,24));
	colorelementLabel.setOpaque (true);
	colorelementLabel.setBorder (BorderFactory.createLineBorder (Color.black));
	panelconstraints.gridx = 1;
	panellayout.setConstraints(colorelementLabel, panelconstraints);
	objects.add (colorelementLabel);
	button = new JButton ("...");
	button.addActionListener (actionlistener);
	button.setActionCommand (OptionsActionListener.COLORELEMENT);
	panelconstraints.gridx = 2;
	panellayout.setConstraints(button, panelconstraints);
	objects.add (button);
	constraints.gridy = 0;
	constraints.gridx = 0;
	layout.setConstraints(objects, constraints);
	colorPanel.add (objects);
	// numbers - boundary
	JPanel numbers = new JPanel ();
	numbers.setLayout (panellayout);
	numbers.setBorder (BorderFactory.createTitledBorder((String)cc.rb.getObject ("options_OptionsDialog_colorPanel_numbers_header")));
	label = new JLabel ((String)cc.rb.getObject ("options_OptionsDialog_colorPanel_numbers_boundary"));
	panelconstraints.gridx = 0;
	panelconstraints.gridy = 0;
	panellayout.setConstraints(label, panelconstraints);
	numbers.add (label);
	numberboundaryLabel = new JLabel ();
	numberboundaryLabel.setPreferredSize (new Dimension (50,24));
	numberboundaryLabel.setOpaque (true);
	numberboundaryLabel.setBorder (BorderFactory.createLineBorder (Color.black));
	panelconstraints.gridx = 1;
	panellayout.setConstraints(numberboundaryLabel, panelconstraints);
	numbers.add (numberboundaryLabel);
	button = new JButton ("...");
	button.addActionListener (actionlistener);
	button.setActionCommand (OptionsActionListener.NUMBERBOUNDARY);
	panelconstraints.gridx = 2;
	panellayout.setConstraints(button, panelconstraints);
	numbers.add (button);
	// numbers - nodes
	panelconstraints.gridy = 1;
	label = new JLabel ((String)cc.rb.getObject ("options_OptionsDialog_colorPanel_numbers_node"));
	panelconstraints.gridx = 0;
	panellayout.setConstraints(label, panelconstraints);
	numbers.add (label);
	numbernodeLabel = new JLabel ();
	numbernodeLabel.setPreferredSize (new Dimension (50,24));
	numbernodeLabel.setOpaque (true);
	numbernodeLabel.setBorder (BorderFactory.createLineBorder (Color.black));
	panelconstraints.gridx = 1;
	panellayout.setConstraints(numbernodeLabel, panelconstraints);
	numbers.add (numbernodeLabel);
	button = new JButton ("...");
	button.addActionListener (actionlistener);
	button.setActionCommand (OptionsActionListener.NUMBERNODE);
	panelconstraints.gridx = 2;
	panellayout.setConstraints(button, panelconstraints);
	numbers.add (button);
	// numbers - edges
	panelconstraints.gridy = 2;
	label = new JLabel ((String)cc.rb.getObject ("options_OptionsDialog_colorPanel_numbers_edge"));
	panelconstraints.gridx = 0;
	panellayout.setConstraints(label, panelconstraints);
	numbers.add (label);
	numberedgeLabel = new JLabel ();
	numberedgeLabel.setPreferredSize (new Dimension (50,24));
	numberedgeLabel.setOpaque (true);
	numberedgeLabel.setBorder (BorderFactory.createLineBorder (Color.black));
	panelconstraints.gridx = 1;
	panellayout.setConstraints(numberedgeLabel, panelconstraints);
	numbers.add (numberedgeLabel);
	button = new JButton ("...");
	button.addActionListener (actionlistener);
	button.setActionCommand (OptionsActionListener.NUMBEREDGE);
	panelconstraints.gridx = 2;
	panellayout.setConstraints(button, panelconstraints);
	numbers.add (button);
	// numbers - elements
	panelconstraints.gridy = 3;
	label = new JLabel ((String)cc.rb.getObject ("options_OptionsDialog_colorPanel_numbers_element"));
	panelconstraints.gridx = 0;
	panellayout.setConstraints(label, panelconstraints);
	numbers.add (label);
	numberelementLabel = new JLabel ();
	numberelementLabel.setPreferredSize (new Dimension (50,24));
	numberelementLabel.setOpaque (true);
	numberelementLabel.setBorder (BorderFactory.createLineBorder (Color.black));
	panelconstraints.gridx = 1;
	panellayout.setConstraints(numberelementLabel, panelconstraints);
	numbers.add (numberelementLabel);
	button = new JButton ("...");
	button.addActionListener (actionlistener);
	button.setActionCommand (OptionsActionListener.NUMBERELEMENT);
	panelconstraints.gridx = 2;
	panellayout.setConstraints(button, panelconstraints);
	numbers.add (button);
	constraints.gridy = 0;
	constraints.gridx = 1;
	layout.setConstraints(numbers, constraints);
	colorPanel.add (numbers);
	// fillcolor palette
	JPanel fill = new JPanel ();
	fill.setLayout (panellayout);
	fill.setBorder (BorderFactory.createTitledBorder((String)cc.rb.getObject ("options_OptionsDialog_colorPanel_fill_header")));
	panelconstraints.gridx = 0;
	panelconstraints.gridy = 0;
	panelconstraints.insets = new Insets (0,0,0,0);
	constraints.gridwidth = 2;
	fill0Label = new JLabel ();
	fill0Label.setPreferredSize (new Dimension (24,24));
	fill0Label.setOpaque (true);
	fill0Label.setBorder (BorderFactory.createLineBorder (Color.black));
	panellayout.setConstraints(fill0Label, panelconstraints);
	fill.add (fill0Label);
	panelconstraints.gridx = 1;
	fill1Label = new JLabel ();
	fill1Label.setPreferredSize (new Dimension (24,24));
	fill1Label.setOpaque (true);
	fill1Label.setBorder (BorderFactory.createLineBorder (Color.black));
	panellayout.setConstraints(fill1Label, panelconstraints);
	fill.add (fill1Label);
	panelconstraints.gridx = 2;
	fill2Label = new JLabel ();
	fill2Label.setPreferredSize (new Dimension (24,24));
	fill2Label.setOpaque (true);
	fill2Label.setBorder (BorderFactory.createLineBorder (Color.black));
	panellayout.setConstraints(fill2Label, panelconstraints);
	fill.add (fill2Label);
	panelconstraints.gridx = 3;
	fill3Label = new JLabel ();
	fill3Label.setPreferredSize (new Dimension (24,24));
	fill3Label.setOpaque (true);
	fill3Label.setBorder (BorderFactory.createLineBorder (Color.black));
	panellayout.setConstraints(fill3Label, panelconstraints);
	fill.add (fill3Label);
	panelconstraints.gridx = 4;
	fill4Label = new JLabel ();
	fill4Label.setPreferredSize (new Dimension (24,24));
	fill4Label.setOpaque (true);
	fill4Label.setBorder (BorderFactory.createLineBorder (Color.black));
	panellayout.setConstraints(fill4Label, panelconstraints);
	fill.add (fill4Label);
	panelconstraints.gridx = 5;
	fill5Label = new JLabel ();
	fill5Label.setPreferredSize (new Dimension (24,24));
	fill5Label.setOpaque (true);
	fill5Label.setBorder (BorderFactory.createLineBorder (Color.black));
	panellayout.setConstraints(fill5Label, panelconstraints);
	fill.add (fill5Label);
	panelconstraints.gridx = 6;
	fill6Label = new JLabel ();
	fill6Label.setPreferredSize (new Dimension (24,24));
	fill6Label.setOpaque (true);
	fill6Label.setBorder (BorderFactory.createLineBorder (Color.black));
	panellayout.setConstraints(fill6Label, panelconstraints);
	fill.add (fill6Label);
	panelconstraints.gridx = 7;
	fill7Label = new JLabel ();
	fill7Label.setPreferredSize (new Dimension (24,24));
	fill7Label.setOpaque (true);
	fill7Label.setBorder (BorderFactory.createLineBorder (Color.black));
	panellayout.setConstraints(fill7Label, panelconstraints);
	fill.add (fill7Label);
	panelconstraints.gridx = 8;
	fill8Label = new JLabel ();
	fill8Label.setPreferredSize (new Dimension (24,24));
	fill8Label.setOpaque (true);
	fill8Label.setBorder (BorderFactory.createLineBorder (Color.black));
	panellayout.setConstraints(fill8Label, panelconstraints);
	fill.add (fill8Label);
	panelconstraints.gridx = 9;
	fill9Label = new JLabel ();
	fill9Label.setPreferredSize (new Dimension (24,24));
	fill9Label.setOpaque (true);
	fill9Label.setBorder (BorderFactory.createLineBorder (Color.black));
	panellayout.setConstraints(fill9Label, panelconstraints);
	fill.add (fill9Label);
	panelconstraints.gridx = 10;
	fill10Label = new JLabel ();
	fill10Label.setPreferredSize (new Dimension (24,24));
	fill10Label.setOpaque (true);
	fill10Label.setBorder (BorderFactory.createLineBorder (Color.black));
	panellayout.setConstraints(fill10Label, panelconstraints);
	fill.add (fill10Label);
	panelconstraints.gridx = 11;
	fill11Label = new JLabel ();
	fill11Label.setPreferredSize (new Dimension (24,24));
	fill11Label.setOpaque (true);
	fill11Label.setBorder (BorderFactory.createLineBorder (Color.black));
	panellayout.setConstraints(fill11Label, panelconstraints);
	fill.add (fill11Label);
	panelconstraints.gridx = 12;
	fill12Label = new JLabel ();
	fill12Label.setPreferredSize (new Dimension (24,24));
	fill12Label.setOpaque (true);
	fill12Label.setBorder (BorderFactory.createLineBorder (Color.black));
	panellayout.setConstraints(fill12Label, panelconstraints);
	fill.add (fill12Label);
	panelconstraints.gridx = 13;
	fill13Label = new JLabel ();
	fill13Label.setPreferredSize (new Dimension (24,24));
	fill13Label.setOpaque (true);
	fill13Label.setBorder (BorderFactory.createLineBorder (Color.black));
	panellayout.setConstraints(fill13Label, panelconstraints);
	fill.add (fill13Label);
	panelconstraints.gridx = 14;
	fill14Label = new JLabel ();
	fill14Label.setPreferredSize (new Dimension (24,24));
	fill14Label.setOpaque (true);
	fill14Label.setBorder (BorderFactory.createLineBorder (Color.black));
	panellayout.setConstraints(fill14Label, panelconstraints);
	fill.add (fill14Label);
	panelconstraints.gridx = 15;
	fill15Label = new JLabel ();
	fill15Label.setPreferredSize (new Dimension (24,24));
	fill15Label.setOpaque (true);
	fill15Label.setBorder (BorderFactory.createLineBorder (Color.black));
	panellayout.setConstraints(fill15Label, panelconstraints);
	fill.add (fill15Label);
	panelconstraints.gridy = 1;
	button = new JButton ("...");
	button.addActionListener (actionlistener);
	button.setActionCommand (OptionsActionListener.FILL0);
	button.setPreferredSize (new Dimension (26,26));
	panelconstraints.gridx = 0;
	panellayout.setConstraints(button, panelconstraints);
	fill.add (button);
	button = new JButton ("...");
	button.addActionListener (actionlistener);
	button.setActionCommand (OptionsActionListener.FILL1);
	button.setPreferredSize (new Dimension (26,26));
	panelconstraints.gridx = 1;
	panellayout.setConstraints(button, panelconstraints);
	fill.add (button);
	button = new JButton ("...");
	button.addActionListener (actionlistener);
	button.setActionCommand (OptionsActionListener.FILL2);
	button.setPreferredSize (new Dimension (26,26));
	panelconstraints.gridx = 2;
	panellayout.setConstraints(button, panelconstraints);
	fill.add (button);
	button = new JButton ("...");
	button.addActionListener (actionlistener);
	button.setActionCommand (OptionsActionListener.FILL3);
	button.setPreferredSize (new Dimension (26,26));
	panelconstraints.gridx = 3;
	panellayout.setConstraints(button, panelconstraints);
	fill.add (button);
	button = new JButton ("...");
	button.addActionListener (actionlistener);
	button.setActionCommand (OptionsActionListener.FILL4);
	button.setPreferredSize (new Dimension (26,26));
	panelconstraints.gridx = 4;
	panellayout.setConstraints(button, panelconstraints);
	fill.add (button);
	button = new JButton ("...");
	button.addActionListener (actionlistener);
	button.setActionCommand (OptionsActionListener.FILL5);
	button.setPreferredSize (new Dimension (26,26));
	panelconstraints.gridx = 5;
	panellayout.setConstraints(button, panelconstraints);
	fill.add (button);
	button = new JButton ("...");
	button.addActionListener (actionlistener);
	button.setActionCommand (OptionsActionListener.FILL6);
	button.setPreferredSize (new Dimension (26,26));
	panelconstraints.gridx = 6;
	panellayout.setConstraints(button, panelconstraints);
	fill.add (button);
	button = new JButton ("...");
	button.addActionListener (actionlistener);
	button.setActionCommand (OptionsActionListener.FILL7);
	button.setPreferredSize (new Dimension (26,26));
	panelconstraints.gridx = 7;
	panellayout.setConstraints(button, panelconstraints);
	fill.add (button);
	button = new JButton ("...");
	button.addActionListener (actionlistener);
	button.setActionCommand (OptionsActionListener.FILL8);
	button.setPreferredSize (new Dimension (26,26));
	panelconstraints.gridx = 8;
	panellayout.setConstraints(button, panelconstraints);
	fill.add (button);
	button = new JButton ("...");
	button.addActionListener (actionlistener);
	button.setActionCommand (OptionsActionListener.FILL9);
	button.setPreferredSize (new Dimension (26,26));
	panelconstraints.gridx = 9;
	panellayout.setConstraints(button, panelconstraints);
	fill.add (button);
	button = new JButton ("...");
	button.addActionListener (actionlistener);
	button.setActionCommand (OptionsActionListener.FILL10);
	button.setPreferredSize (new Dimension (26,26));
	panelconstraints.gridx = 10;
	panellayout.setConstraints(button, panelconstraints);
	fill.add (button);
	button = new JButton ("...");
	button.addActionListener (actionlistener);
	button.setActionCommand (OptionsActionListener.FILL11);
	button.setPreferredSize (new Dimension (26,26));
	panelconstraints.gridx = 11;
	panellayout.setConstraints(button, panelconstraints);
	fill.add (button);
	button = new JButton ("...");
	button.addActionListener (actionlistener);
	button.setActionCommand (OptionsActionListener.FILL12);
	button.setPreferredSize (new Dimension (26,26));
	panelconstraints.gridx = 12;
	panellayout.setConstraints(button, panelconstraints);
	fill.add (button);
	button = new JButton ("...");
	button.addActionListener (actionlistener);
	button.setActionCommand (OptionsActionListener.FILL13);
	button.setPreferredSize (new Dimension (26,26));
	panelconstraints.gridx = 13;
	panellayout.setConstraints(button, panelconstraints);
	fill.add (button);
	button = new JButton ("...");
	button.addActionListener (actionlistener);
	button.setActionCommand (OptionsActionListener.FILL14);
	button.setPreferredSize (new Dimension (26,26));
	panelconstraints.gridx = 14;
	panellayout.setConstraints(button, panelconstraints);
	fill.add (button);
	button = new JButton ("...");
	button.addActionListener (actionlistener);
	button.setActionCommand (OptionsActionListener.FILL15);
	button.setPreferredSize (new Dimension (26,26));
	panelconstraints.gridx = 15;
	panellayout.setConstraints(button, panelconstraints);
	fill.add (button);
	constraints.gridy = 1;
	constraints.gridx = 0;
	constraints.gridwidth = 2;
	layout.setConstraints(fill, constraints);
	colorPanel.add (fill);

	// boundarycolor palette
	JPanel bound = new JPanel ();
	bound.setLayout (panellayout);
	bound.setBorder (BorderFactory.createTitledBorder((String)cc.rb.getObject ("options_OptionsDialog_colorPanel_boundarypalette_header")));
	panelconstraints.gridx = 0;
	panelconstraints.gridy = 0;
	panelconstraints.insets = new Insets (0,0,0,0);
	constraints.gridwidth = 2;
	bound0Label = new JLabel ();
	bound0Label.setPreferredSize (new Dimension (24,24));
	bound0Label.setOpaque (true);
	bound0Label.setBorder (BorderFactory.createLineBorder (Color.black));
	panellayout.setConstraints(bound0Label, panelconstraints);
	bound.add (bound0Label);
	panelconstraints.gridx = 1;
	bound1Label = new JLabel ();
	bound1Label.setPreferredSize (new Dimension (24,24));
	bound1Label.setOpaque (true);
	bound1Label.setBorder (BorderFactory.createLineBorder (Color.black));
	panellayout.setConstraints(bound1Label, panelconstraints);
	bound.add (bound1Label);
	panelconstraints.gridx = 2;
	bound2Label = new JLabel ();
	bound2Label.setPreferredSize (new Dimension (24,24));
	bound2Label.setOpaque (true);
	bound2Label.setBorder (BorderFactory.createLineBorder (Color.black));
	panellayout.setConstraints(bound2Label, panelconstraints);
	bound.add (bound2Label);
	panelconstraints.gridx = 3;
	bound3Label = new JLabel ();
	bound3Label.setPreferredSize (new Dimension (24,24));
	bound3Label.setOpaque (true);
	bound3Label.setBorder (BorderFactory.createLineBorder (Color.black));
	panellayout.setConstraints(bound3Label, panelconstraints);
	bound.add (bound3Label);
	panelconstraints.gridx = 4;
	bound4Label = new JLabel ();
	bound4Label.setPreferredSize (new Dimension (24,24));
	bound4Label.setOpaque (true);
	bound4Label.setBorder (BorderFactory.createLineBorder (Color.black));
	panellayout.setConstraints(bound4Label, panelconstraints);
	bound.add (bound4Label);
	panelconstraints.gridx = 5;
	bound5Label = new JLabel ();
	bound5Label.setPreferredSize (new Dimension (24,24));
	bound5Label.setOpaque (true);
	bound5Label.setBorder (BorderFactory.createLineBorder (Color.black));
	panellayout.setConstraints(bound5Label, panelconstraints);
	bound.add (bound5Label);
	panelconstraints.gridx = 6;
	bound6Label = new JLabel ();
	bound6Label.setPreferredSize (new Dimension (24,24));
	bound6Label.setOpaque (true);
	bound6Label.setBorder (BorderFactory.createLineBorder (Color.black));
	panellayout.setConstraints(bound6Label, panelconstraints);
	bound.add (bound6Label);
	panelconstraints.gridx = 7;
	bound7Label = new JLabel ();
	bound7Label.setPreferredSize (new Dimension (24,24));
	bound7Label.setOpaque (true);
	bound7Label.setBorder (BorderFactory.createLineBorder (Color.black));
	panellayout.setConstraints(bound7Label, panelconstraints);
	bound.add (bound7Label);
	panelconstraints.gridx = 8;
	bound8Label = new JLabel ();
	bound8Label.setPreferredSize (new Dimension (24,24));
	bound8Label.setOpaque (true);
	bound8Label.setBorder (BorderFactory.createLineBorder (Color.black));
	panellayout.setConstraints(bound8Label, panelconstraints);
	bound.add (bound8Label);
	panelconstraints.gridx = 9;
	bound9Label = new JLabel ();
	bound9Label.setPreferredSize (new Dimension (24,24));
	bound9Label.setOpaque (true);
	bound9Label.setBorder (BorderFactory.createLineBorder (Color.black));
	panellayout.setConstraints(bound9Label, panelconstraints);
	bound.add (bound9Label);
	panelconstraints.gridx = 10;
	bound10Label = new JLabel ();
	bound10Label.setPreferredSize (new Dimension (24,24));
	bound10Label.setOpaque (true);
	bound10Label.setBorder (BorderFactory.createLineBorder (Color.black));
	panellayout.setConstraints(bound10Label, panelconstraints);
	bound.add (bound10Label);
	panelconstraints.gridx = 11;
	bound11Label = new JLabel ();
	bound11Label.setPreferredSize (new Dimension (24,24));
	bound11Label.setOpaque (true);
	bound11Label.setBorder (BorderFactory.createLineBorder (Color.black));
	panellayout.setConstraints(bound11Label, panelconstraints);
	bound.add (bound11Label);
	panelconstraints.gridx = 12;
	bound12Label = new JLabel ();
	bound12Label.setPreferredSize (new Dimension (24,24));
	bound12Label.setOpaque (true);
	bound12Label.setBorder (BorderFactory.createLineBorder (Color.black));
	panellayout.setConstraints(bound12Label, panelconstraints);
	bound.add (bound12Label);
	panelconstraints.gridx = 13;
	bound13Label = new JLabel ();
	bound13Label.setPreferredSize (new Dimension (24,24));
	bound13Label.setOpaque (true);
	bound13Label.setBorder (BorderFactory.createLineBorder (Color.black));
	panellayout.setConstraints(bound13Label, panelconstraints);
	bound.add (bound13Label);
	panelconstraints.gridx = 14;
	bound14Label = new JLabel ();
	bound14Label.setPreferredSize (new Dimension (24,24));
	bound14Label.setOpaque (true);
	bound14Label.setBorder (BorderFactory.createLineBorder (Color.black));
	panellayout.setConstraints(bound14Label, panelconstraints);
	bound.add (bound14Label);
	panelconstraints.gridx = 15;
	bound15Label = new JLabel ();
	bound15Label.setPreferredSize (new Dimension (24,24));
	bound15Label.setOpaque (true);
	bound15Label.setBorder (BorderFactory.createLineBorder (Color.black));
	panellayout.setConstraints(bound15Label, panelconstraints);
	bound.add (bound15Label);
	panelconstraints.gridy = 1;
	button = new JButton ("...");
	button.addActionListener (actionlistener);
	button.setActionCommand (OptionsActionListener.BOUND0);
	button.setPreferredSize (new Dimension (26,26));
	panelconstraints.gridx = 0;
	panellayout.setConstraints(button, panelconstraints);
	bound.add (button);
	button = new JButton ("...");
	button.addActionListener (actionlistener);
	button.setActionCommand (OptionsActionListener.BOUND1);
	button.setPreferredSize (new Dimension (26,26));
	panelconstraints.gridx = 1;
	panellayout.setConstraints(button, panelconstraints);
	bound.add (button);
	button = new JButton ("...");
	button.addActionListener (actionlistener);
	button.setActionCommand (OptionsActionListener.BOUND2);
	button.setPreferredSize (new Dimension (26,26));
	panelconstraints.gridx = 2;
	panellayout.setConstraints(button, panelconstraints);
	bound.add (button);
	button = new JButton ("...");
	button.addActionListener (actionlistener);
	button.setActionCommand (OptionsActionListener.BOUND3);
	button.setPreferredSize (new Dimension (26,26));
	panelconstraints.gridx = 3;
	panellayout.setConstraints(button, panelconstraints);
	bound.add (button);
	button = new JButton ("...");
	button.addActionListener (actionlistener);
	button.setActionCommand (OptionsActionListener.BOUND4);
	button.setPreferredSize (new Dimension (26,26));
	panelconstraints.gridx = 4;
	panellayout.setConstraints(button, panelconstraints);
	bound.add (button);
	button = new JButton ("...");
	button.addActionListener (actionlistener);
	button.setActionCommand (OptionsActionListener.BOUND5);
	button.setPreferredSize (new Dimension (26,26));
	panelconstraints.gridx = 5;
	panellayout.setConstraints(button, panelconstraints);
	bound.add (button);
	button = new JButton ("...");
	button.addActionListener (actionlistener);
	button.setActionCommand (OptionsActionListener.BOUND6);
	button.setPreferredSize (new Dimension (26,26));
	panelconstraints.gridx = 6;
	panellayout.setConstraints(button, panelconstraints);
	bound.add (button);
	button = new JButton ("...");
	button.addActionListener (actionlistener);
	button.setActionCommand (OptionsActionListener.BOUND7);
	button.setPreferredSize (new Dimension (26,26));
	panelconstraints.gridx = 7;
	panellayout.setConstraints(button, panelconstraints);
	bound.add (button);
	button = new JButton ("...");
	button.addActionListener (actionlistener);
	button.setActionCommand (OptionsActionListener.BOUND8);
	button.setPreferredSize (new Dimension (26,26));
	panelconstraints.gridx = 8;
	panellayout.setConstraints(button, panelconstraints);
	bound.add (button);
	button = new JButton ("...");
	button.addActionListener (actionlistener);
	button.setActionCommand (OptionsActionListener.BOUND9);
	button.setPreferredSize (new Dimension (26,26));
	panelconstraints.gridx = 9;
	panellayout.setConstraints(button, panelconstraints);
	bound.add (button);
	button = new JButton ("...");
	button.addActionListener (actionlistener);
	button.setActionCommand (OptionsActionListener.BOUND10);
	button.setPreferredSize (new Dimension (26,26));
	panelconstraints.gridx = 10;
	panellayout.setConstraints(button, panelconstraints);
	bound.add (button);
	button = new JButton ("...");
	button.addActionListener (actionlistener);
	button.setActionCommand (OptionsActionListener.BOUND11);
	button.setPreferredSize (new Dimension (26,26));
	panelconstraints.gridx = 11;
	panellayout.setConstraints(button, panelconstraints);
	bound.add (button);
	button = new JButton ("...");
	button.addActionListener (actionlistener);
	button.setActionCommand (OptionsActionListener.BOUND12);
	button.setPreferredSize (new Dimension (26,26));
	panelconstraints.gridx = 12;
	panellayout.setConstraints(button, panelconstraints);
	bound.add (button);
	button = new JButton ("...");
	button.addActionListener (actionlistener);
	button.setActionCommand (OptionsActionListener.BOUND13);
	button.setPreferredSize (new Dimension (26,26));
	panelconstraints.gridx = 13;
	panellayout.setConstraints(button, panelconstraints);
	bound.add (button);
	button = new JButton ("...");
	button.addActionListener (actionlistener);
	button.setActionCommand (OptionsActionListener.BOUND14);
	button.setPreferredSize (new Dimension (26,26));
	panelconstraints.gridx = 14;
	panellayout.setConstraints(button, panelconstraints);
	bound.add (button);
	button = new JButton ("...");
	button.addActionListener (actionlistener);
	button.setActionCommand (OptionsActionListener.BOUND15);
	button.setPreferredSize (new Dimension (26,26));
	panelconstraints.gridx = 15;
	panellayout.setConstraints(button, panelconstraints);
	bound.add (button);
	constraints.gridy = 2;
	constraints.gridx = 0;
	constraints.gridwidth = 2;
	layout.setConstraints(bound, constraints);
	colorPanel.add (bound);

	// highlight - selection

	panelconstraints.insets = new Insets (0,15,3,15);
	JPanel highlight = new JPanel ();
	highlight.setLayout (panellayout);
	highlight.setBorder (BorderFactory.createTitledBorder((String)cc.rb.getObject ("options_OptionsDialog_colorPanel_highlight_header")));
	label = new JLabel ((String)cc.rb.getObject ("options_OptionsDialog_colorPanel_highlight_selection"));
	panelconstraints.gridx = 0;
	panelconstraints.gridy = 0;
	panellayout.setConstraints(label, panelconstraints);
	highlight.add (label);
	highlightselectionLabel = new JLabel ();
	highlightselectionLabel.setPreferredSize (new Dimension (50,24));
	highlightselectionLabel.setOpaque (true);
	highlightselectionLabel.setBorder (BorderFactory.createLineBorder (Color.black));
	panelconstraints.gridx = 1;
	panellayout.setConstraints(highlightselectionLabel, panelconstraints);
	highlight.add (highlightselectionLabel);
	button = new JButton ("...");
	button.addActionListener (actionlistener);
	button.setActionCommand (OptionsActionListener.HIGHLIGHTSELECTION);
	panelconstraints.gridx = 2;
	panellayout.setConstraints(button, panelconstraints);
	highlight.add (button);
	// highlight - rectified
	label = new JLabel ((String)cc.rb.getObject ("options_OptionsDialog_colorPanel_highlight_rect"));
	panelconstraints.gridx = 0;
	panelconstraints.gridy = 1;
	panellayout.setConstraints(label, panelconstraints);
	highlight.add (label);
	highlightrectLabel = new JLabel ();
	highlightrectLabel.setPreferredSize (new Dimension (50,24));
	highlightrectLabel.setOpaque (true);
	highlightrectLabel.setBorder (BorderFactory.createLineBorder (Color.black));
	panelconstraints.gridx = 1;
	panellayout.setConstraints(highlightrectLabel, panelconstraints);
	highlight.add (highlightrectLabel);
	button = new JButton ("...");
	button.addActionListener (actionlistener);
	button.setActionCommand (OptionsActionListener.HIGHLIGHTRECT);
	panelconstraints.gridx = 2;
	panellayout.setConstraints(button, panelconstraints);
	highlight.add (button);
	constraints.gridy = 3;
	constraints.gridx = 0;
	constraints.gridwidth = 1;
	layout.setConstraints(highlight, constraints);
	colorPanel.add (highlight);
	// misc - grid
	JPanel misc = new JPanel ();
	misc.setLayout (panellayout);
	misc.setBorder (BorderFactory.createTitledBorder((String)cc.rb.getObject ("options_OptionsDialog_colorPanel_misc_header")));
	label = new JLabel ((String)cc.rb.getObject ("options_OptionsDialog_colorPanel_misc_grid"));
	panelconstraints.gridx = 0;
	panelconstraints.gridy = 0;
	panellayout.setConstraints(label, panelconstraints);
	misc.add (label);
	miscgridLabel = new JLabel ();
	miscgridLabel.setPreferredSize (new Dimension (50,24));
	miscgridLabel.setOpaque (true);
	miscgridLabel.setBorder (BorderFactory.createLineBorder (Color.black));
	panelconstraints.gridx = 1;
	panellayout.setConstraints(miscgridLabel, panelconstraints);
	misc.add (miscgridLabel);
	button = new JButton ("...");
	button.addActionListener (actionlistener);
	button.setActionCommand (OptionsActionListener.COLORGRID);
	panelconstraints.gridx = 2;
	panellayout.setConstraints(button, panelconstraints);
	misc.add (button);
	// misc - background
	label = new JLabel ((String)cc.rb.getObject ("options_OptionsDialog_colorPanel_misc_background"));
	panelconstraints.gridx = 0;
	panelconstraints.gridy = 1;
	panellayout.setConstraints(label, panelconstraints);
	misc.add (label);
	miscbackgroundLabel = new JLabel ();
	miscbackgroundLabel.setPreferredSize (new Dimension (50,24));
	miscbackgroundLabel.setOpaque (true);
	miscbackgroundLabel.setBorder (BorderFactory.createLineBorder (Color.black));
	panelconstraints.gridx = 1;
	panellayout.setConstraints(miscbackgroundLabel, panelconstraints);
	misc.add (miscbackgroundLabel);
	button = new JButton ("...");
	button.addActionListener (actionlistener);
	button.setActionCommand (OptionsActionListener.COLORBACKGROUND);
	panelconstraints.gridx = 2;
	panellayout.setConstraints(button, panelconstraints);
	misc.add (button);
	// misc - lasso
	label = new JLabel ((String)cc.rb.getObject ("options_OptionsDialog_colorPanel_misc_lasso"));
	panelconstraints.gridx = 0;
	panelconstraints.gridy = 2;
	panellayout.setConstraints(label, panelconstraints);
	misc.add (label);
	misclassoLabel = new JLabel ();
	misclassoLabel.setPreferredSize (new Dimension (50,24));
	misclassoLabel.setOpaque (true);
	misclassoLabel.setBorder (BorderFactory.createLineBorder (Color.black));
	panelconstraints.gridx = 1;
	panellayout.setConstraints(misclassoLabel, panelconstraints);
	misc.add (misclassoLabel);
	button = new JButton ("...");
	button.addActionListener (actionlistener);
	button.setActionCommand (OptionsActionListener.COLORLASSO);
	panelconstraints.gridx = 2;
	panellayout.setConstraints(button, panelconstraints);
	misc.add (button);
	constraints.gridy = 3;
	constraints.gridx = 1;
	constraints.gridwidth = 1;
	layout.setConstraints(misc, constraints);
	colorPanel.add (misc);
	// all done...
	tabs.addTab ((String)cc.rb.getObject ("options_OptionsDialog_colorPanel_tabheader"),null,colorPanel,(String)cc.rb.getObject ("options_OptionsDialog_colorPanel_tabheader_tooltip"));
    }

  /** creates the GUI for the Accelerators tab */
    private void initAcceleratorPanel ()
    {
	acceleratorPanel = new JPanel ();
	acceleratorPanel.setBorder (new EmptyBorder (10,10,10,10));
	acceleratorPanel.setLayout (new BorderLayout());
	// one JTable, left column for functions
	// right column to display the current accelerator
	JPanel tablepanel = new JPanel ();
	tablepanel.setBorder (BorderFactory.createTitledBorder((String)cc.rb.getObject ("options_OptionsDialog_acceleratorPanel_tabletitle")));
	tablemodel = new AcceleratorTableModel ();
	acceleratorTable = new JTable (tablemodel);
	acceleratorTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
	JScrollPane tablescroll = new JScrollPane (acceleratorTable);
	tablescroll.setPreferredSize (new Dimension (500,300));
	ListSelectionModel rowSM = acceleratorTable.getSelectionModel();
	lsListener = new AcceleratorTableListSelectionListener(dialog);
	rowSM.addListSelectionListener(lsListener); 
	tablepanel.add(tablescroll);
	acceleratorPanel.add (tablepanel, BorderLayout.CENTER);
	// modification support
	JPanel selectaccPanel = new JPanel ();
	selectaccPanel.setLayout (new BoxLayout(selectaccPanel, BoxLayout.Y_AXIS));
        selectaccPanel.setBorder(BorderFactory.createTitledBorder((String)cc.rb.getObject ("options_OptionsDialog_acceleratorPanel_southpaneltitle")));
	// radiogroup for modifiers
	JPanel modpanel = new JPanel ();
	modpanel.setLayout (new FlowLayout ());
	ButtonGroup modifiers = new ButtonGroup ();
	CTRLbutton = new JRadioButton ("CTRL       ");
	modifiers.add(CTRLbutton);
	modpanel.add (CTRLbutton);
	SHIFTbutton = new JRadioButton("SHIFT      ");
	modifiers.add (SHIFTbutton);
	modpanel.add (SHIFTbutton);
	ALTbutton = new JRadioButton  ("ALT        ");
	modifiers.add (ALTbutton);
	modpanel.add (ALTbutton);
	NONEbutton = new JRadioButton ("NONE       ");
	modifiers.add (NONEbutton);
	modpanel.add (NONEbutton);
	// combobox for key constants...
	keySelectionComboBox = initKeySelectionCB ();
	modpanel.add (keySelectionComboBox);
	selectaccPanel.add (modpanel);
	// ...and the buttons to apply changes and to remove assignments
	JPanel accbuttons = new JPanel ();
	accbuttons.setLayout (new FlowLayout ());
	applyAcceleratorButton = new JButton ((String)cc.rb.getObject ("general_Apply"));
	applyAcceleratorButton.setToolTipText ((String)cc.rb.getObject("options_OptionsDialog_acceleratorPanel_applybuttontooltip"));
	applyAcceleratorButton.addActionListener (actionlistener);
	applyAcceleratorButton.setActionCommand (OptionsActionListener.APPLY_ACCELERATOR);
	applyAcceleratorButton.setMnemonic (((Integer)cc.rb.getObject("options_OptionsDialog_acceleratorPanel_applybuttonmnemonic")).intValue());
	//	applyAcceleratorButton.setAlignmentX (CENTER_ALIGNMENT);
	accbuttons.add (applyAcceleratorButton);
	removeAcceleratorButton = new JButton ((String)cc.rb.getObject ("options_OptionsDialog_acceleratorPanel_removebutton"));
	removeAcceleratorButton.setToolTipText((String)cc.rb.getObject ("options_OptionsDialog_acceleratorPanel_removebuttontooltip"));
	removeAcceleratorButton.setMnemonic (((Integer)cc.rb.getObject("options_OptionsDialog_acceleratorPanel_removebuttonmnemonic")).intValue());
	removeAcceleratorButton.addActionListener (actionlistener);
	removeAcceleratorButton.setActionCommand (OptionsActionListener.REMOVE_ACCELERATOR);
	//	removeAcceleratorButton.setAlignmentX (CENTER_ALIGNMENT);
	accbuttons.add (removeAcceleratorButton);
	selectaccPanel.add (accbuttons);
	acceleratorPanel.add (selectaccPanel, BorderLayout.SOUTH);
	// ok all done
	tabs.addTab ((String)cc.rb.getObject ("options_OptionsDialog_acceleratorPanel_tabheader"),null,acceleratorPanel,(String)cc.rb.getObject ("options_OptionsDialog_acceleratorPanel_tabheader_tooltip"));
    }

    /**
     *  a little bit ugly here, all valid entries are stored in the
     *  key selection combo box of the ACCELERATORS tab
     *  directly to make sure only supported keys are shown
     */
    private JComboBox initKeySelectionCB ()
    {
	JComboBox combo = new JComboBox();
	combo.addItem ("NONE");
	combo.addItem (Accelerators.getKeyString(KeyEvent.VK_0));
	combo.addItem (Accelerators.getKeyString(KeyEvent.VK_1));
	combo.addItem (Accelerators.getKeyString(KeyEvent.VK_2));
	combo.addItem (Accelerators.getKeyString(KeyEvent.VK_3));
	combo.addItem (Accelerators.getKeyString(KeyEvent.VK_4));
	combo.addItem (Accelerators.getKeyString(KeyEvent.VK_5));
	combo.addItem (Accelerators.getKeyString(KeyEvent.VK_6));
	combo.addItem (Accelerators.getKeyString(KeyEvent.VK_7));
	combo.addItem (Accelerators.getKeyString(KeyEvent.VK_8));
	combo.addItem (Accelerators.getKeyString(KeyEvent.VK_9));
	combo.addItem (Accelerators.getKeyString(KeyEvent.VK_A));
	combo.addItem (Accelerators.getKeyString(KeyEvent.VK_B));
	combo.addItem (Accelerators.getKeyString(KeyEvent.VK_C));
	combo.addItem (Accelerators.getKeyString(KeyEvent.VK_D));
	combo.addItem (Accelerators.getKeyString(KeyEvent.VK_DELETE));
	combo.addItem (Accelerators.getKeyString(KeyEvent.VK_E));
	combo.addItem (Accelerators.getKeyString(KeyEvent.VK_END));
	combo.addItem (Accelerators.getKeyString(KeyEvent.VK_F));
	combo.addItem (Accelerators.getKeyString(KeyEvent.VK_F1));
	combo.addItem (Accelerators.getKeyString(KeyEvent.VK_F10));
	combo.addItem (Accelerators.getKeyString(KeyEvent.VK_F11));
	combo.addItem (Accelerators.getKeyString(KeyEvent.VK_F12));
	combo.addItem (Accelerators.getKeyString(KeyEvent.VK_F2));
	combo.addItem (Accelerators.getKeyString(KeyEvent.VK_F3));
	combo.addItem (Accelerators.getKeyString(KeyEvent.VK_F4));
	combo.addItem (Accelerators.getKeyString(KeyEvent.VK_F5));
	combo.addItem (Accelerators.getKeyString(KeyEvent.VK_F6));
	combo.addItem (Accelerators.getKeyString(KeyEvent.VK_F7));
	combo.addItem (Accelerators.getKeyString(KeyEvent.VK_F8));
	combo.addItem (Accelerators.getKeyString(KeyEvent.VK_F9));
	combo.addItem (Accelerators.getKeyString(KeyEvent.VK_G));
	combo.addItem (Accelerators.getKeyString(KeyEvent.VK_H));
	combo.addItem (Accelerators.getKeyString(KeyEvent.VK_HOME));
	combo.addItem (Accelerators.getKeyString(KeyEvent.VK_I));
	combo.addItem (Accelerators.getKeyString(KeyEvent.VK_INSERT));
	combo.addItem (Accelerators.getKeyString(KeyEvent.VK_J));
	combo.addItem (Accelerators.getKeyString(KeyEvent.VK_K));
	combo.addItem (Accelerators.getKeyString(KeyEvent.VK_L));
	combo.addItem (Accelerators.getKeyString(KeyEvent.VK_M));
	combo.addItem (Accelerators.getKeyString(KeyEvent.VK_N));
	combo.addItem (Accelerators.getKeyString(KeyEvent.VK_O));
	combo.addItem (Accelerators.getKeyString(KeyEvent.VK_P));
	combo.addItem (Accelerators.getKeyString(KeyEvent.VK_PAGE_DOWN));
	combo.addItem (Accelerators.getKeyString(KeyEvent.VK_PAGE_UP));
	combo.addItem (Accelerators.getKeyString(KeyEvent.VK_Q));
	combo.addItem (Accelerators.getKeyString(KeyEvent.VK_R));
	combo.addItem (Accelerators.getKeyString(KeyEvent.VK_S));
	combo.addItem (Accelerators.getKeyString(KeyEvent.VK_T));
	combo.addItem (Accelerators.getKeyString(KeyEvent.VK_U));
	combo.addItem (Accelerators.getKeyString(KeyEvent.VK_V));
	combo.addItem (Accelerators.getKeyString(KeyEvent.VK_W));
	combo.addItem (Accelerators.getKeyString(KeyEvent.VK_X));
	combo.addItem (Accelerators.getKeyString(KeyEvent.VK_Y));
	combo.addItem (Accelerators.getKeyString(KeyEvent.VK_Z));
	combo.setSelectedIndex (0);
	return combo;
    }

    /** creates the GUI for the ADDON tab */
    private void initAddonPanel ()
    {

    }

    /** creates the GUI for the OTHER tab */
    private void initOtherPanel ()
    {
	// basics	
	otherPanel = new JPanel ();
	otherPanel.setBorder (new EmptyBorder (10,10,10,10));
	GridBagLayout layout = new GridBagLayout ();
	GridBagConstraints constraints = new GridBagConstraints ();
	otherPanel.setLayout (layout);
	JLabel label;
	// look&feel selection
	label = new JLabel ((String)cc.rb.getObject ("options_OptionsDialog_otherPanel_lookandfeel")+":");
	label.setDisplayedMnemonic (((Character)cc.rb.getObject ("options_OptionsDialog_otherPanel_lookandfeel_mnemonic")).charValue());
	label.setLabelFor (lookfeelCombo);
	constraints.gridy = 0;
	constraints.gridx = 0;
	layout.setConstraints (label, constraints);
	otherPanel.add (label);
	
	lookfeelCombo = new JComboBox ();
	lookfeelCombo.addItem ("Java Look&Feel");
	lookfeelCombo.addItem ("Motif Look&Feel");
	lookfeelCombo.addItem ("Windows Look&Feel");
	lookfeelCombo.addItem ("Mac Look&Feel");
	constraints.gridx = 1;
	layout.setConstraints (lookfeelCombo, constraints);
	otherPanel.add (lookfeelCombo);


	label = new JLabel ((String)cc.rb.getObject ("options_OptionsDialog_otherPanel_screenx")+":");
	label.setDisplayedMnemonic (((Character)cc.rb.getObject ("options_OptionsDialog_otherPanel_screenx_mnemonic")).charValue());
	constraints.gridy = 1;
	constraints.gridx = 0;
	layout.setConstraints (label, constraints);
	otherPanel.add (label);
	
	constraints.gridx = 1;
	screenx_TF=new JTextField(""+parent.getDrawingArea().screenx,6);
	layout.setConstraints (screenx_TF, constraints);
	otherPanel.add (screenx_TF);


	label = new JLabel ((String)cc.rb.getObject ("options_OptionsDialog_otherPanel_screeny")+":");
	label.setDisplayedMnemonic (((Character)cc.rb.getObject ("options_OptionsDialog_otherPanel_screeny_mnemonic")).charValue());
	constraints.gridy = 2;
	constraints.gridx = 0;
	layout.setConstraints (label, constraints);
	otherPanel.add (label);
	
	constraints.gridx = 1;
	screeny_TF=new JTextField(""+parent.getDrawingArea().screeny,6);
	layout.setConstraints (screeny_TF, constraints);
	otherPanel.add (screeny_TF);
			
	// adding the whole panel to the GUI
	tabs.addTab ((String)cc.rb.getObject ("options_OptionsDialog_otherPanel_tabheader"),null,otherPanel,(String)cc.rb.getObject ("options_OptionsDialog_otherPanel_tabheader_tooltip"));
    }

    /** 
     *  sets the focus to the General tab 
     */
    public void setFocusToGeneral ()
    {
	tabs.setSelectedIndex (tabs.indexOfComponent (generalPanel));
    }

    /** 
     *  sets the focus to the Draw tab 
     */
    public void setFocusToDraw ()
    {
	tabs.setSelectedIndex (tabs.indexOfComponent (drawPanel));
    }

    /** 
     *  sets the focus to the Color tab 
     */
    public void setFocusToColor ()
    {
	tabs.setSelectedIndex (tabs.indexOfComponent (colorPanel));
    }
    /** 
     *  sets the focus to the Accelerator tab 
     */
    public void setFocusToAccelerator ()
    {
	tabs.setSelectedIndex (tabs.indexOfComponent (acceleratorPanel));
    }
    /** 
     *  sets the focus to the Misc tab 
     */
    public void setFocusToMisc ()
    {
	tabs.setSelectedIndex (tabs.indexOfComponent (otherPanel));
    }


   /** 
    *  this is one of the two important methods in this class.
    *  it instantiates a new dialog object and sets up its gui
    *
    *  @param parent - the parent frame for the dialog it is centered on
    */
    public static void initialize(MainFrame parent) 
    {
	dialog = new OptionsDialog (parent);
    }

    /**
     * if initialize() has been called before (i.e. the dialog has been 
     * instanciated), this method just lets the dialog pop up modally on the
     * screen.
     */
    public static void showDialog() 
    {
        if (dialog != null) 
	{
	    dialog.updateGUI ();
	    dialog.lsListener.setParent (dialog);
	    dialog.setLocationRelativeTo (dialog.parent);
            dialog.setVisible(true);
        } 
	else 
	{
            System.err.println ("OptionsDialog requires you to call initialize before calling showDialog.");
        }
    }

    /**
     * shows the dialog with initial focus to the general tab
     */
    public static void showGeneral() 
    {
	dialog.setFocusToGeneral ();
	showDialog ();
    }

    /**
     * shows the dialog with initial focus to the draw tab
     */
    public static void showDraw() 
    {
	dialog.setFocusToDraw ();
	showDialog ();
    }

    /**
     * shows the dialog with initial focus to the Color tab
     */
    public static void showColor() 
    {
	dialog.setFocusToColor ();
	showDialog ();
    }

    /**
     * shows the dialog with initial focus to the accelerator tab
     */
    public static void showAccelerator() 
    {
	dialog.setFocusToAccelerator ();
	showDialog ();
    }

    /**
     * shows the dialog with initial focus to the misc tab
     */
    public static void showMisc() 
    {
	dialog.setFocusToMisc ();
	showDialog ();
    }


    /**
     *  reads all values in the Options class and sets the GUI elements 
     *  accordingly
     */
    public void updateGUI ()
    {
	// general tab
	devhomeTextField.setText (cc.op.get (Options.general_devisorhome));
	generalpathTextField.setText (cc.op.get (Options.general_path));
	trigenTextField.setText (cc.op.get (Options.general_trigen));
	featlibTextField.setText (cc.op.get (Options.general_featlib));
	// draw tab
	drawboundaryCheck.setSelected (Boolean.valueOf(cc.op.get (Options.draw_drawboundary)).booleanValue());
	numberboundaryCheck.setSelected (Boolean.valueOf(cc.op.get (Options.draw_numberboundary)).booleanValue());
	drawnodeCheck.setSelected (Boolean.valueOf(cc.op.get (Options.draw_drawnode)).booleanValue());
	numbernodeCheck.setSelected (Boolean.valueOf(cc.op.get (Options.draw_numbernode)).booleanValue());
	drawedgeCheck.setSelected (Boolean.valueOf(cc.op.get (Options.draw_drawedge)).booleanValue());
	numberedgeCheck.setSelected (Boolean.valueOf(cc.op.get (Options.draw_numberedge)).booleanValue());
	drawelementCheck.setSelected (Boolean.valueOf(cc.op.get (Options.draw_drawelement)).booleanValue());
	numberelementCheck.setSelected (Boolean.valueOf(cc.op.get (Options.draw_numberelement)).booleanValue());
	crosselementCheck.setSelected (Boolean.valueOf(cc.op.get (Options.draw_crosselement)).booleanValue());
	fillelementCheck.setSelected (Boolean.valueOf(cc.op.get (Options.draw_fillelement)).booleanValue());
	drawgridCheck.setSelected (Boolean.valueOf(cc.op.get (Options.draw_drawgrid)).booleanValue());
	drawperimeterCheck.setSelected (Boolean.valueOf(cc.op.get (Options.draw_drawperimeter)).booleanValue());
	drawrectCheck.setSelected (Boolean.valueOf(cc.op.get (Options.draw_drawrect)).booleanValue());
	snapCheck.setSelected (Boolean.valueOf (cc.op.get (Options.snap_enabled)).booleanValue ());
	if (snapCheck.isSelected ())
	{
	    snapXTF.setText (cc.op.get(Options.snap_x));
	    snapYTF.setText (cc.op.get(Options.snap_y));
	}
	toleranceTF.setText (cc.op.get (Options.click_tolerance));
	drawcoordsCheck.setSelected (Boolean.valueOf (cc.op.get (Options.draw_usercoords)).booleanValue ());
	draworimarkerCheck.setSelected (Boolean.valueOf (cc.op.get(Options.draw_orientationmarkers)).booleanValue ());
	epsilonSlider.setValue (Integer.parseInt (cc.op.get (Options.epsilon)));

	// color tab
	colornodeLabel.setBackground (new Color (Integer.parseInt(cc.op.get(Options.color_node))));
	coloredgeLabel.setBackground (new Color (Integer.parseInt(cc.op.get(Options.color_edge))));
	coloredgebaseLabel.setBackground (new Color (Integer.parseInt(cc.op.get(Options.color_edgebase))));
	colorelementLabel.setBackground (new Color (Integer.parseInt(cc.op.get(Options.color_element))));
	numberboundaryLabel.setBackground (new Color (Integer.parseInt(cc.op.get(Options.color_boundarynumber))));
	numbernodeLabel.setBackground (new Color (Integer.parseInt(cc.op.get(Options.color_nodenumber))));
	numberedgeLabel.setBackground (new Color (Integer.parseInt(cc.op.get(Options.color_edgenumber))));
	numberelementLabel.setBackground (new Color (Integer.parseInt(cc.op.get(Options.color_elementnumber))));
	fill0Label.setBackground (new Color (Integer.parseInt(cc.op.get(Options.color_fill0))));
	fill1Label.setBackground (new Color (Integer.parseInt(cc.op.get(Options.color_fill1))));
	fill2Label.setBackground (new Color (Integer.parseInt(cc.op.get(Options.color_fill2))));
	fill3Label.setBackground (new Color (Integer.parseInt(cc.op.get(Options.color_fill3))));
	fill4Label.setBackground (new Color (Integer.parseInt(cc.op.get(Options.color_fill4))));
	fill5Label.setBackground (new Color (Integer.parseInt(cc.op.get(Options.color_fill5))));
	fill6Label.setBackground (new Color (Integer.parseInt(cc.op.get(Options.color_fill6))));
	fill7Label.setBackground (new Color (Integer.parseInt(cc.op.get(Options.color_fill7))));
	fill8Label.setBackground (new Color (Integer.parseInt(cc.op.get(Options.color_fill8))));
	fill9Label.setBackground (new Color (Integer.parseInt(cc.op.get(Options.color_fill9))));
	fill10Label.setBackground (new Color (Integer.parseInt(cc.op.get(Options.color_fill10))));
	fill11Label.setBackground (new Color (Integer.parseInt(cc.op.get(Options.color_fill11))));
	fill12Label.setBackground (new Color (Integer.parseInt(cc.op.get(Options.color_fill12))));
	fill13Label.setBackground (new Color (Integer.parseInt(cc.op.get(Options.color_fill13))));
	fill14Label.setBackground (new Color (Integer.parseInt(cc.op.get(Options.color_fill14))));
	fill15Label.setBackground (new Color (Integer.parseInt(cc.op.get(Options.color_fill15))));
	bound0Label.setBackground (new Color (Integer.parseInt(cc.op.get(Options.color_boundary0))));
	bound1Label.setBackground (new Color (Integer.parseInt(cc.op.get(Options.color_boundary1))));
	bound2Label.setBackground (new Color (Integer.parseInt(cc.op.get(Options.color_boundary2))));
	bound3Label.setBackground (new Color (Integer.parseInt(cc.op.get(Options.color_boundary3))));
	bound4Label.setBackground (new Color (Integer.parseInt(cc.op.get(Options.color_boundary4))));
	bound5Label.setBackground (new Color (Integer.parseInt(cc.op.get(Options.color_boundary5))));
	bound6Label.setBackground (new Color (Integer.parseInt(cc.op.get(Options.color_boundary6))));
	bound7Label.setBackground (new Color (Integer.parseInt(cc.op.get(Options.color_boundary7))));
	bound8Label.setBackground (new Color (Integer.parseInt(cc.op.get(Options.color_boundary8))));
	bound9Label.setBackground (new Color (Integer.parseInt(cc.op.get(Options.color_boundary9))));
	bound10Label.setBackground (new Color (Integer.parseInt(cc.op.get(Options.color_boundary10))));
	bound11Label.setBackground (new Color (Integer.parseInt(cc.op.get(Options.color_boundary11))));
	bound12Label.setBackground (new Color (Integer.parseInt(cc.op.get(Options.color_boundary12))));
	bound13Label.setBackground (new Color (Integer.parseInt(cc.op.get(Options.color_boundary13))));
	bound14Label.setBackground (new Color (Integer.parseInt(cc.op.get(Options.color_boundary14))));
	bound15Label.setBackground (new Color (Integer.parseInt(cc.op.get(Options.color_boundary15))));
	highlightselectionLabel.setBackground (new Color (Integer.parseInt(cc.op.get(Options.color_selection))));
	highlightrectLabel.setBackground (new Color (Integer.parseInt(cc.op.get(Options.color_rect))));
	miscgridLabel.setBackground (new Color (Integer.parseInt(cc.op.get(Options.color_grid))));
	miscbackgroundLabel.setBackground (new Color (Integer.parseInt(cc.op.get(Options.color_background))));
	misclassoLabel.setBackground (new Color (Integer.parseInt(cc.op.get(Options.color_lasso))));


	// accelerators tab
	tablemodel = new AcceleratorTableModel ();
	acceleratorTable.setModel (tablemodel);

	// other tab
	// look and feel
	if (cc.op.get(Options.other_lookandfeel).equals(UIManager.getCrossPlatformLookAndFeelClassName ()))
	    {
		lookfeelCombo.setSelectedItem("Java Look&Feel");
	    }
	if (cc.op.get(Options.other_lookandfeel).equals("com.sun.java.swing.plaf.windows.WindowsLookAndFeel"))
	    {
		lookfeelCombo.setSelectedItem("Windows Look&Feel");
	    }
	if (cc.op.get(Options.other_lookandfeel).equals("com.sun.java.swing.plaf.motif.MotifLookAndFeel"))
	    {
		lookfeelCombo.setSelectedItem("Motif Look&Feel");
	    }
	if (cc.op.get(Options.other_lookandfeel).equals("javax.swing.plaf.mac.MacLookAndFeel"))
	    {
		lookfeelCombo.setSelectedItem("Mac Look&Feel");
	    }
    }

    /**
     *  reads all values from the GUI elements and sets the Options 
     *  class accordingly
     */
    public void updateOptions ()
    {
	// general panel

	cc.op.set (Options.general_devisorhome, devhomeTextField.getText());
	cc.op.set (Options.general_path, generalpathTextField.getText());
	cc.op.set (Options.general_trigen, trigenTextField.getText());
	cc.op.set (Options.general_featlib, featlibTextField.getText());
	cc.op.set (Options.general_helpfile, "file://"+cc.op.get(Options.general_devisorhome)+System.getProperty("file.separator")+"manual"+System.getProperty("file.separator")+"html"+System.getProperty("file.separator")+"DeViSoRGrid2_Manual.html");
	cc.op.set (Options.general_faqfile, "file://"+cc.op.get(Options.general_devisorhome)+System.getProperty("file.separator")+"manual"+System.getProperty("file.separator")+"html"+System.getProperty("file.separator")+"FAQ.html");
	// draw options
	if (drawboundaryCheck.isSelected())
	    cc.op.set (Options.draw_drawboundary, "true");
	else
	    cc.op.set (Options.draw_drawboundary, "false");
	if (numberboundaryCheck.isSelected())
	    cc.op.set (Options.draw_numberboundary, "true");
	else
	    cc.op.set (Options.draw_numberboundary, "false");
	if (drawnodeCheck.isSelected())
	    cc.op.set (Options.draw_drawnode, "true");
	else
	    cc.op.set (Options.draw_drawnode, "false");
	if (numbernodeCheck.isSelected())
	    cc.op.set (Options.draw_numbernode, "true");
	else
	    cc.op.set (Options.draw_numbernode, "false");
	if (drawedgeCheck.isSelected())
	    cc.op.set (Options.draw_drawedge, "true");
	else
	    cc.op.set (Options.draw_drawedge, "false");
	if (numberedgeCheck.isSelected())
	    cc.op.set (Options.draw_numberedge, "true");
	else
	    cc.op.set (Options.draw_numberedge, "false");
	if (drawelementCheck.isSelected())
	    cc.op.set (Options.draw_drawelement, "true");
	else
	    cc.op.set (Options.draw_drawelement, "false");
	if (numberelementCheck.isSelected())
	    cc.op.set (Options.draw_numberelement, "true");
	else
	    cc.op.set (Options.draw_numberelement, "false");
	if (crosselementCheck.isSelected())
	    cc.op.set (Options.draw_crosselement, "true");
	else
	    cc.op.set (Options.draw_crosselement, "false");
	if (fillelementCheck.isSelected())
	    cc.op.set (Options.draw_fillelement, "true");
	else
	    cc.op.set (Options.draw_fillelement, "false");
	if (drawgridCheck.isSelected())
	    cc.op.set (Options.draw_drawgrid, "true");
	else
	    cc.op.set (Options.draw_drawgrid, "false");
	if (drawperimeterCheck.isSelected())
	    cc.op.set (Options.draw_drawperimeter, "true");
	else
	    cc.op.set (Options.draw_drawperimeter, "false");
	if (drawrectCheck.isSelected())
	    cc.op.set (Options.draw_drawrect, "true");
	else
	    cc.op.set (Options.draw_drawrect, "false");
	if (snapCheck.isSelected())
	{
	    cc.op.set (Options.snap_enabled, "true");
	    cc.op.set (Options.snap_x, snapXTF.getText());
	    cc.op.set (Options.snap_y, snapYTF.getText());
	}
	else
	{
	    cc.op.set (Options.snap_enabled, "false");
	}
	cc.op.set (Options.click_tolerance, toleranceTF.getText());
	
        cc.op.set(Options.screenx, screenx_TF.getText());
	cc.op.set(Options.screeny, screeny_TF.getText());

	if (drawcoordsCheck.isSelected())
	    cc.op.set (Options.draw_usercoords, "true");
	else
	    cc.op.set (Options.draw_usercoords, "false");
	if (draworimarkerCheck.isSelected())
	    cc.op.set (Options.draw_orientationmarkers, "true");
	else
	    cc.op.set (Options.draw_orientationmarkers, "false");

	cc.op.set (Options.epsilon, Integer.toString(epsilonSlider.getValue ()));
	// color panel
	cc.op.set (Options.color_node, Integer.toString(colornodeLabel.getBackground().getRGB()));
	cc.op.set (Options.color_edge, Integer.toString(coloredgeLabel.getBackground().getRGB()));
	cc.op.set (Options.color_edgebase, Integer.toString(coloredgebaseLabel.getBackground().getRGB()));
	cc.op.set (Options.color_element, Integer.toString(colorelementLabel.getBackground().getRGB()));
	cc.op.set (Options.color_boundarynumber, Integer.toString(numberboundaryLabel.getBackground().getRGB()));
	cc.op.set (Options.color_nodenumber, Integer.toString(numbernodeLabel.getBackground().getRGB()));
	cc.op.set (Options.color_edgenumber, Integer.toString(numberedgeLabel.getBackground().getRGB()));
	cc.op.set (Options.color_elementnumber, Integer.toString(numberelementLabel.getBackground().getRGB()));
	cc.op.set (Options.color_fill0, Integer.toString(fill0Label.getBackground().getRGB()));
	cc.op.set (Options.color_fill0, Integer.toString(fill0Label.getBackground().getRGB()));
	cc.op.set (Options.color_fill1, Integer.toString(fill1Label.getBackground().getRGB()));
	cc.op.set (Options.color_fill2, Integer.toString(fill2Label.getBackground().getRGB()));
	cc.op.set (Options.color_fill3, Integer.toString(fill3Label.getBackground().getRGB()));
	cc.op.set (Options.color_fill4, Integer.toString(fill4Label.getBackground().getRGB()));
	cc.op.set (Options.color_fill5, Integer.toString(fill5Label.getBackground().getRGB()));
	cc.op.set (Options.color_fill6, Integer.toString(fill6Label.getBackground().getRGB()));
	cc.op.set (Options.color_fill7, Integer.toString(fill7Label.getBackground().getRGB()));
	cc.op.set (Options.color_fill8, Integer.toString(fill8Label.getBackground().getRGB()));
	cc.op.set (Options.color_fill9, Integer.toString(fill9Label.getBackground().getRGB()));
	cc.op.set (Options.color_fill10, Integer.toString(fill10Label.getBackground().getRGB()));
	cc.op.set (Options.color_fill11, Integer.toString(fill11Label.getBackground().getRGB()));
	cc.op.set (Options.color_fill12, Integer.toString(fill12Label.getBackground().getRGB()));
	cc.op.set (Options.color_fill13, Integer.toString(fill13Label.getBackground().getRGB()));
	cc.op.set (Options.color_fill14, Integer.toString(fill14Label.getBackground().getRGB()));
	cc.op.set (Options.color_fill15, Integer.toString(fill15Label.getBackground().getRGB()));
	cc.op.set (Options.color_selection, Integer.toString(highlightselectionLabel.getBackground().getRGB()));
	cc.op.set (Options.color_rect, Integer.toString(highlightrectLabel.getBackground().getRGB()));
	cc.op.set (Options.color_grid, Integer.toString(miscgridLabel.getBackground().getRGB()));
	cc.op.set (Options.color_background, Integer.toString(miscbackgroundLabel.getBackground().getRGB()));
	cc.op.set (Options.color_lasso, Integer.toString(misclassoLabel.getBackground().getRGB()));
	cc.op.set (Options.color_boundary0, Integer.toString(bound0Label.getBackground().getRGB()));
	cc.op.set (Options.color_boundary1, Integer.toString(bound1Label.getBackground().getRGB()));
	cc.op.set (Options.color_boundary2, Integer.toString(bound2Label.getBackground().getRGB()));
	cc.op.set (Options.color_boundary3, Integer.toString(bound3Label.getBackground().getRGB()));
	cc.op.set (Options.color_boundary4, Integer.toString(bound4Label.getBackground().getRGB()));
	cc.op.set (Options.color_boundary5, Integer.toString(bound5Label.getBackground().getRGB()));
	cc.op.set (Options.color_boundary6, Integer.toString(bound6Label.getBackground().getRGB()));
	cc.op.set (Options.color_boundary7, Integer.toString(bound7Label.getBackground().getRGB()));
	cc.op.set (Options.color_boundary8, Integer.toString(bound8Label.getBackground().getRGB()));
	cc.op.set (Options.color_boundary9, Integer.toString(bound9Label.getBackground().getRGB()));
	cc.op.set (Options.color_boundary10, Integer.toString(bound10Label.getBackground().getRGB()));
	cc.op.set (Options.color_boundary11, Integer.toString(bound11Label.getBackground().getRGB()));
	cc.op.set (Options.color_boundary12, Integer.toString(bound12Label.getBackground().getRGB()));
	cc.op.set (Options.color_boundary13, Integer.toString(bound13Label.getBackground().getRGB()));
	cc.op.set (Options.color_boundary14, Integer.toString(bound14Label.getBackground().getRGB()));
	cc.op.set (Options.color_boundary15, Integer.toString(bound15Label.getBackground().getRGB()));

	
	// accelerators panel

	// no need to do anything here, the table writes itself back to the
	// options instance

	//other panel
	String lookandfeel = (String)lookfeelCombo.getSelectedItem ();
	if (lookandfeel.equals("Java Look&Feel"))
	    {
		cc.op.set(Options.other_lookandfeel,UIManager.getCrossPlatformLookAndFeelClassName());
	    }
	if (lookandfeel.equals("Windows Look&Feel"))
	    {
		cc.op.set (Options.other_lookandfeel, "com.sun.java.swing.plaf.windows.WindowsLookAndFeel");
	    }
	if (lookandfeel.equals("Motif Look&Feel"))
	    {
		cc.op.set (Options.other_lookandfeel, "com.sun.java.swing.plaf.motif.MotifLookAndFeel");
	    }
	if (lookandfeel.equals("Mac Look&Feel"))
	    {
		cc.op.set (Options.other_lookandfeel, "javax.swing.plaf.mac.MacLookAndFeel");
	    }
	   

      

    }



    /** components of the dialog */
    private JTabbedPane tabs;
    // general panel
    private JPanel generalPanel;
    private JTextField devhomeTextField;
    private JButton devhomeButton;
    private JTextField generalpathTextField;
    private JButton generalpathButton;
    private JTextField trigenTextField;
    private JButton trigenButton;
    private JTextField featlibTextField;
    private JButton featlibButton;

    // draw panel
    private JPanel drawPanel;
    private JCheckBox drawboundaryCheck;
    private JCheckBox numberboundaryCheck;
    private JCheckBox drawnodeCheck;
    private JCheckBox numbernodeCheck;
    private JCheckBox drawedgeCheck;
    private JCheckBox numberedgeCheck;
    private JCheckBox drawelementCheck;
    private JCheckBox numberelementCheck;
    private JCheckBox crosselementCheck;
    private JCheckBox fillelementCheck;
    private JCheckBox drawgridCheck;
    private JCheckBox drawperimeterCheck;
    private JCheckBox drawrectCheck;
    private JCheckBox snapCheck;
    private JTextField snapXTF, snapYTF;
    private JTextField toleranceTF;
    private JCheckBox drawcoordsCheck;
private JCheckBox draworimarkerCheck;
    private JSlider epsilonSlider;

    // color panel
    private JPanel colorPanel;
    private JLabel colornodeLabel;
    private JLabel colorelementLabel;
    private JLabel coloredgeLabel;
    private JLabel coloredgebaseLabel;
    private JLabel numberboundaryLabel;
    private JLabel numbernodeLabel;
    private JLabel numberelementLabel;
    private JLabel numberedgeLabel;
    private JLabel highlightselectionLabel;
    private JLabel highlightrectLabel;
    private JLabel miscgridLabel;
    private JLabel miscbackgroundLabel;
    private JLabel misclassoLabel;
    private JLabel fill0Label;
    private JLabel fill1Label;
    private JLabel fill2Label;
    private JLabel fill3Label;
    private JLabel fill4Label;
    private JLabel fill5Label;
    private JLabel fill6Label;
    private JLabel fill7Label;
    private JLabel fill8Label;
    private JLabel fill9Label;
    private JLabel fill10Label;
    private JLabel fill11Label;
    private JLabel fill12Label;
    private JLabel fill13Label;
    private JLabel fill14Label;
    private JLabel fill15Label;
    private JLabel bound0Label;
    private JLabel bound1Label;
    private JLabel bound2Label;
    private JLabel bound3Label;
    private JLabel bound4Label;
    private JLabel bound5Label;
    private JLabel bound6Label;
    private JLabel bound7Label;
    private JLabel bound8Label;
    private JLabel bound9Label;
    private JLabel bound10Label;
    private JLabel bound11Label;
    private JLabel bound12Label;
    private JLabel bound13Label;
    private JLabel bound14Label;
    private JLabel bound15Label;    

    // accelerators panel
    private JPanel acceleratorPanel;
    private JTable acceleratorTable;
    private AcceleratorTableModel tablemodel;
    private AcceleratorTableListSelectionListener lsListener;
    private JComboBox keySelectionComboBox;
    private JRadioButton CTRLbutton;
    private JRadioButton SHIFTbutton;
    private JRadioButton ALTbutton;
    private JRadioButton NONEbutton;
    private JButton applyAcceleratorButton;
    private JButton removeAcceleratorButton;
    // addon panel
    private JPanel addonPanel;
    // other panel
    private JPanel otherPanel;
    private JComboBox lookfeelCombo;
    private JTextField screenx_TF;
    private JTextField screeny_TF;

    /**
     *  quick getters and setters for the ones used from outside
     */

    public JTextField getDevHomeTextField ()
    {
	return devhomeTextField;
    }
    public JTextField getPathTextField ()
    {
	return generalpathTextField;
    }
    public JTextField getTrigenTextField ()
    {
	return trigenTextField;
    }
    public JTextField getFeatlibTextField ()
    {
	return featlibTextField;
    }
    public JComboBox getKeySelectionComboBox ()
    {
	return this.keySelectionComboBox;
    }

    public JRadioButton getCTRLbutton()
    {
	return CTRLbutton;
    }

    public JRadioButton getALTbutton()
    {
	return ALTbutton;
    }
    public JRadioButton getSHIFTbutton()
    {
	return SHIFTbutton;
    }
    public JRadioButton getNONEbutton()
    {
	return NONEbutton;
    }
    public JCheckBox getSnapCheck ()
    {
	return snapCheck;
    }
    public JTextField getSnapXTF ()
    {
	return snapXTF;
    }
    public JTextField getSnapYTF ()
    {
	return snapYTF;
    }
    public JSlider getEpsilonSlider ()
    {
	return epsilonSlider;
    }
    public Vector getAcceleratorTableData ()
    {
	return tablemodel.getData();
    }
    public JTable getAcceleratorTable ()
    {
	return acceleratorTable;
    }
    public JLabel getColornodeLabel ()
    {
	return colornodeLabel;
    }
    public JLabel getColoredgeLabel ()
    {
	return coloredgeLabel;
    }
    public JLabel getColoredgebaseLabel ()
    {
	return coloredgebaseLabel;
    }

    public JLabel getColorelementLabel ()
    {
	return colorelementLabel;
    }
    public JLabel getNumberboundaryLabel ()
    {
	return numberboundaryLabel;
    }
    public JLabel getNumbernodeLabel ()
    {
	return numbernodeLabel;
    }
    public JLabel getNumberedgeLabel ()
    {
	return numberedgeLabel;
    }
    public JLabel getNumberelementLabel ()
    {
	return numberelementLabel;
    }
    public JLabel getHighlightselectionLabel ()
    {
	return highlightselectionLabel;
    }
    public JLabel getHighlightrectLabel ()
    {
	return highlightrectLabel;
    }
    public JLabel getMiscgridLabel ()
    {
	return miscgridLabel;
    }
    public JLabel getMiscbackgroundLabel ()
    {
	return miscbackgroundLabel;
    }
    public JLabel getMisclassoLabel ()
    {
	return misclassoLabel;
    }
    public JLabel getFill0Label ()
    {
	return fill0Label;
    }
    public JLabel getFill1Label ()
    {
	return fill1Label;
    }
    public JLabel getFill2Label ()
    {
	return fill2Label;
    }
    public JLabel getFill3Label ()
    {
	return fill3Label;
    }
    public JLabel getFill4Label ()
    {
	return fill4Label;
    }
    public JLabel getFill5Label ()
    {
	return fill5Label;
    }
    public JLabel getFill6Label ()
    {
	return fill6Label;
    }
    public JLabel getFill7Label ()
    {
	return fill7Label;
    }
    public JLabel getFill8Label ()
    {
	return fill8Label;
    }
    public JLabel getFill9Label ()
    {
	return fill9Label;
    }
    public JLabel getFill10Label ()
    {
	return fill10Label;
    }
    public JLabel getFill11Label ()
    {
	return fill11Label;
    }
    public JLabel getFill12Label ()
    {
	return fill12Label;
    }
    public JLabel getFill13Label ()
    {
	return fill13Label;
    }
    public JLabel getFill14Label ()
    {
	return fill14Label;
    }
    public JLabel getFill15Label ()
    {
	return fill15Label;
    }
    public JLabel getBound0Label ()
    {
	return bound0Label;
    }
    public JLabel getBound1Label ()
    {
	return bound1Label;
    }
    public JLabel getBound2Label ()
    {
	return bound2Label;
    }
    public JLabel getBound3Label ()
    {
	return bound3Label;
    }
    public JLabel getBound4Label ()
    {
	return bound4Label;
    }
    public JLabel getBound5Label ()
    {
	return bound5Label;
    }
    public JLabel getBound6Label ()
    {
	return bound6Label;
    }
    public JLabel getBound7Label ()
    {
	return bound7Label;
    }
    public JLabel getBound8Label ()
    {
	return bound8Label;
    }
    public JLabel getBound9Label ()
    {
	return bound9Label;
    }
    public JLabel getBound10Label ()
    {
	return bound10Label;
    }
    public JLabel getBound11Label ()
    {
	return bound11Label;
    }
    public JLabel getBound12Label ()
    {
	return bound12Label;
    }
    public JLabel getBound13Label ()
    {
	return bound13Label;
    }
    public JLabel getBound14Label ()
    {
	return bound14Label;
    }
    public JLabel getBound15Label ()
    {
	return bound15Label;
    }





}


