package devisor2.grid.GUI.event;

import java.awt.event.*;
import java.awt.*;
import java.awt.geom.*;
import java.util.*;
import javax.swing.*;

import devisor2.grid.GUI.framework.*;
import devisor2.grid.options.*;
import devisor2.grid.backend.*;
import devisor2.grid.GUI.dialogs.*;
import devisor2.foundation.*;
import devisor2.foundation.boundary.*;
import devisor2.foundation.elements.*;

/**
 *  This is the king-size mouse-event listener for the DrawingArea. <br>
 *  To help any programmer find his way in this class, we give a short
 *  guided tour, meaning we describe the input mechanism for each element
 *  and all other features and list the events where the corresponding code 
 *  can be found. But before that and to keep the tension up, a little remark:
 *  We decided to define a flag <code>inProgress<\code> which stores the 
 *  current operation. As most actions aren't performed in one event and with 
 *  one click only, we also added a <code>clickCounter<\code> variable to 
 *  store information about intermediate progress. <br>
 *  <br>
 *  INDIRECT INPUT <br>
 *  During indirect input of circles and lines, the length of the line 
 *  (radius) and the angles are displayed at the mouse cursor.
 *  The String variable <code>caption<\code> is used for this. <br>
 *  <br>
 *  INDIRECT NODE INPUT <br>
 *  One click on the drawing area starts the input sequence (corresponding 
 *  code to be found in the <code>mouseClicked<\code> event, code block <code>
 *  DRAW_MODE INDIRECT_MODE, CLICKCOUNTER 0<\code>) Storage variables for 
 *  this are <code>x1,y1<\code>. As visual feedback, a tiny node is added to 
 *  the cursor. Moving the mouse around moves the attached node around 
 *  (<code>mouseMoved<\code> event, code block <code>PROGRESS_ADD_NODE<\code>).
 *  A second click adds the node with the current coordinates to the domain and
 *  sets the listener back to default status (<code>mouseClicked<\code> event,
 *  code block <code>DRAW_MODE INDIRECT_MODE, CLICKCOUNTER 1<\code>). We also
 *  distinguish the cases where a regular node has to be added or a
 *  BoundaryNode.<br>
 *  <br>
 *  INDIRECT LINE INPUT <br>
 *  One click on the drawing area sets the start point for the line (<code>
 *  mouseClicked<\code> event, code block <code>DRAW_MODE INDIRECT_MODE, 
 *  CLICKCOUNTER 0<\code>). Storage variables for this are <code>x1,y1<\code>.
 *  Moving the mouse around draws a line from the start point to the current 
 *  mouse position (<code>mouseMoved<\code> event, code block <code>
 *  PROGRESS_ADD_LINE<\code>). The end point is stored in variables 
 *  <code>x2,y2<\code>.A second click adds the line with end point coordinates
 *  of the current mouse position to the domain and resets the listeners 
 *  status  (<code>mouseClicked<\code> event, code block <code>DRAW_MODE 
 *  INDIRECT_MODE, CLICKCOUNTER 1<\code>).<br>
 *  <br>
 *  INDIRECT MULTILINE INPUT <br>
 *  This is performed just like regular line input, except that after every 
 *  click (without the first click of course) a new line segment is added and
 *  the start point for the next line is set as the end point of the previous.
 *  This is done in a sort of endless loop, and can be stopped by a 
 *  right-click (see below, section "CANCELLING INDIRECT INPUT"). So, the 
 *  corresponding code can be found just where the line code is. <br>
 *  <br>
 *  INDIRECT CIRCLE INPUT <br>
 *  Here, we only describe the CIRCLEP input, the CIRCLEM input is implemented
 *  just the same: One click on the drawing area sets the middle point for the
 *  circle. Storage variables for this are <code>x1,y1<\code>, 
 *  (<code>mouseClicked<\code> event, code block <code>DRAW_MODE INDIRECT_MODE,
 *  CLICKCOUNTER 0<\code>). Moving the mouse around specifies the radius, as 
 *  visual feedback, a circle with radius |middle point - current mouse 
 *  position| is drawn. The corresponding code can be found in <code>mouseMoved
 *  <\code> event, code block <code>PROGRESS_ADD_CIRCLEP, CLICKCOUNTER 1<\code>
 *  , as temporary storage variables <code>x2,y2,r<\code> are used. A second 
 *  click fixes the radius (<code>mouseClicked<\code> event, code block <code>
 *  DRAW_MODE INDIRECT_MODE, CLICKCOUNTER 1<\code>) and the start angle setting
 *  is started. Following mouse moves draw a line from the middle point to the
 *  current mouse position (<code>mouseMoved<\code> event, code block <code>
 *  PROGRESS_ADD_CIRCLEP, CLICKCOUNTER 2<\code>, using the additional variable
 *  <code>a1<\code>). The third click fixes the start angle (<code>mouseClicked
 *  <\code> event, code block <code>DRAW_MODE INDIRECT_MODE, CLICKCOUNTER 2
 *  <\code>) and the end angle setting is started (using the additional 
 *  variables <code>a2,arc<\code>. Now, mouse moves result in painting a line 
 *  from the middle point to the current mouse position, and as visual 
 *  feedback, an arc is drawn from the start angle to the end angle (<code>
 *  mouseMoved<\code> event, code block <code>PROGRESS_ADD_CIRCLEP, 
 *  CLICKCOUNTER 3<\code>). Finally, the 4th click fixes the end angle, 
 *  a circle segment is added to the domain, and the listener is reset 
 *  (<code>mouseClicked<\code> event, code block <code>DRAW_MODE INDIRECT_MODE,
 *  CLICKCOUNTER 3<\code>). <br>
 *  <br>
 *  INDIRECT TRI INPUT <br>
 *  Any click that does not hit a node results in cancelling the whole adding 
 *  operation! The first click sets the first node (<code>mouseClicked<\code>
 *  event, <code>DRAW_MODE INDIRECT_MODE, CLICKCOUNTER 0<\code>, stored in 
 *  variable <code>n1<\code>, using the variables <code>x1,y1,x2,y2<\code> as 
 *  temporary storage), mouse moves result in a line drawing from the first 
 *  node to the current mouse position (<code>mouseMoved<\code> event, code 
 *  block <code>PROGRESS_ADD_TRI, CLICKCOUNTER 1<\code>). A second click sets 
 *  the second node (<code>DRAW_MODE INDIRECT_MODE, CLICKCOUNTER 1<\code>, 
 *  stored in variable <code>n2<\code>). Upcoming mouse moves draw again a 
 *  line (<code>mouseMoved<\code> event, code block <code>PROGRESS_ADD_TRI, CLICKCOUNTER 2<\code>), and the third click sets the third node and finally adds 
 *  the tri element, reseting the listener  (<code>DRAW_MODE INDIRECT_MODE,
 *  CLICKCOUNTER 2<\code>, stored in variable <code>n3<\code>).<br>
 *  <br>
 *  INDIRECT QUAD INPUT <br>
 *  This is implemented just like TRI input, except that a fourth click is 
 *  neccessary. Naturally, a 4th variable  <code>n4<\code> is used. <br>
 *  <br>
 *  INDIRECT MACRO INPUT <br>
 *  This is implemented just like QUAD input. <br>
 *  <br>
 *  CANCELLING INDIRECT INPUT <br>
 *  At any time during indirect input, a single right-click cancels the 
 *  current operation, no element or segment is added, and a redraw is forced.
 *  The listener is reset. <br>
 *  <br>
 *  <br>
 *  <br>
 *  DIRECT INPUT <br>
 *  In direct input mode, a single click displays the corresponding dialog, 
 *  additional clicks update the dialog with the click point. This can be 
 *  found in the <code>mouseclicked<\code> event, all in code block 
 *  <code>DRAW MODE DIRECT MODE<\code>.<br>
 *  <br>
 *  <br>
 *  <br>
 *  PROPERTY QUERIES <br>
 *  Property queries are implemented just like direct input, to be found in
 *  the <code>mouseclicked<\code> event, all in code block 
 *  <code>DRAW MODE PROPERTY MODE<\code>.<br>
 *  <br>
 *  <br>
 *  <br>
 *  SELECTING <br>
 *  Selecting is the default operation. There are two ways of selecting 
 *  items on the drawing area, either clicking on a single item or throwing
 *  a lasso around several elements. Only items completely inside the 
 *  rectangular area of the rubberband are selected, the current select
 *  flags and the draw options serve as a filter. All selecting is handled
 *  in the <code>mouseClicked<\code> and <code>mouseMoved<\code> methods,
 *  the actual selection management is performed in the 
 *  <code>GridSelector<\code> class. Note that the CTRL key ist used for
 *  discontinous or additive selecting, in other words, <br>
 *  <br>
 *  <br>
 *  MOVING, SCALING, ROTATING and MIRRORING<br>
 *  Every time one or several items become selected, a box is drawn around 
 *  them with nine hotspots. Depending on the mode and desired operation, 
 *  various constellations for hotspots are possible. For example, nodes are
 *  not rotatable, and circles are only uniformly scalable. Generally, a 
 *  click on the center hotspot starts the moving operation which is 
 *  performed via the drag&drop technique, to be found in the corresponding
 *  drag events. Temporarily variables used are x1 and y1 for the initial 
 *  center of the select box. <br>
 *  The other hotspots located at the corners of the box are used for 
 *  resizing, which is performed via the drag and drop technique.  The 
 *  clicked hotspot coords are saved in the x1 and y1 variables, the coords 
 *  of the opposite hotspot in x2 and y2 respectively, the first ones are 
 *  used to determine the scale factor, the other ones for the follow-up 
 *  translation. <br>
 *  When SHIFT is held down, the corner hotspots disappear, as rotating can 
 *  only be initiated with the four center hotspots. The actual rotate 
 *  operation is not surprisingly again performed via drag and drop. <br>
 *  When circles are part of the selection of only one single node is 
 *  is selected, only some hotspots are visible. The GridHotspotter class
 *  in the backend package is responsible for this. <br>
 *  For mirroring, direct mode just brings up a dialog where the user can enter
 *  the line against which to mirror. Otherwise, the user can draw this line
 *  by mouse. The corresponding code fragments can be found just where LINE
 *  input takes place, so see there for details. <br>
 *  As always, a right-click cancels the current operation.
 *  <br>
 *  <br>
 *  ZOOMING <br>
 *  For setting a new user-defined zoom level, the GridSelector is misused,
 *  in other words, lasso-throwing is implemented just as described before
 *  in the selection section.<br>
 *  For zooming in by a constant factor, a single click suffices, thus,
 *  the corresponding code can be found in the <code>mouseClicked<\code>
 *  event handler, section <code>zoom<\code>.
 *  <br>
 *  <br>
 *  DIALOG-BASED MODES <br>
 *  This is actually very straight-forward, all corresponding code can be 
 *  found in the <code>mouseClicked<\code> event handler. Usually, only the
 *  dialog is updated with the values extracted from the currently selected
 *  item on the drawing area. <br>
 *  <br>
 *  <br>
 *  USER COORD FEEDBACK <br>
 *  This is done in the <code>mouseMoved<\code> event if the corresponding
 *  option has been set. <br>
 *
 *
 *  @author Dominik Goeddeke
 *  @version everything but zooming
 */
public class GridListener implements MouseMotionListener, MouseListener
{
    // the MainFrame reference
    private MainFrame root;
    
    // the ControlCenter reference
    protected ControlCenter cc = ControlCenter.getMyself();

    // reference to the mainframe's drawing area
    private DrawingArea da;

    // reference to the GridSelector cooperating with this listener
    private GridSelector gs;

    // reference to the Graphics context of the DrawingArea
    private Graphics g;

    // indicates what's currently in progress (use constants provided)
    private int inProgress;

    // storage for indirect adding (stores clickpoints)
    private int x1,y1,x2,y2,r;
    private int cx,cy;
    private double a1,a2, arc;
    private Node n1,n2,n3,n4;
    // storage for the caption
    private String caption;
    // flag to store if mouse is over a segment (for BoundaryNodeInput)
    private boolean isOverSegment;
    // and the Segment itself
    private Segment overSegment;
    // storage for the hotspot last clicked on
    private int hotspot = gs.HOTSPOT_NONE;
    // and for the last hotspotmask that was in use
    private boolean[] saveHotspotMask;

    // counter for the clicks since last add
    private int clickCounter = 0;

    // a stupid flag needed for managing simultaneous keyboard and mouse input
    private boolean shiftStarted = false;

    // stores the topleft of the select box at the time of cloning
    // (set by the storeCopyPasteAnchor() method)
    private Point copyPasteAnchor;

    // stores the hotspotMask at the time of cloning 
    // (set by the storeCopyPasteAnchor() method)
    private boolean[] copyPasteMask;

    /**
     *  the vector of GridItems which are currently selected
     *  updated every time the select box drawing process is finished
     */
    private Vector selectedItems = new Vector ();


    int rectstatus;
    int rectindex;
    Node[] rectsave;
    /**************************************************************/

    /**
     *  The constructor just sets references correctly
     *  and initializes some variables
     *
     *  @param mf - the MainFrame reference
     *  @param da - the drawing area this listener listens to
     */    
    public GridListener(MainFrame mf, DrawingArea da) 
    {
	root = mf;
	this.da = da;
	gs = new GridSelector (mf, da);
	inProgress = PROGRESS_NOTHING;
    }



    /*********************primnitive getters and setters************/

    /**
     *  @return true if rotate has been initiated via the SHIFT button
     */
    public boolean getShiftStarted ()
    {
	return shiftStarted;
    }


    /**
     *  sets the SHIFT started flag (see the keyboard events in MainFrame
     *  for a detailed description)
     */
    public void setShiftStarted (boolean c)
    {
	shiftStarted = c;
    }

    /**
     *  @return the current hotspot (uses constants from GridSelector clasS)
     */
    public int getHotspot ()
    {
	return hotspot;
    }

    /**
     *  @return the current Vector of selected Items (all of class GridItems)
     */
    public Vector getSelectedItems ()
    {
	return selectedItems;
    }

    /**
     *  sets the selected items
     */
    public void setSelectedItems (Vector newselection)
    {
	selectedItems = newselection;
    }

    /**
     *  clears all selected items and updates the GUI respectively
     */
    public void clearSelection ()
    {
	Enumeration enum = selectedItems.elements ();
	while (enum.hasMoreElements())
	{
	    ((GridItem)enum.nextElement()).unsetMarker();
	}
	selectedItems = new Vector ();
	da.repaint ();
    }


    /**
     *  returns the current GridSelector instance connected to this listener
     */
    public GridSelector getSelector ()
    {
	return gs;
    }

    /**
     *  saves the current saveHotspotMask used for copy&paste
     */
    public void setSaveHotspotMask (boolean[] m)
    {
	saveHotspotMask = m;
    }

    /**
     *  @return the current copy&paste saveHotspotMask
     */
    public boolean[] getSaveHotspotMask ()
    {
	return saveHotspotMask;
    }

    /**
     *  @return the current inProgress status (see constants below)
     */
    public int getInProgress ()
    {
	return inProgress;
    }

    /**
     *  stores the current topleft point of the select box for further
     *  affine transform when the paste in copyandpaste is called
     */
    public void storeCopyPasteAnchor ()
    {
	copyPasteAnchor = gs.getTopLeftHotspot ();
	copyPasteMask = gs.getHotspotMask ();
    }


    /**
     *  event handler for the mousePressed event. See above for a detailed
     *  description of which operation belongs to which event handler.
     */
    public void mousePressed(MouseEvent e) 
    {
	// apply snap if neccessary
	int x = calcSnapX(e);
	int y = calcSnapY(e);

	// zooming
	if (cc.mode == ControlCenter.ZOOM_MODE && cc.zoomType == ControlCenter.ZOOM_SET)
	{
	    inProgress = PROGRESS_SETZOOM;
	    // the select box is used for zooming as well :-)
	    gs.startSelectBox (x,y);
	    return;
	}
	hotspot = gs.getClickedHotspot (e.getX(), e.getY());
	// translate / rotate
	if (hotspot != gs.HOTSPOT_NONE)
	{
	    // depending on the number of the hotspot, start scale & rotate
	    if (hotspot == gs.HOTSPOT_CENTERCENTER)
	    {
		if (gs.isBoundaryNodesOnly())
		{
		    JOptionPane.showMessageDialog (root, (String)cc.rb.getObject("gridlistener_error_boundarymove"), (String)cc.rb.getObject ("errors_selectionerror"), JOptionPane.ERROR_MESSAGE);
		    return;
		}
		else
		{
		    x1 = x;
		    y1 = y;
		    gs.updateHotspots ();
		    gs.translateSelectBox (x,y,true);
		    inProgress = PROGRESS_TRANSLATE;
		    return;
		}
	    }
	    else
	    {
		if (e.isShiftDown ())
		{
		    //		    System.out.println ("Some bug in rotating, use the diaog");
		    //		    inProgress = PROGRESS_ROTATE;
		    //		    gs.updateHotspots ();
		    //		    gs.startRotateSelectBox (hotspot);
		    //		    gs.rotateSelectBox (x,y, hotspot, true);
		    return;
		}
		else
		{
		    inProgress = PROGRESS_RESIZE;
		    // save old clicked coords
		    Point p = gs.getClickedHotspotCoords (hotspot);
		    x1 = p.x;
		    y1 = p.y;
		    // save opposite coords
		    p = gs.getOppositeHotspotCoords (hotspot);
		    x2 = p.x;
		    y2 = p.y;
		    // and start scaling GUI feedback
		    gs.updateHotspots ();
		    gs.startScaleSelectBox (hotspot);
		    gs.scaleSelectBox (x,y,hotspot,true);
		    return;
		}
	    }

	}

	// select box
	if (hotspot == gs.HOTSPOT_NONE && cc.mode != cc.DRAW_MODE && cc.mode != cc.MIRROR_MODE && !(cc.mode == cc.ZOOM_MODE && cc.zoomType == cc.ZOOM_IN))
        {
	    inProgress = PROGRESS_SELECTBOX;
	    if (!selectedItems.isEmpty ())
	    {
		gs.updateHotspots ();
	    }
	    gs.startSelectBox (x,y);
	    return;
	}
    }



    /**
     *  event handler for the mouseDragged event. See above.
     */
    public void mouseDragged(MouseEvent e) 
    {
	int x = calcSnapX(e);
	int y = calcSnapY(e);
	double[] p1 = {x,y};
	double[] p2 = new double[2];
	cc.gc.invtrf.transform(p1,0,p2,0,1);
	
	switch (inProgress)
  	{
	case PROGRESS_SETZOOM :
	    {
		gs.updateSelectBox (x,y);
		break;
	    }
	case PROGRESS_SELECTBOX :
	    {
		gs.updateSelectBox (x,y);
		break;
	    }
	case PROGRESS_ROTATE :
	    {
		gs.rotateSelectBox (x,y, hotspot, false);
		break;
	    }
	case PROGRESS_RESIZE :
	    {
		gs.scaleSelectBox (x,y,hotspot,false);
		break;
	    }
	case PROGRESS_TRANSLATE :
	    {
		gs.translateSelectBox (x,y, false);
		break;
	    }
	}
    }
    

    /**
     *  Event handler for the mouse released event. see above.
     */
    public void mouseReleased(MouseEvent e) 
    {
	int x = calcSnapX(e);
	int y = calcSnapY(e);
	// retransform and get other neccessary information
	int[] trf = cc.gc.fromScreenToWorld(x,y);
	int tx = trf[0]; // "transformed x"
	int ty = trf[1];
	// and in user space (real world coordinates)
	double ux = cc.gc.fromWorldToUser(tx);
	double uy = cc.gc.fromWorldToUser(ty);

	switch (inProgress)
	{
	case PROGRESS_SETZOOM :
	{
	    gs.updateSelectBox (x,y);
	    Rectangle selectbox = gs.getSelectBox ();
	    Rectangle worldbox = cc.gc.boxFromScreenToWorld(selectbox);
	    // if box not point
	    if (selectbox.width > 0 && selectbox.height > 0)
	    {
		// rescale to full drawingarea size
		gs.clearSelectBox ();
	        root.getAdjustment().newZoom(worldbox);
		int tol = Integer.parseInt(cc.op.get (Options.click_tolerance));
		cc.dc.tolerance = cc.gc.SingleFromScreenToWorld(tol);
		cc.gc.gtk.fixBoundingBoxes ();
	    }
	    else
	    {
		// do nothing
	    }
	    inProgress = PROGRESS_NOTHING;
	    break;
	}
	case PROGRESS_SELECTBOX :
	{
	    // 1) update the select box in GridSelector
	    gs.updateSelectBox (x,y);
	    // 2) get selected items and calc their perimeter
	    // depending on CRTL status, delete selection first or leave it
	    selectedItems = gs.getSelectedItems (e.isControlDown(), selectedItems);
	    if (!selectedItems.isEmpty ())
	    {
		if (gs.containsCircles ())
		{
		    gs.setHotspotMask (GridSelector.MASK_CIRCLES);
		}
		else
		{
		    if (gs.isNodesOnly ())
		    {
			gs.setHotspotMask (GridSelector.MASK_NODES);
		    }
		    else
		    {
			gs.setHotspotMask (GridSelector.MASK_ALL);
		    }
		}
	    }
	    da.repaint ();
  	    Runnable updateHotspots = new Runnable() 
  		{
  		    public void run() { calcPerimeter(true); }
  		};
  	    SwingUtilities.invokeLater(updateHotspots);
	    // 4) clear up
	    inProgress = PROGRESS_NOTHING;
	    break;
	}
	case PROGRESS_ROTATE :
	{
	    gs.rotateSelectBox (x,y, hotspot, true);
	    Point anchor = gs.getCenterHotspot ();
	    int[] world = cc.gc.fromScreenToWorld(anchor.x, anchor.y);
	    double angle = -gs.getRotateAngle ();
	    AffineTransform newtrafo = AffineTransform.getRotateInstance (angle, world[0], world[1]);
	    GridTransformation trafo = new GridTransformation (newtrafo, Math.toDegrees(angle), 1, 1, 0, 0, world[0], world[1]);
	    // to avoid rounding errors, rotate is undone in an unusual manner
	    // the position on screen of each affected item is stored
	    // so that undo just needs to reset those positions
	    // the code might seem a bit lengthy though
	    // convention: in the info object vector pushed on the stack, the 
	    // index 0 is reserved for the hotspotmask to be saved.
	    // all other items are again object arrays themselves, the index 0
	    // is used for a reference to the item, the follow-up indices are
	    // used to store positions, radius etc
	    Vector infos = new Vector ();
	    infos.add (gs.getHotspotMask ());
	    Enumeration enum = selectedItems.elements ();
	    while (enum.hasMoreElements ())
	    {
		GridItem item = (GridItem)enum.nextElement ();
		Object[] info = {item, item.getParameters()};
		infos.add (info);
	    }
 	    cc.um.notifyUndo (cc.um.UNDO_ROTATE, selectedItems, infos);
	    // apply trafo and done
	    applyTransformation (trafo, true);
	    inProgress = PROGRESS_NOTHING;
	    //gs.updateHotspots ();
	    break;
	}
	case PROGRESS_RESIZE :
	{
	    double factorx, factory;
	    gs.scaleSelectBox (x,y,hotspot,true);
	    gs.endScaleSelectBox ();
	    // calc scale factor as quotient of new and old hotspot coords
	    // relative to the opposite hotspot (which has not moved)
	    Point newclick = gs.getClickedHotspotCoords (hotspot);
	    if ((x1-x2) != 0)
		factorx = (double)(newclick.x - x2)/(double)(x1-x2);
	    else
		factorx = 1.0;
	    if ((y1-y2) != 0)
		factory = (double)(newclick.y - y2)/(double)(y1-y2);
	    else
		factory = 1.0;
	    if (Math.abs(factorx) < 0.01)
		factorx = 1.0;
	    if (Math.abs(factory) < 0.01)
		factory = 1.0;
	    AffineTransform newtrafo = AffineTransform.getScaleInstance (factorx, factory);
	    GridTransformation trafo = new GridTransformation (newtrafo,0,factorx, factory, 0,0,0,0);
	    // to avoid rounding errors, scale is undone in an unusual manner:
	    // the position on screen of each affected item is stored
	    // so that undo just needs to reset those positions
	    // the code might seem a bit lengthy though
	    // convention: in the info object vector pushed on the stack, the 
	    // index 0 is reserved for the hotspotmask to be saved.
	    // all other items are again object arrays themselves, the index 0
	    // is used for a reference to the item, the follow-up indices are
	    // used to store positions, radius etc
	    Vector infos = new Vector ();
	    infos.add (gs.getHotspotMask ());
	    Enumeration enum = selectedItems.elements ();
	    while (enum.hasMoreElements ())
	    {
		GridItem item = (GridItem)enum.nextElement ();
		Object[] info = {item, item.getParameters()};
		infos.add (info);
	    }
 	    cc.um.notifyUndo (cc.um.UNDO_SCALE, selectedItems, infos);
	    // apply trafo without repainting or anything
	    applyTransformation (trafo, false);
	    // calc neccessary translation which shifts the opposite hotspot
	    // back to where it was
	    Point newopp = gs.getOppositeHotspotCoords (hotspot);
	    int[] oldoppworld = cc.gc.fromScreenToWorld(x2,y2);
	    int[] oppworld = cc.gc.fromScreenToWorld(newopp.x, newopp.y);
	    int transx = oldoppworld[0] - oppworld[0];
	    int transy = oldoppworld[1] - oppworld[1];
	    newtrafo = AffineTransform.getTranslateInstance (transx, transy);
	    GridTransformation trafo2 = new GridTransformation (newtrafo, 0,1,1,transx, transy,0,0);
	    // apply trafo and repaint
	    applyTransformation (trafo2,true);
	    //	    Object[] infos = new Object[3];
	    //	    infos[0] = trafo;
	    //	    infos[1] = trafo2;
	    //	    infos[2] = gs.getHotspotMask ();
	    //	    cc.um.notifyUndo (cc.um.UNDO_SCALE, selectedItems, infos);
	    inProgress = PROGRESS_NOTHING;
	    break;
	}
	case PROGRESS_TRANSLATE :
	{
	    gs.translateSelectBox (x,y,true);
	    // calc transformation matrix
	    int transx = cc.gc.SingleFromScreenToWorld(x-x1);
	    int transy = cc.gc.SingleFromScreenToWorld(y1-y);
	    AffineTransform newtrafo = AffineTransform.getTranslateInstance (transx,transy);
	    GridTransformation trafo = new GridTransformation (newtrafo,0,1,1,transx, transy,0,0);
	    // prepare undo
	    Vector infos = new Vector ();
	    infos.add (gs.getHotspotMask ());
	    Enumeration enum = selectedItems.elements ();
	    while (enum.hasMoreElements ())
	    {
		GridItem item = (GridItem)enum.nextElement ();
		Object[] info = {item, item.getParameters()};
		infos.add (info);
	    }
 	    cc.um.notifyUndo (cc.um.UNDO_TRANSLATE, selectedItems, infos);
	    // perform trafo and exit
	    applyTransformation (trafo,true);
	    inProgress = PROGRESS_NOTHING;
	    break;
	}
	}
    }

    /**
     *  a little convenience method: applies the given transformation to
     *  all selected items, performs a repaint
     *  and recalculates the perimeter of the items
     *
     *  @param trafo - a homogenous 2D AffineTransform instance
     *  @param paint - true results in complete repaint, 
     *                 false in pure trafo without repaint
     */
    public void applyTransformation (GridTransformation trafo, boolean paint)
    {
	Enumeration enum = selectedItems.elements ();
	cc.gc.gtk.clearAllTransformedFlags ();
	while (enum.hasMoreElements())
	{
	    ((GridItem)enum.nextElement()).performTransformation (trafo);
	}
	if (paint)
	{
	    da.repaint ();
	    Runnable updatePerimeter = new Runnable() 
  		{
  		    public void run() { calcPerimeter (true); }
  		};
  	    SwingUtilities.invokeLater(updatePerimeter);
	}
	else
	{
	    calcPerimeter (false);
	}
    }

    /**
     *  calculates the perimeter of the selected items and updates the GUI
     *  @param paint - with or without painting
     */
    public void calcPerimeter (boolean paint)
    {
	Rectangle perimeterworld = new Rectangle ();
	try
        {
	    perimeterworld = ((GridItem)selectedItems.firstElement()).getPerimeter();
	} catch (NoSuchElementException exc) {}
	if (!perimeterworld.equals (new Rectangle ()))
	{
	    Enumeration enum = selectedItems.elements ();
	    while (enum.hasMoreElements ())
	    {
		perimeterworld = perimeterworld.union (((GridItem)enum.nextElement ()).getPerimeter ());
	    }
	    // pass perimeter to GridSelector and add hotspots
	    int[] screen1 = cc.gc.fromWorldToScreen(perimeterworld.x, perimeterworld.y);
	    int[] screen2 = cc.gc.fromWorldToScreen(perimeterworld.x+perimeterworld.width, perimeterworld.y+perimeterworld.height);
	    Rectangle perimeter = new Rectangle (screen1[0], screen1[1],screen2[0] - screen1[0], screen2[1] - screen1[1]);
	    gs.setPerimeter (perimeter);
	    if (paint)
		gs.updateHotspots ();
	}
    }

    


    /**
     *  event handler for the mouseMoved event. mostly takes care of the 
     *  indirect adding of segments and elements
     */
    public void mouseMoved(MouseEvent e) 
    {
	int x = calcSnapX(e);
	int y = calcSnapY(e);
	int[] trf = cc.gc.fromScreenToWorld(x,y);
	g = (Graphics)da.getGraphics ();
	// give user coord feedback
	if (cc.dc.displaycoords)
	{   
	    g.setColor (da.getBackground());
	    g.fillRect (da.getWidth()-50, 0,da.getWidth(), 30);
	    String sx = Double.toString(cc.gc.fromWorldToUser(trf[0]));
	    String sy = Double.toString(cc.gc.fromWorldToUser(trf[1]));
	    if (sx.length() >= 6)
		sx = sx.substring(0,5);
	    if (sy.length() >= 6)
		sy = sy.substring(0,5);
	    g.setColor (Color.black);
	    g.setFont (cc.dc.font_numbering);
	    g.drawString("x: "+sx, da.getWidth()-50, 10);
	    g.drawString("y: "+sy, da.getWidth()-50, 25);
	}

	switch (inProgress)
	{
	case PROGRESS_ADD_NODE :
	{
	    // draw over old rect
	    g.setXORMode (da.getBackground ());
	    g.setColor (cc.dc.color_lasso);
	    if (isOverSegment)
		g.drawRect (x1-4,y1-4,8,8);
	    else
		g.drawRect (x1-1,y1-1,2,2);
	    // update and draw new one
	    x1 = x;
	    y1 = y;
	    checkOverSegment (x,y);
	    if (isOverSegment)
		g.drawRect (x1-4,y1-4,8,8);
	    else
		g.drawRect (x1-1,y1-1,2,2);
	    g.setPaintMode ();
	    break;
	}
	case PROGRESS_ADD_LINE : // line and multiline are handled the same
	case PROGRESS_ADD_MULTILINE :
	case PROGRESS_MIRROR :  // the line is used for mirroring too
	{
	    g.setXORMode (da.getBackground ());
	    g.setColor (cc.dc.color_lasso);
	    // paint over old line and caption
	    g.drawLine (x1,y1,x2,y2);
	    if (cc.dc.displaycoords && !(inProgress == PROGRESS_MIRROR))
		g.drawString (caption,x2,y2);
	    // paint new line and caption
	    g.drawLine (x1,y1,x,y);
	    if (cc.dc.displaycoords && !(inProgress == PROGRESS_MIRROR))
	    {
		double[] p0 = {x1,y1};
		double[] trf0 = new double[2];
		cc.gc.invtrf.transform(p0,0,trf0,0,1);
		caption = Double.toString(cc.gc.fromWorldToUser((int)Math.sqrt ((double)((trf[0]-trf0[0])*(trf[0]-trf0[0]) + (trf[1]-trf0[1])*(trf[1]-trf0[1])))));
		g.drawString (caption,x,y);
	    }
	    // store new end coords
	    x2 = x;
	    y2 = y;
	    g.setPaintMode ();
	    break;
	}
	case PROGRESS_ADD_TRI : // elements
	case PROGRESS_ADD_QUAD :
	case PROGRESS_ADD_MACRO :
	{
	    g.setXORMode (da.getBackground ());
	    g.setColor (cc.dc.color_lasso);
	    // paint over old line
	    g.drawLine (x1,y1,x2,y2);
	    // paint new line
	    g.drawLine (x1,y1,x,y);
	    // store new end coords
	    x2 = x;
	    y2 = y;
	    g.setPaintMode ();
	    break;
	}
	case PROGRESS_ADD_CIRCLEP : // circles
	case PROGRESS_ADD_CIRCLEM :
	{
	    g.setXORMode (da.getBackground ());
	    g.setColor (cc.dc.color_lasso);
	    if (clickCounter == 1)
	    {
		// the center is already set, we are currently specifying r
		// unfortunately, java does not support specifying the
		// center, but only the top left corner of the surroundung
		// rectangle. so some tedious calculations are neccessary

		// paint over old circle
		g.drawOval (x1 - r, y1 - r,2*r,2*r);
		if (cc.dc.displaycoords)
		{
		    g.drawString (caption,cx,cy);
		}
		// calculate new radius
		r = (int)Math.sqrt ((x1-x)*(x1-x) + (y1-y)*(y1-y));
		// draw new circle
		g.drawOval (x1 - r, y1 - r,2*r,2*r);
		if (cc.dc.displaycoords)
		{
		    int lr = cc.gc.SingleFromScreenToWorld (r);
		    caption = Double.toString(cc.gc.fromWorldToUser(lr));
		    g.drawString (caption,x,y);
		}
		x2 = x1;
		y2 = y1;
		cx = x; 
		cy = y;
		g.setPaintMode ();
	    }
	    else
	    {
		if (clickCounter == 2)
		{
		    // center and radius are already specified
		    // now for the start angle, which is set by a line
		    // remove old line
		    g.drawLine (x1,y1,x2,y2);
		    if (cc.dc.displaycoords)
			g.drawString (caption,cx,cy);
		    // draw line
		    g.drawLine (x1,y1,x,y);
		    x2 = x; y2 = y;
		    // calc start angle
		    double h1 = x2-x1;
		    double h2 = y2-y1;
		    a1 = Math.acos(h1 / Math.sqrt(h1*h1+h2*h2));
		    if (y2 > y1)
			a1 = 2*Math.PI - a1;
		    a2 = a1;
		   if (cc.dc.displaycoords)
		    {
			caption = Double.toString (Math.toDegrees(a1));
			if (caption.length() >= 8)
			    caption = caption.substring(0,7);
			g.drawString (caption,x,y);
		    }
		   cx = x;
		   cy = y;
		}
		else
		{
		    if (clickCounter == 3)
		    {
			// now for the end angle, which is set by a line 
			// and the arc being drawn as visual feedback
			// draw over old line
			g.setXORMode (da.getBackground());
			g.setColor (cc.dc.color_lasso);
			g.drawLine(x1,y1,x2, y2);
			if (cc.dc.displaycoords)
			    g.drawString(caption,cx,cy);
			// draw over old arc
			int darc = (int)(180*arc/Math.PI);
			int dstart = (int)(180*a1/Math.PI);
			g.drawArc(x1-r,y1-r,2*r,2*r,dstart,darc);
			// calc new end angle
			x2 = x;
			y2 = y;
			double h1 = x2-x1;
			double h2 = y2-y1;
			a2 = Math.acos(h1 / Math.sqrt(h1*h1+h2*h2));
			// full circle
			if (y2 > y1)
			    a2 = 2*Math.PI - a2;
			// take care of orientation
			if (inProgress == PROGRESS_ADD_CIRCLEP)
			{
			    if (a2<a1) 
				a2=a2+2*Math.PI;
			}
			else
			{
			    if (a2>a1) 
				a2=a2-2*Math.PI;
			}
			// draw new line
			g.setColor (cc.dc.color_lasso);
			g.drawLine(x1,y1,x2,y2);
			// draw new arc
			arc= a2-a1;
			// convert to degrees
			darc = (int)(180*arc/Math.PI);
			dstart = (int)(180*a1/Math.PI);
			g.drawArc(x1-r,y1-r,2*r,2*r,dstart,darc);
			if (cc.dc.displaycoords)
			{
			    caption = Double.toString (Math.toDegrees(a2));
			    if (caption.length() >= 8)
				caption = caption.substring(0,7);
			    g.drawString (caption,x,y);
			}
			cx = x;
			cy = y;
			g.setPaintMode ();
		}
		}
	    }
	    break;
	}
	case PROGRESS_SETZOOM :
	    {
		gs.updateSelectBox (x,y);
		break;
	    }
	}
    }
    
    /**
     *  the mouseClicked event handler takes care of it all
     */
    public void mouseClicked(MouseEvent e) 
    {
	int x = calcSnapX(e);
	int y = calcSnapY(e);
	// retransform and get other neccessary information
	int[] trf = cc.gc.fromScreenToWorld(x,y);
	int tx = trf[0]; // "transformed x"
	int ty = trf[1];
	// click coords without snap
	int xs = e.getX();
	int ys = e.getY();
	int[] trfs = cc.gc.fromScreenToWorld(xs,ys);
	// and in user space (real world coordinates)
	double ux = cc.gc.fromWorldToUser(tx);
	double uy = cc.gc.fromWorldToUser (ty);;
	// get clicked-on element
	GridItem clickedItem = gs.getClickedItem (trfs[0],trfs[1]);
	// if in no mode, the clicked item is added to the selection
	if (inProgress == PROGRESS_NOTHING && cc.mode == cc.NO_MODE && clickedItem != null)
	{
	    boolean added = false;
	    boolean removed = false;
	    if (e.isControlDown())
	    {
		if (selectedItems.contains (clickedItem))
		{
		    selectedItems.removeElement (clickedItem);
		    clickedItem.unsetMarker ();
		    removed = true;
		}
		else
		{
		    selectedItems.addElement (clickedItem);
		    clickedItem.setMarker ();
		    added = true;
		}
	    }
	    else
	    {
		Enumeration enum = selectedItems.elements ();
		while (enum.hasMoreElements())
		    ((GridItem)enum.nextElement()).unsetMarker ();
		selectedItems = new Vector ();
		selectedItems.addElement (clickedItem);
		clickedItem.setMarker ();
	    }
	    if (!removed)
	    {
		if (clickedItem instanceof SegmentCircle)
		    gs.setHotspotMask (GridSelector.MASK_CIRCLES);
		else
		{
		    if (clickedItem instanceof Node && !added)
			gs.setHotspotMask (GridSelector.MASK_NODES);
		    else
			gs.setHotspotMask (GridSelector.MASK_ALL);
		}
	    }
	    da.repaint ();
	    // dont ask, i hate xor mode painting
	    Runnable updateHotspots1 = new Runnable() 
  		{
  		    public void run() { calcPerimeter(true); }
  		};
  	    SwingUtilities.invokeLater(updateHotspots1);
	    Runnable updateHotspots2 = new Runnable() 
  		{
  		    public void run() { calcPerimeter(true); }
  		};
  	    SwingUtilities.invokeLater(updateHotspots2);
	    return;
	}
	// if indirect input is active, start or finish adding
	if (cc.mode == cc.DRAW_MODE && !cc.directMode)
	{
	    // right mouse button cancels all drawing of the current element
	    if ((e.getModifiers() & InputEvent.BUTTON3_MASK)!=0)
	    {
		inProgress = PROGRESS_NOTHING;
		clickCounter = 0;
		da.repaint ();
		return;
	    }
	    else
	    {
		// the number of clicks since the last add determines what
		// to do...
		if (clickCounter == 0)
		{
		    switch (cc.addType)
		    {
		    case ControlCenter.ADD_NODE :
		    {
			// init node input and store coordinates
			inProgress = PROGRESS_ADD_NODE;
			clickCounter = 1;
			x1 = x;
			y1 = y;
			// draw first tiny rect
			g.setXORMode (da.getBackground());
			g.setColor (cc.dc.color_lasso);
			checkOverSegment (x,y);
			if (isOverSegment)
			    g.drawRect (x1-4,y1-4,8,8);
			else
			    g.drawRect (x1-1,y1-1,2,2);
			g.setPaintMode ();
			break;
		    }
		    case ControlCenter.ADD_LINE :
		    {
			if (checkEmptyBoundary())
			{
			    // save the clickpoint as start point for the line
			    inProgress = PROGRESS_ADD_LINE;
			    clickCounter = 1;
			    x1 = x2 = x;
			    y1 = y2 = y;
			    caption = "";
			}
			    break;
		    }
		    case ControlCenter.ADD_MULTILINE :
		    {
			if (checkEmptyBoundary())
			{
			    // save the clickpoint as start point for the line
			    inProgress = PROGRESS_ADD_MULTILINE;
			    clickCounter = 1;
			    x1 = x2 = x;
			    y1 = y2 = y;
			    caption = "";
			}
			break;
		    }
		    case ControlCenter.ADD_CIRCLEP :
		    {
			if (checkEmptyBoundary())
			{
			    inProgress = PROGRESS_ADD_CIRCLEP;
			    clickCounter = 1;
			    // clickpoint is set as center
			    x1 = x;
			    y1 = y;
			    cx = x; 
			    cy = y;
			    // radius is 0, angles are set to full circle
			    r = 0;
			    a1 = 0;
			    a2 = 2*Math.PI;
			    caption = "";
			}
			break;
		    }
		    case ControlCenter.ADD_CIRCLEM :
		    {
			if (checkEmptyBoundary())
			{
			    inProgress = PROGRESS_ADD_CIRCLEM;
			    clickCounter = 1;
			    // init temp variables
			    x1 = x;
			    y1 = y;
			    r = 0;
			    a1 = 2*Math.PI;;
			    a2 = 0;
			    caption = "";
			}
			break;
		    }
		    case ControlCenter.ADD_TRI :
		    {
			if (clickedItem != null && clickedItem instanceof Node)
			{
			    inProgress = PROGRESS_ADD_TRI;
			    clickCounter = 1;
			    // save node that has been hit...
			    n1 = (Node)clickedItem;
			    // save the clickpoint as start point for the line
			    x1 = x2 = x;
			    y1 = y2 = y;
			}
			else
			{
			    // tell everybody to fuck off & cancel operation
			    inProgress = PROGRESS_NOTHING;
			    JOptionPane.showMessageDialog (root, (String)cc.rb.getObject("gridlistener_error_nonodehit"), (String)cc.rb.getObject ("errors_gridinputerror"), JOptionPane.ERROR_MESSAGE);
			    da.repaint ();
			}
			break;
		    }
		    case ControlCenter.ADD_QUAD :
		    {
			if (clickedItem != null && clickedItem instanceof Node)
			{
			    inProgress = PROGRESS_ADD_QUAD;
			    clickCounter = 1;
			    // save node that has been hit...
			    n1 = (Node)clickedItem;
			    // save the clickpoint as start point for the line
			    x1 = x2 = x;
			    y1 = y2 = y;
			}
			else
			{
			    // tell everybody to fuck off & cancel operation
			    inProgress = PROGRESS_NOTHING;
			    JOptionPane.showMessageDialog (root, (String)cc.rb.getObject("gridlistener_error_nonodehit"), (String)cc.rb.getObject ("errors_gridinputerror"), JOptionPane.ERROR_MESSAGE);
			    da.repaint ();
			}
			break;
		    }
		    case ControlCenter.ADD_MACRO :
		    {
			if (clickedItem != null && clickedItem instanceof Node)
			{
			    clickCounter = 1;
			    inProgress = PROGRESS_ADD_MACRO;
			    // save node that has been hit...
			    n1 = (Node)clickedItem;
			    // save the clickpoint as start point for the line
			    x1 = x2 = x;
			    y1 = y2 = y;
			}
			else
			{
			    // tell everybody to fuck off & cancel operation
			    inProgress = PROGRESS_NOTHING;
			    JOptionPane.showMessageDialog (root, (String)cc.rb.getObject("gridlistener_error_nonodehit"), (String)cc.rb.getObject ("errors_gridinputerror"), JOptionPane.ERROR_MESSAGE);
			    da.repaint ();
			}
			break;
		    }
			case ControlCenter.ADD_CLIPBOARD :
			{
			    // if selection is visible, remove hotspots
			    if (!selectedItems.isEmpty())
				gs.updateHotspots ();
			    // calc affine transform 
			    int[] oldp = cc.gc.fromScreenToWorld(copyPasteAnchor.x, copyPasteAnchor.y);
			    AffineTransform trafo = AffineTransform.getTranslateInstance (trfs[0] - oldp[0], trfs[1] - oldp[1]);
			    GridTransformation gtrafo = new GridTransformation (trafo,0,1,1,trfs[0] - oldp[0], trfs[1] - oldp[1],0,0);
			    // apply trafo to clones
			    Vector clones = da.getCloner ().getClones ();
			    Enumeration enum = clones.elements ();
			    while (enum.hasMoreElements ())
			    {
				GridItem item = (GridItem)enum.nextElement();
				item.clearTransformedFlag ();
			    }
			    enum = clones.elements();
			    while (enum.hasMoreElements())
			    {
				((GridItem)enum.nextElement()).performTransformation (gtrafo);
			    }
			    // add all to Domain
			    enum = clones.elements ();
			    Vector bounds = new Vector ();
			    Vector clonedBounds = new Vector ();
			    Vector savedbnodes = new Vector ();
			    Vector savedelements = new Vector ();
			    Vector nextToClone = new Vector ();
			    while (enum.hasMoreElements ())
			    {
				GridItem item = (GridItem)enum.nextElement();
				// segments have two different clone modes
				// unfortunately :-)
				if (item instanceof Segment)
				{
				    // in APPEND mode, all cloned segments
				    // remain in their boundary and are
				    // appended with new segmentnumber
				    Segment s = (Segment)item;
				    if (cc.copymode == cc.APPEND)
				    {
					Boundary newboundary = s.getBoundary ();
					if (!cc.d.db.isBoundary (s.getBoundary()))
					{
					    if (bounds.contains (s.getBoundary()))
					    {
						int index = bounds.indexOf (s.getBoundary ());
						newboundary = (Boundary)clonedBounds.elementAt (index);
					    }
					    else
					    {
						cc.d.db.addBoundary ();
						//						System.out.println ("added boundary "+(cc.d.db.getBoundaryCount()-1));
						bounds.add (s.getBoundary());
						clonedBounds.add (cc.d.db.getBoundary (cc.d.db.getBoundaryCount()-1));
						newboundary = cc.d.db.getBoundary (cc.d.db.getBoundaryCount()-1);
					    }
					}
					s.setBoundary (newboundary);
					cc.d.db.addSegment (s.getBoundary(),s);
					nextToClone.add (s);
				    }
				    if (cc.copymode == cc.KEEPSTRUCTURE)
				    {
					// in KEEPSTRUCTURE MODE, a new 
					// boundary is created for every 
					// existing one, and clones are added
					// to the new boundary
					if (!bounds.contains (s.getBoundary()))
					{
					    cc.d.db.addBoundary ();
					    Boundary b = cc.d.db.getBoundary (cc.d.db.getBoundaryCount()-1);
					    bounds.add (s.getBoundary ());
					    clonedBounds.add (b);

					}
					int origindex = bounds.indexOf(s.getBoundary());
					Boundary cb = (Boundary)clonedBounds.get (origindex);
					cc.d.db.addSegment (cb, s);
					nextToClone.add (s);
				    }
				}
				else
				{
				    // boundary nodes are added after all
				    // segments are done
				    if (item instanceof BoundaryNode)
				    {
					savedbnodes.add (item);
				    }
				    else
				    { 
					if (item instanceof Cell2D)
					{
					    savedelements.add (item);
					}
					else
					{
					    // the rest is added just fine
					    cc.d.add (item);
					    nextToClone.add (item);
					}
				    }
				}
			    }
			    // take care of boundary nodes
  			    enum = savedbnodes.elements ();
			    boolean shitewegotanerror = false;
  			    while (enum.hasMoreElements ())
  			    {
  				BoundaryNode b = (BoundaryNode)enum.nextElement ();
				BoundaryNode newnode = cc.d.db.reconfigureBoundaryNode (b);
				if (newnode == null)
				{
				    shitewegotanerror = true;
				}
				else
				{
				    b = newnode;
				}
  				cc.d.adjustNode (b);
  				nextToClone.add (b);
  			    }
			    if (shitewegotanerror)
			    {
				JOptionPane.showMessageDialog (root, (String)cc.rb.getObject("gridlistener_error_clone"), (String)cc.rb.getObject ("errors_cloneerror"), JOptionPane.ERROR_MESSAGE);
			    }
			    // take care of elements at last
			    enum = savedelements.elements ();
			    while (enum.hasMoreElements())
			    {
				cc.d.add ((Cell2D)enum.nextElement());
			    }
			    cc.d.db.updateNumbering ();
			    // the newly cloned items become selected
			    clearSelection ();
			    selectedItems = clones;
			    gs.setHotspotMask (copyPasteMask);
			    calcPerimeter (false);
			    // clone a fresh set for the next paste
			    GridCloner cloner = new GridCloner (nextToClone);
			    cloner.createClone ();
			    da.setCloner (cloner);
			    storeCopyPasteAnchor ();
			    // done
			    da.repaint ();
			    Runnable paintHotspots = new Runnable() 
  			    {
  				public void run() { gs.updateHotspots(); }
  			    };
  			    SwingUtilities.invokeLater(paintHotspots);
			    cc.gc.gtk.updateBoundaryCombos ();
			    // store undo information
			    Object[] info = new Object[2];
			    info[0] = gs.getHotspotMask();
			    info[1] = cc.gc.workingBoundary;
			    cc.um.notifyUndo (cc.um.UNDO_PASTE, selectedItems, info);

			}
		    }
		}
		else
		{
		    if (clickCounter == 1)
		    {
			switch (inProgress)
			{
			case PROGRESS_ADD_NODE :
			{
			    inProgress = PROGRESS_NOTHING;
			    clickCounter = 0;
			    // insert a node with the transformed coordinates
			    if (isOverSegment)
			    {
				Node tmp = new Node (tx,ty);
				double parvalue = overSegment.getParameterValue (tmp);
				parvalue += overSegment.getSegmentNumber ();
				if (parvalue == (double)overSegment.getBoundary().getSegmentCount())
				{
				    parvalue = 0;
				    overSegment = overSegment.getBoundary().getSegment (0);
				}
				BoundaryNode n = new BoundaryNode (parvalue, overSegment.getBoundary(), overSegment);
				cc.d.adjustNode (n);
				Vector tmpvec = new Vector ();
				tmpvec.add (n);
				cc.um.notifyUndo (cc.um.UNDO_ADDNODE, tmpvec, null);
			    }
			    else
			    {
				Node t = new Node(tx,ty);
				cc.d.adjustNode(t);
				Vector tmpvec = new Vector ();
				tmpvec.add (t);
				cc.um.notifyUndo (cc.um.UNDO_ADDNODE, tmpvec, null);
			    }
			    cc.d.updateNodeNumbers ();
			    da.repaint ();
			    break;
			}
			case PROGRESS_ADD_LINE :
			{
			    inProgress = PROGRESS_NOTHING;
			    clickCounter = 0;
			    caption = "";
			    if (x == x1 && y == y1 )
			    {
				JOptionPane.showMessageDialog (root, (String)cc.rb.getObject("gridlistener_error_samepointtwice"), (String)cc.rb.getObject ("errors_gridinputerror"), JOptionPane.ERROR_MESSAGE);
			    }
			    else
			    {
				// create new line object
				int[] trf1 = cc.gc.fromScreenToWorld(x1,y1);
				int tx1 = trf1[0];
				int ty1 = trf1[1];
				
				SegmentLine newline = new SegmentLine (tx1,ty1,tx,ty, SegmentLine.TWOPOINT);
				cc.d.db.addSegment (cc.gc.workingBoundary,newline);
				Vector tmpvec = new Vector ();
				tmpvec.add (newline);
				cc.um.notifyUndo (cc.um.UNDO_ADDLINE, tmpvec, null);
				cc.gc.gtk.updateBoundaryCombos ();
			    }
			    // give visual feedback
			    da.repaint ();
			    break;
			}
			case PROGRESS_ADD_MULTILINE :
			{
			    if (x == x1 && y == y1 )
			    {
				JOptionPane.showMessageDialog (root, (String)cc.rb.getObject("gridlistener_error_samepointtwice"), (String)cc.rb.getObject ("errors_gridinputerror"), JOptionPane.ERROR_MESSAGE);
				inProgress = PROGRESS_NOTHING;
				clickCounter = 0;
				caption = "";
			    }
			    else
			    {
				// create new line object
				int[] trf1 = cc.gc.fromScreenToWorld(x1,y1);
				int tx1 = trf1[0];
				int ty1 = trf1[1];
				
				SegmentLine newline = new SegmentLine (tx1,ty1,tx,ty, SegmentLine.TWOPOINT);

				cc.d.db.addSegment (cc.gc.workingBoundary,newline);
				cc.gc.gtk.updateBoundaryCombos ();
				Vector tmpvec = new Vector ();
				tmpvec.add (newline);
				cc.um.notifyUndo (cc.um.UNDO_ADDLINE, tmpvec, null);
				// update storage variables
				caption = "";
				x1 = x2 =x;
				y1 = y2 = y;
			    }
			    // give visual feedback
			    da.repaint ();
			    break;
			}
			case PROGRESS_ADD_CIRCLEP :
			case PROGRESS_ADD_CIRCLEM :
			{
			    if (r == 0)
			    {
				JOptionPane.showMessageDialog (root, (String)cc.rb.getObject("gridlistener_error_radiuszero"), (String)cc.rb.getObject ("errors_gridinputerror"), JOptionPane.ERROR_MESSAGE);
				inProgress = PROGRESS_NOTHING;
				clickCounter = 0;
				da.repaint ();
				break;
			    }
			    else
			    {
				clickCounter++;
				arc = 0;
				double h1 = x2-x1;
				double h2 = y2-y1;
				a1 = Math.acos(h1 / Math.sqrt(h1*h1+h2*h2));
				if (y2 > y1)
				    a1 = 2*Math.PI - a1;
				a2 = a1;
				break;
			    }
			}
			case PROGRESS_ADD_TRI :
			case PROGRESS_ADD_QUAD :
			case PROGRESS_ADD_MACRO :
			{
			    if (clickedItem != null && clickedItem instanceof Node)
			    {
				clickCounter++;
				// save node that has been hit...
				n2 = (Node)clickedItem;
				// check for wrong input
				if (n1 == n2)
				{
				    JOptionPane.showMessageDialog (root, (String)cc.rb.getObject("gridlistener_error_nodetwicehit"), (String)cc.rb.getObject ("errors_gridinputerror"), JOptionPane.ERROR_MESSAGE);
				    inProgress = PROGRESS_NOTHING;
				    clickCounter = 0;
				    da.repaint ();
				}
				else
				{
				// update storage variables
				    x1 = x2 =x;
				    y1 = y2 = y;
				}
			    }
			    else
			    {
				// tell everybody to fuck off & cancel operation
				inProgress = PROGRESS_NOTHING;
				clickCounter = 0;
				
				JOptionPane.showMessageDialog (root, (String)cc.rb.getObject("gridlistener_error_nonodehit"), (String)cc.rb.getObject ("errors_gridinputerror"), JOptionPane.ERROR_MESSAGE);
				da.repaint ();
			    }
			    break;
			}
			}
		    }
		    else
		    {
			if (clickCounter == 2)
			{
			    switch (inProgress)
			    {
			    case PROGRESS_ADD_CIRCLEP :
			    case PROGRESS_ADD_CIRCLEM :
				{
				    
				clickCounter++;
				// paint over full circle to prepare 
				// angle setting
				g.setXORMode (da.getBackground());
				g.setColor (cc.dc.color_lasso);
				g.drawOval (x1 - r, y1 - r,2*r,2*r);
				// init arc variable
				arc = 0;
				g.setPaintMode ();
				break;
			    }
			    case PROGRESS_ADD_TRI :
			    {
				if (clickedItem != null && clickedItem instanceof Node)
				{
				    n3 = (Node)clickedItem;
				    Node[] nodes = {n1,n2,n3};
				    double vek1 = cc.d.db.crossProduct(2,0,1,nodes);
				    if ((vek1<0.0))
				    {
					// create edges and add them
					Edge e1, e2, e3;
					e1 = cc.d.adjustEdge (n1,n2);
					e2 = cc.d.adjustEdge (n2,n3);
					e3 = cc.d.adjustEdge (n3,n1);
					cc.d.updateEdgeNumbers ();
					// create and add tri 
					Tri tri = new Tri (e1,e2,e3);
					Vector tmpvec = new Vector ();
					tmpvec.add (e1);
					tmpvec.add (e2);
					tmpvec.add (e3);
					tmpvec.add (tri);
					cc.d.addCell (tri);
					cc.d.updateCellNumbers ();
					cc.um.notifyUndo (cc.um.UNDO_ADDTRI, tmpvec, null);
				    }
				    else
				    {
					JOptionPane.showMessageDialog (root, (String)cc.rb.getObject("gridlistener_error_nodeorientation"), (String)cc.rb.getObject ("errors_gridinputerror"), JOptionPane.ERROR_MESSAGE);
				    }
				}
				else
				{
				    // tell everybody to fuck off & cancel operation
				    JOptionPane.showMessageDialog (root, (String)cc.rb.getObject("gridlistener_error_nonodehit"), (String)cc.rb.getObject ("errors_gridinputerror"), JOptionPane.ERROR_MESSAGE);
				}
				inProgress = PROGRESS_NOTHING;
				clickCounter = 0;
				da.repaint ();
				break;
			    }
			    case PROGRESS_ADD_QUAD :
			    case PROGRESS_ADD_MACRO :
			    {
				if (clickedItem != null && clickedItem instanceof Node)
				{
				    clickCounter++;
				    // save node that has been hit...
				    n3 = (Node)clickedItem;
				    // check for wrong input
				    if (n1 == n3 || n2 == n3)
				    {
					JOptionPane.showMessageDialog (root, (String)cc.rb.getObject("gridlistener_error_nodetwicehit"), (String)cc.rb.getObject ("errors_gridinputerror"), JOptionPane.ERROR_MESSAGE);
					inProgress = PROGRESS_NOTHING;
					clickCounter = 0;
					da.repaint ();
				    }
				    else
				    {
					// update storage variables
					x1 = x2 =x;
					y1 = y2 = y;
				    }
				}
				else
				{
				    // tell everybody to fuck off & cancel operation
				    inProgress = PROGRESS_NOTHING;
				    clickCounter = 0;
				    JOptionPane.showMessageDialog (root, (String)cc.rb.getObject("gridlistener_error_nonodehit"), (String)cc.rb.getObject ("errors_gridinputerror"), JOptionPane.ERROR_MESSAGE);
				    da.repaint ();
				}
				break;
			    }
			    }
			}
			else
			{
			    if (clickCounter == 3)
			    {
				switch (inProgress)
				{
				case PROGRESS_ADD_CIRCLEP :
				{
				    inProgress = PROGRESS_NOTHING;
				    clickCounter = 0;
				    // retransform to user space
				    int[] trf1 = cc.gc.fromScreenToWorld(x1,y1);
				    int radius = cc.gc.SingleFromScreenToWorld(r);
				    double dstart = 180*a1/Math.PI;
				    double dend = 180*a2/Math.PI;
				    if (dstart > dend)
					JOptionPane.showMessageDialog (root, (String)cc.rb.getObject ("dialogs_circle_error_orientation"), (String)cc.rb.getObject ("errors_parseerror"), JOptionPane.ERROR_MESSAGE);
				    else
				    {
					if ( (Math.abs(360-(((int)(dend-dstart))%360)) < 10) || (dstart == dend))
					{
					    dend = dstart + 360.0;
					}
					if (dstart > dend)
					{
					    double tmp = dstart;
					    dstart = dend;
					    dend = tmp;
					}
					SegmentCircle newcirc = new SegmentCircle (trf1[0],trf1[1],radius,dstart,dend);
					cc.d.db.addSegment (cc.gc.workingBoundary,newcirc); 
					cc.gc.gtk.updateBoundaryCombos ();
					Vector tmpvec = new Vector ();
					tmpvec.add (newcirc);
					cc.um.notifyUndo (cc.um.UNDO_ADDCIRCLE, tmpvec, null);
				    }
				    // give visual feedback
				    da.repaint ();
				    break;
				}
				case PROGRESS_ADD_CIRCLEM :
				{
				    inProgress = PROGRESS_NOTHING;
				    clickCounter = 0;
				    int[] trf1 = cc.gc.fromScreenToWorld(x1,y1);
				    int radius = cc.gc.SingleFromScreenToWorld(r);
				    double dstart = 180*a1/Math.PI;
				    double dend = 180*a2/Math.PI;
				    if (dstart < dend)
					JOptionPane.showMessageDialog (root, (String)cc.rb.getObject ("dialogs_circle_error_orientation"), (String)cc.rb.getObject ("errors_parseerror"), JOptionPane.ERROR_MESSAGE);
				    else
				    {
					if ( (Math.abs(360-(((int)(dend-dstart))%360)) < 10) || (dstart == dend))
					{
					    dend = dstart + 360.0;
					}
					if (dstart < dend)
					{
					    double tmp = dstart;
					    dstart = dend;
					    dend = tmp;
					}
					SegmentCircle newcirc = new SegmentCircle (trf1[0],trf1[1],radius,dstart,dend);
					cc.d.db.addSegment (cc.gc.workingBoundary,newcirc);
					cc.gc.gtk.updateBoundaryCombos ();
					Vector tmpvec = new Vector ();
					tmpvec.add (newcirc);
					cc.um.notifyUndo (cc.um.UNDO_ADDCIRCLE, tmpvec, null);
				    }
				    // give visual feedback
				    da.repaint ();
				    break;
				}
				case PROGRESS_ADD_QUAD :
				{
				    if (clickedItem != null && clickedItem instanceof Node)
				    {
					n4 = (Node)clickedItem;
					// check for correct orientation
					Node[] nodes = {n1,n2,n3,n4};
					double vek1 = cc.d.db.crossProduct (3,0,1,nodes);
					double vek2 = cc.d.db.crossProduct (1,2,3,nodes);
					if ((vek1<0.0) && (vek2<0.0))
					{
					    // create edges and add them
					    Edge e1, e2, e3,e4;
					    e1 = cc.d.adjustEdge (n1,n2);
					    e2 = cc.d.adjustEdge (n2,n3);
					    e3 = cc.d.adjustEdge (n3,n4);
					    e4 = cc.d.adjustEdge (n4,n1);
					    cc.d.updateEdgeNumbers ();
					    // create and add Quad
					    Quad quad = new Quad ();
					    quad.setCell (e1,e2,e3,e4);
					    cc.d.addCell (quad);
					    Vector tmpvec = new Vector ();
					    tmpvec.add (e1);
					    tmpvec.add (e2);
					    tmpvec.add (e3);
					    tmpvec.add (e4);
					    tmpvec.add (quad);
					    cc.um.notifyUndo (cc.um.UNDO_ADDQUAD, tmpvec, null);
					    cc.d.updateCellNumbers ();
					}
					else
					{
					    JOptionPane.showMessageDialog (root, (String)cc.rb.getObject("gridlistener_error_nodeorientation"), (String)cc.rb.getObject ("errors_gridinputerror"), JOptionPane.ERROR_MESSAGE);

					}
				    }
				    else
				    {
					// tell everybody to fuck off & cancel operation
					JOptionPane.showMessageDialog (root, (String)cc.rb.getObject("gridlistener_error_nonodehit"), (String)cc.rb.getObject ("errors_gridinputerror"), JOptionPane.ERROR_MESSAGE);
				    }
				    inProgress = PROGRESS_NOTHING;
				    clickCounter = 0;
				    da.repaint ();
				    break;
				}
				case PROGRESS_ADD_MACRO :
				{
				    if (clickedItem != null && clickedItem instanceof Node)
				    {
					n4 = (Node)clickedItem;
					// check for correct orientation
					Node[] nodes = {n1,n2,n3,n4};
					double vek1 = cc.d.db.crossProduct (3,0,1,nodes);
					double vek2 = cc.d.db.crossProduct (1,2,3,nodes);
					if ((vek1<0.0) && (vek2<0.0))
					{
					    // create edges and add them
					    Edge e1, e2, e3,e4;
					    e1 = cc.d.adjustEdge (n1,n2);
					    e2 = cc.d.adjustEdge (n2,n3);
					    e3 = cc.d.adjustEdge (n3,n4);
					    e4 = cc.d.adjustEdge (n4,n1);
					    cc.d.updateEdgeNumbers ();
					    // create and add macro 
					    Macro macro = new Macro (1,1,1,e1,e2,e3,e4);
					    cc.d.addCell (macro);
					    cc.d.updateCellNumbers ();
					    Vector tmpvec = new Vector ();
					    tmpvec.add (e1);
					    tmpvec.add (e2);
					    tmpvec.add (e3);
					    tmpvec.add (e4);
					    tmpvec.add (macro);
					    cc.um.notifyUndo (cc.um.UNDO_ADDMACRO, tmpvec, null);
					}
					else
					{
					    JOptionPane.showMessageDialog (root, (String)cc.rb.getObject("gridlistener_error_nodeorientation"), (String)cc.rb.getObject ("errors_gridinputerror"), JOptionPane.ERROR_MESSAGE);

					}
				    }
				    else
				    {
					// tell everybody to fuck off & cancel operation
					JOptionPane.showMessageDialog (root, (String)cc.rb.getObject("gridlistener_error_nonodehit"), (String)cc.rb.getObject ("errors_gridinputerror"), JOptionPane.ERROR_MESSAGE);
					da.repaint ();
				    }
				    inProgress = PROGRESS_NOTHING;
				    clickCounter = 0;
				    da.repaint ();
				    break;
				}
				}
				
			    }
			    
			}
		    }
		}
	    }
	}
	
	// if direct input is active in add-mode, show corresponding dialogs
	if (cc.mode == cc.DRAW_MODE && cc.directMode)
	{
	    switch (cc.addType)
	    {
	    case ControlCenter.ADD_NODE :
		{
		    // init the dialog
		    BoundaryNodeDialog dialog = (BoundaryNodeDialog)cc.dm.get (cc.dm.BOUNDARYNODEDIALOG);
		    int tab = 0;
		    if (dialog.isVisible())
			tab = dialog.getSelectedTab ();
		    cc.dm.hide (cc.dm.BOUNDARYNODEDIALOG);
		    dialog.setTitle ((String)cc.rb.getObject("dialogs_boundarynode_caption_direct"));
		    dialog.setSource (null);
		    dialog.setCoords (ux,uy,0);
		    dialog.setSelectedTab (tab);
		    dialog.setVisible (true);
		    break;
		}
	    case ControlCenter.ADD_LINE :
		{
		    if (checkEmptyBoundary())
		    {
			// init the dialog (as the dialog is recycled in property mode, the caption must be set individually)
			LineDialog dialog = (LineDialog)cc.dm.get (cc.dm.LINEDIALOG);
			cc.dm.hide (cc.dm.LINEDIALOG);
			dialog.setTitle ((String)cc.rb.getObject("dialogs_line_caption_direct"));
			dialog.setSource (null);
			dialog.setStart (ux,uy);
			dialog.setEnd (ux,uy);
			dialog.setVisible (true);
		    }
		    break;
		}
	    case ControlCenter.ADD_CIRCLEP :
		{
		    if (checkEmptyBoundary())
		    {
			CircleDialog dialog = (CircleDialog)cc.dm.get (cc.dm.CIRCLEDIALOG);
			cc.dm.hide (cc.dm.CIRCLEDIALOG);
			dialog.setTitle ((String)cc.rb.getObject("dialogs_circle_caption_direct_p"));
			dialog.setSource (null);
			dialog.setCenter (ux,uy);
			dialog.setRadius (0.5);
			dialog.setAngles (0,360);
			dialog.setOrientation (CircleDialog.POS_ORIENTATION);
			dialog.setVisible (true);
		    }
		    break;
		}
	    case ControlCenter.ADD_CIRCLEM :
		{
		    if (checkEmptyBoundary())
		    {
			CircleDialog dialog = (CircleDialog)cc.dm.get (cc.dm.CIRCLEDIALOG);
			cc.dm.hide (cc.dm.CIRCLEDIALOG);
			dialog.setTitle ((String)cc.rb.getObject("dialogs_circle_caption_direct_m"));
			dialog.setSource (null);
			dialog.setCenter (ux,uy);
			dialog.setRadius (0.5);
			dialog.setAngles (0,360);
			dialog.setOrientation (CircleDialog.POS_ORIENTATION);
			dialog.setVisible (true);
		    }
		    break;
		}
	    case ControlCenter.ADD_TRI :
		{
		    TriDialog dialog = (TriDialog)cc.dm.get (cc.dm.TRIDIALOG);
		    cc.dm.hide (cc.dm.TRIDIALOG);
		    dialog.setTitle ((String)cc.rb.getObject("dialogs_tri_caption_direct"));
		    dialog.setSource (null);
		    // if a node has been hit, use its index as initial value
		    // for all three input fields, else no initial value
		    // is set
		    if (clickedItem != null && clickedItem instanceof Node)
		    {
			int nodeindex = ((Node)clickedItem).getNumber ();
			dialog.setNode1 (nodeindex);
			dialog.setNode2 (nodeindex);
			dialog.setNode3 (nodeindex);
		    }
		    else
		    {
			dialog.setNode1 (0);
			dialog.setNode2 (0);
			dialog.setNode3 (0);
		    }
		    dialog.setVisible (true);
		    break;
		}
	    case ControlCenter.ADD_QUAD :
		{
		    QuadDialog dialog = (QuadDialog)cc.dm.get (cc.dm.QUADDIALOG);
		    cc.dm.hide (cc.dm.QUADDIALOG);
		    dialog.setTitle ((String)cc.rb.getObject("dialogs_quad_caption_direct"));
		    dialog.setSource (null);
		    // if a node has been hit, use its index as initial value
		    // for all three input fields, else no initial value
		    // is set
		    if (clickedItem != null && clickedItem instanceof Node)
		    {
			int nodeindex = ((Node)clickedItem).getNumber ();
			dialog.setNode1 (nodeindex);
			dialog.setNode2 (nodeindex);
			dialog.setNode3 (nodeindex);
			dialog.setNode4 (nodeindex);
		    }
		    else
		    {
			dialog.setNode1 (0);
			dialog.setNode2 (0);
			dialog.setNode3 (0);
			dialog.setNode4 (0);
		    }
		    dialog.setVisible (true);
		    break;
		}
	    case ControlCenter.ADD_MACRO :
		{
		    MacroDialog dialog = (MacroDialog)cc.dm.get (cc.dm.MACRODIALOG);
		    cc.dm.hide (cc.dm.MACRODIALOG);
		    dialog.setTitle ((String)cc.rb.getObject("dialogs_macro_caption_direct"));
		    dialog.setSource (null);
		    // if a node has been hit, use its index as initial value
		    // for all three input fields, else no initial value
		    // is set
		    if (clickedItem != null && clickedItem instanceof Node)
		    {
			int nodeindex = ((Node)clickedItem).getNumber ();
			dialog.setNode1 (nodeindex);
			dialog.setNode2 (nodeindex);
			dialog.setNode3 (nodeindex);
			dialog.setNode4 (nodeindex);
		    }
		    else
		    {
			dialog.setNode1 (0);
			dialog.setNode2 (0);
			dialog.setNode3 (0);
			dialog.setNode4 (0);
		    }
		    dialog.setSelectedTab (0);
		    dialog.setVisible (true);
		    break;
		}
	    }
	    return;
	}// end of (cc.mode == cc.ADD_MODE && cc.directinput)
	// property queries
	if (cc.mode == ControlCenter.PROPERTY_MODE)
	{
	    // get type of element at mouse position
	    int elemType = 0;
	    if (clickedItem != null)
	    {
		if (clickedItem instanceof SegmentLine)
		    elemType = ELEMENT_LINE;
		else
		{
		    if (clickedItem instanceof SegmentCircle)
			elemType = ELEMENT_CIRCLE;
		    else
		    {
			if (clickedItem instanceof Tri)
			    elemType = ELEMENT_TRI;
			else
			{
			    if (clickedItem instanceof Macro)
				elemType = ELEMENT_MACRO;
			    else
			    {
				if (clickedItem instanceof Quad)
				    elemType = ELEMENT_QUAD;
				else
				{
				    if (clickedItem instanceof BoundaryNode)
					elemType = ELEMENT_BOUNDARYNODE;
				    else
					if (clickedItem instanceof Node)
					    elemType = ELEMENT_NODE;
				}
			    }
			}
		    }
		}
	    }
	    // depending on the element type, extract neccessary information 
	    // and bring up corresponding dialog
	    switch (elemType)
	    {
	    case ELEMENT_LINE :
		{
		    // get the line object
		    SegmentLine line = (SegmentLine)clickedItem;
		    LineDialog dialog = (LineDialog)cc.dm.get (cc.dm.LINEDIALOG);
		    cc.dm.hide (cc.dm.LINEDIALOG);
		    dialog.setTitle ((String)cc.rb.getObject("dialogs_line_caption_property"));
		    dialog.setSource (line);
		    dialog.setVisible (true);
		    break;
		}
	    case ELEMENT_CIRCLE :
		{
		    // get the circle object at click position
		    SegmentCircle circle = (SegmentCircle)clickedItem;
		    // get the circle's orientation
		    boolean orientation;
		    if (circle.getStartAngle() > circle.getEndAngle())
			orientation = CircleDialog.POS_ORIENTATION;
		    else
			orientation = CircleDialog.NEG_ORIENTATION;
		    // call the dialog
		    CircleDialog dialog = (CircleDialog)cc.dm.get (cc.dm.CIRCLEDIALOG);
		    cc.dm.hide (cc.dm.CIRCLEDIALOG);
		    if (orientation == CircleDialog.POS_ORIENTATION)
			dialog.setTitle ((String)cc.rb.getObject("dialogs_circle_caption_property_p"));
		    else
			dialog.setTitle ((String)cc.rb.getObject("dialogs_circle_caption_property_m"));
		    dialog.setSource (circle);
		    dialog.setOrientation (orientation);
		    dialog.setVisible (true);
		    break;
		}
	    case ELEMENT_TRI :
		{
		    // get the Tri object at click position
		    Tri tri = (Tri)clickedItem;
		    TriDialog dialog = (TriDialog)cc.dm.get (cc.dm.TRIDIALOG);
		    cc.dm.hide (cc.dm.TRIDIALOG);
		    dialog.setTitle ((String)cc.rb.getObject("dialogs_tri_caption_property"));
		    dialog.setSource (tri);
		    dialog.setVisible (true);
		    break;
		}
	    case ELEMENT_QUAD :
		{
		    // get the quad object at click position
		    Quad quad = (Quad)clickedItem;
		    // update the dialog with node indices
		    QuadDialog dialog = (QuadDialog)cc.dm.get (cc.dm.QUADDIALOG);
		    cc.dm.hide (cc.dm.QUADDIALOG);
		    dialog.setTitle ((String)cc.rb.getObject("dialogs_quad_caption_property"));
		    dialog.setSource (quad);
		    dialog.setVisible (true);
		    break;
		}
	    case ELEMENT_MACRO :
		{
		    // get the macro object at click position
		    Macro macro = (Macro)clickedItem;
		    // update the dialog with node indices
		    MacroDialog dialog = (MacroDialog)cc.dm.get (cc.dm.MACRODIALOG);
		    cc.dm.hide (cc.dm.MACRODIALOG);
		    dialog.setTitle ((String)cc.rb.getObject("dialogs_macro_caption_property"));
		    dialog.setSource (macro);
		    dialog.setSelectedTab (1);
		    dialog.setVisible (true);
		    break;
		}
	    case ELEMENT_NODE :
		{
		    Node node = (Node)clickedItem;
		    BoundaryNodeDialog dialog = (BoundaryNodeDialog)cc.dm.get (cc.dm.BOUNDARYNODEDIALOG);
		    cc.dm.hide (cc.dm.BOUNDARYNODEDIALOG);
		    dialog.setTitle ((String)cc.rb.getObject("dialogs_boundarynode_caption_property"));
		    dialog.setSource (node);
		    dialog.setSelectedTab (0);
		    dialog.setVisible (true);
		    break;
		}
		case ELEMENT_BOUNDARYNODE :
		{
		    BoundaryNode node = (BoundaryNode)clickedItem;
		    BoundaryNodeDialog dialog = (BoundaryNodeDialog)cc.dm.get (cc.dm.BOUNDARYNODEDIALOG);
		    cc.dm.hide (cc.dm.BOUNDARYNODEDIALOG);
		    dialog.setTitle ((String)cc.rb.getObject("dialogs_boundarynode_caption_property"));
		    dialog.setSource (node);
		    dialog.setSelectedTab (1);
		    dialog.setVisible (true);
		    break;
		}
	    }

	}

	// dialog modes...
	if (cc.mode == ControlCenter.EDGESTATUS_MODE)
	{
	    if (clickedItem instanceof Edge)
	    {
		Edge edge = (Edge)clickedItem;
		EdgeStatusDialog dialog = (EdgeStatusDialog)cc.dm.get (cc.dm.EDGESTATUSDIALOG);
		if (! e.isControlDown ())
		{
		    // left click sets values from dialog to edge
		    edge.setStatus (dialog.getStatus ());
		    edge.setBC (dialog.getBC());
		}
		else
		{
		    // left + ctrl reads values from edge into dialog
		    dialog.setStatus (edge.getStatus ());
		    dialog.setBC (edge.getBC ());
		}
	    }
	    return;
	}
	if (cc.mode == ControlCenter.PARALLELBLOCK_MODE)
	{
	    // get initial values for the dialog
	    if (clickedItem instanceof Macro)
	    {
		Macro mac = ((Macro)clickedItem);
		int pb = mac.getParallelBlock();
		int mb = mac.getMatrixBlock();
		int ref = mac.getRefinement(0);
		int  aiso = mac.getAnisotropyMode();
		double[] anf = mac.getAnisotropyFactors();
		ParallelBlockDialog dialog = (ParallelBlockDialog)cc.dm.get (cc.dm.PARALLELBLOCKDIALOG);
		dialog.setValues(pb,mb,ref,aiso,anf);
	    }
	    return;
	}
	if (cc.mode == ControlCenter.REFINE_MODE)
	{
	    // get initial values for the dialog

	    // update the dialog
	    //	    System.out.println ("Refine mode dialog updated");
	    return;
	}

	// rectify element mode
	if (cc.mode == ControlCenter.RECTIFYELEMENT_MODE)
	{
	    // cycle through rectify permutations
	    xs = e.getX();
	    ys = e.getY();
	    trfs = cc.gc.fromScreenToWorld(xs,ys);
	    clickedItem = gs.getClickedItem (trfs[0],trfs[1]);
	    if (e.getButton()==MouseEvent.BUTTON3)
	    {
		rectstatus=-1;		    
		Node[] n = ((Macro)cc.d.getCell(rectindex)).getNodes();
		for (int i=0; i<4; i++ ) 
		{
		    n[i].setCoord(rectsave[i].getCoord());
                }
	    }
	    else
		if (clickedItem!=null)
		    if ((clickedItem instanceof Quad) || (clickedItem instanceof Macro))
		    {
			Node[] n = ((Macro)clickedItem).getNodes();
			int cellindex = ((Macro)clickedItem).getNumber();
	                if (cellindex!=rectindex) rectstatus=-1;
			if (rectstatus==-1)
			{
			    rectsave=new Node[4];
			    for (int i=0; i<4; i++ ) 
			    {
				rectsave[i]=new Node();
				rectsave[i].setCoord(n[i].getCoord());
			    }
			    rectstatus=1;
			    rectindex=cellindex;
			}
			switch(rectstatus)
			{
			case 1:
			    if (!(n[2] instanceof BoundaryNode) &&
				(!(n[3] instanceof BoundaryNode)))
			    {
				for (int i=0; i<4; i++ ) n[i].setCoord(rectsave[i].getCoord());
				cc.d.rectifyElement(n[0],n[1],n[2],n[3]);
				da.repaint ();
			    }
			    rectstatus=2;
			    break;
			case 2:
			    if (!(n[3] instanceof BoundaryNode) &&
				(!(n[0] instanceof BoundaryNode)))
			    {
				for (int i=0; i<4; i++ ) n[i].setCoord(rectsave[i].getCoord());
				cc.d.rectifyElement(n[1],n[2],n[3],n[0]);
				da.repaint ();
			    }     
			    rectstatus=3;
			    break;
			case 3:
			    if (!(n[1] instanceof BoundaryNode) &&
				(!(n[0] instanceof BoundaryNode)))
			    {
				for (int i=0; i<4; i++ ) n[i].setCoord(rectsave[i].getCoord());
				cc.d.rectifyElement(n[2],n[3],n[0],n[1]);
				da.repaint ();
			    }     
			    rectstatus=4;
			    break;
			case 4:
			    if (!(n[1] instanceof BoundaryNode) &&
				(!(n[2] instanceof BoundaryNode)))
			    {
				for (int i=0; i<4; i++ ) n[i].setCoord(rectsave[i].getCoord());
				cc.d.rectifyElement(n[3],n[0],n[1],n[2]);
				da.repaint ();
			    }     
			    rectstatus=1;
			    break;
			}
		    }
	    return;
	}

	// zooming in
	if (cc.mode == ControlCenter.ZOOM_MODE && cc.zoomType == ControlCenter.ZOOM_IN)
	{
	    // perform standard zoom in operation at event point

            int xlen=cc.dc.screen_width/10;
            int ylen=cc.dc.screen_height/10;
	    
	    Rectangle selectbox = new Rectangle(x-xlen/2,y-ylen/2,xlen,ylen);
	    Rectangle worldbox = cc.gc.boxFromScreenToWorld(selectbox);

	    root.getAdjustment().newZoom(worldbox);
	    int tol = Integer.parseInt(cc.op.get (Options.click_tolerance));
	    cc.dc.tolerance = cc.gc.SingleFromScreenToWorld(tol);
	    cc.gc.gtk.fixBoundingBoxes ();

	    return;
	}

	// last up, mirror mode
	if (cc.mode == cc.MIRROR_MODE && !cc.directMode)
	{  
	    // right mouse button cancels all drawing of the current element
	    if ((e.getModifiers() & InputEvent.BUTTON3_MASK)!=0)
	    {
		inProgress = PROGRESS_NOTHING;
		clickCounter = 0;
		da.repaint ();
		return;
	    }
	    else
	    {
		// the number of clicks determines what to do...
		if (clickCounter == 0)
		{
		    // save the clickpoint as start point for the line
		    inProgress = PROGRESS_MIRROR;
		    clickCounter = 1;
		    x1 = x2 = x;
		    y1 = y2 = y;
		    caption = "";
		    return;
		}
		else
		{
		    inProgress = PROGRESS_NOTHING;
		    clickCounter = 0;
		    caption = "";
		    if (x == x1 && y == y1 )
		    {
			JOptionPane.showMessageDialog (root, (String)cc.rb.getObject("gridlistener_error_samepointtwice"), (String)cc.rb.getObject ("errors_gridinputerror"), JOptionPane.ERROR_MESSAGE);
		    }
		    else
		    {
			// start mirroring
			int[] trf1 = cc.gc.fromScreenToWorld(x1,y1);
			int tx1 = trf1[0];
			int ty1 = trf1[1];
			cc.gc.gtk.mirror (tx1,ty1,tx,ty);
			// give visual feedback
			da.repaint ();
			calcPerimeter (true);
			root.getMainActionListener ().disableMirrorMode ();
			int[] mirrorline = {tx1,ty1,tx,ty};
			Object[] info = new Object[2];
			info[0] = mirrorline;
			info[1] = gs.getHotspotMask ();
			cc.um.notifyUndo (cc.um.UNDO_MIRROR, selectedItems, info);
			return;
		    }
		}
	    }
	    return;
	}
    }

    /**
     *  shows an error message if the domain does not contain a boundary
     *  @return true <=> the domain contains a boundary
     */
    private boolean checkEmptyBoundary ()
    {
	if (cc.gc.workingBoundary == null)
	{
	    JOptionPane.showMessageDialog (root, (String)cc.rb.getObject("gridlistener_error_noboundary"), (String)cc.rb.getObject ("errors_gridinputerror"), JOptionPane.ERROR_MESSAGE);
	    return false;
	}
	return true;
    }



    /**
     *  checks if the cursor at given position is located over a segment. 
     *  sets the result in the isOverSegment flag and stores the segment in
     *  in the overSegment variable
     *  @param x,y - screen coord cursor location
     */
    private void checkOverSegment (int x, int y)
    {
	isOverSegment = false;
	overSegment = null;
	int[] withoutsnap = cc.gc.fromScreenToWorld(x, y);
	Enumeration bounds = cc.d.db.getBoundaries ().elements ();
	while (bounds.hasMoreElements () && !isOverSegment)
	{
	    Boundary actBound = (Boundary)bounds.nextElement ();
	    Enumeration segs = actBound.getSegments().elements();
	    while (segs.hasMoreElements() && !isOverSegment)
	    {
		Segment actseg = (Segment)segs.nextElement();
		if (actseg.isOnBoundary (withoutsnap[0], withoutsnap[1]))
		{
		    isOverSegment = true;
		    overSegment = actseg;
		}
	    }
	}
    }

  
    
    
    public void mouseEntered(MouseEvent e) 
    {
	// no need to implement this event...
    }
    
    
    public void mouseExited(MouseEvent e) 
    {
	// no need to implement this event
    }
  
    

    // *****************************************************
    // basic utility functions

    /** 
     *  transforms the application coordinate of the event point into world
     *  coordinates
     */
    private int calcSnapX(MouseEvent e)
    {
	try
	{
	    if ((cc.gc.snapon) && (cc.d.validpx!=null))
		//  	 //   return Math.round(e.getX()/cc.gc.vsnapx)*cc.gc.vsnapx;
		{
		    int i=0;
		    int d=(cc.d.validpx[1]-cc.d.validpx[0])/2;
		    while(e.getX()>(cc.d.validpx[i++]+d));
		    return cc.d.validpx[i-1];
		}
	    else
		return e.getX();
	}
	catch (Exception ex)
	{
	    return e.getX ();
	}
    }
	
    /** 
     *  transforms the application coordinate of the event point into world
     *  coordinates
     */
    private int calcSnapY(MouseEvent e)
    {
	// ugly but only temporarily :-)
	try
	{
	    if ((cc.gc.snapon) && (cc.d.validpy!=null))
	    {
		int i=0;
		int d=(cc.d.validpy[0]-cc.d.validpy[1])/2;
		while(e.getY()<cc.d.validpy[i++]-d);
		return cc.d.validpy[i-1];
	    }
	    //  	//	    return Math.round(e.getY()/cc.gc.vsnapy)*cc.gc.vsnapy;
	    else
		return e.getY();
	}
	catch (Exception ex)
	{
	    return e.getY ();
	}
    }



    // constants for the inProgress variable
    public static final int PROGRESS_NOTHING = 8900;
    public static final int PROGRESS_SETZOOM = 8901;
    public static final int PROGRESS_SELECTBOX = 8902;
    public static final int PROGRESS_ROTATE = 8903;
    public static final int PROGRESS_RESIZE = 8904;
    public static final int PROGRESS_TRANSLATE = 8905;
    public static final int PROGRESS_ADD_LINE = 8906;
    public static final int PROGRESS_ADD_MULTILINE = 8907;
    public static final int PROGRESS_ADD_CIRCLEP = 8908;
    public static final int PROGRESS_ADD_CIRCLEM = 8909;
    public static final int PROGRESS_ADD_TRI = 8910;
    public static final int PROGRESS_ADD_QUAD = 8911;
    public static final int PROGRESS_ADD_MACRO = 8912;
    public static final int PROGRESS_ADD_NODE = 8913;
    public static final int PROGRESS_MIRROR = 8914;
 

    // constants for the element types (needed when determing which type
    // of element has been clicked on
    public static final int ELEMENT_NODE = 11111;
    public static final int ELEMENT_LINE = 11112;
    public static final int ELEMENT_CIRCLE = 11113;
    public static final int ELEMENT_TRI = 11115;
    public static final int ELEMENT_QUAD = 11116;
    public static final int ELEMENT_MACRO = 11117;
    public static final int ELEMENT_BOUNDARYNODE = 11118;
    
}

