package devisor2.grid.GUI.event;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.*;
import java.util.*;

import devisor2.grid.GUI.dialogs.*;
import devisor2.grid.GUI.framework.*;
import devisor2.grid.GUI.help.*;
import devisor2.grid.options.*;
import devisor2.foundation.*;
import devisor2.foundation.base.*;
import devisor2.foundation.boundary.*;
import devisor2.grid.backend.*;
import devisor2.grid.backend.spellchecker.*;
import devisor2.grid.backend.undo.*;

/** 
 *  This class serves as the event listener for MainMenu and Toolbar. It is 
 *  modeled to be some sort of event multicaster more than an event handler.
 *  <br>
 *  It works like this:<br>
 *  Every user command that is initiated via the 
 *  MainMenu, ToolBar or a keyboard accelerator is caught in this class,
 *  because this class is the ActionListener for all these classes. 
 *  Every operation has therefore a so called ActionCommand assigned to
 *  it which is a unique String constant. Supported constants must be
 *  declared in this class.<br>
 *  The actionPerformed() method then determines which if-else-block holds
 *  the event handling code for the incoming Action Event. As this method is
 *  rather big and hard to read, I tried to order the blocks according to
 *  the main menu.<br>
 *  As this class is more an event multicaster than an actual event handler,
 *  it only takes care of updating the modes the application can be
 *  in (e.g. if in add mode, which type of element is addable), of updating
 *  the GUI (toolbar and statusbar), and last of all, the corresponding method
 *  in the core classes is invoked to perform the actual work. See the 
 *  detailed "Zap Brannagan's Big Book of Operations & Modes" for more about
 *  mode changes dependent on performed operations (located in your 
 *  manual/specs directory).<br>
 *
 *  There are several reasons for this modeling:<br>
 *  (1) We decided to use a "lightweight" GUI, which means there should be 
 *      little to no code related to actual application functionality in
 *      the GUI classes.<br>
 *  (2) With all the mode management in this one class, it is way easier
 *      to maintain a consistent mode at every time during runtime.
 *      This means especially that all core event handlers just have to 
 *      determine what mode the application is in to decide which method 
 *      call is the appropriate one for the current situation.<br>
 *  (3) We decided to separate distinctly the three parts of the application,
 *      that is the GUI (menu, toolbar, dialogs), the drawing area (the center
 *      panel of the mainframe with it's action listeners) and the core
 *      classes that provide the data structures and algorithms.
 *      So, messing around with modes in these three parts would have made
 *      the application much more complicated.<br>
 *  As a result, we have a natural hierarchical structure: The top GUI just
 *  handles user input and is responsible for mode integrity. All parts of
 *  the application can get information about the current mode via their
 *  ControlCenter reference, but are not allowed to change modes. The drawing
 *  area just determines what mode the application is in, extracts user
 *  input coordinates (i.e. mouse-clicks) and calls the appropriate methods
 *  of the core classes.
 *
 *
 *  @author Dominik Goeddeke
 *  @version completing
 */

public class MainActionListener implements ActionListener
{

    /** 
     * the MainFrame reference 
     */
    private MainFrame parent;

    /**
     *  reference to the ControlCenter
     */
    private ControlCenter cc;

    /** 
     *  the constructor just sets the references correctly 
     *
     *  @param parent - the MainFrame reference
     */
    public MainActionListener (MainFrame parent)
    {
	this.parent = parent;
	cc = ControlCenter.getMyself ();
    }

    /**
     *  restores the default mode and submodes
     *  a detailed description can be found in the spec mentioned.
     */
    private void restoreDefaultMode ()
    {
	// general note: setSelectedItem() in Comboboxes causes an 
	// ActionEvent, so I had to remove the actionlistener before 
	// updating the GUI. looks like some bug has become a feature... :-)

	// the add Type is set to Nothing
	cc.addType = cc.ADD_NOTHING;
	parent.getToolBar().getAddTypeCombo().removeActionListener (this);
	parent.getToolBar().getAddTypeCombo().setSelectedIndex (0);
	parent.getToolBar().getAddTypeCombo().addActionListener (this);
	parent.getToolBar().getPasteToggle().setSelected (false);
	parent.getMainMenu().getPasteItem().setSelected (false);
	parent.getMainMenu().getNewLineItem().setSelected (false);
	parent.getMainMenu().getNewMultiLineItem().setSelected (false);
	parent.getMainMenu().getNewCirclePItem().setSelected (false);
	parent.getMainMenu().getNewCircleMItem().setSelected (false);
	parent.getMainMenu().getNewTriItem().setSelected (false);
	parent.getMainMenu().getNewQuadItem().setSelected (false);
	parent.getMainMenu().getNewMacroItem().setSelected (false);
	parent.getMainMenu().getNewNodeItem().setSelected (false);
	// disable dialog-based modes
	parent.getMainMenu().getViewPropertiesItem().setSelected (false);
	parent.getToolBar().getPropertiesToggle().setSelected (false);
	parent.getMainMenu().getEdgeStatusItem().setSelected (false);
	parent.getToolBar().getEdgeStatusToggle().setSelected (false);
	parent.getMainMenu().getRectifyElementItem().setSelected (false);
	parent.getToolBar().getRectifyElementToggle().setSelected (false);
	parent.getMainMenu().getParallelBlockItem().setSelected (false);
	parent.getToolBar().getParallelblockToggle().setSelected (false);
	parent.getMainMenu().getRefineItem().setSelected (false);
	parent.getMainMenu ().getMirrorItem().setSelected (false);
	// add hide commands to all dialogs here...
	try
	{
	    cc.dm.hideAll ();
	}
	catch (NullPointerException ex) {}
	// disable Zooming
	cc.zoomType = cc.ZOOM_NONE;
	parent.getMainMenu().getSetZoomItem().setSelected (false);
	parent.getToolBar().getSetZoomToggle().setSelected (false);
	parent.getMainMenu().getZoomPlusItem().setSelected (false);
	parent.getToolBar ().getZoomInToggle().setSelected (false);
    }

    /**
     *  sets the application to default mode
     *  @see Zap Brannagan's Big Book of Operations & Modes
     */
    public void restoreStartUpMode ()
    {
	restoreDefaultMode ();
	// the select flags are all set to true
	cc.selectNodes = true;
	cc.selectElements = true;
	cc.selectSegments = true;
	cc.selectEdges = false;
	parent.getToolBar().getSelectNodeToggle().setSelected (true);
	parent.getToolBar().getSelectElementToggle().setSelected (true);
	parent.getToolBar().getSelectSegmentToggle().setSelected (true);
	parent.getMainMenu().getSelectNodesItem().setSelected (true);
	parent.getMainMenu().getSelectElementsItem().setSelected (true);
	parent.getMainMenu().getSelectSegmentsItem().setSelected (true);
	parent.getToolBar().getSelectEdgesToggle().setSelected (false);
	parent.getMainMenu().getSelectEdgesItem().setSelected (false);
	// clear copy mode
	parent.getMainMenu().getCopymodeItem().setSelected (cc.APPEND);
	parent.getToolBar().getCopymodeToggle().setSelected (cc.APPEND);
	cc.copymode = cc.APPEND;
	// switch to no mode
	cc.mode = cc.NO_MODE;
    }

    /**
     *  disables the mirror mode
     *  (called by the GridListener after a indirect mirroring was successful
     *   in order to keep the selection alive)
     */
    public void disableMirrorMode ()
    {
	cc.mode = cc.NO_MODE;
	parent.getMainMenu().getMirrorItem().setSelected (false);
    }



    /** the actionPerformed method is just a king-size switch-block
     *  all actions this method performs must be declared as String
     *  constants right in this class (see below)
     */
    public void actionPerformed(ActionEvent e) 
    {
	/* Action Command NEW */   
	if (e.getActionCommand ().equals(NEW))
	{
	    // ask for confirmation
	    int confirm = JOptionPane.NO_OPTION;
	    if (cc.d.hasChanged())
	    {
		confirm = JOptionPane.showConfirmDialog (parent,(String)cc.rb.getObject("general_message_unsaveddata"),(String)cc.rb.getObject ("general_message_unsaveddata_header"),JOptionPane.YES_NO_OPTION);
		if (confirm != JOptionPane.YES_OPTION)
		    return;
	    }
	    // if approved 
	    restoreStartUpMode ();
	    cc.d = new Domain ();
	    cc.gc.gtk.updateBoundaryCombos();
	    parent.getDrawingArea().repaint ();
	    // update visual feedback in the statusbar
	    parent.getStatusBar ().display (cc.getModeString());
	    return;
	}

	/* Action Command OPEN */   
	if (e.getActionCommand ().equals(OPEN))
	{

	    // ask for confirmation
	    int confirm = JOptionPane.NO_OPTION;
	    if (cc.d.hasChanged ())
	    {
		confirm = JOptionPane.showConfirmDialog (parent,(String)cc.rb.getObject("general_message_unsaveddata"),(String)cc.rb.getObject ("general_message_unsaveddata_header"),JOptionPane.YES_NO_OPTION);
		if (confirm != JOptionPane.YES_OPTION)
		    return;
	    }
	    // show a load dialog with supported formats INP, PRM, TRI, FEAST
	    FilePickerFilter defaultfilter = new FilePickerFilter (FilePickerFilter.FILTER_DEFAULT);
	    FilePickerFilter ucdfilter = new FilePickerFilter (FilePickerFilter.FILTER_UCD);
	    FilePickerFilter featfilter = new FilePickerFilter (FilePickerFilter.FILTER_FEAT);
	    FilePickerFilter feastfilter = new FilePickerFilter (FilePickerFilter.FILTER_FEAST);
	    FilePicker picker = new FilePicker (parent,(String)cc.rb.getObject ("dialogs_loaddialog_caption"), (String)cc.rb.getObject ("dialogs_loaddialog_button"), cc.op.get (Options.general_path), ucdfilter , true);
	    picker.addChoosableFileFilter (featfilter);
	    picker.addChoosableFileFilter (feastfilter);
	    picker.addChoosableFileFilter (defaultfilter);
	    int returnvalue = picker.showDialog ();
	    // if user accepted a file in the dialog...
	    if (returnvalue == JFileChooser.APPROVE_OPTION)
	    {
		//		JOptionPane.showMessageDialog (parent, "You selected: \n "+picker.getSelectedFile () + "","INFO", JOptionPane.INFORMATION_MESSAGE);
		// try to load and parse the file into the specified format
		File target = picker.getSelectedFile ();
		
                String format = picker.getExtension(target);
		
		if (target == null) 
		    return;
		cc.d = new Domain ();
		cc.d.loadDomain(picker.getSelectedFile()+"", format);
		parent.getDrawingArea().repaint ();
		cc.gc.workingBoundary = cc.d.db.getLastBoundary ();
		cc.d.db.updateNumbering ();
		// go to startup mode
		cc.gc.gtk.updateBoundaryCombos();
		restoreStartUpMode ();
		parent.getStatusBar ().display (cc.getModeString());
	    }
	    return;
	}

	/* Action Command SAVE */   
	if (e.getActionCommand ().equals(SAVE))
	{
	    // just overwrite the existing file
	    if (cc.d.getFileformat() != null && cc.d.getFilename() != null)
	    {
		cc.d.saveDomain (cc.d.getFilename(), cc.d.getFileformat(), false);
	    }
	    else
	    {
		JOptionPane.showMessageDialog (parent, "Try saveas first to give your domain a proper name and format","ERROR", JOptionPane.ERROR_MESSAGE);
	    }
	    return;
	}

	/* Action Command SAVEAS_UCD */   
	if (e.getActionCommand ().equals(SAVEAS_UCD))
	{
	    // determine current file type

	    // if other than UCD, show warning message

	    // show a saveas dialog with supported format INP
	    FilePickerFilter ucdfilter = new FilePickerFilter (FilePickerFilter.FILTER_UCD);
	    FilePicker picker = new FilePicker (parent,(String)cc.rb.getObject ("dialogs_saveucddialog_caption"), (String)cc.rb.getObject ("dialogs_savedialog_button"), cc.op.get (Options.general_path), ucdfilter, true);
	    int returnvalue = picker.showDialog ();
	    // if user accepted a file in the dialog...
	    if (returnvalue == JFileChooser.APPROVE_OPTION)
	    {
		// note that the filename returned already ends with .inp
		String filename = picker.getSelectedFile ().getAbsolutePath();
		// try to parse and save the file as UCD
		
		JOptionPane.showMessageDialog (parent, "You selected: \n "+ filename + "","INFO", JOptionPane.INFORMATION_MESSAGE);
	    }
	    return;
	}

	/* Action Command SAVEAS_FEAT */   
	if (e.getActionCommand ().equals(SAVEAS_FEAT))
	{
	    // show a saveas dialog with supported format PRM, TRI
	    FilePickerFilter featfilter = new FilePickerFilter (FilePickerFilter.FILTER_FEAT);
	    FilePicker picker = new FilePicker (parent,(String)cc.rb.getObject ("dialogs_savefeatdialog_caption"), (String)cc.rb.getObject ("dialogs_savedialog_button"), cc.op.get (Options.general_path), featfilter, true);
	    int returnvalue = picker.showDialog ();
	    // if user accepted a file in the dialog...
	    if (returnvalue == JFileChooser.APPROVE_OPTION)
	    {
	//    System.out.println("DDR "+picker.getPurePath ());
	    
	//	String filename = picker.getSelectedFile ().getAbsolutePath();
		String filename = picker.getPurePath();
		cc.d.saveDomain(filename, "tri",false);
	    }
	    return;
	}

	/* Action Command SAVEAS_FEAST */   
	if (e.getActionCommand ().equals(SAVEAS_FEAST))
	{
	    // show a saveas dialog with supported format FEAST
	    FilePicker picker = new FilePicker (parent,(String)cc.rb.getObject ("dialogs_savefeastdialog_caption"), (String)cc.rb.getObject ("dialogs_savedialog_button"), cc.op.get (Options.general_path), new FilePickerFilter (FilePickerFilter.FILTER_FEAST), true);
	    int returnvalue = picker.showDialog ();
	    // if user accepted a file in the dialog...
	    if (returnvalue == JFileChooser.APPROVE_OPTION)
	    {
		String filename = picker.getSelectedFile ().getAbsolutePath();
		cc.d.saveDomain(filename, "feast",false);
	    }
	    return;
	}
	/* Action Command MERGE */   
	if (e.getActionCommand ().equals(MERGE))
	{
	    // show a load dialog with supported format DEFAULT
	    FilePicker picker = new FilePicker (parent,(String)cc.rb.getObject ("dialogs_mergedialog_caption"), (String)cc.rb.getObject ("dialogs_mergedialog_button"), cc.op.get (Options.general_path), new FilePickerFilter (FilePickerFilter.FILTER_DEFAULT), true);
	    int returnvalue = picker.showDialog ();
	    // if user accepted a file in the dialog...
	    if (returnvalue == JFileChooser.APPROVE_OPTION)
	    {
		// determine file extension and therefore file type
		JOptionPane.showMessageDialog (parent, "You selected: \n "+picker.getSelectedFile () + "","INFO", JOptionPane.INFORMATION_MESSAGE);

		// display warning message if some features of the new file 
		// will be lost

		// perform the actual merging
		
		// display the new domain
	    }
	    return;
	}

	/* Action Command PRINT */   
	if (e.getActionCommand ().equals(PRINT))
	{
	    FilePickerFilter psfilter = new FilePickerFilter (FilePickerFilter.FILTER_PS);
	    FilePicker picker = new FilePicker (parent,(String)cc.rb.getObject ("dialogs_printdialog_caption"), (String)cc.rb.getObject ("dialogs_printdialog_button"), cc.op.get (Options.general_path), psfilter, true);
	    int returnvalue = picker.showDialog ();
	    // if user accepted a file in the dialog...
	    if (returnvalue == JFileChooser.APPROVE_OPTION)
	    {
		String filename = picker.getSelectedFile ().getAbsolutePath();
	        PSGraphics g = new PSGraphics(filename,cc.dc.used_screen_width, cc.dc.used_screen_height,false);
		cc.d.draw(g);
		g.close();
	    }
	    
	    
	    return;
	}

	if (e.getActionCommand ().equals(PRINTLS))
	{
	    FilePickerFilter psfilter = new FilePickerFilter (FilePickerFilter.FILTER_PS);
	    FilePicker picker = new FilePicker (parent,(String)cc.rb.getObject ("dialogs_printlsdialog_caption"), (String)cc.rb.getObject ("dialogs_printdialog_button"), cc.op.get (Options.general_path), psfilter, true);
	    int returnvalue = picker.showDialog ();
	    // if user accepted a file in the dialog...
	    if (returnvalue == JFileChooser.APPROVE_OPTION)
	    {
		String filename = picker.getSelectedFile ().getAbsolutePath();
	        PSGraphics g = new PSGraphics(filename,cc.dc.used_screen_width, cc.dc.used_screen_height,true);
		cc.d.draw(g);
		g.close();
	    }
	    
	    
	    return;
	}
	
	/* Action Command EXIT */   
	if (e.getActionCommand ().equals(EXIT))
	{
	    parent.shutDownApplication ();
	    return;
	}

	/* Action Command UNDO */   
	if (e.getActionCommand ().equals(UNDO))
	{
	    boolean painthotspots = cc.um.performUndo ();
	    parent.getDrawingArea().repaint ();
	    if (painthotspots)
	    {
		Runnable paintHotspots = new Runnable() 
		{
		    public void run() { parent.getDrawingArea().getListener().getSelector().updateHotspots(); }
		};
		SwingUtilities.invokeLater(paintHotspots);
	    }
	    parent.getStatusBar ().display (cc.getModeString());
	    return;
	}

	/* Action Command REDO */   
	if (e.getActionCommand ().equals(REDO))
	{
	    boolean painthotspots = cc.um.performRedo ();
	    parent.getDrawingArea().repaint ();
	    if (painthotspots)
	    {
		Runnable paintHotspots = new Runnable() 
		{
		    public void run() { parent.getDrawingArea().getListener().getSelector().updateHotspots(); }
		};
		SwingUtilities.invokeLater(paintHotspots);
	    }
	    parent.getStatusBar ().display (cc.getModeString());
	    return;
	}

	/* Action Command CUT */   
	if (e.getActionCommand ().equals(CUT))
	{  
	    // get selected items
	    Vector items = parent.getDrawingArea().getListener().getSelectedItems();
	    // HACK: deny CUT when BoundaryNodes are selected
	    Enumeration checkNodes = items.elements ();
	    boolean containsBNs = false;
	    while (checkNodes.hasMoreElements())
	    {
		if (checkNodes.nextElement() instanceof BoundaryNode)
		    containsBNs = true;
	    }
	    if (containsBNs)
	    {
		JOptionPane.showMessageDialog (parent, "CUT is not supported when BoundaryNodes are selected!","WARNING", JOptionPane.ERROR_MESSAGE);
		return;
	    }
	    if ((!items.isEmpty()) && (!containsBNs))
	    {
		// clone them
		GridCloner cloner = new GridCloner (items);
		parent.getDrawingArea().setCloner (cloner);
		cloner.createClone ();
		parent.getDrawingArea().getListener().storeCopyPasteAnchor ();
		// prepare undo
		Object[] info = new Object[2];
		info[0] = parent.getDrawingArea().getListener().getSelector().getHotspotMask();
		info[1] = cc.gc.workingBoundary;
		Vector implicitlydeleted = cc.gc.gtk.delete (items);
		Enumeration enum = implicitlydeleted.elements ();
		Vector tmpvec = new Vector ();
		while (enum.hasMoreElements ())
		    tmpvec.add (enum.nextElement ());
		enum = items.elements ();
		while (enum.hasMoreElements ())
		    tmpvec.add (enum.nextElement ());
		cc.um.notifyUndo (cc.um.UNDO_CUT, tmpvec, info);
		// refresh
		cc.d.db.updateNumbering ();
	    }
	    parent.getStatusBar ().display (cc.getModeString());
	    parent.getDrawingArea().repaint ();
	    return;
	}

	/* Action Command COPY */   
	if (e.getActionCommand ().equals(COPY))
	{
	    // if the current selection is not empty, call the copy operation
	    Vector items = parent.getDrawingArea().getListener ().getSelectedItems();
	    if (!items.isEmpty())
	    {
		GridCloner cloner = new GridCloner (items);
		parent.getDrawingArea().setCloner (cloner);
		cloner.createClone ();
		parent.getDrawingArea().getListener().storeCopyPasteAnchor ();
	    }
	    // done
	    parent.getStatusBar ().display (cc.getModeString());
	    return;
	}

	/* Action Command PASTE */   
	if (e.getActionCommand ().equals(PASTE))
	{
	    // update modes
	    if (cc.addType == cc.ADD_CLIPBOARD)
	    {
		// disable pasting
		restoreDefaultMode ();
		cc.mode = cc.NO_MODE;
		cc.addType = cc.ADD_NOTHING;
	    }
	    else
	    {
		// enable pasting
		restoreDefaultMode ();
		cc.mode = cc.DRAW_MODE;
		cc.addType = cc.ADD_CLIPBOARD;
		parent.getMainMenu().getPasteItem().setSelected (true);
		parent.getToolBar().getPasteToggle().setSelected (true);
		parent.getToolBar().getAddTypeCombo().removeActionListener (this);
		parent.getToolBar().getAddTypeCombo().setSelectedIndex (1);
		parent.getToolBar().getAddTypeCombo().addActionListener (this);
		if (cc.directMode)
		{
		    JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject("errors_directnotsupported"), (String)cc.rb.getObject ("errors_directerror"), JOptionPane.ERROR_MESSAGE);
		}
	    }
	    parent.getStatusBar ().display (cc.getModeString());    
	    return;
	}

	/** Action Command COPYMODE */
	if (e.getActionCommand ().equals (COPYMODE))
	{ 
	    if (cc.copymode==cc.KEEPSTRUCTURE)
	    {
		// disable copymode
		parent.getMainMenu().getCopymodeItem().setSelected (cc.APPEND);
		parent.getToolBar().getCopymodeToggle().setSelected (cc.APPEND);
		cc.copymode = cc.APPEND;
	    }
	    else
	    {
		// enable copymode
		parent.getMainMenu().getCopymodeItem().setSelected (cc.KEEPSTRUCTURE);
		parent.getToolBar().getCopymodeToggle().setSelected (cc.KEEPSTRUCTURE);
		cc.copymode = cc.KEEPSTRUCTURE;
	    }
	    parent.getStatusBar ().display (cc.getModeString());
	    return;
	}

	/* Action Command DELETE */
	if (e.getActionCommand ().equals(DELETE))
	{
	    // get selected items
	    Vector items = parent.getDrawingArea().getListener().getSelectedItems();
	    if (items.isEmpty())
	    {
		JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject("errors_emptyselection"), (String)cc.rb.getObject ("errors_selectionerror"), JOptionPane.ERROR_MESSAGE);
	    }
	    else
	    {
		Object[] info = new Object[2];
		info[0] = parent.getDrawingArea().getListener().getSelector().getHotspotMask();
		info[1] = cc.gc.workingBoundary;
		Vector implicitlydeleted = cc.gc.gtk.delete (items);
		Enumeration enum = implicitlydeleted.elements ();
		Vector tmpvec = new Vector ();
		while (enum.hasMoreElements ())
		    tmpvec.add (enum.nextElement ());
		enum = items.elements ();
		while (enum.hasMoreElements ())
		    tmpvec.add (enum.nextElement ());
		cc.um.notifyUndo (cc.um.UNDO_DELETE, tmpvec, info);
	    }
	    parent.getStatusBar ().display (cc.getModeString());
	    parent.getDrawingArea().repaint ();
	    return;
	}

	/* Action Command DELETEWB */
	if (e.getActionCommand ().equals(DELETEWB))
	{
	    // add working boundary to temp selection
	    Vector items = cc.gc.workingBoundary.getSegments();
	    Enumeration enum = items.elements();
	    //	    while (enum.hasMoreElements())
	    //	System.out.println (enum.nextElement());
	    if (items.isEmpty())
	    {
		JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject("errors_emptyselection"), (String)cc.rb.getObject ("errors_selectionerror"), JOptionPane.ERROR_MESSAGE);
	    }
	    else
	    {
		cc.gc.gtk.delete (items);
	    }
	    parent.getStatusBar ().display (cc.getModeString());
	    parent.getDrawingArea().repaint ();
	    return;
	}

	/* Action Command SCALE */   
	if (e.getActionCommand ().equals(SCALE))
	{
	    Vector items = parent.getDrawingArea().getListener().getSelectedItems();
	    if (!items.isEmpty())
	    {
		int hs = parent.getDrawingArea().getListener().getHotspot ();
		//ScaleDialog.setSources (parent.getDrawingArea().getListener().getSelector().getOppositeHotspotCoords (hs),hs);
		cc.dm.get (cc.dm.SCALEDIALOG).setVisible (true);
	    }
	    else
	    {
		JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject("errors_emptyselection"), (String)cc.rb.getObject ("errors_selectionerror"), JOptionPane.ERROR_MESSAGE);
	    }
	    parent.getStatusBar ().display (cc.getModeString());
	    return;
	}

	/* Action Command ROTATE */   
	if (e.getActionCommand ().equals(ROTATE))
	{ 
	    Vector items = parent.getDrawingArea().getListener().getSelectedItems();
	    if (!items.isEmpty())
	    {
		cc.dm.get (cc.dm.ROTATEDIALOG).setVisible (true);
	    }
	    else
	    {
		JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject("errors_emptyselection"), (String)cc.rb.getObject ("errors_selectionerror"), JOptionPane.ERROR_MESSAGE);
	    }
	    parent.getStatusBar ().display (cc.getModeString());
	    return;
	}

	/* Action Command MIRROR */   
	if (e.getActionCommand ().equals(MIRROR))
	{ 
	    if (cc.mode == cc.MIRROR_MODE)
	    {
		// disable 
		restoreDefaultMode ();
		cc.mode = cc.NO_MODE;
	    }
	    else
	    {
		// enable
		Vector items = parent.getDrawingArea().getListener().getSelectedItems();
		if (!items.isEmpty())
		{
		    restoreDefaultMode ();
		    cc.mode = cc.MIRROR_MODE;
		    parent.getMainMenu().getMirrorItem().setSelected (true);
		    // if in directMode, show the dialog
		    if (cc.directMode)
		    {
			cc.dm.get (cc.dm.MIRRORDIALOG).setVisible (true);
		    }
		}
		else
		{
		    JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject("errors_emptyselection"), (String)cc.rb.getObject ("errors_selectionerror"), JOptionPane.ERROR_MESSAGE);
		    parent.getMainMenu().getMirrorItem().setSelected (false);
		}
	    }
	    parent.getStatusBar ().display (cc.getModeString());
	    return;
	}

	/* Action Command SELECT BOUNDARY (MENU VERSION) */   
	if (e.getActionCommand ().equals(SELECTBOUNDARY))
	{
	    // a little dialog is shown
	    if (cc.d.db.getBoundaryCount () == 0)
	    {
		JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject("dialogs_changeb_error_nobounds"), (String)cc.rb.getObject ("errors_notfounderror"), JOptionPane.ERROR_MESSAGE);
	    }
	    else
	    {
		cc.dm.get (cc.dm.CHANGEBOUNDARYDIALOG).setVisible (true);
		cc.gc.gtk.updateBoundaryCombos ();
	    }
	    parent.getStatusBar ().display (cc.getModeString());
	    return;
	}

	/* Action Command SELECT BOUNDARY (TOOLBAR VERSION) */   
	if (e.getActionCommand ().equals(SELECTBOUNDARYCOMBO))
	{
	    // extract boundary from combobox
	    if (parent.getToolBar().getSelectBoundaryCombo().getSelectedItem()!= "-1")
	    {
		try
		{
		    cc.gc.workingBoundary = (Boundary)cc.d.db.getBoundary(Integer.parseInt (((String)parent.getToolBar().getSelectBoundaryCombo().getSelectedItem()))-1);
		}
		catch (NumberFormatException ex) {}
	    }
	    // update statusbar
	    parent.getStatusBar ().display (cc.getModeString());
	    return;
	}

	/* Action Command ZOOM_SETZOOM */   
	if (e.getActionCommand ().equals(ZOOM_SETZOOM))
	{
	    // update modes
	    if (cc.mode == cc.ZOOM_MODE)
	    {
		// disable zooming
		restoreDefaultMode ();
		cc.mode = cc.NO_MODE;
	    }
	    else
	    {
		// enable zooming with ZoomType SET
		restoreDefaultMode ();
		cc.mode = cc.ZOOM_MODE;
		cc.zoomType = cc.ZOOM_SET;
		parent.getMainMenu().getSetZoomItem().setSelected (true);
		parent.getToolBar().getSetZoomToggle().setSelected (true);
	    }
	    // thats it
	    parent.getStatusBar ().display (cc.getModeString());
	    return;
	}

	/* Action Command ZOOM_RESETZOOM */   
	if (e.getActionCommand ().equals(ZOOM_RESETZOOM))
	{
	    ((GridAdjustment)parent.getAdjustment()).setDefault();
	    int tol = Integer.parseInt(cc.op.get (Options.click_tolerance));
	    cc.dc.tolerance = cc.gc.SingleFromScreenToWorld(tol);
	    cc.gc.gtk.fixBoundingBoxes ();
	    restoreDefaultMode ();
	    cc.mode = cc.NO_MODE;
	    parent.getStatusBar ().display (cc.getModeString());
	    return;
	}

	/* Action Command ZOOM_ZOOMPLUS */   
	if (e.getActionCommand ().equals(ZOOM_ZOOMPLUS))
	{
	    // update modes
	    if (cc.mode == cc.ZOOM_MODE)
	    {
		// disable zooming
		restoreDefaultMode ();
		cc.mode = cc.NO_MODE;
		parent.getMainMenu().getZoomPlusItem().setSelected (false);
		parent.getToolBar().getZoomInToggle().setSelected (false);
	    }
	    else
	    {
		// enable zooming with ZoomType IN
		restoreDefaultMode ();
		cc.mode = cc.ZOOM_MODE;
		cc.zoomType = cc.ZOOM_IN;
		parent.getMainMenu().getZoomPlusItem().setSelected (true);
		parent.getToolBar().getZoomInToggle().setSelected (true);
	    }
	    parent.getStatusBar ().display (cc.getModeString());
	    return;
	}

	/* Action Command ZOOM_ZOOMMINUS */   
	if (e.getActionCommand ().equals(ZOOM_ZOOMMINUS))
	{
	    ((GridAdjustment)parent.getAdjustment()).preZoom();
	    int tol = Integer.parseInt(cc.op.get (Options.click_tolerance));
	    cc.dc.tolerance = cc.gc.SingleFromScreenToWorld(tol);
	    cc.gc.gtk.fixBoundingBoxes ();
	    parent.getStatusBar ().display (cc.getModeString());
	    return;
	}

	/* Action Command SELECTNODES */   
	if (e.getActionCommand ().equals(SELECTNODES))
	{
	    if (cc.selectNodes)
	    {
		// disable node mask
		parent.getMainMenu().getSelectNodesItem().setSelected (false);
		parent.getToolBar().getSelectNodeToggle().setSelected (false);
		cc.selectNodes = false;
	    }
	    else
	    {
		// enable node mask
		parent.getMainMenu().getSelectNodesItem().setSelected (true);
		parent.getToolBar().getSelectNodeToggle().setSelected (true);
		cc.selectNodes = true;
	    }
	    parent.getStatusBar ().display (cc.getModeString());
	    return;
	}

	/* Action Command SELECTELEMENTS */   
	if (e.getActionCommand ().equals(SELECTELEMENTS))
	{
	    if (cc.selectElements)
	    {
		// disable elements mask
		parent.getMainMenu().getSelectElementsItem().setSelected (false);
		parent.getToolBar().getSelectElementToggle().setSelected (false);
		cc.selectElements = false;
	    }
	    else
	    {
		// enable elements mask
		parent.getMainMenu().getSelectElementsItem().setSelected (true);
		parent.getToolBar().getSelectElementToggle().setSelected (true);
		cc.selectElements = true;
	    }
	    parent.getStatusBar ().display (cc.getModeString());
	    return;
	}

	/* Action Command SELECTSEGMENTS */   
	if (e.getActionCommand ().equals(SELECTSEGMENTS))
	{
	    if (cc.selectSegments)
	    {
		// disable segments mask
		parent.getMainMenu().getSelectSegmentsItem().setSelected (false);
		parent.getToolBar().getSelectSegmentToggle().setSelected (false);
		cc.selectSegments = false;
	    }
	    else
	    {
		// enable segments mask
		parent.getMainMenu().getSelectSegmentsItem().setSelected (true);
		parent.getToolBar().getSelectSegmentToggle().setSelected (true);
		cc.selectSegments = true;
	    }
	    parent.getStatusBar ().display (cc.getModeString());
	    return;
	}
	/* Action Command SELECTEDGES */   
	if (e.getActionCommand ().equals(SELECTEDGES))
	{
	    if (cc.selectEdges)
	    {
		// disable edges mask
		parent.getMainMenu().getSelectEdgesItem().setSelected (false);
		parent.getToolBar().getSelectEdgesToggle().setSelected (false);
		cc.selectEdges = false;
	    }
	    else
	    {
		// enable segments mask
		parent.getMainMenu().getSelectEdgesItem().setSelected (true);
		parent.getToolBar().getSelectEdgesToggle().setSelected (true);
		cc.selectEdges = true;
	    }
	    parent.getStatusBar ().display (cc.getModeString());
	    return;
	}
	/* Action Command PROPERTIES */   
	if (e.getActionCommand ().equals(PROPERTIES))
	{
	    if (cc.mode == cc.PROPERTY_MODE)
	    {
		// disable property mode
		restoreDefaultMode ();
		cc.mode = cc.NO_MODE;
	    }
	    else
	    {
		// enable property mode
		restoreDefaultMode ();
		cc.mode = cc.PROPERTY_MODE;
		parent.getMainMenu().getViewPropertiesItem().setSelected (true);
		parent.getToolBar().getPropertiesToggle().setSelected (true);
	    }
	    parent.getStatusBar ().display (cc.getModeString());
	    return;
	}


	/* Action Command NEWBOUNDARY */   
	if (e.getActionCommand ().equals(NEWBOUNDARY))
	{
	 
	    cc.d.db.addBoundary ();
	    cc.gc.workingBoundary = cc.d.db.getLastBoundary();
	    cc.gc.gtk.updateBoundaryCombos ();
	    parent.getStatusBar ().display (cc.getModeString());
	    return;
	}

	/* Action Command NEWANALYTIC */   
	if (e.getActionCommand ().equals(NEWANALYTIC))
	{
	    // show corresponding dialog
	    parent.getStatusBar ().display (cc.getModeString());
	    return;
	}

	/* Action Command DIRECTINPUT */   
	if (e.getActionCommand ().equals(DIRECTINPUT))
	{
	    if (!cc.directMode)
	    {
		// enable di
		parent.getMainMenu().getDirectInputItem().setSelected (true);
		parent.getToolBar().getDirectInputToggle().setSelected (true);
		cc.directMode = true;
	    }
	    else
	    {
		// disable di
		parent.getMainMenu().getDirectInputItem().setSelected (false);
		parent.getToolBar().getDirectInputToggle().setSelected (false);
		cc.directMode = false;
	    }
	    parent.getStatusBar ().display (cc.getModeString());
	    return;
	}

	/* Action Command NEWLINE */   
	if (e.getActionCommand ().equals(NEWLINE))
	{   
	    // update modes
	    if (cc.addType == cc.ADD_LINE && cc.mode == cc.DRAW_MODE)
	    {
		// disable 
		restoreDefaultMode ();
		cc.mode = cc.NO_MODE;
	    }
	    else
	    {
		// enable
		restoreDefaultMode ();
		cc.mode = cc.DRAW_MODE;
		cc.addType = cc.ADD_LINE;
		parent.getMainMenu().getNewLineItem().setSelected (true);
		parent.getToolBar().getAddTypeCombo().removeActionListener(this);
		parent.getToolBar().getAddTypeCombo().setSelectedIndex (2);
		parent.getToolBar().getAddTypeCombo().addActionListener (this);
		// if directmode, show the dialog
		if (cc.directMode)
	        {
		    LineDialog dialog = (LineDialog)cc.dm.get (cc.dm.LINEDIALOG);
		    cc.dm.hide (cc.dm.LINEDIALOG);
		    dialog.setTitle ((String)cc.rb.getObject("dialogs_line_caption_direct"));
		    dialog.setSource (null);
		    dialog.setVisible (true);
		}
	    }
	    parent.getStatusBar ().display (cc.getModeString());
	    return;
	}

	/* Action Command NEWMULTILINE */   
	if (e.getActionCommand ().equals(NEWMULTILINE))
	{
	    // update modes
	    if (cc.addType == cc.ADD_MULTILINE && cc.mode == cc.DRAW_MODE)
	    {
		// disable 
		restoreDefaultMode ();
		cc.mode = cc.NO_MODE;
	    }
	    else
	    {
		// enable
		restoreDefaultMode ();
		cc.mode = cc.DRAW_MODE;
		cc.addType = cc.ADD_MULTILINE;
		parent.getMainMenu().getNewMultiLineItem().setSelected (true);
		parent.getToolBar().getAddTypeCombo().removeActionListener(this);
		parent.getToolBar().getAddTypeCombo().setSelectedIndex (3);
		parent.getToolBar().getAddTypeCombo().addActionListener (this);
		if (cc.directMode)
		{
		    JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject("errors_directnotsupported"), (String)cc.rb.getObject ("errors_directerror"), JOptionPane.ERROR_MESSAGE);
		}
	    }
	    parent.getStatusBar ().display (cc.getModeString());
	    return;
	}

	/* Action Command NEWCIRCLEP */   
	if (e.getActionCommand ().equals(NEWCIRCLEP))
	{
	    // update modes
	    if (cc.addType == cc.ADD_CIRCLEP && cc.mode == cc.DRAW_MODE)
	    {
		// disable 
		restoreDefaultMode ();
		cc.mode = cc.NO_MODE;
	    }
	    else
	    {
		// enable
		restoreDefaultMode ();
		cc.mode = cc.DRAW_MODE;
		cc.addType = cc.ADD_CIRCLEP;
		parent.getMainMenu().getNewCirclePItem().setSelected (true);
		parent.getToolBar().getAddTypeCombo().removeActionListener(this);
		parent.getToolBar().getAddTypeCombo().setSelectedIndex (4);
		parent.getToolBar().getAddTypeCombo().addActionListener (this);
		// if directmode, show the dialog
		if (cc.directMode)
	        {
		    CircleDialog dialog = (CircleDialog)cc.dm.get (cc.dm.CIRCLEDIALOG);
		    cc.dm.hide (cc.dm.CIRCLEDIALOG);
		    dialog.setTitle ((String)cc.rb.getObject("dialogs_circle_caption_direct_p"));
		    dialog.setSource (null);
		    dialog.setOrientation (CircleDialog.POS_ORIENTATION);
		    dialog.setVisible (true);
		}
	    }
	    parent.getStatusBar ().display (cc.getModeString());
	    return;
	}

	/* Action Command NEWCIRCLEM */   
	if (e.getActionCommand ().equals(NEWCIRCLEM))
	{
	    // update modes
	    if (cc.addType == cc.ADD_CIRCLEM && cc.mode == cc.DRAW_MODE)
	    {
		// disable 
		restoreDefaultMode ();
		cc.mode = cc.NO_MODE;
	    }
	    else
	    {
		// enable
		restoreDefaultMode ();
		cc.mode = cc.DRAW_MODE;
		cc.addType = cc.ADD_CIRCLEM;
		parent.getMainMenu().getNewCircleMItem().setSelected (true);
		parent.getToolBar().getAddTypeCombo().removeActionListener(this);
		parent.getToolBar().getAddTypeCombo().setSelectedIndex (5);
		parent.getToolBar().getAddTypeCombo().addActionListener (this);
		// if directmode, show the dialog
		if (cc.directMode)
	        {
		    CircleDialog dialog = (CircleDialog)cc.dm.get (cc.dm.CIRCLEDIALOG);
		    cc.dm.hide (cc.dm.CIRCLEDIALOG);
		    dialog.setTitle ((String)cc.rb.getObject("dialogs_circle_caption_direct_m"));
		    dialog.setSource (null);
		    dialog.setOrientation (CircleDialog.NEG_ORIENTATION);
		    dialog.setVisible (true);
		}
	    }
	    parent.getStatusBar ().display (cc.getModeString());
	    return;
	}

	/* Action Command NEWTRI */   
	if (e.getActionCommand ().equals(NEWTRI))
	{
	    // update modes
	    if (cc.addType == cc.ADD_TRI && cc.mode == cc.DRAW_MODE)
	    {
		// disable 
		restoreDefaultMode ();
		cc.mode = cc.NO_MODE;
	    }
	    else
	    {
		// enable
		restoreDefaultMode ();
		cc.mode = cc.DRAW_MODE;
		cc.addType = cc.ADD_TRI;
		parent.getMainMenu().getNewTriItem().setSelected (true);
		parent.getToolBar().getAddTypeCombo().removeActionListener(this);
		parent.getToolBar().getAddTypeCombo().setSelectedIndex (6);
		parent.getToolBar().getAddTypeCombo().addActionListener (this);
		// if directmode show dialog
		if (cc.directMode)
		{
		    TriDialog dialog = (TriDialog)cc.dm.get (cc.dm.TRIDIALOG);
		    cc.dm.hide (cc.dm.TRIDIALOG);
		    dialog.setTitle ((String)cc.rb.getObject("dialogs_tri_caption_direct"));
		    dialog.setSource (null);
		    dialog.setNode1 (0);
		    dialog.setNode2 (0);
		    dialog.setNode3 (0);
		    dialog.setVisible (true);
		}

	    }
	    parent.getStatusBar ().display (cc.getModeString());
	    return;
	}

	/* Action Command NEWQUAD */   
	if (e.getActionCommand ().equals(NEWQUAD))
	{
	    // update modes
	    if (cc.addType == cc.ADD_QUAD && cc.mode == cc.DRAW_MODE)
	    {
		// disable 
		restoreDefaultMode ();
		cc.mode = cc.NO_MODE;
	    }
	    else
	    {
		// enable
		restoreDefaultMode ();
		cc.mode = cc.DRAW_MODE;
		cc.addType = cc.ADD_QUAD;
		parent.getMainMenu().getNewQuadItem().setSelected (true);
		parent.getToolBar().getAddTypeCombo().removeActionListener(this);
		parent.getToolBar().getAddTypeCombo().setSelectedIndex (7);
		parent.getToolBar().getAddTypeCombo().addActionListener (this);
		// if directmode show dialog
		if (cc.directMode)
		{
		    QuadDialog dialog = (QuadDialog)cc.dm.get (cc.dm.QUADDIALOG);
		    cc.dm.hide (cc.dm.QUADDIALOG);
		    dialog.setTitle ((String)cc.rb.getObject("dialogs_quad_caption_direct"));
		    dialog.setSource (null);
		    dialog.setNode1 (0);
		    dialog.setNode2 (0);
		    dialog.setNode3 (0);
		    dialog.setNode4 (0);
		    dialog.setVisible (true);
		}
	    }
	    parent.getStatusBar ().display (cc.getModeString());
	    return;
	}

	/* Action Command NEWMACRO */   
	if (e.getActionCommand ().equals(NEWMACRO))
	{
	    // update modes
	    if (cc.addType == cc.ADD_MACRO && cc.mode == cc.DRAW_MODE)
	    {
		// disable 
		restoreDefaultMode ();
		cc.mode = cc.NO_MODE;
	    }
	    else
	    {
		// enable
		restoreDefaultMode ();
		cc.mode = cc.DRAW_MODE;
		cc.addType = cc.ADD_MACRO;
		parent.getMainMenu().getNewMacroItem().setSelected (true);
		parent.getToolBar().getAddTypeCombo().removeActionListener(this);
		parent.getToolBar().getAddTypeCombo().setSelectedIndex (8);
		parent.getToolBar().getAddTypeCombo().addActionListener (this);
		// if directmode show dialog
		if (cc.directMode)
		{
		    MacroDialog dialog = (MacroDialog)cc.dm.get (cc.dm.MACRODIALOG);
		    cc.dm.hide (cc.dm.MACRODIALOG);
		    dialog.setTitle ((String)cc.rb.getObject("dialogs_macro_caption_direct"));
		    dialog.setSource (null);
		    dialog.setNode1 (0);
		    dialog.setNode2 (0);
		    dialog.setNode3 (0);
		    dialog.setNode4 (0);
		    dialog.setVisible (true);
		}
	    }
	    parent.getStatusBar ().display (cc.getModeString());
	    return;
	}
  
	/* Action Command NEWNODE */   
	if (e.getActionCommand ().equals(NEWNODE))
	{
	    // update modes
	    if (cc.addType == cc.ADD_NODE && cc.mode == cc.DRAW_MODE)
	    {
		// disable 
		restoreDefaultMode ();
		cc.mode = cc.NO_MODE;
	    }
	    else
	    {
		// enable
		restoreDefaultMode ();
		cc.mode = cc.DRAW_MODE;
		cc.addType = cc.ADD_NODE;
		parent.getMainMenu().getNewNodeItem().setSelected (true);
		parent.getToolBar().getAddTypeCombo().removeActionListener(this);
		parent.getToolBar().getAddTypeCombo().setSelectedIndex (9);
		parent.getToolBar().getAddTypeCombo().addActionListener (this);
		// if in directMode, show the dialog
		if (cc.directMode)
		{
		    BoundaryNodeDialog dialog = (BoundaryNodeDialog)cc.dm.get (cc.dm.BOUNDARYNODEDIALOG);
		    cc.dm.hide (cc.dm.BOUNDARYNODEDIALOG);
		    dialog.setTitle ((String)cc.rb.getObject("dialogs_boundarynode_caption_direct"));
		    dialog.setSource (null);
		    dialog.setVisible (true);
		}
	    }
	    parent.getStatusBar ().display (cc.getModeString());
	    return;
	}

	/* Action Command ADJUSTBOUNDARIES */   
	if (e.getActionCommand ().equals(ADJUSTBOUNDARIES))
	{
	    cc.d.db.continousBoundary();
	    parent.getStatusBar ().display (cc.getModeString());
	    return;
	}
 
	/* Action Command LINKBOUNDARIES */   
	if (e.getActionCommand ().equals(LINKBOUNDARIES))
	{
	    // place method call here
	    parent.getStatusBar ().display (cc.getModeString());
	    return;
	}

	/* Action Command FROMTRI2PRM */   
	if (e.getActionCommand ().equals(FROMTRI2PRM))
	{
	    // place method call here
            cc.d.construct_boundary();
	    parent.getDrawingArea().repaint ();
	    cc.gc.workingBoundary = cc.d.db.getLastBoundary ();
	    cc.gc.gtk.updateBoundaryCombos ();
	    return;
	}
	
	/* Action Command SWAPBOUNDARYORIENTATION */   
	if (e.getActionCommand ().equals(SWAPBOUNDARYORIENTATION))
	{
	    //            System.out.println("Starting swap orientation");
	    cc.d.db.swapOrientation (cc.gc.workingBoundary);
	    parent.getStatusBar ().display (cc.getModeString());
	    return;
	}
	/* Action Command DOMAINPERIMETER */   
	if (e.getActionCommand ().equals(DOMAINPERIMETER))
	{
	    PerimeterDialog dialog = (PerimeterDialog)cc.dm.get (cc.dm.PERIMETERDIALOG);
	    dialog.setVisible (true);
	    return;
	}
  
	/* Action Command DOMAINSTATISTICS */   
	if (e.getActionCommand ().equals(DOMAINSTATISTICS))
	{
	    TextFrame.initialize (parent, "DomainStatistics", "waiting for the sun...");
	    TextFrame.setText (cc.d.getStatistics ());
	    TextFrame.showDialog ();
	    return;
	}

	/* Action Command MULTIPLEBOUNDARYNODES */   
	if (e.getActionCommand ().equals(MULTIPLEBOUNDARYNODES))
	{
	    if (cc.gc.workingBoundary == null)
	    {
		JOptionPane.showMessageDialog (parent, (String)cc.rb.getObject("gridlistener_error_nosegments"), (String)cc.rb.getObject ("errors_gridinputerror"), JOptionPane.ERROR_MESSAGE);
	    }
	    else
	    {
		MultipleBNDialog dialog = (MultipleBNDialog)cc.dm.get (cc.dm.MULTIPLEBNDIALOG);
		dialog.setVisible (true);
	    }
	    parent.getStatusBar ().display (cc.getModeString());
	    return;
	}

	/* Action Command EDGESTATUS */   
	if (e.getActionCommand ().equals(EDGESTATUS))
	{
	    // update modes
	    if (cc.mode == cc.EDGESTATUS_MODE)
	    {
		// disable 
		restoreDefaultMode ();
		cc.mode = cc.NO_MODE;
	    }
	    else
	    {
		// enable
		restoreDefaultMode ();
		cc.mode = cc.EDGESTATUS_MODE;
		parent.getMainMenu().getEdgeStatusItem().setSelected (true);
		parent.getToolBar().getEdgeStatusToggle().setSelected (true);
		EdgeStatusDialog dialog = (EdgeStatusDialog)cc.dm.get (cc.dm.EDGESTATUSDIALOG);
		dialog.setVisible (true);
	    }
	    parent.getStatusBar ().display (cc.getModeString());
	    return;
	}

	/* Action Command PARALLELBLOCK */   
	if (e.getActionCommand ().equals(PARALLELBLOCK))
	{
	    // update modes
	    if (cc.mode == cc.PARALLELBLOCK_MODE)
	    {
		// disable 
		restoreDefaultMode ();
		cc.mode = cc.NO_MODE;
	    }
	    else
	    {
		// enable
		restoreDefaultMode ();
		cc.mode = cc.PARALLELBLOCK_MODE;
		parent.getMainMenu().getParallelBlockItem().setSelected (true);
		parent.getToolBar().getParallelblockToggle().setSelected (true);
		ParallelBlockDialog dialog = (ParallelBlockDialog)cc.dm.get (cc.dm.PARALLELBLOCKDIALOG);
		dialog.setVisible (true);
	    }
	    parent.getStatusBar ().display (cc.getModeString());
	    return; 
	}

	/* Action Command REFINE */   
	if (e.getActionCommand ().equals(REFINE))
	{
	    // update modes
	    if (cc.mode == cc.REFINE_MODE)
	    {
		// disable 
		restoreDefaultMode ();
		cc.mode = cc.NO_MODE;
	    }
	    else
	    {
		// enable
		restoreDefaultMode ();
		cc.mode = cc.REFINE_MODE;
		parent.getMainMenu().getRefineItem().setSelected (true);
		RefineDialog dialog = (RefineDialog)cc.dm.get (cc.dm.REFINEDIALOG);
		dialog.setVisible (true);
	    }
	    parent.getStatusBar ().display (cc.getModeString());
	    return; 
	}
	
	/* Action Command SPELLCHECKER */   
	if (e.getActionCommand ().equals(SPELLCHECKER))
	{
	    SpellCheckerDialog dialog = (SpellCheckerDialog)cc.dm.get (cc.dm.SPELLCHECKERDIALOG);
	    dialog.setVisible (true);
	    parent.getStatusBar ().display (cc.getModeString());
	}

	/* Action Command MERGEBOUNDARIES */   
	if (e.getActionCommand ().equals(MERGEBOUNDARIES))
	{
	    MergeBoundariesDialog dialog = (MergeBoundariesDialog)cc.dm.get (cc.dm.MERGEBOUNDARIESDIALOG);
	    dialog.setVisible (true);
	    parent.getStatusBar ().display (cc.getModeString());
	}

	   

	/* Action Command LOADALANCING */   
	if (e.getActionCommand ().equals(LOADBALANCING))
	{
	    LoadBalancingDialog dialog = (LoadBalancingDialog)cc.dm.get (cc.dm.LOADBALANCINGDIALOG);
	    dialog.setVisible (true);
	    parent.getStatusBar ().display (cc.getModeString());
	    return;
	}
	/* Action Command RECTIFYELEMENT */   
	if (e.getActionCommand ().equals(RECTIFYELEMENT))
	{
	    // update modes
	    if (cc.mode == cc.RECTIFYELEMENT_MODE)
	    {
		// disable 
		restoreDefaultMode ();
		cc.mode = cc.NO_MODE;
	    }
	    else
	    {
		// enable
		restoreDefaultMode ();
		cc.mode = cc.RECTIFYELEMENT_MODE;
		parent.getMainMenu().getRectifyElementItem().setSelected (true);
		parent.getToolBar().getRectifyElementToggle().setSelected (true);
	    }
	    parent.getStatusBar ().display (cc.getModeString());
	    return;
	}
	/* Action Command GENERALOPTIONS */   
	if (e.getActionCommand ().equals(GENERALOPTIONS))
	{
	    OptionsDialog.showGeneral ();
	    return;
	}
	/* Action Command DRAWOPTIONS */   
	if (e.getActionCommand ().equals(DRAWOPTIONS))
	{
	    OptionsDialog.showDraw ();
	    return;
	}
	/* Action Command COLOROPTIONS */   
	if (e.getActionCommand ().equals(COLOROPTIONS))
	{
	    OptionsDialog.showColor ();
	    return;
	}
	/* Action Command ACCELERATOROPTIONS */   
	if (e.getActionCommand ().equals(ACCELERATOROPTIONS))
	{
	    OptionsDialog.showAccelerator ();
	    return;
	}
	/* Action Command MISCOPTIONS */   
	if (e.getActionCommand ().equals(MISCOPTIONS))
	{
	    OptionsDialog.showMisc ();
	    return;
	}
	/* Action Command MANUAL */   
	if (e.getActionCommand ().equals(MANUAL))
	{
	    cc.help.show ();
	    parent.getStatusBar ().display (cc.getModeString());
	    return;
	}
	/* Action Command FAQ */   
	if (e.getActionCommand ().equals(FAQ))
	{
	    cc.faq.show ();
	    parent.getStatusBar ().display (cc.getModeString());
	    return;
	}
	/* Action Command README */   
	if (e.getActionCommand ().equals(README))
	{
	    ReadmeFrame.showDialog();
	    parent.getStatusBar ().display (cc.getModeString());
	    return;
	}
	/* Action Command ABOUT */   
	if (e.getActionCommand ().equals(ABOUT))
	{ 
	    InfoDialog.showDialog ();
	    parent.getStatusBar ().display (cc.getModeString());
	    return;
	}
	/* Action Command ADDTYPECHANGED */   
	if (e.getActionCommand ().equals(ADDTYPECHANGED))
	{ 
	    // determine the selection and call this method again with the
	    // according action command (as if the menu had been clicked)
	    switch (parent.getToolBar().getAddTypeCombo().getSelectedIndex())
	    {
	    case 0 : // switch to no mode
		{
		    restoreDefaultMode ();
		    cc.mode = cc.NO_MODE;
		    parent.getStatusBar ().display (cc.getModeString());
		    return;
		}
	    case 1 :
		{
		    actionPerformed (new ActionEvent (e.getSource(),e.getID(),PASTE,e.getModifiers())); 
		}
	    case 2 : 
		{
		    actionPerformed (new ActionEvent (e.getSource(),e.getID(),NEWLINE,e.getModifiers())); 
		    return;
		}
	    case 3 : 
		{
		    actionPerformed (new ActionEvent (e.getSource(),e.getID(),NEWMULTILINE,e.getModifiers())); 
		    return;
		}
	    case 4 : 
		{
		    actionPerformed (new ActionEvent (e.getSource(),e.getID(),NEWCIRCLEP,e.getModifiers())); 
		    return;
		}
	    case 5 : 
		{
		    actionPerformed (new ActionEvent (e.getSource(),e.getID(),NEWCIRCLEM,e.getModifiers())); 
		    return;
		}
	    case 6 : 
		{
		    actionPerformed (new ActionEvent (e.getSource(),e.getID(),NEWTRI,e.getModifiers())); 
		    return;
		}
	    case 7 : 
		{
		    actionPerformed (new ActionEvent (e.getSource(),e.getID(),NEWQUAD,e.getModifiers())); 
		    return;
		}
	    case 8 : 
		{
		    actionPerformed (new ActionEvent (e.getSource(),e.getID(),NEWMACRO,e.getModifiers())); 
		    return;
		}
	    case 9 : 
		{
		    actionPerformed (new ActionEvent (e.getSource(),e.getID(),NEWNODE,e.getModifiers())); 
		    return;
		}
	    }
	    return;
	}
	    

    }

    // file menu
    public static final String NEW = "NEW";
    public static final String OPEN = "OPEN";
    public static final String SAVE = "SAVE";
    public static final String SAVEAS_UCD = "SAVEAS_UCD";
    public static final String SAVEAS_FEAT = "SAVEAS_FEAT";
    public static final String SAVEAS_FEAST = "SAVEAS_FEAST";
    public static final String MERGE = "MERGE";
    public static final String PRINT = "PRINT";
    public static final String PRINTLS = "PRINTLS";
    public static final String EXIT = "EXIT";
    //edit menu
    public static final String UNDO = "UNDO";
    public static final String REDO = "REDO";
    public static final String CUT = "CUT";
    public static final String COPY = "COPY";
    public static final String PASTE = "PASTE";
    public static final String COPYMODE = "COPYMODE";
    public static final String DELETE = "DELETE";
    public static final String DELETEWB = "DELETEWB";
    public static final String SCALE = "SCALE";
    public static final String ROTATE = "ROTATE";
    public static final String MIRROR = "MIRROR";
    public static final String SELECTBOUNDARY = "SELECTBOUNDARY";

    // view menu
    public static final String ZOOM_SETZOOM = "ZOOM_SETZOOM";
    public static final String ZOOM_RESETZOOM = "ZOOM_RESETZOOM";
    public static final String ZOOM_ZOOMPLUS = "ZOOM_ZOOMPLUS";
    public static final String ZOOM_ZOOMMINUS = "ZOOM_ZOOMMINUS";
    public static final String SELECTNODES = "SELECTNODES";
    public static final String SELECTELEMENTS = "SELECTELEMENTS";
    public static final String SELECTSEGMENTS = "SELECTSEGMENTS";
    public static final String SELECTEDGES = "SELECTEDGES";
    public static final String PROPERTIES = "PROPERTIES";
    // domain menu
    public static final String NEWBOUNDARY = "NEWBOUNDARY";
    public static final String NEWANALYTIC = "NEWANALYTIC";
    public static final String DIRECTINPUT = "DIRECTINPUT";
    public static final String NEWLINE = "NEWLINE";
    public static final String NEWMULTILINE = "NEWMULTILINE";
    public static final String NEWCIRCLEP = "NEWCIRCLEP";
    public static final String NEWCIRCLEM = "NEWCIRCLEM";
    public static final String NEWTRI = "NEWTRI";
    public static final String NEWQUAD = "NEWQUAD";
    public static final String NEWMACRO = "NEWMACRO";
    public static final String NEWNODE = "NEWNODE";
    public static final String ADJUSTBOUNDARIES = "ADJUSTBOUNDARIES";
    public static final String LINKBOUNDARIES = "LINKBOUNDARIES";
    public static final String FROMTRI2PRM = "FROMTRI2PRM";
    public static final String SWAPBOUNDARYORIENTATION = "SWAPBOUNDARYORIENTATION";
    public static final String DOMAINPERIMETER = "DOMAINPERIMETER";
    public static final String DOMAINSTATISTICS = "DOMAINSTATISTICS";
    // grid menu
    public static final String MULTIPLEBOUNDARYNODES = "MULTIPLEBOUNDARYNODES";
    public static final String EDGESTATUS = "EDGESTATUS";
    public static final String PARALLELBLOCK = "PARALLELBLOCK";
    public static final String LOADBALANCING = "LOADBALANCING";
    public static final String RECTIFYELEMENT = "RECTIFYELEMENT";
    public static final String REFINE = "REFINE";
    public static final String SPELLCHECKER = "SPELLCHECKER";
    public static final String MERGEBOUNDARIES = "MERGEBOUNDARIES";
    // options menu
    public static final String GENERALOPTIONS = "GENERALOPTIONS";
    public static final String DRAWOPTIONS = "DRAWOPTIONS";
    public static final String COLOROPTIONS = "COLOROPTIONS";
    public static final String ACCELERATOROPTIONS = "ACCELERATOROPTIONS";
    public static final String MISCOPTIONS = "MISCOPTIONS";
    
    // help menu
    public static final String MANUAL = "MANUAL";
    public static final String FAQ = "FAQ";
    public static final String README = "README";
    public static final String ABOUT = "ABOUT";
    // some special features available via toolbar only
    public static final String SELECTBOUNDARYCOMBO = "SELECTBOUNDARYCOMBO";
    public static final String ADDTYPECHANGED = "ADDTYPECHANGED";

}

