package devisor2.grid.GUI.framework;


import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.awt.geom.*;

import devisor2.grid.options.*;
import devisor2.grid.GUI.dialogs.*;
import devisor2.foundation.elements.*;
import devisor2.grid.GUI.event.*;
import devisor2.grid.backend.*;
import devisor2.grid.backend.spellchecker.*;



/** 
 *  This is the main window of the DeViSoRGrid2 application. It hosts the
 *  menu and toolbar, the statusbar, and of course the central drawing area.
 *  All these can be accessed from outside via the corresponding get methods.
 *  <br>
 *  Except serving as a container for all these classes, the MainFrame class
 *  takes care of repositioning the dockable dialogs whenever the MainFrame
 *  is moved or resized. This is done only in the two event handlers for the 
 *  corresponding Component Events, and in the <code>shutDownApplication() <\code>
 *  method, which also takes care to store the options so that the user can 
 *  continue to work right where he was when the application is restarted.
 *  <br>
 *  Last but not least, every resizing must be followed by a recalculation
 *  of the transformations to be applied to the elements, so this is handled
 *  here as well. <br>
 *  Some remarks on layouting: The basic layout is of course BorderLayout,
 *  which means that menu, toolbar and statusbar are given as much space as 
 *  they require, leaving the large center for the drawing area. The
 *  MainFrame always appears centered on the screen, and to allow for various
 *  screen sizes, its initial size is about 80% of the whole screen. 
 *  Anyhow, we strongly recommend using a resolution of at least 1024 by 768
 *  pixels. <br>
 *  Due to some bug-or-feature, this class is also the keylistener responsible
 *  for catching CTRL- and SHIFT-pressed events.

 *
 *  @author Dominik Goeddeke
 *  @version release candidate
 *  @see all other classes of the application :-)
 */

public class MainFrame extends JFrame //implements KeyListener
{

    /** 
     *  reference to the menu 
     */
    private MainMenu menu;

    /**
     *  the statusbar is encapsulated into a class of its own 
     */
    private StatusBar statusbar;

    /**
     *  the toolbar is encapsulated into a class of its own 
     */
    private ToolBar toolbar;

    /** 
     *  the container for the drawing area
     */
    private JScrollPane centerfold;
    /**
     *  the drawing area itself
     */
    private DrawingArea drawingArea;

    /**
     *  the former origin (before moving the frame)
     */
    private Point lastOrigin = new Point (0,0);

    /**
     *  The ActionListener for both ToolBar and Menu
     */
    private MainActionListener mainActionListener;

    /** reference to the ControlCenter */
    private ControlCenter cc = ControlCenter.getMyself ();    


    /** reference to scrollbar right */
    private JScrollBar balkenrechts;

    /** reference to scrollbar left */
    private JScrollBar balkenunten;    

    /** reference to adjustment listener */    
    private GridAdjustment gal;
    

    /** 
     *  This constructor just creates a new MainFrame object and adds
     *  subsequently the toolbar, statusbar, main menu and centerfold area.
     *  it also adds a listener to make sure the application can't be killed
     *  by just closing the main window. The component listeners responsible
     *  for induced movements of the dockable dialogs are also created.
     *  
     *  The positioning works as follows: If a value has been stored in the 
     *  Options instance during last shutdown, it is used to display the
     *  MainFrame at its recent position, if not, the default values are used.
     */
    public MainFrame ()
    {
	super();
	// connect to locale and options
	cc = ControlCenter.getMyself ();
	
	// set title
	setTitle (cc.wS("GUI_framework_Mainframe_frametitle"));
	// add layout manager
	getContentPane().setLayout (new BorderLayout ());
	setDefaultCloseOperation (DO_NOTHING_ON_CLOSE);
	// init action listener
	mainActionListener = new MainActionListener (this);
	// add menu 
	menu = new MainMenu (this);
	setJMenuBar (menu);
	// add toolbar
	toolbar = new ToolBar (this);
	getContentPane().add (toolbar, BorderLayout.NORTH);
	// add statusbar
	JPanel south = new JPanel ();
	south.setLayout (new BoxLayout (south, BoxLayout.Y_AXIS));
	statusbar = new StatusBar ();
	statusbar.display ("DeViSoRGrid 2");
	// finally the main display area
	drawingArea = new DrawingArea(this);
	getContentPane().add (drawingArea, BorderLayout.CENTER);
	
	balkenrechts=new JScrollBar(JScrollBar.VERTICAL);
	balkenunten=new JScrollBar(JScrollBar.HORIZONTAL);
	
	gal = new GridAdjustment(this,drawingArea);
	
	balkenrechts.addAdjustmentListener(gal);
	balkenunten.addAdjustmentListener(gal);
	
	gal.setSliderDefault(balkenrechts);
	gal.setSliderDefault(balkenunten);
	
	getContentPane().add (balkenrechts, BorderLayout.EAST);
	south.add (balkenunten);
	south.add (statusbar);
	getContentPane().add (south, BorderLayout.SOUTH);
	
	//addKeyListener (this);
	addKeyListener (new KeyAdapter ()
	    {
		/**
		 *  the key-typed method is not needed
		 */
		public void keyTyped (KeyEvent e)
		{
		    System.out.println ("key typed");
		}

		/**
		 *  This method takes care of letting disappear some hotspots
		 *  whenever the SHIFT key is pressed.
		 */
		public void keyPressed (KeyEvent e)
		{
		    System.out.println ("key pressed");
		    drawingArea.getListener().setShiftStarted (false);
		    if (e.getKeyCode() == KeyEvent.VK_SHIFT)
			{ 
			    System.out.println ("CTRL pressed");
			    if (!drawingArea.getListener().getSelectedItems().isEmpty ())
				if (drawingArea.getListener().getInProgress() == GridListener.PROGRESS_NOTHING)
				    {
					System.out.println ("inside");
					drawingArea.getListener().getSelector().updateHotspots ();
					drawingArea.getListener().setSaveHotspotMask(drawingArea.getListener().getSelector().getHotspotMask ());
					drawingArea.getListener().getSelector().setHotspotMask (GridSelector.MASK_ROTATE);
					drawingArea.getListener().getSelector().updateHotspots ();
					drawingArea.getListener().setShiftStarted (true);
				    }
			}
		}
		
		/**
		 *  This method takes care of letting reappear some hotspots
		 *  whenever the SHIFT key is released.
		 */
		public void keyReleased (KeyEvent e)
		{
		    System.out.println ("key released");
		    if (e.getKeyCode() == KeyEvent.VK_SHIFT)
			{
			    if (!drawingArea.getListener().getSelectedItems().isEmpty ())
				{
				    if (drawingArea.getListener().getInProgress() == GridListener.PROGRESS_NOTHING)
					if (drawingArea.getListener().getShiftStarted())
					    {
						drawingArea.getListener().getSelector().updateHotspots ();
						drawingArea.getListener().getSelector().setHotspotMask (drawingArea.getListener().getSaveHotspotMask());
						drawingArea.getListener().getSelector().updateHotspots ();
					    }
				}
			}
		}
	    });
	// the listeners
	// add window listener (an anonymous class suffices)
	addWindowListener( new WindowAdapter () 
	    {
		public void windowClosing(WindowEvent e) 
		{
		    shutDownApplication ();
		}
	    });
	// add component listener responsible for trailing dialogs
	addComponentListener (new ComponentAdapter ()
	    {
		public void componentMoved (ComponentEvent e)
		{
		    final Point newPos = getLocationOnScreen ();
		    final Point lastOrig = lastOrigin;
		    if (isVisible())
		    {
			Runnable updateDialogs = new Runnable ()
			{
			    public void run ()
			    {
				//Point newPos = getLocationOnScreen ();
				int dx = lastOrig.x - newPos.x;
				int dy = lastOrig.y - newPos.y;
				try
				{
				    for (int i=0; i<DialogManager.DIALOGCOUNT;i++)
				    {
					DockableDialog dialog = cc.dm.get (i);
					Point lastDiag = dialog.getPositionOnScreen ();
					Point newDiag = new Point (lastDiag.x - dx, lastDiag.y - dy);
					dialog.removeMoveListener ();
					dialog.setPositionOnScreen (newDiag);
					dialog.addMoveListener ();
				    }
				}
				catch (NullPointerException ex) {}
			    }};
			SwingUtilities.invokeLater (updateDialogs);
			// update the position variables
			lastOrigin = newPos;
		    }
		}
			
		public void componentResized (ComponentEvent e)
		{
		    int newwidth = getDrawingArea().getWidth();
		    int newheight = getDrawingArea().getHeight();
                    cc.dc.screen_width = newwidth;
                    cc.dc.screen_height = newheight;

                    cc.gc.setTransformation();


                    if ((0!=cc.dc.screen_width) && (0!=cc.dc.screen_height) && (cc.gc.mf!=null))
                    {		    
		      newwidth=cc.gc.SingleFromScreenToWorld(newwidth);
		      newheight=cc.gc.SingleFromScreenToWorld(newheight);

		      cc.dc.act_x1=cc.dc.act_x1;
		      cc.dc.act_x2=cc.dc.act_x1+newwidth;
		      cc.dc.act_y1=cc.dc.act_y2-newheight;
		      cc.dc.act_y2=cc.dc.act_y2;
		    
		      getAdjustment().adjustZoom();
		      cc.d.ecke1=null;
		      
		      	    cc.gc.mf.getDrawingArea().repaint ();

		    }
		    
		    if (isVisible ())
		    {
			// default dock Margin for all dialogs is 10 %
			final int margin = (int)(getWidth()/10);
			Runnable updateDialogs = new Runnable ()
			{
			    public void run ()
			    {
				try
				{
				    for (int i=0; i<DialogManager.DIALOGCOUNT;i++)
				    {
					DockableDialog dialog = cc.dm.get (i);
					dialog.setDockMargin (margin);
					// depending on the dock status of each
					// dialog, translate them if neccessary
					if (dialog.isDocked())
					    dialog.rePosition ();
				    }
				}
				catch (NullPointerException ex) {}
			    }};
			SwingUtilities.invokeLater (updateDialogs);
			// update position flags
			lastOrigin = getLocationOnScreen ();
			// update DrawControl
			//			int tol = Integer.parseInt(cc.op.get (Options.click_tolerance));
			//			cc.dc.tolerance = cc.gc.SingleFromScreenToWorld(tol);
			cc.gc.gtk.fixBoundingBoxes ();
		    }
		}
	    });

	// leave the house...
	pack();
	// size and positioning
	Dimension screen = getToolkit().getScreenSize();
	//  try
//  	{
//  	    setSize (Integer.parseInt (cc.op.get (Options.mainframe_width)), Integer.parseInt (cc.op.get (Options.mainframe_height)));
//  	    setLocation (Integer.parseInt (cc.op.get (Options.mainframe_x)), Integer.parseInt (cc.op.get (Options.mainframe_x)));
//  	}
//  	catch (NumberFormatException ex)
//  	{
//  	    // use default size and position
//  	    setSize ((int)(screen.getSize().width * 0.8), (int)(screen.getSize().height * 0.8));
//  	    setLocation((screen.getSize().width - getSize().width) / 2,(screen.getSize().height - getSize().height) / 2);
//  	}
    }

    /**
     *  returns the current toolbar object of the frame  
     */
    public ToolBar getToolBar ()
    {
	return toolbar;
    }
    
    /** 
     *  returns the current statusbar object of the frame  
     */
    public StatusBar getStatusBar ()
    {
	return statusbar;
    }

    /** 
     *  returns the current MainMenu object of the frame 
     */
    public MainMenu getMainMenu ()
    {
	return menu;
    }

    /** 
     *  returns the current MainActionListener object of the frame 
     */
    public MainActionListener getMainActionListener ()
    {
	return mainActionListener;
    }

   /** 
    *  resets the MainMenu object of the frame 
    *  @param newmenu - the new MainMenu instance
    */
    public void setMainMenu (MainMenu newmenu)
    {
	menu = newmenu;
	validate();
	repaint ();
    }

    /**
     *  returns the current DrawingArea
     */
    public DrawingArea getDrawingArea ()
    {
	return drawingArea;
    }
    

    /**
     *  returns the select scrollbar object
     */
    public JScrollBar getScrollBar(int sel)
    {
      if (sel==JScrollBar.VERTICAL)
        return balkenunten;
      else
	return balkenrechts;
    }

    /**
     *  returns the adjustment object
     */
    public GridAdjustment getAdjustment()
    {
      return gal;
    }
        
    /** 
     *  shuts down the current instance of the application,
     *  requesting user confirmation.
     *  saves the current option settings to the .devisorgrid file in the
     *  users homedir.
     */
    public void shutDownApplication ()
    {
	int quit = JOptionPane.NO_OPTION;
	int unsaved = JOptionPane.NO_OPTION;
	// ask for confirmation
	quit = JOptionPane.showConfirmDialog (this, (String)cc.rb.getObject ("general_message_quit"), (String)cc.rb.getObject ("general_message_quit_header"),JOptionPane.YES_NO_OPTION);
	// check for unsaved data...
	if (quit == JOptionPane.YES_OPTION)
	{
	    if (cc.d.hasChanged())
	    {
		unsaved = JOptionPane.showConfirmDialog (this, (String)cc.rb.getObject ("general_message_unsaveddata"), (String)cc.rb.getObject ("general_message_unsaveddata_header"),JOptionPane.YES_NO_OPTION);
	    }
	    // only quit if everything is ok
	    if ( quit == JOptionPane.YES_OPTION && ( (cc.d.hasChanged() == true && unsaved == JOptionPane.YES_OPTION) || (cc.d.hasChanged() == false)))
	    {
		for (int i=0; i<DialogManager.DIALOGCOUNT;i++)
		{
		    cc.dm.hideAll ();
		}
		// store MainFrame position and size
		cc.op.set (Options.mainframe_width, Integer.toString(getWidth ()));
		cc.op.set (Options.mainframe_height, Integer.toString(getHeight ()));
		cc.op.set (Options.mainframe_x, Integer.toString(getLocationOnScreen().x));
		cc.op.set (Options.mainframe_y, Integer.toString(getLocationOnScreen().y));
		// save options
		
		// quit
		System.exit (4711);
	    }
	}
    }






    
   
}// end of class MainFrame


