package devisor2.grid.GUI.framework;


import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import devisor2.grid.options.*;
import devisor2.grid.GUI.event.*;
import devisor2.grid.backend.spellchecker.*;

/** 
 *  Defines the main menu of the MainFrame <br>
 *  <br>
 *  It is connected to the application via the MainFrame reference, and 
 *  functionality is available via the ActionCommands declared in the 
 *  MainActionListener class.<br>
 *  Note: The accelerators can be changed by the user (without re-compiling)
 *  in the OptionsDialog class, therefore, if more functionality is 
 *  subsequently added to the application, more accelerators have to be
 *  registered with the Options class and the corresponding Action Listener
 *  of the dialog mentioned above. Note that only accelerators are accepted
 *  by the mentioned dialog whose keys start with the prefix accelerator_.
 *  These keys are declared in the Options class.<br>
 *  The mnemonics are part of the locale support class, therefore, the
 *  Locale_xx_yy class has to be edited and recompiled if mnemonics are
 *  going to be altered.<br>
 *
 *  Summary: To add one new menu item, do the following:<br>
 *  (1) create two new entries in the Locale class, one for the caption and 
 *      one for the mnemonic<br>
 *  (2) create a new ActionCommand constant for the item in the 
 *      MainActionListener class <br>
 *  (3) add one if - block with the new ActionCommand to the 
 *      MainActionListener's actionPerformed method <br>
 *  (4) create a new accelerator constant in the Options class, remember it
 *      has to start with the prefix accelelator_ <br>
 *  (5) add a default value for this in the Options class' initDefaults method <br>
 *  (6) add an entry for it in the Options class' isAlreadyAccelerator method<br>
 *  (7) use all these constants to create the new item, see below how this is 
 *      accomplished (basic swing menu creation) <br>
 *  (8) add your entry to the setAccelerators method (in this class) <br>
 *
 *  @author Dominik Goeddeke
 *  @version done, all FEAST disabled
 */

public class MainMenu extends JMenuBar
{

    /** the parent frame of this menu */
    private MainFrame parent;

    /** the ControlCenter reference */
    private ControlCenter cc;
    /** the submenus */
    private JMenu fileMenu, editMenu, viewMenu, domainMenu, gridMenu, optionsMenu, helpMenu;

    /** one actionlistener responsible for all action events from this menu */
    private MainActionListener menuActionListener;

    /** 
     *  sets up a new instance of the MainMenu including all submenus
     *  @param MainFrame parent - the MainFrame the menu is added to
     */
    public MainMenu (MainFrame parent)
    {
	super ();
	/* set up local variables */
	this.parent = parent;
	menuActionListener = parent.getMainActionListener ();
	cc = ControlCenter.getMyself ();
	/* init FILE menu */
	initFileMenu ();
	add(fileMenu);
	/* init EDIT menu */
	initEditMenu ();
	add(editMenu);
	/* init VIEW menu */
	initViewMenu ();
	add (viewMenu);
	/* init DOMAIN menu */
	initDomainMenu ();
	add(domainMenu);
	/* init GRID menu */
	initGridMenu ();
	add (gridMenu);
	/* init OPTIONS menu */
	initOptionsMenu ();
	add(optionsMenu);
	/* init HELP menu */
	initHelpMenu ();
	add(helpMenu);
	/* set accelerators */
	setAccelerators ();
    }

    /**
     *  sets corresponding accelerators of all menu entries to the value 
     *  stored in the global current Options instance
     */
    public void setAccelerators ()
    {
	if ((cc.op.get (Options.accelerator_new).equals ("NONE")))
	    filenew.setAccelerator (null);
	else
	    filenew.setAccelerator (Accelerators.getAccelerator (cc.op.get(Options.accelerator_new)));

	if ((cc.op.get (Options.accelerator_open).equals ("NONE")))
	    fileopen.setAccelerator (null);
	else
	    fileopen.setAccelerator (Accelerators.getAccelerator (cc.op.get(Options.accelerator_open)));
	if ((cc.op.get (Options.accelerator_save).equals ("NONE")))
	    filesave.setAccelerator (null);
	else
	    filesave.setAccelerator (Accelerators.getAccelerator (cc.op.get(Options.accelerator_save)));
	if ((cc.op.get (Options.accelerator_saveas_ucd).equals ("NONE")))
	    filesaveasucd.setAccelerator (null);
	else
	    filesaveasucd.setAccelerator (Accelerators.getAccelerator (cc.op.get(Options.accelerator_saveas_ucd)));
	if ((cc.op.get (Options.accelerator_saveas_feat).equals ("NONE")))
	    filesaveasfeat.setAccelerator (null);
	else
	    filesaveasfeat.setAccelerator (Accelerators.getAccelerator (cc.op.get(Options.accelerator_saveas_feat)));
	if ((cc.op.get (Options.accelerator_saveas_feast).equals ("NONE")))
	    filesaveasfeast.setAccelerator (null);
	else
	    filesaveasfeast.setAccelerator (Accelerators.getAccelerator (cc.op.get(Options.accelerator_saveas_feast)));
	if ((cc.op.get (Options.accelerator_merge).equals ("NONE")))
	    filemergedomain.setAccelerator (null);
	else
	    filemergedomain.setAccelerator (Accelerators.getAccelerator (cc.op.get(Options.accelerator_merge)));

	if ((cc.op.get (Options.accelerator_print).equals ("NONE")))
	    fileprint.setAccelerator (null);
	else
	    fileprint.setAccelerator (Accelerators.getAccelerator (cc.op.get(Options.accelerator_print)));

	if ((cc.op.get (Options.accelerator_printls).equals ("NONE")))
	    fileprintls.setAccelerator (null);
	else
	    fileprintls.setAccelerator (Accelerators.getAccelerator (cc.op.get(Options.accelerator_printls)));


	if ((cc.op.get (Options.accelerator_exit).equals ("NONE")))
	    fileexit.setAccelerator (null);
	else
	    fileexit.setAccelerator (Accelerators.getAccelerator (cc.op.get(Options.accelerator_exit)));
	if ((cc.op.get (Options.accelerator_undo).equals ("NONE")))
	    editundo.setAccelerator (null);
	else
	    editundo.setAccelerator (Accelerators.getAccelerator (cc.op.get(Options.accelerator_undo)));
	if ((cc.op.get (Options.accelerator_redo).equals ("NONE")))
	    editredo.setAccelerator (null);
	else
	    editredo.setAccelerator (Accelerators.getAccelerator (cc.op.get(Options.accelerator_redo)));
	if ((cc.op.get (Options.accelerator_cut).equals ("NONE")))
	    editcut.setAccelerator (null);
	else
	    editcut.setAccelerator (Accelerators.getAccelerator (cc.op.get(Options.accelerator_cut)));
	if ((cc.op.get (Options.accelerator_copy).equals ("NONE")))
	    editcopy.setAccelerator (null);
	else
	    editcopy.setAccelerator (Accelerators.getAccelerator (cc.op.get(Options.accelerator_copy)));
	if ((cc.op.get (Options.accelerator_paste).equals ("NONE")))
	    editpaste.setAccelerator (null);
	else
	    editpaste.setAccelerator (Accelerators.getAccelerator (cc.op.get(Options.accelerator_paste)));
	if ((cc.op.get (Options.accelerator_copymode).equals ("NONE")))
	    editcopymode.setAccelerator (null);
	else
	    editcopymode.setAccelerator (Accelerators.getAccelerator (cc.op.get(Options.accelerator_copymode)));
	if ((cc.op.get (Options.accelerator_delete).equals ("NONE")))
	    editdelete.setAccelerator (null);
	else
	    editdelete.setAccelerator (Accelerators.getAccelerator (cc.op.get(Options.accelerator_delete)));
	if ((cc.op.get (Options.accelerator_deleteworkingboundary).equals ("NONE")))
	    editdeletewb.setAccelerator (null);
	else
	    editdeletewb.setAccelerator (Accelerators.getAccelerator (cc.op.get(Options.accelerator_deleteworkingboundary)));
	if ((cc.op.get (Options.accelerator_scale).equals ("NONE")))
	    editscale.setAccelerator (null);
	else
	    editscale.setAccelerator (Accelerators.getAccelerator (cc.op.get(Options.accelerator_scale)));
	if ((cc.op.get (Options.accelerator_rotate).equals ("NONE")))
	    editrotate.setAccelerator (null);
	else
	    editrotate.setAccelerator (Accelerators.getAccelerator (cc.op.get(Options.accelerator_rotate)));
	if ((cc.op.get (Options.accelerator_mirror).equals ("NONE")))
	    editmirror.setAccelerator (null);
	else
	    editmirror.setAccelerator (Accelerators.getAccelerator (cc.op.get(Options.accelerator_mirror)));
	if ((cc.op.get (Options.accelerator_selectboundary).equals ("NONE")))
	    editselectboundary.setAccelerator (null);
	else
	    editselectboundary.setAccelerator (Accelerators.getAccelerator (cc.op.get(Options.accelerator_selectboundary)));
	if ((cc.op.get (Options.accelerator_zoom_setzoom).equals ("NONE")))
	    viewzoomsetzoom.setAccelerator (null);
	else
	    viewzoomsetzoom.setAccelerator (Accelerators.getAccelerator (cc.op.get(Options.accelerator_zoom_setzoom)));
	if ((cc.op.get (Options.accelerator_zoom_resetzoom).equals ("NONE")))
	    viewzoomresetzoom.setAccelerator (null);
	else
	    viewzoomresetzoom.setAccelerator (Accelerators.getAccelerator (cc.op.get(Options.accelerator_zoom_resetzoom)));
	if ((cc.op.get (Options.accelerator_zoom_zoomplus).equals ("NONE")))
	    viewzoomzoomplus.setAccelerator (null);
	else
	    viewzoomzoomplus.setAccelerator (Accelerators.getAccelerator (cc.op.get(Options.accelerator_zoom_zoomplus)));
	if ((cc.op.get (Options.accelerator_zoom_zoomminus).equals ("NONE")))
	    viewzoomzoomminus.setAccelerator (null);
	else
	    viewzoomzoomminus.setAccelerator (Accelerators.getAccelerator (cc.op.get(Options.accelerator_zoom_zoomminus)));
	if ((cc.op.get (Options.accelerator_selectnodes).equals ("NONE")))
	    viewselectnodes.setAccelerator (null);
	else
	    viewselectnodes.setAccelerator (Accelerators.getAccelerator (cc.op.get(Options.accelerator_selectnodes)));
	if ((cc.op.get (Options.accelerator_selectelements).equals ("NONE")))
	    viewselectelements.setAccelerator (null);
	else
	    viewselectelements.setAccelerator (Accelerators.getAccelerator (cc.op.get(Options.accelerator_selectelements)));
	if ((cc.op.get (Options.accelerator_selectsegments).equals ("NONE")))
	    viewselectsegments.setAccelerator (null);
	else
	    viewselectsegments.setAccelerator (Accelerators.getAccelerator (cc.op.get(Options.accelerator_selectsegments)));
	if ((cc.op.get (Options.accelerator_selectedges).equals ("NONE")))
	    viewselectedges.setAccelerator (null);
	else
	    viewselectedges.setAccelerator (Accelerators.getAccelerator (cc.op.get(Options.accelerator_selectedges)));
	if ((cc.op.get (Options.accelerator_viewproperties).equals ("NONE")))
	    viewproperties.setAccelerator (null);
	else
	    viewproperties.setAccelerator (Accelerators.getAccelerator (cc.op.get(Options.accelerator_viewproperties)));
	if ((cc.op.get (Options.accelerator_newboundary).equals ("NONE")))
	    domainnewboundary.setAccelerator (null);
	else
	    domainnewboundary.setAccelerator (Accelerators.getAccelerator (cc.op.get(Options.accelerator_newboundary)));
	if ((cc.op.get (Options.accelerator_directinput).equals ("NONE")))
	    domaindirectinput.setAccelerator (null);
	else
	    domaindirectinput.setAccelerator (Accelerators.getAccelerator (cc.op.get(Options.accelerator_directinput)));
	if ((cc.op.get (Options.accelerator_newline).equals ("NONE")))
	    domainnewline.setAccelerator (null);
	else
	    domainnewline.setAccelerator (Accelerators.getAccelerator (cc.op.get(Options.accelerator_newline)));
	if ((cc.op.get (Options.accelerator_newmultiline).equals ("NONE")))
	    domainnewmultiline.setAccelerator (null);
	else
	    domainnewmultiline.setAccelerator (Accelerators.getAccelerator (cc.op.get(Options.accelerator_newmultiline)));
	if ((cc.op.get (Options.accelerator_newcirclep).equals ("NONE")))
	    domainnewcirclep.setAccelerator (null);
	else
	    domainnewcirclep.setAccelerator (Accelerators.getAccelerator (cc.op.get(Options.accelerator_newcirclep)));
	if ((cc.op.get (Options.accelerator_newcirclem).equals ("NONE")))
	    domainnewcirclem.setAccelerator (null);
	else
	    domainnewcirclem.setAccelerator (Accelerators.getAccelerator (cc.op.get(Options.accelerator_newcirclem)));
	if ((cc.op.get (Options.accelerator_newanalytic).equals ("NONE")))
	    domainnewanalytic.setAccelerator (null);
	else
	    domainnewanalytic.setAccelerator (Accelerators.getAccelerator (cc.op.get(Options.accelerator_newanalytic)));
	if ((cc.op.get (Options.accelerator_newtri).equals ("NONE")))
	    domainnewtri.setAccelerator (null);
	else
	    domainnewtri.setAccelerator (Accelerators.getAccelerator (cc.op.get(Options.accelerator_newtri)));
	if ((cc.op.get (Options.accelerator_newquad).equals ("NONE")))
	    domainnewquad.setAccelerator (null);
	else
	    domainnewquad.setAccelerator (Accelerators.getAccelerator (cc.op.get(Options.accelerator_newquad)));
	if ((cc.op.get (Options.accelerator_newmacro).equals ("NONE")))
	    domainnewmacro.setAccelerator (null);
	else
	    domainnewmacro.setAccelerator (Accelerators.getAccelerator (cc.op.get(Options.accelerator_newmacro)));
	if ((cc.op.get (Options.accelerator_newnode).equals ("NONE")))
	    domainnewnode.setAccelerator (null);
	else
	    domainnewnode.setAccelerator (Accelerators.getAccelerator (cc.op.get(Options.accelerator_newnode)));
	if ((cc.op.get (Options.accelerator_adjustboundaries).equals ("NONE")))
	    domainadjustboundaries.setAccelerator (null);
	else
	    domainadjustboundaries.setAccelerator (Accelerators.getAccelerator (cc.op.get(Options.accelerator_adjustboundaries)));

	if ((cc.op.get (Options.accelerator_linkboundaries).equals ("NONE")))
	    domainlinkboundaries.setAccelerator (null);
	else
	    domainlinkboundaries.setAccelerator (Accelerators.getAccelerator (cc.op.get(Options.accelerator_linkboundaries)));

	if ((cc.op.get (Options.accelerator_fromtri2prm).equals ("NONE")))
	    domainfromtri2prm.setAccelerator (null);
	else
	    domainfromtri2prm.setAccelerator (Accelerators.getAccelerator (cc.op.get(Options.accelerator_fromtri2prm)));

	if ((cc.op.get (Options.accelerator_swapboundaryorientation).equals ("NONE")))
	    domainswapboundaryorientation.setAccelerator (null);
	else
	    domainswapboundaryorientation.setAccelerator (Accelerators.getAccelerator (cc.op.get(Options.accelerator_swapboundaryorientation)));
	if ((cc.op.get (Options.accelerator_domainperimeter).equals ("NONE")))
	    domainperimeter.setAccelerator (null);
	else
	    domainperimeter.setAccelerator (Accelerators.getAccelerator (cc.op.get(Options.accelerator_domainperimeter)));
	if ((cc.op.get (Options.accelerator_domainstatistics).equals ("NONE")))
	    domainstatistics.setAccelerator (null);
	else
	    domainstatistics.setAccelerator (Accelerators.getAccelerator (cc.op.get(Options.accelerator_domainstatistics)));
	if ((cc.op.get (Options.accelerator_multipleboundarynodes).equals ("NONE")))
	    gridmultipleboundarynodes.setAccelerator (null);
	else
	    gridmultipleboundarynodes.setAccelerator (Accelerators.getAccelerator (cc.op.get(Options.accelerator_multipleboundarynodes)));
	if ((cc.op.get (Options.accelerator_edgestatus).equals ("NONE")))
	    gridedgestatus.setAccelerator (null);
	else
	    gridedgestatus.setAccelerator (Accelerators.getAccelerator (cc.op.get(Options.accelerator_edgestatus)));
	if ((cc.op.get (Options.accelerator_parallelblock).equals ("NONE")))
	    gridparallelblock.setAccelerator (null);
	else
	    gridparallelblock.setAccelerator (Accelerators.getAccelerator (cc.op.get(Options.accelerator_parallelblock)));
	if ((cc.op.get (Options.accelerator_loadbalancing).equals ("NONE")))
	    gridloadbalancing.setAccelerator (null);
	else
	    gridloadbalancing.setAccelerator (Accelerators.getAccelerator (cc.op.get(Options.accelerator_loadbalancing)));
	if ((cc.op.get (Options.accelerator_rectifyelement).equals ("NONE")))
	    gridrectifyelement.setAccelerator (null);
	else
	    gridrectifyelement.setAccelerator (Accelerators.getAccelerator (cc.op.get(Options.accelerator_rectifyelement)));
	if ((cc.op.get (Options.accelerator_refine).equals ("NONE")))
	    gridrefine.setAccelerator (null);
	else
	    gridrefine.setAccelerator (Accelerators.getAccelerator (cc.op.get(Options.accelerator_refine)));
	if ((cc.op.get (Options.accelerator_spellchecker).equals ("NONE")))
	    gridspellchecker.setAccelerator (null);
	else
	    gridspellchecker.setAccelerator (Accelerators.getAccelerator (cc.op.get(Options.accelerator_spellchecker)));
	if ((cc.op.get (Options.accelerator_mergeboundaries).equals ("NONE")))
	    gridmergeboundaries.setAccelerator (null);
	else
	    gridmergeboundaries.setAccelerator (Accelerators.getAccelerator (cc.op.get(Options.accelerator_mergeboundaries)));
	if ((cc.op.get (Options.accelerator_generaloptions).equals ("NONE")))
	    optionsgeneral.setAccelerator (null);
	else
	    optionsgeneral.setAccelerator (Accelerators.getAccelerator (cc.op.get(Options.accelerator_generaloptions)));
	if ((cc.op.get (Options.accelerator_drawoptions).equals ("NONE")))
	    optionsdraw.setAccelerator (null);
	else
	    optionsdraw.setAccelerator (Accelerators.getAccelerator (cc.op.get(Options.accelerator_drawoptions)));
	if ((cc.op.get (Options.accelerator_coloroptions).equals ("NONE")))
	    optionscolor.setAccelerator (null);
	else
	    optionscolor.setAccelerator (Accelerators.getAccelerator (cc.op.get(Options.accelerator_coloroptions)));
	if ((cc.op.get (Options.accelerator_acceleratoroptions).equals ("NONE")))
	    optionsaccelerator.setAccelerator (null);
	else
	    optionsaccelerator.setAccelerator (Accelerators.getAccelerator (cc.op.get(Options.accelerator_acceleratoroptions)));
	if ((cc.op.get (Options.accelerator_miscoptions).equals ("NONE")))
	    optionsmisc.setAccelerator (null);
	else
	    optionsmisc.setAccelerator (Accelerators.getAccelerator (cc.op.get(Options.accelerator_miscoptions)));
	if ((cc.op.get (Options.accelerator_manual).equals ("NONE")))
	    helpmanual.setAccelerator (null);
	else
	    helpmanual.setAccelerator (Accelerators.getAccelerator (cc.op.get(Options.accelerator_manual)));

	if ((cc.op.get (Options.accelerator_faq).equals ("NONE")))
	    helpfaq.setAccelerator (null);
	else
	    helpfaq.setAccelerator (Accelerators.getAccelerator (cc.op.get(Options.accelerator_faq)));

	if ((cc.op.get (Options.accelerator_readme).equals ("NONE")))
	    helpreadme.setAccelerator (null);
	else
	    helpreadme.setAccelerator (Accelerators.getAccelerator (cc.op.get(Options.accelerator_readme)));


	if ((cc.op.get (Options.accelerator_about).equals ("NONE")))
	    helpabout.setAccelerator (null);
	else
	    helpabout.setAccelerator (Accelerators.getAccelerator (cc.op.get(Options.accelerator_about)));
    }



    /** sets up the file menu */
    private void initFileMenu ()
    {
	JMenu submenu;
	fileMenu = new JMenu ((String)cc.rb.getObject("menu_file_header"));
        fileMenu.setMnemonic (((Integer)cc.rb.getObject("menu_file_header_mnemonic")).intValue());
	/* NEW */
	filenew = new JMenuItem ((String)cc.rb.getObject("menu_file_new"));
	filenew.setMnemonic (((Integer)cc.rb.getObject("menu_file_new_mnemonic")).intValue());
	filenew.setActionCommand (MainActionListener.NEW);
	filenew.addActionListener (menuActionListener);
	fileMenu.add (filenew);
	fileMenu.addSeparator ();
	/* OPEN */
        fileopen= new JMenuItem((String)cc.rb.getObject("menu_file_open"));
	fileopen.setMnemonic(((Integer)cc.rb.getObject("menu_file_open_mnemonic")).intValue());
	fileopen.setActionCommand (MainActionListener.OPEN);
	fileopen.addActionListener (menuActionListener);
	fileMenu.add(fileopen);
	/* SAVE */
	filesave = new JMenuItem ((String)cc.rb.getObject("menu_file_save"));
	filesave.setMnemonic (((Integer)cc.rb.getObject("menu_file_save_mnemonic")).intValue());
	filesave.setActionCommand (MainActionListener.SAVE);
	filesave.addActionListener (menuActionListener);
	fileMenu.add (filesave);
	/* submenu for FILE - SAVE AS */
	submenu = new JMenu((String)cc.rb.getObject("menu_file_saveas"));
        submenu.setMnemonic(((Integer)cc.rb.getObject("menu_file_saveas_mnemonic")).intValue());
        filesaveasucd = new JMenuItem((String)cc.rb.getObject("menu_file_saveas_ucd"));
	filesaveasucd.setMnemonic(((Integer)cc.rb.getObject("menu_file_saveas_ucd_mnemonic")).intValue());
	filesaveasucd.setActionCommand (MainActionListener.SAVEAS_UCD);
	filesaveasucd.addActionListener (menuActionListener);
	filesaveasucd.setEnabled (false);
	submenu.add(filesaveasucd);
        filesaveasfeat = new JMenuItem((String)cc.rb.getObject("menu_file_saveas_feat"));
	filesaveasfeat.setMnemonic(((Integer)cc.rb.getObject("menu_file_saveas_feat_mnemonic")).intValue());
	filesaveasfeat.setActionCommand (MainActionListener.SAVEAS_FEAT);
	filesaveasfeat.addActionListener (menuActionListener);
	submenu.add(filesaveasfeat);
	filesaveasfeast = new JMenuItem((String)cc.rb.getObject("menu_file_saveas_feast"));
	filesaveasfeast.setMnemonic (((Integer)cc.rb.getObject("menu_file_saveas_feast_mnemonic")).intValue());
	filesaveasfeast.setActionCommand (MainActionListener.SAVEAS_FEAST);
	filesaveasfeast.addActionListener (menuActionListener);
	filesaveasfeast.setEnabled (true);
        submenu.add(filesaveasfeast);
        fileMenu.add(submenu);
	/* MERGE DOMAIN */
	filemergedomain = new JMenuItem((String)cc.rb.getObject("menu_file_merge"));
	filemergedomain.setMnemonic(((Integer)cc.rb.getObject("menu_file_merge_mnemonic")).intValue());
	filemergedomain.setActionCommand (MainActionListener.MERGE);
	filemergedomain.addActionListener (menuActionListener);
	filemergedomain.setEnabled (false);
	fileMenu.add(filemergedomain);
	fileMenu.addSeparator ();
	/* PRINT */
	fileprint = new JMenuItem ((String)cc.rb.getObject("menu_file_print"));
	fileprint.setMnemonic (((Integer)cc.rb.getObject("menu_file_print_mnemonic")).intValue());
	fileprint.setActionCommand (MainActionListener.PRINT);
	fileprint.addActionListener (menuActionListener);
	//fileprint.setEnabled (false);
	fileMenu.add (fileprint);
	/* PRINTLS */
	fileprintls = new JMenuItem ((String)cc.rb.getObject("menu_file_printls"));
	fileprintls.setMnemonic (((Integer)cc.rb.getObject("menu_file_printls_mnemonic")).intValue());
	fileprintls.setActionCommand (MainActionListener.PRINTLS);
	fileprintls.addActionListener (menuActionListener);
	//fileprint.setEnabled (false);
	fileMenu.add (fileprintls);
	fileMenu.addSeparator ();
	/* EXIT */
	fileexit = new JMenuItem ((String)cc.rb.getObject("menu_file_exit"));
	fileexit.setMnemonic (((Integer)cc.rb.getObject("menu_file_exit_mnemonic")).intValue());
	fileexit.setActionCommand (MainActionListener.EXIT);
	fileexit.addActionListener (menuActionListener);
	fileMenu.add (fileexit);
    }

    /** set up the edit menu */
    private void initEditMenu ()
    {
	JMenu submenu;
	editMenu = new JMenu ((String)cc.rb.getObject("menu_edit_header"));
        editMenu.setMnemonic (((Integer)cc.rb.getObject("menu_edit_header_mnemonic")).intValue());
	/* UNDO */
	editundo = new JMenuItem ((String)cc.rb.getObject("menu_edit_undo"));
	editundo.setMnemonic (((Integer)cc.rb.getObject("menu_edit_undo_mnemonic")).intValue());
	editundo.setActionCommand (MainActionListener.UNDO);
	editundo.addActionListener (menuActionListener);
	editMenu.add (editundo);
	/* REDO */
	editredo = new JMenuItem ((String)cc.rb.getObject("menu_edit_redo"));
	editredo.setMnemonic (((Integer)cc.rb.getObject("menu_edit_redo_mnemonic")).intValue());
	editredo.setActionCommand (MainActionListener.REDO);
	editredo.addActionListener (menuActionListener);
	editMenu.add (editredo);
	editMenu.addSeparator ();
	/* CUT */
	editcut = new JMenuItem ((String)cc.rb.getObject("menu_edit_cut"));
	editcut.setMnemonic (((Integer)cc.rb.getObject("menu_edit_cut_mnemonic")).intValue());
	editcut.setActionCommand (MainActionListener.CUT);
	editcut.addActionListener (menuActionListener);
	editMenu.add (editcut);
	/* COPY */
	editcopy = new JMenuItem ((String)cc.rb.getObject("menu_edit_copy"));
	editcopy.setMnemonic (((Integer)cc.rb.getObject("menu_edit_copy_mnemonic")).intValue());
	editcopy.setActionCommand (MainActionListener.COPY);
	editcopy.addActionListener (menuActionListener);
	editMenu.add (editcopy);
	/* PASTE */
	editpaste = new JCheckBoxMenuItem ((String)cc.rb.getObject("menu_edit_paste"));
	editpaste.setMnemonic (((Integer)cc.rb.getObject("menu_edit_paste_mnemonic")).intValue());
	editpaste.setActionCommand (MainActionListener.PASTE);
	editpaste.addActionListener (menuActionListener);
	editMenu.add (editpaste);
	editMenu.addSeparator ();
	/* COPYMODE */
	editcopymode = new JCheckBoxMenuItem ((String)cc.rb.getObject("menu_edit_copymode"));
	editcopymode.setMnemonic (((Integer)cc.rb.getObject("menu_edit_copymode_mnemonic")).intValue());
	editcopymode.setActionCommand (MainActionListener.COPYMODE);
	editcopymode.addActionListener (menuActionListener);
	editMenu.add (editcopymode);
	editMenu.addSeparator ();
	/* DELETE */
	editdelete = new JMenuItem ((String)cc.rb.getObject("menu_edit_delete"));
	editdelete.setMnemonic (((Integer)cc.rb.getObject("menu_edit_delete_mnemonic")).intValue());
	editdelete.setActionCommand (MainActionListener.DELETE);
	editdelete.addActionListener (menuActionListener);
	editMenu.add (editdelete);
	/* DELETEWB */
	editdeletewb = new JMenuItem ((String)cc.rb.getObject("menu_edit_deletewb"));
	editdeletewb.setMnemonic (((Integer)cc.rb.getObject("menu_edit_deletewb_mnemonic")).intValue());
	editdeletewb.setActionCommand (MainActionListener.DELETEWB);
	editdeletewb.addActionListener (menuActionListener);
	editMenu.add (editdeletewb);
	/* SCALE */
	editscale = new JMenuItem ((String)cc.rb.getObject("menu_edit_scale"));
	editscale.setMnemonic (((Integer)cc.rb.getObject("menu_edit_scale_mnemonic")).intValue());
	editscale.setActionCommand (MainActionListener.SCALE);
	editscale.addActionListener (menuActionListener);
	editMenu.add (editscale);
	/* ROTATE */
	editrotate = new JMenuItem ((String)cc.rb.getObject("menu_edit_rotate"));
	editrotate.setMnemonic (((Integer)cc.rb.getObject("menu_edit_rotate_mnemonic")).intValue());
	editrotate.setActionCommand (MainActionListener.ROTATE);
	editrotate.addActionListener (menuActionListener);
	editMenu.add (editrotate);
	/* MIRROR */
	editmirror = new JCheckBoxMenuItem ((String)cc.rb.getObject("menu_edit_mirror"));
	editmirror.setMnemonic (((Integer)cc.rb.getObject("menu_edit_mirror_mnemonic")).intValue());
	editmirror.setActionCommand (MainActionListener.MIRROR);
	editmirror.addActionListener (menuActionListener);
	editMenu.add (editmirror);
	editMenu.addSeparator ();
	/* SELECTBOUNDARY */
	editselectboundary = new JMenuItem ((String)cc.rb.getObject("menu_edit_selectboundary"));
	editselectboundary.setMnemonic (((Integer)cc.rb.getObject("menu_edit_selectboundary_mnemonic")).intValue());
	editselectboundary.setActionCommand (MainActionListener.SELECTBOUNDARY);
	editselectboundary.addActionListener (menuActionListener);
	editMenu.add (editselectboundary);
    }

    /** set up the view menu */
    private void initViewMenu ()
    {
	viewMenu = new JMenu ((String)cc.rb.getObject("menu_view_header"));
        viewMenu.setMnemonic (((Integer)cc.rb.getObject("menu_view_header_mnemonic")).intValue());
	/* SET ZOOM  */
        viewzoomsetzoom = new JCheckBoxMenuItem((String)cc.rb.getObject("menu_view_zoom_setzoom"));
	viewzoomsetzoom.setMnemonic(((Integer)cc.rb.getObject("menu_view_zoom_setzoom_mnemonic")).intValue());
	viewzoomsetzoom.setActionCommand (MainActionListener.ZOOM_SETZOOM);
	viewzoomsetzoom.addActionListener (menuActionListener);
	viewMenu.add(viewzoomsetzoom);
	/* RESET ZOOM */
	viewzoomresetzoom = new JMenuItem((String)cc.rb.getObject("menu_view_zoom_resetzoom"));
	viewzoomresetzoom.setMnemonic(((Integer)cc.rb.getObject("menu_view_zoom_resetzoom_mnemonic")).intValue());
	viewzoomresetzoom.setActionCommand (MainActionListener.ZOOM_RESETZOOM);
	viewzoomresetzoom.addActionListener (menuActionListener);
	viewMenu.add(viewzoomresetzoom);
	/* ZOOM+ */
	viewzoomzoomplus = new JCheckBoxMenuItem((String)cc.rb.getObject("menu_view_zoom_zoomplus"));
	viewzoomzoomplus.setMnemonic(((Integer)cc.rb.getObject("menu_view_zoom_zoomplus_mnemonic")).intValue());
	viewzoomzoomplus.setActionCommand (MainActionListener.ZOOM_ZOOMPLUS);
	viewzoomzoomplus.addActionListener (menuActionListener);
	viewMenu.add(viewzoomzoomplus);
	/* ZOOM- */
	viewzoomzoomminus = new JMenuItem((String)cc.rb.getObject("menu_view_zoom_zoomminus"));
	viewzoomzoomminus.setMnemonic(((Integer)cc.rb.getObject("menu_view_zoom_zoomminus_mnemonic")).intValue());
	viewzoomzoomminus.setActionCommand (MainActionListener.ZOOM_ZOOMMINUS);
	viewzoomzoomminus.addActionListener (menuActionListener);
	viewMenu.add(viewzoomzoomminus);
	viewMenu.addSeparator ();
	/* SELECT NODES TOGGLE */
	viewselectnodes = new JCheckBoxMenuItem ((String)cc.rb.getObject("menu_view_selectnodes"));
	viewselectnodes.setMnemonic(((Integer)cc.rb.getObject("menu_view_selectnodes_mnemonic")).intValue());
	viewselectnodes.setActionCommand (MainActionListener.SELECTNODES);
	viewselectnodes.addActionListener (menuActionListener);
	viewMenu.add (viewselectnodes);
	/* SELECT ELEMENTS TOGGLE */
	viewselectelements = new JCheckBoxMenuItem ((String)cc.rb.getObject("menu_view_selectelements"));
	viewselectelements.setMnemonic(((Integer)cc.rb.getObject("menu_view_selectelements_mnemonic")).intValue());
	viewselectelements.setActionCommand (MainActionListener.SELECTELEMENTS);
	viewselectelements.addActionListener (menuActionListener);
	viewMenu.add (viewselectelements);
	/* SELECT SEGMENTS TOGGLE */
	viewselectsegments = new JCheckBoxMenuItem ((String)cc.rb.getObject("menu_view_selectsegments"));
	viewselectsegments.setMnemonic(((Integer)cc.rb.getObject("menu_view_selectsegments_mnemonic")).intValue());
	viewselectsegments.setActionCommand (MainActionListener.SELECTSEGMENTS);
	viewselectsegments.addActionListener (menuActionListener);
	viewMenu.add (viewselectsegments);
	/* SELECT EDGES TOGGLE */
	viewselectedges = new JCheckBoxMenuItem ((String)cc.rb.getObject("menu_view_selectedges"));
	viewselectedges.setMnemonic(((Integer)cc.rb.getObject("menu_view_selectedges_mnemonic")).intValue());
	viewselectedges.setActionCommand (MainActionListener.SELECTEDGES);
	viewselectedges.addActionListener (menuActionListener);
	viewMenu.add (viewselectedges);
	viewMenu.addSeparator ();
	/* VIEW PROPERTIES TOGGLE */
	viewproperties = new JCheckBoxMenuItem ((String)cc.rb.getObject("menu_view_properties"));
	viewproperties.setMnemonic(((Integer)cc.rb.getObject("menu_view_properties_mnemonic")).intValue());
	viewproperties.setActionCommand (MainActionListener.PROPERTIES);
	viewproperties.addActionListener (menuActionListener);
	viewMenu.add (viewproperties);
    }

    /** set up the Domain menu */
    private void initDomainMenu ()
    {
	JMenu submenu;
	domainMenu = new JMenu ((String)cc.rb.getObject("menu_domain_header"));
        domainMenu.setMnemonic (((Integer)cc.rb.getObject("menu_domain_header_mnemonic")).intValue());
	/* NEW BOUNDARY CONTAINER */
	domainnewboundary = new JMenuItem((String)cc.rb.getObject("menu_domain_newboundary"));
	domainnewboundary.setMnemonic(((Integer)cc.rb.getObject("menu_domain_newboundary_mnemonic")).intValue());
	domainnewboundary.setActionCommand (MainActionListener.NEWBOUNDARY);
	domainnewboundary.addActionListener (menuActionListener);
	domainMenu.add(domainnewboundary);
	/* ANALYTIC DESCRIPTION */
	domainnewanalytic = new JMenuItem((String)cc.rb.getObject("menu_domain_newanalytic"));
	domainnewanalytic.setMnemonic(((Integer)cc.rb.getObject("menu_domain_newanalytic_mnemonic")).intValue());
	domainnewanalytic.setActionCommand (MainActionListener.NEWANALYTIC);
	domainnewanalytic.addActionListener (menuActionListener);
	domainnewanalytic.setEnabled (false);
	domainMenu.add(domainnewanalytic);
	domainMenu.addSeparator ();
	/* DIRECT INPUT */
	domaindirectinput = new JCheckBoxMenuItem((String)cc.rb.getObject("menu_domain_directinput"));
	domaindirectinput.setMnemonic(((Integer)cc.rb.getObject("menu_domain_directinput_mnemonic")).intValue());
	domaindirectinput.setActionCommand (MainActionListener.DIRECTINPUT);
	domaindirectinput.addActionListener (menuActionListener);
	domainMenu.add(domaindirectinput);
	/* NEW SEGMENT submenu */
	submenu = new JMenu((String)cc.rb.getObject("menu_domain_newsegment"));
        submenu.setMnemonic(((Integer)cc.rb.getObject("menu_domain_newsegment_mnemonic")).intValue());
        domainnewline = new JCheckBoxMenuItem((String)cc.rb.getObject("menu_domain_newline"));
	domainnewline.setMnemonic(((Integer)cc.rb.getObject("menu_domain_newline_mnemonic")).intValue());
	domainnewline.setActionCommand (MainActionListener.NEWLINE);
	domainnewline.addActionListener (menuActionListener);
	submenu.add(domainnewline);
	domainnewmultiline = new JCheckBoxMenuItem((String)cc.rb.getObject("menu_domain_newmultiline"));
	domainnewmultiline.setMnemonic(((Integer)cc.rb.getObject("menu_domain_newmultiline_mnemonic")).intValue());
	domainnewmultiline.setActionCommand (MainActionListener.NEWMULTILINE);
	domainnewmultiline.addActionListener (menuActionListener);
	submenu.add(domainnewmultiline);
	domainnewcirclep = new JCheckBoxMenuItem((String)cc.rb.getObject("menu_domain_newcirclep"));
	domainnewcirclep.setMnemonic(((Integer)cc.rb.getObject("menu_domain_newcirclep_mnemonic")).intValue());
	domainnewcirclep.setActionCommand (MainActionListener.NEWCIRCLEP);
	domainnewcirclep.addActionListener (menuActionListener);
	submenu.add(domainnewcirclep);
	domainnewcirclem = new JCheckBoxMenuItem((String)cc.rb.getObject("menu_domain_newcirclem"));
	domainnewcirclem.setMnemonic(((Integer)cc.rb.getObject("menu_domain_newcirclem_mnemonic")).intValue());
	domainnewcirclem.setActionCommand (MainActionListener.NEWCIRCLEM);
	domainnewcirclem.addActionListener (menuActionListener);
	submenu.add(domainnewcirclem);
	domainMenu.add(submenu);
	/* NEW ELEMENT submenu */
	submenu = new JMenu((String)cc.rb.getObject("menu_domain_newelement"));
        submenu.setMnemonic(((Integer)cc.rb.getObject("menu_domain_newelement_mnemonic")).intValue());
	domainnewtri = new JCheckBoxMenuItem((String)cc.rb.getObject("menu_domain_newtri"));
	domainnewtri.setMnemonic(((Integer)cc.rb.getObject("menu_domain_newtri_mnemonic")).intValue());
	domainnewtri.setActionCommand (MainActionListener.NEWTRI);
	domainnewtri.addActionListener (menuActionListener);
	submenu.add(domainnewtri);
	domainnewquad = new JCheckBoxMenuItem((String)cc.rb.getObject("menu_domain_newquad"));
	domainnewquad.setMnemonic(((Integer)cc.rb.getObject("menu_domain_newquad_mnemonic")).intValue());
	domainnewquad.setActionCommand (MainActionListener.NEWQUAD);
	domainnewquad.addActionListener (menuActionListener);
	submenu.add(domainnewquad);
	domainnewmacro = new JCheckBoxMenuItem((String)cc.rb.getObject("menu_domain_newmacro"));
	domainnewmacro.setMnemonic(((Integer)cc.rb.getObject("menu_domain_newmacro_mnemonic")).intValue());
	domainnewmacro.setActionCommand (MainActionListener.NEWMACRO);
	domainnewmacro.addActionListener (menuActionListener);
	submenu.add(domainnewmacro);
	domainMenu.add(submenu);
	/* NEW NODE */
	domainnewnode = new JCheckBoxMenuItem((String)cc.rb.getObject("menu_domain_newnode"));
	domainnewnode.setMnemonic(((Integer)cc.rb.getObject("menu_domain_newnode_mnemonic")).intValue());
	domainnewnode.setActionCommand (MainActionListener.NEWNODE);
	domainnewnode.addActionListener (menuActionListener);
	domainMenu.add(domainnewnode);
	domainMenu.addSeparator ();
	/* ADJUST BOUNDARIES */
	domainadjustboundaries = new JMenuItem((String)cc.rb.getObject("menu_domain_adjustboundaries"));
	domainadjustboundaries.setMnemonic(((Integer)cc.rb.getObject("menu_domain_adjustboundaries_mnemonic")).intValue());
	domainadjustboundaries.setActionCommand (MainActionListener.ADJUSTBOUNDARIES);
	domainadjustboundaries.addActionListener (menuActionListener);
	domainMenu.add(domainadjustboundaries);
	/* LINK BOUNDARIES */
	domainlinkboundaries = new JMenuItem((String)cc.rb.getObject("menu_domain_linkboundaries"));
	domainlinkboundaries.setMnemonic(((Integer)cc.rb.getObject("menu_domain_linkboundaries_mnemonic")).intValue());
	domainlinkboundaries.setActionCommand (MainActionListener.LINKBOUNDARIES);
	domainlinkboundaries.addActionListener (menuActionListener);
	domainlinkboundaries.setEnabled(false);
	domainMenu.add(domainlinkboundaries);

	/* FROMTRI2PRM  */
	domainfromtri2prm = new JMenuItem((String)cc.rb.getObject("menu_domain_fromtri2prm"));
	domainfromtri2prm.setMnemonic(((Integer)cc.rb.getObject("menu_domain_fromtri2prm_mnemonic")).intValue());
	domainfromtri2prm.setActionCommand (MainActionListener.FROMTRI2PRM);
	domainfromtri2prm.addActionListener (menuActionListener);
	domainMenu.add(domainfromtri2prm);
	
	/* SWAP BOUNDARY ORIENTATION */
	domainswapboundaryorientation = new JMenuItem((String)cc.rb.getObject("menu_domain_swapboundaryorientation"));
	domainswapboundaryorientation.setMnemonic(((Integer)cc.rb.getObject("menu_domain_swapboundaryorientation_mnemonic")).intValue());
	domainswapboundaryorientation.setActionCommand (MainActionListener.SWAPBOUNDARYORIENTATION);
	domainswapboundaryorientation.addActionListener (menuActionListener);
	domainMenu.add(domainswapboundaryorientation);
	domainMenu.addSeparator ();
	/* DOMAIN PERIMETER */
	domainperimeter = new JMenuItem((String)cc.rb.getObject("menu_domain_domainperimeter"));
	domainperimeter.setMnemonic(((Integer)cc.rb.getObject("menu_domain_domainperimeter_mnemonic")).intValue());
	domainperimeter.setActionCommand (MainActionListener.DOMAINPERIMETER);
	domainperimeter.addActionListener (menuActionListener);
	domainMenu.add(domainperimeter);
	domainMenu.addSeparator ();
	/* DOMAIN STATISTICS */
	domainstatistics = new JMenuItem((String)cc.rb.getObject("menu_domain_domainstatistics"));
	domainstatistics.setMnemonic(((Integer)cc.rb.getObject("menu_domain_domainstatistics_mnemonic")).intValue());
	domainstatistics.setActionCommand (MainActionListener.DOMAINSTATISTICS);
	domainstatistics.addActionListener (menuActionListener);
	domainMenu.add(domainstatistics);
    }

   /** set up the Grid menu */
    private void initGridMenu ()
    {
	JMenu submenu;
	gridMenu = new JMenu ((String)cc.rb.getObject("menu_grid_header"));
        gridMenu.setMnemonic (((Integer)cc.rb.getObject("menu_grid_header_mnemonic")).intValue());
	/* SET MULTIPLE BOUNDARY NODES */
	gridmultipleboundarynodes = new JMenuItem((String)cc.rb.getObject("menu_grid_multipleboundarynodes"));
	gridmultipleboundarynodes.setMnemonic(((Integer)cc.rb.getObject("menu_grid_multipleboundarynodes_mnemonic")).intValue());
	gridmultipleboundarynodes.setActionCommand (MainActionListener.MULTIPLEBOUNDARYNODES);
	gridmultipleboundarynodes.addActionListener (menuActionListener);
	gridMenu.add(gridmultipleboundarynodes);
	gridMenu.addSeparator ();
	/* SET EDGE STATUS */
	gridedgestatus = new JCheckBoxMenuItem((String)cc.rb.getObject("menu_grid_edgestatus"));
	gridedgestatus.setMnemonic(((Integer)cc.rb.getObject("menu_grid_edgestatus_mnemonic")).intValue());
	gridedgestatus.setActionCommand (MainActionListener.EDGESTATUS);
	gridedgestatus.addActionListener (menuActionListener);
	gridMenu.add(gridedgestatus);
	gridMenu.addSeparator ();
	/* SET PARALLEL BLOCK */
	gridparallelblock = new JCheckBoxMenuItem((String)cc.rb.getObject("menu_grid_parallelblock"));
	gridparallelblock.setMnemonic(((Integer)cc.rb.getObject("menu_grid_parallelblock_mnemonic")).intValue());
	gridparallelblock.setActionCommand (MainActionListener.PARALLELBLOCK);
	gridparallelblock.addActionListener (menuActionListener);
	gridMenu.add(gridparallelblock);
	gridMenu.addSeparator ();
	/* LOADBALANCING */
	gridloadbalancing = new JMenuItem((String)cc.rb.getObject("menu_grid_loadbalancing"));
	gridloadbalancing.setMnemonic(((Integer)cc.rb.getObject("menu_grid_loadbalancing_mnemonic")).intValue());
	gridloadbalancing.setActionCommand (MainActionListener.LOADBALANCING);
	gridloadbalancing.addActionListener (menuActionListener);
	gridMenu.add(gridloadbalancing);
	gridMenu.addSeparator ();
	/* RECTIFY ELEMENT */
	gridrectifyelement = new JCheckBoxMenuItem((String)cc.rb.getObject("menu_grid_rectifyelement"));
	gridrectifyelement.setMnemonic(((Integer)cc.rb.getObject("menu_grid_rectifyelement_mnemonic")).intValue());
	gridrectifyelement.setActionCommand (MainActionListener.RECTIFYELEMENT);
	gridrectifyelement.addActionListener (menuActionListener);
	gridMenu.add(gridrectifyelement);
	gridMenu.addSeparator ();
	/* REFINE */
	gridrefine = new JCheckBoxMenuItem((String)cc.rb.getObject("menu_grid_refine"));
	gridrefine.setMnemonic(((Integer)cc.rb.getObject("menu_grid_refine_mnemonic")).intValue());
	gridrefine.setActionCommand (MainActionListener.REFINE);
	gridrefine.addActionListener (menuActionListener);
	gridMenu.add(gridrefine);
	gridMenu.addSeparator ();
	/* SPELLCHECKER */
	gridspellchecker = new JMenuItem((String)cc.rb.getObject("menu_grid_spellchecker"));
	gridspellchecker.setMnemonic(((Integer)cc.rb.getObject("menu_grid_spellchecker_mnemonic")).intValue());
	gridspellchecker.setActionCommand (MainActionListener.SPELLCHECKER);
	gridspellchecker.addActionListener (menuActionListener);
	gridMenu.add(gridspellchecker);
	gridMenu.addSeparator ();
	/* MERGEBOUNDARIES */
	gridmergeboundaries = new JMenuItem((String)cc.rb.getObject("menu_grid_mergeboundaries"));
	gridmergeboundaries.setMnemonic(((Integer)cc.rb.getObject("menu_grid_mergeboundaries_mnemonic")).intValue());
	gridmergeboundaries.setActionCommand (MainActionListener.MERGEBOUNDARIES);
	gridmergeboundaries.addActionListener (menuActionListener);
	gridMenu.add(gridmergeboundaries);
    }


    /** set up the  options menu */
    private void initOptionsMenu ()
    {
	JMenuItem menuItem;
	JMenu submenu;
	optionsMenu = new JMenu((String)cc.rb.getObject ("menu_options_header"));
	optionsMenu.setMnemonic(((Integer)cc.rb.getObject("menu_options_header_mnemonic")).intValue());
	// GENERAL
        optionsgeneral = new JMenuItem ((String)cc.rb.getObject("menu_options_general"));
	optionsgeneral.setMnemonic(((Integer)cc.rb.getObject("menu_options_general_mnemonic")).intValue());
	optionsgeneral.setActionCommand (MainActionListener.GENERALOPTIONS);
	optionsgeneral.addActionListener (menuActionListener);
	optionsMenu.add (optionsgeneral);
	// DRAW
        optionsdraw = new JMenuItem ((String)cc.rb.getObject("menu_options_draw"));
	optionsdraw.setMnemonic(((Integer)cc.rb.getObject("menu_options_draw_mnemonic")).intValue());
	optionsdraw.setActionCommand (MainActionListener.DRAWOPTIONS);
	optionsdraw.addActionListener (menuActionListener);
	optionsMenu.add (optionsdraw);
	// COLOR
        optionscolor = new JMenuItem ((String)cc.rb.getObject("menu_options_color"));
	optionscolor.setMnemonic(((Integer)cc.rb.getObject("menu_options_color_mnemonic")).intValue());
	optionscolor.setActionCommand (MainActionListener.COLOROPTIONS);
	optionscolor.addActionListener (menuActionListener);
	optionsMenu.add (optionscolor);
	// ACCELERATORS
        optionsaccelerator = new JMenuItem ((String)cc.rb.getObject("menu_options_accelerator"));
	optionsaccelerator.setMnemonic(((Integer)cc.rb.getObject("menu_options_accelerator_mnemonic")).intValue());
	optionsaccelerator.setActionCommand (MainActionListener.ACCELERATOROPTIONS);
	optionsaccelerator.addActionListener (menuActionListener);
	optionsMenu.add (optionsaccelerator);
	// MISC
        optionsmisc = new JMenuItem ((String)cc.rb.getObject("menu_options_misc"));
	optionsmisc.setMnemonic(((Integer)cc.rb.getObject("menu_options_misc_mnemonic")).intValue());
	optionsmisc.setActionCommand (MainActionListener.MISCOPTIONS);
	optionsmisc.addActionListener (menuActionListener);
	optionsMenu.add (optionsmisc);
    }

    /** sets up the help menu */
    private void initHelpMenu ()
    {
	JMenu submenu;
	helpMenu = new JMenu ((String)cc.rb.getObject("menu_help_header"));
        helpMenu.setMnemonic (((Integer)cc.rb.getObject("menu_help_header_mnemonic")).intValue());
	/* MANUAL */
	helpmanual = new JMenuItem((String)cc.rb.getObject("menu_help_manual"));
	helpmanual.setMnemonic(((Integer)cc.rb.getObject("menu_help_manual_mnemonic")).intValue());
	helpmanual.setActionCommand (MainActionListener.MANUAL);
	helpmanual.addActionListener (menuActionListener);
	helpMenu.add(helpmanual);
	/* FAQ */
	helpfaq = new JMenuItem((String)cc.rb.getObject("menu_help_faq"));
	helpfaq.setMnemonic(((Integer)cc.rb.getObject("menu_help_faq_mnemonic")).intValue());
	helpfaq.setActionCommand (MainActionListener.FAQ);
	helpfaq.addActionListener (menuActionListener);
	helpMenu.add(helpfaq);
	/* README */
	helpreadme = new JMenuItem((String)cc.rb.getObject("menu_help_readme"));
	helpreadme.setMnemonic(((Integer)cc.rb.getObject("menu_help_readme_mnemonic")).intValue());
	helpreadme.setActionCommand (MainActionListener.README);
	helpreadme.addActionListener (menuActionListener);
	helpMenu.add(helpreadme);
	/* ABOUT */
	helpMenu.addSeparator ();
	helpabout = new JMenuItem((String)cc.rb.getObject("menu_help_about"));
	helpabout.setMnemonic(((Integer)cc.rb.getObject("menu_help_about_mnemonic")).intValue());
	helpabout.setActionCommand (MainActionListener.ABOUT);
	helpabout.addActionListener (menuActionListener);
	helpMenu.add(helpabout);

    }

    // getters and setters for CheckBoxMenuItems
    public JCheckBoxMenuItem getPasteItem ()
    {
	return editpaste;
    }
    public JCheckBoxMenuItem getCopymodeItem ()
    {
	return editcopymode;
    }
    public JCheckBoxMenuItem getSetZoomItem ()
    {
	return viewzoomsetzoom;
    }
    public JCheckBoxMenuItem getZoomPlusItem ()
    {
	return viewzoomzoomplus;
    }
    public JCheckBoxMenuItem getSelectNodesItem ()
    {
	return viewselectnodes;
    }
    public JCheckBoxMenuItem getSelectElementsItem ()
    {
	return viewselectelements;
    }
    public JCheckBoxMenuItem getSelectSegmentsItem ()
    {
	return viewselectsegments;
    }
    public JCheckBoxMenuItem getSelectEdgesItem ()
    {
	return viewselectedges;
    }
    public JCheckBoxMenuItem getViewPropertiesItem ()
    {
	return viewproperties;
    }
    public JCheckBoxMenuItem getDirectInputItem ()
    {
	return domaindirectinput;
    }
    public JCheckBoxMenuItem getNewLineItem ()
    {
	return domainnewline;
    }
    public JCheckBoxMenuItem getNewMultiLineItem ()
    {
	return domainnewmultiline;
    }
    public JCheckBoxMenuItem getNewCirclePItem ()
    {
	return domainnewcirclep;
    }
    public JCheckBoxMenuItem getNewCircleMItem ()
    {
	return domainnewcirclem;
    }
    public JCheckBoxMenuItem getNewTriItem ()
    {
	return domainnewtri;
    }
    public JCheckBoxMenuItem getNewQuadItem ()
    {
	return domainnewquad;
    }
    public JCheckBoxMenuItem getNewMacroItem ()
    {
	return domainnewmacro;
    }
    public JCheckBoxMenuItem getNewNodeItem ()
    {
	return domainnewnode;
    }
    public JCheckBoxMenuItem getEdgeStatusItem ()
    {
	return gridedgestatus;
    }
    public JCheckBoxMenuItem getParallelBlockItem ()
    {
	return gridparallelblock;
    }
    public JCheckBoxMenuItem getRectifyElementItem ()
    {
	return gridrectifyelement;
    }
    public JCheckBoxMenuItem getRefineItem ()
    {
	return gridrefine;
    }
    public JCheckBoxMenuItem getMirrorItem ()
    {
	return editmirror;
    }




    private JMenuItem filenew;
    private JMenuItem fileopen;
    private JMenuItem filesave;
    private JMenuItem filesaveasucd;
    private JMenuItem filesaveasfeat;
    private JMenuItem filesaveasfeast;
    private JMenuItem filemergedomain;
    private JMenuItem fileprint;
    private JMenuItem fileprintls;
    private JMenuItem fileexit;
    private JMenuItem editundo;
    private JMenuItem editredo;
    private JMenuItem editcut;
    private JMenuItem editcopy;
    private JCheckBoxMenuItem editpaste;
    private JCheckBoxMenuItem editcopymode;
    private JMenuItem editdelete;
    private JMenuItem editdeletewb;
    private JMenuItem editscale;
    private JMenuItem editrotate;
    private JCheckBoxMenuItem editmirror;
    private JMenuItem editselectboundary;
    private JCheckBoxMenuItem viewzoomsetzoom;
    private JMenuItem viewzoomresetzoom;
    private JCheckBoxMenuItem viewzoomzoomplus;
    private JMenuItem viewzoomzoomminus;
    private JCheckBoxMenuItem viewselectnodes;
    private JCheckBoxMenuItem viewselectelements;
    private JCheckBoxMenuItem viewselectsegments;
    private JCheckBoxMenuItem viewselectedges;
    private JCheckBoxMenuItem viewproperties;
    private JMenuItem domainnewboundary;
    private JMenuItem domainnewanalytic;
    private JCheckBoxMenuItem domaindirectinput;
    private JCheckBoxMenuItem domainnewline;
    private JCheckBoxMenuItem domainnewmultiline;
    private JCheckBoxMenuItem domainnewcirclep;
    private JCheckBoxMenuItem domainnewcirclem;
    private JCheckBoxMenuItem domainnewtri;
    private JCheckBoxMenuItem domainnewquad;
    private JCheckBoxMenuItem domainnewmacro;
    private JCheckBoxMenuItem domainnewnode;
    private JMenuItem domainadjustboundaries;
    private JMenuItem domainlinkboundaries;
    private JMenuItem domainfromtri2prm;
    private JMenuItem domainswapboundaryorientation;
    private JMenuItem domainperimeter;
    private JMenuItem domainstatistics;
    private JMenuItem gridmultipleboundarynodes;
    private JCheckBoxMenuItem gridedgestatus;
    private JCheckBoxMenuItem gridparallelblock;
    private JMenuItem gridloadbalancing;
    private JCheckBoxMenuItem gridrectifyelement;
    private JCheckBoxMenuItem gridrefine;
    private JMenuItem gridspellchecker;
    private JMenuItem gridmergeboundaries;
    private JMenuItem optionsgeneral;
    private JMenuItem optionsdraw;
    private JMenuItem optionscolor;
    private JMenuItem optionsaccelerator;
    private JMenuItem optionsmisc;
    private JMenuItem helpmanual;
    private JMenuItem helpfaq;
    private JMenuItem helpreadme;
    private JMenuItem helpabout;

}

