package devisor2.grid.backend;

import java.awt.event.*;
import java.awt.*;
import java.awt.geom.*;
import java.util.*;

import devisor2.grid.GUI.framework.*;
import devisor2.grid.options.*;
import devisor2.grid.GUI.event.*;
import devisor2.foundation.elements.*;
import devisor2.foundation.base.*;
import devisor2.foundation.boundary.*;
import devisor2.foundation.*;


/**
 *  This class is responsible for managing the select box which can be dragged
 *  open on the DrawingArea. All event handling is taken care of in the 
 *  GridListener class, here just the neccessary methods for drawing and
 *  changing the select box are provided. <br>
 *  Additionally, this class provides the container for selections (in other 
 *  words, the internal selection management) and methods to modify it.<br>
 *
 *  @author Dominik Goeddeke
 *  @version 0.9 release candidate
 */
public abstract class BaseSelector
{
    // the MainFrame reference
    protected MainFrame root;
    
    // the ControlCenter reference
    protected ControlCenter cc = ControlCenter.getMyself();

    // the rectangle objects for select box painting
    protected Rectangle currentRect = null;
    protected Rectangle rectToDraw = null;
    protected Rectangle previousRectDrawn = new Rectangle();
    protected Rectangle perimeter;
    
    // reference to the mainframe's drawing area
    protected DrawingArea da;

    // reference to the Graphics context of the DrawingArea
    protected Graphics2D g;

    // flag to indicate the special hotspotting for nodes
    protected boolean nodesonly;
    // yet another flag which signals that circles are part the selection.
    // if that is so, only uniform scaling is allowed
    protected boolean containsCircles = false;

    // flag to indicate that boundary nodes are part of the selection
    // if that is so, translating is not allowed
    protected boolean boundaryNodesOnly = false;


    final static float dash1[] = {10.0f};

    final static BasicStroke dashed = new BasicStroke(1.0f, BasicStroke.CAP_BUTT, BasicStroke.JOIN_MITER, 10.0f, dash1, 0.0f);


    /**
     *  The constructor just sets references correctly
     *  and initializes some variables
     *
     *  @param mf - the MainFrame reference
     *  @param da - the drawing area where the select box will be painted on
     */    
    public BaseSelector (MainFrame mf, DrawingArea da) 
    {
	this.root = mf;
	this.da = da;
    }

    /**
     *  updates the graphics handle
     */
    public void setGraphics (Graphics g)
    {
	this.g = (Graphics2D)g;
    }


    /**
     *  @return true <=> circles are part of the current selection
     */
    public boolean containsCircles ()
    {
	return containsCircles;
    }

    /**
     *  @return true <=> boundarynodes part of the current selection
     */
    public boolean isBoundaryNodesOnly ()
    {
	return boundaryNodesOnly;
    }

    /**
     *  @return true <=> the selection contains a single node
     */
    public boolean isNodesOnly ()
    {
	return nodesonly;
    }


    /////////////////////////////////////////////////////////
    //
    // SELECT BOX HANDLING
    //
    ////////////////////////////////////////////////////////


    /**
     *  returns the current SelectBox as a Rectangle object
     *  (in screen coordinates, not in world coordinates)
     */
    public Rectangle getSelectBox ()
    {
	return rectToDraw;
    }

    /**
     *  paints the selection box to the drawing area's graphics context
     */
    public void paintSelectBox ()
    {
        if (currentRect != null) 
	{
	    // get the current graphics context
	    g = (Graphics2D)da.getGraphics ();
	    // set colors right
	    g.setXORMode (da.getBackground ());
	    g.setColor (cc.dc.color_lasso);
	    // temporarily, turn off all transformation, because the select
	    // box works with screen coordinates only
	    AffineTransform restoretrans = g.getTransform ();
	    AffineTransform newtrans = new AffineTransform ();
	    g.setTransform (newtrans);
	    Stroke savestroke = g.getStroke();
	    g.setStroke (dashed);
	    // remove the old rectangle
	    g.drawRect(previousRectDrawn.x, previousRectDrawn.y, previousRectDrawn.width - 1, previousRectDrawn.height - 1);
	    // ... and draw the new one
            g.drawRect(rectToDraw.x, rectToDraw.y, rectToDraw.width - 1, rectToDraw.height - 1);
	    //restore old transformation
	    g.setTransform (restoretrans);
	    g.setStroke (savestroke);
	    g.setPaintMode ();
	}
    }

    /**
     *  fixes some pervert degraded select box constellations
     *
     *  @param comp - the DrawingArea's actual size
     */
    private void updateDrawableRect(int compWidth, int compHeight) 
    {
        int x = currentRect.x;
        int y = currentRect.y;
        int width = currentRect.width;
        int height = currentRect.height;
        //Make the width and height positive, if necessary.
        if (width < 0) 
	{
            width = 0 - width;
            x = x - width + 1; 
            if (x < 0) 
	    {
                width += x; 
                x = 0;
            }
        }
        if (height < 0) 
	{
            height = 0 - height;
            y = y - height + 1; 
            if (y < 0) 
	    {
                height += y; 
                y = 0;
            }
        }
        //The rectangle shouldn't extend past the drawing area.
        if ((x + width) > compWidth) 
	{
            width = compWidth - x;
        }
        if ((y + height) > compHeight) 
	{
            height = compHeight - y;
        }
        //Update rectToDraw after saving old value.
        if (rectToDraw != null) 
	{
            previousRectDrawn.setBounds(rectToDraw.x, rectToDraw.y, rectToDraw.width, rectToDraw.height);
            rectToDraw.setBounds(x, y, width, height);
        } 
	else 
	{
            rectToDraw = new Rectangle(x, y, width, height);
        }
    }  


    /**
     *  starts the rectangle drawing process
     *  @param x,y - the event point of the pressedEvent
     */
    public void startSelectBox (int x, int y)
    {
	currentRect = new Rectangle(x, y, 0, 0);
	updateDrawableRect(da.getWidth(), da.getHeight());
	paintSelectBox ();
    }


    
    /** 
     *  Update the size of the select box and redraw it.
     *
     * @param x,y - the event points during the dragged event
     */
    public void updateSelectBox (int x, int y) 
    {
	currentRect.setSize(x - currentRect.x, y - currentRect.y);
	updateDrawableRect(da.getWidth(), da.getHeight());
	paintSelectBox ();
    }

    /**
     *  clears the gui feedback of the rectangle last drawn
     */
    public void clearSelectBox ()
    {
	g.setXORMode (da.getBackground ());
	g.setColor (cc.dc.color_lasso);
	g.drawRect (rectToDraw.x, rectToDraw.y, rectToDraw.width - 1, rectToDraw.height - 1);
	currentRect = null;
	rectToDraw = null;
	previousRectDrawn = new Rectangle ();
	g.setPaintMode ();
    }

//      /**
//       *  @return the node index at the given position
//       *  @param x,y - world coords
//       */
//      public Node getClickedNode (int x, int y)
//      {
//  	Enumeration enum = cc.d.nodes.elements ();
//  	while (enum.hasMoreElements())
//  	{
//  	    Node n = (Node)enum.nextElement();
//  	    if (n.isHit (x,y))
//  	    {
//  		return cc.d.getNodeIndex (n);
//  	    }
//  	}
//  	return -1;
//      }

    /**
     *  returns the GridItem which is positioned at the given world coords
     *  @return GridItem or null
     *  @param x,y - world coords!
     */
    public GridItem getClickedItem (int x, int y)
    {
	//System.out.println ("Tolerance: "+cc.dc.tolerance);
	//System.out.println ("Event Point: "+x+","+y);
	// try nodes first
	if (cc.selectNodes)
	{
	    Enumeration enum = cc.d.nodes.elements ();
	    while (enum.hasMoreElements())
	    {
		Node n = (Node)enum.nextElement();
		if (n.isHit (x,y))
		{
		    return n;
		}
	    }
	}
	// then edges
	if (cc.selectEdges)
	{
	    Enumeration enum = cc.d.edges.elements ();
	    while (enum.hasMoreElements())
	    {
	    
		Edge e = (Edge)enum.nextElement();
		if (e.isHit (x,y))
		{
		    return e;
		}
	    }
	}
	// then cells
	if (cc.selectElements)
	{
	    Enumeration enum = cc.d.cells.elements ();
	    while (enum.hasMoreElements())
	    {
		Cell e = (Cell)enum.nextElement();
		if (e.isHit (x,y))
		{
		    return e;
		}
	    }
	}
	// then segments
	if (cc.selectSegments)
	{
	    Enumeration enum = cc.d.db.getBoundaries().elements ();
	    while (enum.hasMoreElements ())
	    {
		Enumeration segenum = ((Boundary)enum.nextElement()).segments.elements();
		while (segenum.hasMoreElements ())
		{
		    GridItem n = (GridItem)segenum.nextElement ();
		    if (n.isHit (x,y))
		    {
			return n;
		    }
		}

	    }
	}
	return null;
    }

    /**
     *  returns all selected GridItems inside the currently 
     *  used rubber band / lasso. Also updates the contains... flags
     *
     *  @param additive - if true, selection is added to ...
     *  @param previous - previously selected items
     *  @return a Vector of GridItems
     */
    public Vector getSelectedItems (boolean additive, Vector previous)
    {
	Vector items = new Vector ();
	if (additive)
	{
	    Enumeration enum = previous.elements ();
	    while (enum.hasMoreElements())
		items.addElement (enum.nextElement());
	}
	else
	{
	    containsCircles = false;
	    boundaryNodesOnly = false;
	}
	// transform rect into world coordinates
	int[] topleft = cc.gc.fromScreenToWorld(rectToDraw.x, rectToDraw.y);
	int[] bottomright = cc.gc.fromScreenToWorld(rectToDraw.x + rectToDraw.width, rectToDraw.y + rectToDraw.height);
	// add all nodes inside the rubber band
	int nodecounter = 0;
	if (cc.selectNodes)
	{
	    Enumeration enum = cc.d.nodes.elements ();
	    while (enum.hasMoreElements ())
	    {
		Node n = (Node)enum.nextElement ();
		if (n.isInside (topleft, bottomright))
		{
		    if (!items.contains (n))
			items.addElement (n);
		    n.setMarker ();
		    nodecounter++;
		    if (n instanceof BoundaryNode)
			boundaryNodesOnly = true;
		}
		else
		{
		    n.unsetMarker ();
		}
	    }
	}
	if (nodecounter==1)
	    nodesonly = true;
	else
	    nodesonly = false;
	// then edges
	if (cc.selectEdges)
	{
	    Enumeration enum = cc.d.edges.elements ();
	    while (enum.hasMoreElements ())
	    {
		Edge e = (Edge)enum.nextElement ();
		if (e.isInside (topleft, bottomright))
		{
		    if (!items.contains (e))
			items.addElement (e);
		    e.setMarker ();
		}
		else
		{
		    e.unsetMarker ();
		}
	    }
	}
	// then elements
	if (cc.selectElements)
	{
	    Enumeration enum = cc.d.cells.elements ();
	    while (enum.hasMoreElements ())
	    {
		GridItem n = (GridItem)enum.nextElement ();
		if (n.isInside (topleft, bottomright))
		{
		    if (!items.contains (n))
			items.addElement (n);
		    n.setMarker ();
		    nodesonly = false;
		    boundaryNodesOnly = false;
		}
		else
		{
		    n.unsetMarker ();
		}

	    }
	}
	// segments
	if (cc.selectSegments)
	{
	    Enumeration enum = cc.d.db.getBoundaries().elements ();
	    while (enum.hasMoreElements ())
	    {
		Enumeration segenum = ((Boundary)enum.nextElement()).segments.elements();
		while (segenum.hasMoreElements ())
		{
		    GridItem n = (GridItem)segenum.nextElement ();
		    if (n.isInside (topleft, bottomright))
		    {
			if (!items.contains (n))
			    items.addElement (n);
			n.setMarker ();
			if (n instanceof SegmentCircle)
			    containsCircles = true;
			nodesonly = false;
			boundaryNodesOnly = false;
		    }
		    else
		    {
			n.unsetMarker ();
		    }
		}
		
	    }
	}
	return items;
    }
}






