package devisor2.grid.backend;

import java.awt.event.*;
import java.awt.*;
import java.awt.geom.*;
import java.util.*;

import devisor2.grid.GUI.framework.*;
import devisor2.grid.options.*;
import devisor2.grid.GUI.event.*;
import devisor2.foundation.elements.*;
import devisor2.foundation.base.*;
import devisor2.foundation.boundary.*;
import devisor2.foundation.*;


/**
 *  This class performs the actual cloning which is basically used for
 *  the copy&paste operation. Here is how it works to make sure that not only
 *  shallow copies are created, but that after cloning, the object references
 *  are correct. <br>
 *  A Vector of selected items is passed as input for this class.
 *  Firstly, the given nodes are cloned, which means all the nodes contained 
 *  in the vector and the nodes implicitely contained (as part of Tri and 
 *  Quad elements or as BoundaryNodes). The original ones and the cloned
 *  copies are both stored in an array at the same index so that later on, it
 *  is still possible to figure out what node is a clone of what node.<br>
 *  After that, the elements are cloned shallowly, and for every node
 *  contained in the original, its clone is assigned to the element clone. <br>
 *  After that, parent-child-relationships are updated in the cloned elements.
 *  Last but not least, the affine transform to the new position is
 *  applied, and everything is added to the Domain. <br>
 *  This class provides methods to perform that, but is not intelligent:
 *  The whole copy&paste operation is controlled by the GridListener and 
 *  MainActionListener classes. See there for more details about how
 *  this class is to be used.
 *
 *  @author Dominik Goeddeke
 *  @version should work fine, lets call it release candidate
 */
public class GridCloner
{
   // nothing new here
    protected ControlCenter cc;

    // a reference to the vector of selected items
    protected Vector originals;

    // the originals and clones
    protected Node[] origNodes, clonedNodes;
    protected Edge[] origEdges, clonedEdges;
    protected Cell2D[] origCells, clonedCells;
    protected Segment[] origSegments, clonedSegments;
    protected Boundary[] origBounds, clonedBounds;


    /**
     *  the constructor just sets the references correctly
     */
    public GridCloner (Vector itemsToBeCloned)
    {
	cc = ControlCenter.getMyself ();
	originals = itemsToBeCloned;
    }


    /**
     *  returns all cloned items in one vector
     */
    public Vector getClones ()
    {
	Vector v = new Vector ();
	if (clonedNodes != null)
	    for (int i=0; i<clonedNodes.length; i++)
		v.add (clonedNodes[i]);
	if (clonedEdges != null)
	    for (int i=0; i<clonedEdges.length; i++)
		v.add (clonedEdges[i]);
	if (clonedCells != null)
	    for (int i=0; i<clonedCells.length; i++)
		v.add (clonedCells[i]);
	if (clonedSegments != null)
	    for (int i=0; i<clonedSegments.length; i++)
		v.add (clonedSegments[i]);
	return v;
    }

    /**
     *  @return the originals of the boundaries (same indexing)
     */
    public Boundary[] getOrigBoundaries ()
    {
	return origBounds;
    }

    /**
     *  @return all clones of the boundaries (deep/full clones)
     */
    public Boundary[] getClonedBoundaries ()
    {
	return clonedBounds;
    }

    /**
     *  get cloned Segment of the given Segment
     */
    public Segment getClone (Segment s)
    {
	if (origSegments != null)
	{
	    int index = 0;
	    while (index < origSegments.length)
	    {
		if (s == origSegments[index])
		    return clonedSegments[index];
		else
		    index++;
	    }
	}
	return null;
    }

    /**
     *  @return the original segment belonging to the given segment on the given boundary
     */
    public Segment getOrigSegment (int index, int bno)
    {
	if (origSegments != null)
	{
	    int i=0;
	    while (i < origSegments.length)
	    {
		Segment s = origSegments[i];
		if (s.getSegmentNumber() == index && s.getBoundaryNumber ()==bno)
		    return s;
		else
		    i++;
	    }
		
	}
	return null;
    }


    /**
     *  creates a deep clone of the given items
     *  and stores it in the corresponding inner variable
     */
    public void createClone ()
    {
	Vector clones = new Vector();
	// add nodes to nodearray and clone
	gatherNodes ();
	cloneNodes ();
	// same for cells (and implicitly edges)
	gatherCells ();
	cloneCells ();
	// last for segments
	gatherSegments ();
	cloneSegments ();
	// Boundaries needed for cut paste
	cloneBoundaries ();

	// debugging: output everything

	//  System.out.println ("origNodes");
//  	print (origNodes);
//  	System.out.println ("clonedNodes");
//  	print (clonedNodes);
//  	System.out.println ("origEdges");
//  	print (origEdges);
//  	System.out.println ("clonedEdges");
//  	print (clonedEdges);
//  	System.out.println ("origCells");
//  	print (origCells);
//  	System.out.println ("clonedCells");
//  	print (clonedCells);
//  	System.out.println ("origSegments");
//  	print (origSegments);
//  	System.out.println ("clonedSegments");
//  	print (clonedSegments);
//  	System.out.println ("origBoundaries");
//  	print (origBounds);
//  	System.out.println ("clonedBoundaries");
//  	print (clonedBounds);

    }


    /**
     *  gathers all explicitly and implicitly selected nodes and fills
     *  the array orig_nodes.
     */
    private void gatherNodes ()
    {
	Node[] nodesarray = new Node[1];
	Vector tmp = new Vector ();
	// first, all nodes explicitly selected
	Enumeration enum = originals.elements ();
	while (enum.hasMoreElements ())
	{
	    GridItem item = (GridItem)enum.nextElement ();
	    if (item instanceof Node || item instanceof BoundaryNode)
		tmp.addElement (item);
	}
	// then, all nodes implicitly selected and not added yet
	enum = null;
	enum = originals.elements ();
	while (enum.hasMoreElements ())
	{
	    GridItem item = (GridItem)enum.nextElement ();
	    // Cells first (Tri, Quad, Macro)
	    if (item instanceof Cell)
	    {
		Node[] nds = ((Cell)item).updateCellNodes ();
		for (int i=0; i<nds.length; i++)
		{
		    if (!tmp.contains (nds[i]))
			tmp.addElement (nds[i]);
		}
	    }
	}
	// copy all into an array of NODES et voila
	origNodes = (Node[])tmp.toArray (nodesarray);
	if (origNodes[0]==null)
	    origNodes = null;
    }


    /**
     *  clones the Nodes array 
     */
    private void cloneNodes ()
    {
	if (origNodes != null)
	{
	    clonedNodes = new Node[origNodes.length];
	    for (int i=0; i<origNodes.length; i++)
	    {
		if (origNodes[i] instanceof BoundaryNode)
		    clonedNodes[i] = (BoundaryNode)origNodes[i].clone();
		else
		    clonedNodes[i] = (Node)origNodes[i].clone ();
	    }
	}
    }


    /**
     *  collects the Cells into an array origCells, as for full clones, the 
     *  edges are also required, this is performed here as well.
     */
    private void gatherCells ()
    {
	// Cells first
	Cell2D[] cellarray = new Cell2D[1];
	Vector tmp = new Vector ();
	Enumeration enum = originals.elements ();
	while (enum.hasMoreElements ())
	{
	    GridItem item = (GridItem)enum.nextElement ();
	    if (item instanceof Cell2D)
		tmp.addElement (item);
	}
	origCells = (Cell2D[])tmp.toArray (cellarray);
	if (origCells[0] == null)
	    origCells = null;
	// then edges
	Edge[] edgearray = new Edge[1];
	tmp = new Vector ();
	if (origCells != null)
	{
	    for (int c=0; c<origCells.length; c++)
	    {
		for (int i=0; i<origCells[c].getEdges().length; i++)
		    if (!tmp.contains (origCells[c].getEdge(i)))
			tmp.addElement (origCells[c].getEdge (i));
	    }
	}
	else
	{
	    enum = originals.elements ();
	    while (enum.hasMoreElements ())
	    {
		GridItem item = (GridItem)enum.nextElement ();
		if (item instanceof Edge)
		    tmp.addElement (item);
	    }
	}
	origEdges = (Edge[])tmp.toArray (edgearray);
	if (origEdges[0] == null)
	    origEdges = null;
    }


    /**
     *  firstly, clones the edges, but still keeps references to the original 
     *  nodes. After that, clones the Cells using the new clones of the edges.
     */
    private void cloneCells ()
    {
	int index;
	if (origEdges != null)
	{
	    // (1) Edges shallowly
	    clonedEdges = new Edge[origEdges.length];
	    for (int i=0; i<origEdges.length; i++)
		clonedEdges[i] = (Edge)origEdges[i].clone ();
	    // (2) Edges deeply
	    for (int i=0; i<origEdges.length; i++)
		for (int j=0; j<=1; j++)
		{
		    index = locateNode (origEdges[i].getNode(j));
		    if (index != -1)  // should not happen due to construction
		    {
			clonedEdges[i].setNode (j, clonedNodes[index]);
		    }
		}
	}
	if (origCells != null)
	{
	    // (3) Cells shallowly
	    clonedCells = new Cell2D[origCells.length];
	    for (int i=0; i<origCells.length; i++)
	    {
		clonedCells[i] = (Cell2D)origCells[i].clone();
	    }
	    // (4) Cells deeply
	    for (int i=0; i<origCells.length; i++)
	    {
		Edge[] cellEdges = origCells[i].getEdges ();
		Edge[] newEdges = new Edge [cellEdges.length];
		for (int j=0; j<cellEdges.length; j++)
		{
		    index = locateEdge (cellEdges[j]);
		    newEdges[j] = clonedEdges[index];
		}
		clonedCells[i].setCell (origCells[i].getNumber(), newEdges);
	    }
	}
    }


    /**
     *  gathers all Segments in the collection and stores them in the 
     *  origSegments array
     */
    private void gatherSegments ()
    {
	Segment[] segmentarray = new Segment[1];
	Vector tmp = new Vector ();
	Enumeration enum = originals.elements ();
	while (enum.hasMoreElements ())
	{
	    GridItem item = (GridItem)enum.nextElement ();
	    if (item instanceof Segment)
		tmp.addElement (item);
	}
	origSegments = (Segment[])tmp.toArray (segmentarray);
	if (origSegments[0] == null)
	    origSegments = null;
    }

    /**
     *  clones the segments in the origSegments vector and stores
     *  the clones in the clonedSegments array
     */
    private void cloneSegments ()
    {
	if (origSegments != null)
	{
	    clonedSegments = new Segment[origSegments.length];
	    for (int i=0; i<origSegments.length; i++)
	    {
		clonedSegments[i] = (Segment)origSegments[i].clone ();
	    }
	}
    }

    /**
     *  clones Boundaries and sets references in boundary nodes and segments
     */
    private void cloneBoundaries ()
    {
	// first shallowly
	Boundary[] bb = new Boundary[1];
	origBounds = (Boundary[])cc.d.db.getBoundaries().toArray (bb);
	if (origBounds[0] == null)
	    origBounds = null;
	if (origBounds != null)
	{
	    clonedBounds = new Boundary[origBounds.length];
	    for (int i=0; i< origBounds.length; i++)
		clonedBounds[i] = (Boundary)origBounds[i];
	}
	else
	{
	    clonedBounds = null;
	}
    }

    /**
     *  simple linear search for the given node in the origNodes array.
     *  object reference suffices, equals is not what we want.
     *  @return -1 if not found, index otherwise
     */
    private int locateNode (Node n)
    {
	int index = 0;
	if (origNodes != null)
	{
	    while (index < origNodes.length)
	    {
		if (n == origNodes[index])
		    return index;
		else
		    index++;
	    }
	}
	return -1;
    }

    /**
     *  same for Edges
     */
    private int locateEdge (Edge e)
    {
	if (origEdges != null)
	{
	    int index = 0;
	    while (index < origEdges.length)
	    {
		if (e == origEdges[index])
		    return index;
		else
		    index++;
	    }
	}
	return -1;
    }

    /**
     *  debug purposes: prints out the contents of the given array
     */
    public void print (Object[] array)
    {
	if (array != null)
	    for (int i=0; i<array.length; i++)
		System.out.println (array[i]);
	else
	    System.out.println ("empty");
    }

}












