package devisor2.grid.backend;

import java.awt.event.*;
import java.awt.*;
import java.awt.geom.*;
import java.util.*;

import devisor2.grid.GUI.framework.*;
import devisor2.grid.options.*;
import devisor2.grid.GUI.event.*;
import devisor2.foundation.elements.*;
import devisor2.foundation.base.*;
import devisor2.foundation.boundary.*;
import devisor2.foundation.*;


/**
 *  This class adds the hotspotting mechanism to the BaseSelector.
 *
 *  @author Dominik Goeddeke
 *  @version 0.9 release candidate
 */
public abstract class GridHotspotter extends BaseSelector
{

    /**
     *  this array holds the flags which hotspots are currently active
     *  and are therefore to be drawn and have to be sensitive to user input
     *  read it top to bottom left to right
     */
    protected boolean[] hotspotMask;

    /**
     *  constant to indicate that no hotspot has been hit
     */
    public static final int HOTSPOT_NONE = -1;

    /**
     * constants to indicate the number of the hotspot that has been hit
     */
    public static final int HOTSPOT_TOPLEFT = 10;
    public static final int HOTSPOT_TOPCENTER = 11;
    public static final int HOTSPOT_TOPRIGHT = 12;
    public static final int HOTSPOT_CENTERLEFT = 13;
    public static final int HOTSPOT_CENTERCENTER = 14;
    public static final int HOTSPOT_CENTERRIGHT = 15;
    public static final int HOTSPOT_BOTTOMLEFT = 16;
    public static final int HOTSPOT_BOTTOMCENTER = 17;
    public static final int HOTSPOT_BOTTOMRIGHT = 18;

    /**
     *  the hotspot mask for all hotspots
     */
    public static final boolean[] MASK_ALL = { true, true, true, true, true, true, true, true, true };

    /**
     *  the hotspot mask for nodes only
     */
    public static final boolean[] MASK_NODES = { false, false, false, false, true, false, false, false, false };

    /**
     *  the hotspot mask for rotating
     */
    public static final boolean[] MASK_ROTATE = { false, true, false, true, false, true, false, true, false };

    /**
     *  the hotspot mask for scaling when circles are involved
     */
    public static final boolean[] MASK_CIRCLES = {true, false, true, false, true, false, true, false, true };

  
    /**
     *  The constructor just sets references correctly
     *  and initializes some variables
     *
     *  @param mf - the MainFrame reference
     *  @param da - the drawing area where the select box will be painted on
     */    
    public GridHotspotter (MainFrame mf, DrawingArea da) 
    {
	super (mf,da);
    }




    ////////////////////////////////////////////////////////
    //
    // HOTSPOT HANDLING
    //
    ///////////////////////////////////////////////////////

    /**
     *  sets the hotspot mask to the given value, please use MASK??? constants
     *  provided
     *  @param mask - the new gotspot mask, use constants provided
     */
    public void setHotspotMask (boolean[] mask)
    {
	hotspotMask = mask;
    }

    /**
     *  @return the currently active hotspot mask
     */
    public boolean[] getHotspotMask ()
    {
	return hotspotMask;
    }

    /**
     *  updates the hotspot display, as XOR painting is used, this method
     *  draws hotspots if there are none, and removes them if they are 
     *  already visible. Of course, the current hotspot mask is used.
     *  @see setHotspotMask
     */
    public void updateHotspots ()
    {
	if (hotspotMask == null)
	{
	    //	    System.out.println ("Fuck you. Set the hotspot mask first");
	    return;
	}
	// switch on correct XOR mode
	g.setXORMode (da.getBackground ());
	g.setColor (cc.dc.color_lasso);
	// draw top row
	if (hotspotMask[0])
	    g.fillRect (perimeter.x-4,perimeter.y-4,8,8);
	if (hotspotMask[1])
	    g.fillRect (perimeter.x+perimeter.width/2-4,perimeter.y-4,8,8);
	if (hotspotMask[2])
	    g.fillRect (perimeter.x+perimeter.width-4,perimeter.y-4,8,8);
	// draw center row
	if (hotspotMask[3])
	    g.fillRect (perimeter.x-4,perimeter.y+perimeter.height/2-4,8,8);
	if (hotspotMask[4])
	    g.fillRect (perimeter.x+perimeter.width/2-4,perimeter.y+perimeter.height/2-4,8,8);
	if (hotspotMask[5])
	    g.fillRect (perimeter.x+perimeter.width-4,perimeter.y+perimeter.height/2-4,8,8);
	// draw bottom row
	if (hotspotMask[6])
	    g.fillRect (perimeter.x-4,perimeter.y+perimeter.height-4,8,8);
	if (hotspotMask[7])
	    g.fillRect (perimeter.x+perimeter.width/2-4,perimeter.y+perimeter.height-4,8,8);
	if (hotspotMask[8])
	    g.fillRect (perimeter.x+perimeter.width-4,perimeter.y+perimeter.height-4,8,8);
	if (hotspotMask != MASK_NODES)
	{
	    Stroke savestroke = g.getStroke();
	    g.setStroke (dashed);
	    g.drawLine (perimeter.x,perimeter.y, perimeter.x+perimeter.width, perimeter.y);
	    g.drawLine (perimeter.x+perimeter.width, perimeter.y,perimeter.x+perimeter.width, perimeter.y+perimeter.height);
	    g.drawLine (perimeter.x+perimeter.width, perimeter.y+perimeter.height,perimeter.x, perimeter.y+perimeter.height);
	    g.drawLine (perimeter.x, perimeter.y+perimeter.height,perimeter.x, perimeter.y);
	    g.setStroke (savestroke);
	}
	g.setPaintMode ();
    }

   
    /**
     *  @return the screen coords of the center hotspot
     */
    public Point getCenterHotspot ()
    {
	return new Point (perimeter.x+perimeter.width/2, perimeter.y + perimeter.height/2);
    }

    /**
     *  same for the topleft corner
     */
    public Point getTopLeftHotspot ()
    {
	return new Point (perimeter.x, perimeter.y);
    }


    /**
     *  @return the screen coords of the clicked hotspot
     */
    public Point getClickedHotspotCoords (int hotspot)
    {
	switch (hotspot)
	    {
	    case HOTSPOT_TOPLEFT :
		return new Point (perimeter.x, perimeter.y);
	    case HOTSPOT_TOPCENTER :
		return new Point (perimeter.x+perimeter.width/2, perimeter.y);
	    case HOTSPOT_TOPRIGHT :
		return new Point (perimeter.x + perimeter.width, perimeter.y);
	    case HOTSPOT_CENTERLEFT :
		return new Point (perimeter.x, perimeter.y+perimeter.height/2);
	    case HOTSPOT_CENTERCENTER :
		return new Point (perimeter.x+perimeter.width/2, perimeter.y + perimeter.height/2);
	    case HOTSPOT_CENTERRIGHT :
		return new Point (perimeter.x + perimeter.width, perimeter.y+perimeter.height/2);
	    case HOTSPOT_BOTTOMLEFT :
		return new Point (perimeter.x, perimeter.y+perimeter.height);
	    case HOTSPOT_BOTTOMCENTER :
		return new Point (perimeter.x+perimeter.width/2, perimeter.y + perimeter.height);
	    case HOTSPOT_BOTTOMRIGHT :
		return new Point (perimeter.x + perimeter.width, perimeter.y+perimeter.height);
	    }
	return null;
    }


    /**
     *  @param hotspot - the hotspot last clicked on
     *  @return the screen coords of the hotspot opposite the one clicked on,
     *  e.g. if hotspot equals TOPLEFT, the coords of the BOTTOMRIGHT hotspot
     *  are returned
     */
    public Point getOppositeHotspotCoords (int hotspot)
    {
	switch (hotspot)
	    {
	    case HOTSPOT_TOPLEFT :
		return new Point (perimeter.x+perimeter.width, perimeter.y+perimeter.height);
	    case HOTSPOT_TOPCENTER :
		return new Point (perimeter.x+perimeter.width/2, perimeter.y+perimeter.height);
	    case HOTSPOT_TOPRIGHT :
		return new Point (perimeter.x, perimeter.y + perimeter.height);
	    case HOTSPOT_CENTERLEFT :
		return new Point (perimeter.x+perimeter.width, perimeter.y+perimeter.height/2);
	    case HOTSPOT_CENTERRIGHT :
		return new Point (perimeter.x, perimeter.y + perimeter.height/2);
	    case HOTSPOT_BOTTOMLEFT :
		return new Point (perimeter.x+perimeter.width, perimeter.y);
	    case HOTSPOT_BOTTOMCENTER :
		return new Point (perimeter.x+perimeter.width/2, perimeter.y);
	    case HOTSPOT_BOTTOMRIGHT :
		return new Point (perimeter.x, perimeter.y);
	    }
	return null;
    }

   
    /**
     *  sets the perimeter rectangle used for scaling, rotating and 
     *  translating, especially for the visual feedback for these
     *  operations
     */
    public void setPerimeter (Rectangle peri)
    {
	perimeter = peri;
	// fix size and orientation
	if (!nodesonly)
	{
	    if (perimeter.width < 0)
	    {
		perimeter.x = perimeter.x + perimeter.width;
		perimeter.width = Math.abs (perimeter.width);
	    }
	    if (perimeter.height < 0)
	    {
		perimeter.y = perimeter.y + perimeter.height;
		perimeter.height = Math.abs (perimeter.height);
	    }
	    if (perimeter.width < 26)
		perimeter.grow ((30-perimeter.width)/2,0);
	    if (perimeter.height < 26)
		perimeter.grow (0,(30-perimeter.height)/2);
	}
    }

    /**
     *  determines what hotspot has been hit (use constants provided). Only 
     *  hotspots that siut the hotspot mask currently set are accepted.
     *  If no hotspot is currently visible, alwazs NONE is returned.
     *
     *  @param x,y - the click point in screen coords
     *  @return one of the hotspot constants (see below)
     */
    public int getClickedHotspot (int x, int y)
    {
	if (perimeter != null && hotspotMask != null)
	{
	    // top row
	    if (hotspotMask[0])
		if (x >= perimeter.x-4 && x <= perimeter.x+4 && y >=perimeter.y-4 && y <= perimeter.y+4)
		    return HOTSPOT_TOPLEFT;
	    if (hotspotMask[1])
		if (x >= perimeter.x+perimeter.width/2-4 && x <= perimeter.x+perimeter.width/2+4 && y >=perimeter.y-4 && y <= perimeter.y+4)
		    return HOTSPOT_TOPCENTER;
	    if (hotspotMask[2])
		if (x >= perimeter.x+perimeter.width-4 && x <= perimeter.x+perimeter.width+4 && y >=perimeter.y-4 && y <= perimeter.y+4)
		    return HOTSPOT_TOPRIGHT;
	    // center row
	    if (hotspotMask[3])
		if (x >= perimeter.x-4 && x <= perimeter.x+4 && y >=perimeter.y+perimeter.height/2-4 && y <= perimeter.y+perimeter.height/2+4)
		    return HOTSPOT_CENTERLEFT;
	    if (hotspotMask[4])
		if (x >= perimeter.x+perimeter.width/2 -4 && x <= perimeter.x+perimeter.width/2 +4 && y >=perimeter.y+perimeter.height/2 -4 && y <= perimeter.y+perimeter.height/2 +4)
		    return HOTSPOT_CENTERCENTER;
	    if (hotspotMask[5])
		if (x >= perimeter.x+perimeter.width-4 && x <= perimeter.x+perimeter.width+4 && y >=perimeter.y+perimeter.height/2-4 && y <= perimeter.y+perimeter.height/2+4)
		    return HOTSPOT_CENTERRIGHT;   
	    // bottom row
	    if (hotspotMask[6])
		if (x >= perimeter.x-4 && x <= perimeter.x+4 && y >=perimeter.y+perimeter.height-4 && y <= perimeter.y+perimeter.height+4)
		    return HOTSPOT_BOTTOMLEFT;
	    if (hotspotMask[7])
		if (x >= perimeter.x+perimeter.width/2-4 && x <= perimeter.x+perimeter.width/2+4 && y >=perimeter.y+perimeter.height-4 && y <= perimeter.y+perimeter.height+4)
		    return HOTSPOT_BOTTOMCENTER;
	    if (hotspotMask[8])
		if (x >= perimeter.x+perimeter.width-4 && x <= perimeter.x+perimeter.width+4 && y >=perimeter.y+perimeter.height-4 && y <= perimeter.y+perimeter.height+4)
		    return HOTSPOT_BOTTOMRIGHT;
  	}
	return HOTSPOT_NONE;
    }

}
