package devisor2.grid.backend;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;

import devisor2.grid.GUI.dialogs.*;
import devisor2.grid.options.*;
import devisor2.foundation.*;
import devisor2.foundation.base.*;
import devisor2.foundation.boundary.*;
import devisor2.foundation.elements.*;
import devisor2.grid.GUI.framework.*;
import devisor2.grid.GUI.event.*;

/**
 *  This class provides some useful methods to handle objects on the grid.
 *  - Cutting / deleting 
 *  - updating boundary information
 *  - fixing line elements after zooming
 *  - mirroring
 *
 *  @author Dominik Goeddeke
 *  @version beta
 */
public class GridToolkit
{

    private ControlCenter cc;

    private MainFrame parent;

    public GridToolkit (MainFrame parent)
    {
	this.parent = parent;
	cc = ControlCenter.getMyself ();
    }


    /**
     *  deletes the given GridItems.
     *  If nodes are selectable, all Boundary nodes are deleted without
     *  further delay, if nodes are not selectable, bns are transformed
     *  into regular nodes and permanently (!) lose all parametrisation,
     *  all element information is kept
     *
     *  @param useless - the GridItems to be deleted
     *  @return a vector of implicitly deleted items, neccessary for undo
     */
    public Vector delete (Vector useless)
    {
	Vector savedsegs = new Vector ();
	Vector impldeleted = new Vector ();
	boolean wbdeleted = false;
	Enumeration enum = useless.elements();
	while (enum.hasMoreElements())
	{
	    GridItem item = (GridItem)enum.nextElement();
	    if (item instanceof Segment)
		savedsegs.add (item);
	    if (item instanceof Node)
	    {
		Node n = (Node)item;
		if (n.getParents()!=null)
		{
		    Edge[] adges = n.getParents();
		    for(int e=0;e<adges.length;e++)
		    {
			if (adges[e]!=null)
			{
			    Cell[] zells = (Cell[])adges[e].getParents();
			    if (zells!=null)
			    {
				for(int c=0;c<zells.length;c++)
				{
				    if (zells[c]!=null) 
				    {
					impldeleted.add (zells[c]);
				    }
				}
			    }
			}
		    }				
		}
	    }
	}
	enum = useless.elements ();
	while (enum.hasMoreElements ())
	{
	    GridItem item = (GridItem)enum.nextElement ();
	    cc.d.del (item);
	}
	// delete all empty boundaries
	enum = cc.d.db.getBoundaries().elements();
	while (enum.hasMoreElements ())
	{
	    Boundary b = (Boundary)enum.nextElement();
	    if (b.getSegmentCount()==0)
	    {
		cc.d.db.delBoundary(b);
		if (b == cc.gc.workingBoundary)
		    wbdeleted = true;
	    }
	}
	// convert boundary nodes to inner nodes on deleted segments
	convertBoundaryNodes (savedsegs);
	// fix parametrisation of boundary nodes on follow-up segments
	fixBoundaryNodeParametrisation ();
	// fix working boundary and GUI feedback
	if (wbdeleted)
	{
	    cc.gc.workingBoundary = cc.d.db.getLastBoundary ();
	    updateBoundaryCombos ();
	}
	cc.d.db.updateNumbering ();
	return impldeleted;
    }

    /**
     *  on all given segments, boundary nodes are converted to inner nodes,
     *  keeping all element structure
     */
    private void convertBoundaryNodes (Vector savedsegs)
    {
	Vector deletableNodes = new Vector ();
	Vector replacementNodes = new Vector ();
	Enumeration enum = cc.d.getNodes ().elements ();
	while (enum.hasMoreElements())
	{
	    Node n = (Node)enum.nextElement();
	    if (n instanceof BoundaryNode)
	    {
		BoundaryNode bn = (BoundaryNode)n;
		if (savedsegs.contains (bn.getSegment()))
		{
		    Node newnode = new Node (bn.getCoord(0), bn.getCoord(1));
		    Edge[] edges = bn.getParents ();
		    if (edges != null)
			for (int i=0; i<edges.length;i++)
			{
			    int index = 0;
			    if (edges[i].getNode (0) != bn)
				index = 1;
			    edges[i].setNode (index,newnode);
			    newnode.addParent (edges[i]);
			}
		    deletableNodes.add (bn);
		    replacementNodes.add (newnode);
		}
	    }
	}
	enum = deletableNodes.elements ();
	while (enum.hasMoreElements())
	{
	    // hard-coded kick out of the nodes vector to keep elements 
	    // where they are
	    cc.d.getNodes().remove ((Node)enum.nextElement());
	}
	enum = replacementNodes.elements ();
	while (enum.hasMoreElements())
	{
	    cc.d.adjustNode ((Node)enum.nextElement ());
	}
    }

    /**
     *  fixes the parametrisation of all boundary nodes on follow-up
     *  segments of the given ones
     */
    private void fixBoundaryNodeParametrisation ()
    {
	Enumeration enum = cc.d.getNodes().elements ();
	while (enum.hasMoreElements ())
	{
	    Node node = (Node)enum.nextElement ();
	    if (node instanceof BoundaryNode)
	    {
		BoundaryNode bn = (BoundaryNode)node;
		bn = cc.d.db.reconfigureBoundaryNode (bn);
	    }
	}

    }

    /**
     *  clears the transformed flag of all elements, segments and nodes in
     *  the domain
     */
    public void clearAllTransformedFlags ()
    {
	Enumeration enum = cc.d.nodes.elements ();
	while (enum.hasMoreElements())
	    ((Node)enum.nextElement()).clearTransformedFlag ();
	enum = cc.d.edges.elements ();
	while (enum.hasMoreElements())
	    ((Edge)enum.nextElement()).clearTransformedFlag ();
	enum = cc.d.cells.elements ();
	while (enum.hasMoreElements())
	    ((Cell)enum.nextElement()).clearTransformedFlag ();
	enum = cc.d.db.getBoundaries().elements ();
	while (enum.hasMoreElements ())
	{
	    Enumeration segs = ((Boundary)enum.nextElement()).segments.elements();
	    while (segs.hasMoreElements())
		((Segment)segs.nextElement()).clearTransformedFlag();
	}
    }

    /**
     *  updates all comboboxes in the application which hold boundary numbers
     */
    public void updateBoundaryCombos ()
    {
	Runnable updatecombos = new Runnable() 
	    {
		public void run() 
		{ 
		    cc.dm.refreshAllGUI ();
		    MainActionListener listener = parent.getMainActionListener ();
		    JComboBox toolbarcombo = parent.getToolBar().getSelectBoundaryCombo();
		    toolbarcombo.removeActionListener (listener);
		    toolbarcombo.removeAllItems ();
		    if (cc.gc.workingBoundary == null)
		    {
			toolbarcombo.addItem ("-1");
			toolbarcombo.setSelectedItem ("-1");
		    }
		    else
		    {
			for (int i=0; i< cc.d.db.getBoundaryCount(); i++)
			    toolbarcombo.addItem (Integer.toString(i+1));
			toolbarcombo.setSelectedItem (Integer.toString(cc.gc.workingBoundary.getNumber ()+1));
		    }
		    toolbarcombo.addActionListener (listener);
		}
	    };
	SwingUtilities.invokeLater(updatecombos);
    }


    /**
     *  after zooming, fixes the bounding boxes of edges and segmentlines
     *  which must be updated as the tolerance factor in user coords
     *  has changed
     */
    public void fixBoundingBoxes ()
    {
	Enumeration enum = cc.d.getEdges().elements ();
	while (enum.hasMoreElements())
	    ((Edge)enum.nextElement()).updateInnerData ();
	enum = cc.d.db.getBoundaries().elements();
	while (enum.hasMoreElements())
	{
	    Boundary b = (Boundary)enum.nextElement();
	    Enumeration segs = b.getSegments().elements();
	    while (segs.hasMoreElements())
	    {
		Segment s = (Segment)segs.nextElement ();
		if (s instanceof SegmentLine)
		    s.updateInnerData ();
	    }
	}
    }


    /**
     *  mirrors all selected items at the line from (x1,y1) to (x2,y2)
     */
    public void mirror (int x1, int y1, int x2, int y2)
    {
	// get selected items
	final Vector selectedItems = parent.getDrawingArea().getListener().getSelectedItems();
	// get normal vector of this line and scale down to length 1
	double sx = x2-x1;
	double sy = y2-y1;
	double[] n = {sy,sx};
	if (Math.abs (sx) > Math.abs (sy))
	    n[1] = -n[1];
	else
	    n[0] = -n[0];
	double l = Math.sqrt (n[0]*n[0] + n[1]*n[1]);
	n[0] /= l;
	n[1] /= l;
	// clear transformation flag of all selected items
	Enumeration enum = selectedItems.elements ();
	while (enum.hasMoreElements ())
	    ((GridItem)enum.nextElement()).clearTransformedFlag ();
	// and mirror them
	enum = selectedItems.elements();
	while (enum.hasMoreElements())
	{
	    ((GridItem)enum.nextElement()).performMirror (n[0], n[1], x1,y1);
	}
	parent.getDrawingArea().repaint ();
	final GridSelector gs = parent.getDrawingArea().getListener().getSelector ();
	final ControlCenter cc = this.cc;
	Runnable updatePerimeter = new Runnable() 
	{
	    public void run() 
	    { 
		Rectangle perimeterworld = new Rectangle ();
		try
		{
		    perimeterworld = ((GridItem)selectedItems.firstElement()).getPerimeter();
		} catch (NoSuchElementException exc) {}
		if (!perimeterworld.equals (new Rectangle ()))
		{
		    Enumeration enum = selectedItems.elements ();
		    while (enum.hasMoreElements ())
		    {
			perimeterworld = perimeterworld.union (((GridItem)enum.nextElement ()).getPerimeter ());
		    }
		    // pass perimeter to GridSelector and add hotspots
		    int[] screen1 = cc.gc.fromWorldToScreen(perimeterworld.x, perimeterworld.y);
		    int[] screen2 = cc.gc.fromWorldToScreen(perimeterworld.x+perimeterworld.width, perimeterworld.y+perimeterworld.height);
		    Rectangle perimeter = new Rectangle (screen1[0], screen1[1],screen2[0] - screen1[0], screen2[1] - screen1[1]);
		    gs.setPerimeter (perimeter);
		    gs.updateHotspots ();
		}
	    }
	};
	SwingUtilities.invokeLater(updatePerimeter);
    }
}
