package devisor2.grid.backend.spellchecker;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.table.*;

import java.util.*;

import devisor2.grid.options.*;
import devisor2.foundation.elements.*;
import devisor2.foundation.base.*;
import devisor2.foundation.boundary.*;

/**
 *  The underlying table model for the JTable on the 
 *  SpellCheckerDialog
 *  <br>
 *  The whole functionality is distrubuted via several methods which are 
 *  called by the constructor. Each of these methods searches for certain 
 *  'spelling errors' and adds them to the list.
 *  <br>
 *  The implementation is pretty straightforward, see the API 
 *  (AbstractTableModel) for details.
 *
 *  @author Dominik Goeddeke
 *  @version 1.0 (fully tested)
 */
class SpellCheckerTableModel extends AbstractTableModel
{
    /**
     *  the names of the columns
     */
    private String[] columnNames;

    private SpellCheckerFilter filter;

    /**
     *  the data stored in the table, see below for explanation
     */
    private Vector data;
    /**
     *  the usual control center reference
     */
    private ControlCenter cc;

    /**
     *  creates a new instance, searches the global Options instance for
     *  all keys starting with "accelerator" and takes these as the data 
     *  source.
     */
    public SpellCheckerTableModel (SpellCheckerFilter filt)
    {
	data = new Vector ();
	cc = ControlCenter.getMyself ();
	filter = filt;
	columnNames = new String[2];
	columnNames[0] = (String)cc.rb.getObject ("spellchecker_tablemodel_columnheader_type");
	columnNames[1] = (String)cc.rb.getObject ("spellchecker_tablemodel_columnheader_number");
	if (filter.isIsolatedNodes ())
	    searchIsolatedNodes ();
	if (filter.isInnerAngles())
	    searchInnerAngles (filter.getInnerAnglesThreshold());
	if (filter.isAspectRatio())
	    searchAspectRatios (filter.getAspectRatioThreshold());
	if (filter.isNeighbouring())
	    searchNeighbouring (filter.getNeighbouringThreshold());
    }

    /**
     *  adds all isolated nodes to the list
     */
    private void searchIsolatedNodes ()
    {
	Enumeration nodes = cc.d.getNodes().elements();
	while (nodes.hasMoreElements())
	{
	    Node n = (Node)nodes.nextElement();
	    if (n.getParents()==null || n.getParents().length==0)
	    {
		String[] pair = new String[2];
		pair[0] = cc.wS("spellchecker_tablemodel_isolatednode");
		pair[1] = Integer.toString (n.getNumber()+1);
		data.add (pair);
	    }
	}
    }

    /**
     *  adds all inner angle deformations to the list
     */
    private void searchInnerAngles (double threshold)
    {
	Enumeration cells = cc.d.getCells().elements();
	while (cells.hasMoreElements())
	{
	    Cell2D c = (Cell2D)cells.nextElement();
	    if (c.getSmallestInnerAngle() <= threshold)
	    {
		String[] pair = new String[2];
		pair[0] = cc.wS("spellchecker_tablemodel_innerangle");
		pair[1] = Integer.toString (c.getNumber()+1);
		data.add (pair);
	    }
	}
    }

    /**
     *  adds all elements with large aspect ratios to the list
     */
    private void searchAspectRatios (double threshold)
    {
	Enumeration cells = cc.d.getCells().elements();
	while (cells.hasMoreElements())
	{
	    Cell2D c = (Cell2D)cells.nextElement();
	    if (c.getAspectRatio() <= threshold)
	    {
		String[] pair = new String[2];
		pair[0] = cc.wS("spellchecker_tablemodel_aspectratio");
		pair[1] = Integer.toString (c.getNumber()+1);
		data.add (pair);
	    }
	}
    }

    /**
     *  adds all elements with large area difference to neighbours to the list
     */
    private void searchNeighbouring (double threshold)
    {
	Enumeration cells = cc.d.getCells().elements();
	while (cells.hasMoreElements())
	{
	    Cell2D c = (Cell2D)cells.nextElement();
	    Cell[] neighs = (Cell[])c.updateCellNeighbours();
	    boolean found = false;
	    int i=0;
	    while (!found && i<neighs.length)
	    {
		double q = c.getArea()/((Cell2D)neighs[i]).getArea();
		if (q > threshold)
		    found = true;
		else
		    i++;
	    }
	    if (found)
	    {
		String[] pair = new String[2];
		pair[0] = cc.wS("spellchecker_tablemodel_neighbours");
		pair[1] = Integer.toString (c.getNumber()+1);
		data.add (pair);
	    }
	}
    }


    /**
     *  returns if the cell at (row,col) is editable
     */
    public boolean isCellEditable(int row, int column) 
    {
	return false;
    }

    /**
     *  returns the number of columns in the table
     */
    public int getColumnCount() 
    {
        return columnNames.length;
    }

    /**
     *returns the number of rows in the table
     */
    public int getRowCount() 
    {
        return data.size ();
    }

    /**
     *  returns the name of the column at the given position
     */
    public String getColumnName(int col) 
    {
        return columnNames[col];
    }

    /**
     *  returns the value stored in the table at the given position
     */
    public Object getValueAt(int row, int col) 
    {
	if (col == 0)
	    return ((String[])data.elementAt(row))[0];
	else
	    return ((String[])data.elementAt(row))[1];
    }

    /**
     *  returns the data source of the TableModel.
     *  Data structure: the index in the vector corresponds with the row
     *  in the table, each entry is a two-dimensional array of strings,
     *  first entry is the accelerator key from the Options instance,
     *  second entry is the assigned shortcut.
     *
     *  @return a vector of String[2] objects, the first element being a key,
     *          the second element being the value
     */
    public Vector getData ()
    {
	return data;
    }




}







