package devisor2.grid.backend.undo;


import devisor2.grid.options.*;
import devisor2.grid.GUI.framework.*;
import devisor2.grid.GUI.event.*;
import devisor2.grid.GUI.dialogs.*;
import devisor2.foundation.*;
import devisor2.foundation.elements.*;
import devisor2.foundation.boundary.*;
import devisor2.foundation.base.*;
import devisor2.grid.backend.*;

import java.util.*;
import java.awt.geom.*;

/**
 *  This class is responsible for managing all UNDO information. It basically 
 *  provides three core methods:<br>
 *  <code>notifyUndo()<\code> is used whenever an undoable operation takes
 *  place. All information neccessary to take back the operation is passed
 *  to this method, and an UndoContainer object is created and pushed on the 
 *  UndoStack.<br>
 *  <code>performUndo()<\code> is called whenever the UNDO operation is 
 *  called. This method is a bit lengthy to implement, scroll down and you 
 *  will see why. <br>
 *  <code>performRedo()<\code> is called whenever the REDO operation is 
 *  called. Note that only the undos can be redone that took place BEFORE the
 *  last regular operation. <br>
 *  Additionally, this class provides constants for all undoable operations.
 *
 *  Take a look at the technical paper about UNDO if you want to learn more
 *  about the full undo implementation.
 *
 *  @see UndoStack, UndoContainer
 *  @author Dominik Goeddeke
 *
 */
public class UndoManager
{

    /**
     *  The all important ControlCenter reference linking to the rest of the
     *  application
     */
    private ControlCenter cc;

    /**
     *  The equally important MainFrame reference
     */
    private MainFrame parent;

    /**
     *  The UndoStack reference used to store undoable operations
     */
    private UndoStack stack;

    /**
     *  a stack of the same type is used for redoing
     */
    private UndoStack redostack;

    /**
     *  The constructor is used to set references correctly. It also 
     *  initialises an empty UndoStack.
     */
    public UndoManager (MainFrame parent)
    {
	this.parent = parent;
	cc = ControlCenter.getMyself ();
	stack = new UndoStack ();
	redostack = new UndoStack ();
    }

    /**
     *  This method requires all information used for the operation to be 
     *  added to the UndoManager as they were. It then transforms them if 
     *  neccessary (the idea ist to split the work between the two important 
     *  methods) and pushes them onto the Stack.
     *
     *  @param opCode - one of the constants provided in this class
     *  @param items - a Vector of all affected items
     *  @param info - additional information needed to take back the given
     *                operation
     */
    public void notifyUndo (int opCode, Vector items, Object info)
    {
	boolean clearredo = false;
	switch (opCode)
	{
	case UNDO_ADDNODE :
	case UNDO_ADDTRI :
	case UNDO_ADDQUAD :
	case UNDO_ADDMACRO :
	case UNDO_ADDLINE :
	case UNDO_ADDCIRCLE :
	case UNDO_PASTE :
	case UNDO_DELETE :
	case UNDO_CUT :
	    {
		stack.push (new UndoContainer (opCode, items, info));
		clearredo = true;
		break;
	    }
	case UNDO_SCALE :
	case UNDO_ROTATE :
	case UNDO_MIRROR :
	case UNDO_TRANSLATE :
	    {
		stack.push (new UndoContainer (opCode, items, info));
		clearredo = false;
		break;
	    }
	case UNDO_PROPERTYNODE :
	case UNDO_PROPERTYBNODE :
	case UNDO_PROPERTYLINE :
	case UNDO_PROPERTYCIRC :
	    {
		stack.push (new UndoContainer (opCode, items, info));
		clearredo = false;
		break;
	    }
	default :
	    {
		//		System.out.println ("This operation is not yet supported.");
		break;
	    }
	}
	if (clearredo)
	    redostack = new UndoStack ();
    }


    /**
     *  This method takes the top element from the Stack and performs
     *  the corresponding undo operation.
     *
     *  @return true if hotspots need to be repainted
     */
    public boolean performUndo ()
    {
	UndoContainer container;
	try
	{
	    container = stack.popp ();
	}
	catch (EmptyStackException ex)
	{
	    //	    System.out.println ("UndoStack is empty! No operation is taken back.");
	    return false;
	}
	switch (container.getOpCode())
	{
	case UNDO_ADDNODE :
	    {
		cc.d.delNode ((Node)container.getItems().firstElement ());
		redostack.push (container);
		break;
	    }
	case UNDO_ADDTRI :
	    {
		Tri tri = (Tri)container.getItems().elementAt (3);
		cc.d.delCell (tri);
		redostack.push (container);
		break;
	    }
	case UNDO_ADDQUAD :
	    {
		Quad quad = (Quad)container.getItems().elementAt (4);
		cc.d.delCell (quad);
		redostack.push (container);
		break;
	    }
	case UNDO_ADDMACRO :
	    {
		Macro macro = (Macro)container.getItems().elementAt (4);
		cc.d.delCell (macro);
		redostack.push (container);
		break;
	    }
	case UNDO_ADDLINE :
	    {
		SegmentLine line = (SegmentLine)container.getItems().firstElement ();
		cc.d.db.delSegment (line);
		cc.gc.gtk.updateBoundaryCombos ();
		redostack.push (container);
		break;
	    }
	case UNDO_ADDCIRCLE :
	    {
		SegmentCircle circle = (SegmentCircle)container.getItems().firstElement ();
		cc.d.db.delSegment (circle);
		cc.gc.gtk.updateBoundaryCombos ();
		redostack.push (container);
		break;
	    }
	case UNDO_DELETE :
	    {
		undoDelete (container);
		redostack.push (container);
		return true;
	    }
	case UNDO_SCALE :
	case UNDO_TRANSLATE :
	case UNDO_ROTATE :
	    {
		GridListener gl = parent.getDrawingArea ().getListener ();
		// prepare redo
		Vector info = new Vector ();
		info.add (gl.getSelector ().getHotspotMask ());
		Enumeration enum = container.getItemEnum ();
		while (enum.hasMoreElements ())
		{
		    GridItem item = (GridItem)enum.nextElement ();
		    Object[] obj = {item, item.getParameters()};
		    info.add (obj);
		}
		redostack.push (new UndoContainer (container.getOpCode(), container.getItems(), info));
		// perform undo
		gl.clearSelection ();
  		enum = container.getItemEnum ();
  		while (enum.hasMoreElements ())
  		    ((GridItem)enum.nextElement()).setMarker ();
  		gl.setSelectedItems (container.getItems ());
		info = (Vector)container.getInfo();
  		gl.getSelector().setHotspotMask ((boolean[])info.firstElement());
		enum = info.elements ();
		enum.nextElement();
		while (enum.hasMoreElements ())
		{
		    Object[] obj = (Object[])enum.nextElement ();
		    GridItem item = (GridItem)obj[0];
		    double[] params = (double[])obj[1];
		    item.setParameters (params);
		}
		gl.calcPerimeter (false);
		return true;
	    }
	case UNDO_MIRROR :
	    {
		Object[] info = (Object[])container.getInfo ();
		GridListener gl = parent.getDrawingArea ().getListener ();
		gl.clearSelection ();
		Enumeration enum = container.getItemEnum ();
		while (enum.hasMoreElements ())
		    ((GridItem)enum.nextElement()).setMarker ();
		gl.setSelectedItems (container.getItems ());
		gl.getSelector().setHotspotMask ((boolean[])info[1]);
		int[] line = (int[])info[0];
		cc.gc.gtk.mirror (line[0], line[1], line[2], line[3]);
		gl.calcPerimeter (false);
		redostack.push (container);
		return true;
	    }
	case UNDO_CUT :
	    {
		undoDelete (container);
		GridCloner cloner = new GridCloner (container.getItems());
		parent.getDrawingArea().setCloner (cloner);
		cloner.createClone ();
		parent.getDrawingArea().getListener().storeCopyPasteAnchor ();
		redostack.push (container);

		return true;
	    }
	case UNDO_PASTE :
	    {
		Enumeration enum = container.getItemEnum ();
		while (enum.hasMoreElements ())
		{
		    cc.d.del ((GridItem)enum.nextElement ());
		}
		GridCloner cloner = new GridCloner (container.getItems());
		parent.getDrawingArea().setCloner (cloner);
		cloner.createClone ();
		parent.getDrawingArea().getListener().storeCopyPasteAnchor ();
		redostack.push (container);
		break;
	    }
	case UNDO_PROPERTYNODE :
	    {
		Node n = (Node)container.getItems ().firstElement ();
		// prepare redo
		int[] info = new int[3];
		info[0] = n.getCoord (0);
		info[1] = n.getCoord (1);
		info[2] = n.getNPR ();
		redostack.push (new UndoContainer (UNDO_PROPERTYNODE, container.getItems(), info));
		// perform undo
		info = (int[])container.getInfo ();
		n.setCoord (info[0], info[1]);
		n.setNPR (info[2]);
		BoundaryNodeDialog dialog = (BoundaryNodeDialog)cc.dm.get (cc.dm.BOUNDARYNODEDIALOG);
		dialog.setSource (n);
		break;
	    }
	case UNDO_PROPERTYBNODE :
	    {
		BoundaryNode n = (BoundaryNode)container.getItems ().firstElement ();
		// prepare redo
		Object[] info = new Object[2];
		info[0] = n.getBoundary ();
		info[1] = new Double (n.getParaValue ());
		redostack.push (new UndoContainer (cc.um.UNDO_PROPERTYBNODE, container.getItems(), info));
		// perform undo
		info = (Object[])container.getInfo ();
		n.setBoundary ((Boundary)info[0]);
		n.setParaValue (((Double)info[1]).doubleValue ());
		n.updateBoundaryNode ();
		cc.d.updateNodeNumbers ();
		BoundaryNodeDialog dialog = (BoundaryNodeDialog)cc.dm.get (cc.dm.BOUNDARYNODEDIALOG);
		dialog.setSource (n);
		break;
	    }
	case UNDO_PROPERTYLINE :
	    {
		SegmentLine l = (SegmentLine)container.getItems ().firstElement ();
		// prepare redo
		int[] info = new int[4];
		info[0] = l.getStartPoint ()[0];
		info[1] = l.getStartPoint ()[1];
		info[2] = l.getEndPoint ()[0];
		info[3] = l.getEndPoint ()[1];
		redostack.push (new UndoContainer (cc.um.UNDO_PROPERTYLINE, container.getItems(), info));
		// perform undo
		info = (int[])container.getInfo ();
		l.setStartPoint (info[0], info[1]);
		l.setEndPoint (info[2], info[3]);
		LineDialog dialog = (LineDialog)cc.dm.get (cc.dm.LINEDIALOG);
		dialog.setSource (l);
		break;
	    }
	case UNDO_PROPERTYCIRC :
	    {
		SegmentCircle c = (SegmentCircle)container.getItems ().firstElement ();
		// prepare redo
		int[] info = new int[5];
		info[0] = c.getMidPoint ()[0];
		info[1] = c.getMidPoint ()[1];
		info[2] = c.getRadius ();
		info[3] = (int)c.getStartAngle ();
		info[4] = (int)c.getEndAngle ();
		redostack.push (new UndoContainer (cc.um.UNDO_PROPERTYCIRC, container.getItems(), info));
		// perform undo
		info = (int[])container.getInfo ();
		c.setMidPoint (info[0],info[1]);
		c.setRadius (info[2]);
		c.setStartAngle (info[3]);
		c.setEndAngle (info[4]);
		CircleDialog dialog = (CircleDialog)cc.dm.get (cc.dm.CIRCLEDIALOG);
		dialog.setSource (c);
		break;
	    }
	}
	return false;
    }


    /**
     *  this method performs a redo step
     *  @return true if hotspots need to be repainted
     */
    public boolean performRedo ()
    {
	UndoContainer container;
	try
	{
	    container = redostack.popp ();
	}
	catch (EmptyStackException ex)
	{
	    //	    System.out.println ("The RedoStack is empty! Note that the RedoStack is emptied every time an undoable action is performed.");
	    return false;
	}
	switch (container.getOpCode())
	{
	case UNDO_ADDNODE :
	    {
		// add node again
		cc.d.adjustNode ((Node)container.getItems().firstElement());
		stack.push (container);
		break;
	    }
	case UNDO_ADDTRI :
	    {
		Edge e1 = (Edge)container.getItems().elementAt (0);
		Edge e2 = (Edge)container.getItems().elementAt (1);
		Edge e3 = (Edge)container.getItems().elementAt (2);
		Tri tri = (Tri)container.getItems().elementAt (3);
		cc.d.adjustEdge (e1);
		cc.d.adjustEdge (e2);
		cc.d.adjustEdge (e3);
		cc.d.addCell (tri);
		stack.push (container);
		break;
	    }
	case UNDO_ADDQUAD :
	    {
		Edge e1 = (Edge)container.getItems().elementAt (0);
		Edge e2 = (Edge)container.getItems().elementAt (1);
		Edge e3 = (Edge)container.getItems().elementAt (2);
		Edge e4 = (Edge)container.getItems().elementAt (3);
		Quad quad = (Quad)container.getItems().elementAt (4);
		cc.d.adjustEdge (e1);
		cc.d.adjustEdge (e2);
		cc.d.adjustEdge (e3);
		cc.d.adjustEdge (e4);
		cc.d.addCell (quad);
		stack.push (container);
		break;
	    }
	case UNDO_ADDMACRO :
	    {
		Edge e1 = (Edge)container.getItems().elementAt (0);
		Edge e2 = (Edge)container.getItems().elementAt (1);
		Edge e3 = (Edge)container.getItems().elementAt (2);
		Edge e4 = (Edge)container.getItems().elementAt (3);
		Macro macro = (Macro)container.getItems().elementAt (4);
		cc.d.adjustEdge (e1);
		cc.d.adjustEdge (e2);
		cc.d.adjustEdge (e3);
		cc.d.adjustEdge (e4);
		cc.d.addCell (macro);
		stack.push (container);
		break;
	    }
	case UNDO_ADDLINE :
	    {
		SegmentLine line = (SegmentLine)container.getItems().firstElement ();
		// perform undo
		Boundary b = line.getBoundary ();
		if (!cc.d.db.isBoundary (b))
		    cc.d.db.addBoundary (b);
		cc.d.db.addSegment (b, line);
		cc.gc.gtk.updateBoundaryCombos ();
		stack.push (container);
		break;
	    }
	case UNDO_ADDCIRCLE :
	    {
		SegmentCircle circle = (SegmentCircle)container.getItems().firstElement ();
		Boundary b = circle.getBoundary ();
		if (!cc.d.db.isBoundary (b))
		    cc.d.db.addBoundary (b);
		cc.d.db.addSegment (b, circle);
		cc.gc.gtk.updateBoundaryCombos ();
		stack.push (container);
		break;
	    }
	case UNDO_DELETE :
	    {
		Enumeration enum = container.getItemEnum ();
		while (enum.hasMoreElements())
		    cc.d.del ((GridItem)enum.nextElement());
		stack.push (container);
		break;
	    }
	case UNDO_TRANSLATE :
	case UNDO_ROTATE :
	case UNDO_SCALE :
	    {
		GridListener gl = parent.getDrawingArea ().getListener ();
		// prepare undo
		Vector info = new Vector ();
		info.add (gl.getSelector ().getHotspotMask ());
		Enumeration enum = container.getItemEnum ();
		while (enum.hasMoreElements ())
		{
		    GridItem item = (GridItem)enum.nextElement ();
		    Object[] obj = {item, item.getParameters()};
		    info.add (obj);
		}
		stack.push (new UndoContainer (container.getOpCode(), container.getItems(), info));
		// perform redo
		gl.clearSelection ();
  		enum = container.getItemEnum ();
  		while (enum.hasMoreElements ())
  		    ((GridItem)enum.nextElement()).setMarker ();
  		gl.setSelectedItems (container.getItems ());
		info = (Vector)container.getInfo();
  		gl.getSelector().setHotspotMask ((boolean[])info.firstElement());
		enum = info.elements ();
		enum.nextElement();
		while (enum.hasMoreElements ())
		{
		    Object[] obj = (Object[])enum.nextElement ();
		    GridItem item = (GridItem)obj[0];
		    double[] params = (double[])obj[1];
		    item.setParameters (params);
		}
		gl.calcPerimeter (false);
		return true;
	    }
	case UNDO_MIRROR :
	    {
		Object[] info = (Object[])container.getInfo ();
		GridListener gl = parent.getDrawingArea ().getListener ();
		gl.clearSelection ();
		Enumeration enum = container.getItemEnum ();
		while (enum.hasMoreElements ())
		    ((GridItem)enum.nextElement()).setMarker ();
		gl.setSelectedItems (container.getItems ());
		gl.getSelector().setHotspotMask ((boolean[])info[1]);
		int[] line = (int[])info[0];
		cc.gc.gtk.mirror (line[0], line[1], line[2], line[3]);
		gl.calcPerimeter (false);
		stack.push (container);
		return true;
	    }
	case UNDO_CUT :
	    {
		GridListener gl = parent.getDrawingArea ().getListener ();
		Enumeration enum = container.getItemEnum ();
		while (enum.hasMoreElements())
		    cc.d.del ((GridItem)enum.nextElement());
		stack.push (container);
		GridCloner cloner = new GridCloner (container.getItems());
		parent.getDrawingArea().setCloner (cloner);
		cloner.createClone ();
		parent.getDrawingArea().getListener().storeCopyPasteAnchor ();
		redostack.push (container);
		gl.calcPerimeter (false);
		return true;
	    }
	case UNDO_PASTE :
	    {
		GridListener gl = parent.getDrawingArea ().getListener ();
		undoDelete (container);
		GridCloner cloner = new GridCloner (container.getItems());
		parent.getDrawingArea().setCloner (cloner);
		cloner.createClone ();
		parent.getDrawingArea().getListener().storeCopyPasteAnchor ();
		redostack.push (container);
		stack.push (container);
		gl.calcPerimeter (false);
		break;
	    }
	case UNDO_PROPERTYNODE:
	    {
		Node n = (Node)container.getItems ().firstElement ();
		// prepare undo
		int[] info = new int[3];
		info[0] = n.getCoord (0);
		info[1] = n.getCoord (1);
		info[2] = n.getNPR ();
		stack.push (new UndoContainer (UNDO_PROPERTYNODE, container.getItems(), info));
		// perform redo
		info = (int[])container.getInfo ();
		n.setCoord (info[0], info[1]);
		n.setNPR (info[2]);
		BoundaryNodeDialog dialog = (BoundaryNodeDialog)cc.dm.get (cc.dm.BOUNDARYNODEDIALOG);
		dialog.setSource (n);
		break;
	    }
	case UNDO_PROPERTYBNODE:
	    {
		BoundaryNode n = (BoundaryNode)container.getItems ().firstElement ();
		// prepare undo
		Object[] info = new Object[2];
		info[0] = n.getBoundary ();
		info[1] = new Double (n.getParaValue ());
		redostack.push (new UndoContainer (cc.um.UNDO_PROPERTYBNODE, container.getItems(), info));
		// perform undo
		info = (Object[])container.getInfo ();
		n.setBoundary ((Boundary)info[0]);
		n.setParaValue (((Double)info[1]).doubleValue ());
		n.updateBoundaryNode ();
		cc.d.updateNodeNumbers ();
		BoundaryNodeDialog dialog = (BoundaryNodeDialog)cc.dm.get (cc.dm.BOUNDARYNODEDIALOG);
		dialog.setSource (n);
		break;
	    }
	case UNDO_PROPERTYLINE:
	    {
		SegmentLine l = (SegmentLine)container.getItems ().firstElement ();
		// prepare undo
		int[] info = new int[4];
		info[0] = l.getStartPoint ()[0];
		info[1] = l.getStartPoint ()[1];
		info[2] = l.getEndPoint ()[0];
		info[3] = l.getEndPoint ()[1];
		stack.push (new UndoContainer (cc.um.UNDO_PROPERTYLINE, container.getItems(), info));
		// perform redo
		info = (int[])container.getInfo ();
		l.setStartPoint (info[0], info[1]);
		l.setEndPoint (info[2], info[3]);
		LineDialog dialog = (LineDialog)cc.dm.get (cc.dm.LINEDIALOG);
		dialog.setSource (l);
		break;
	    }
	case UNDO_PROPERTYCIRC:
	    {
		SegmentCircle c = (SegmentCircle)container.getItems ().firstElement ();
		// prepare undo
		int[] info = new int[5];
		info[0] = c.getMidPoint ()[0];
		info[1] = c.getMidPoint ()[1];
		info[2] = c.getRadius ();
		info[3] = (int)c.getStartAngle ();
		info[4] = (int)c.getEndAngle ();
		stack.push (new UndoContainer (cc.um.UNDO_PROPERTYCIRC, container.getItems(), info));
		// perform redo
		info = (int[])container.getInfo ();
		c.setMidPoint (info[0],info[1]);
		c.setRadius (info[2]);
		c.setStartAngle (info[3]);
		c.setEndAngle (info[4]);
		CircleDialog dialog = (CircleDialog)cc.dm.get (cc.dm.CIRCLEDIALOG);
		dialog.setSource (c);
		break;
	    }
	}
	return false;
    }


    /**
     *  outsourced because needed for CUT and DELETE
     */
    private void undoDelete (UndoContainer container)
    {
	//	System.out.println ("undoing delete");
	//System.out.println ("separating deleted items into segs, elements, nodes");
	Vector nodes = new Vector ();
	Vector elems = new Vector ();
	Vector segs = new Vector ();
	Enumeration enum = container.getItemEnum ();
	while (enum.hasMoreElements ())
	{
	    GridItem next = (GridItem)enum.nextElement ();
	    if (next instanceof Node || next instanceof BoundaryNode)
		nodes.add (next);
	    if (next instanceof Cell2D)
		elems.add (next);
	    if (next instanceof Segment)
		segs.add (next);
	}
	//	System.out.println ("adding segs first");
	enum = segs.elements ();
	while (enum.hasMoreElements ())
	{
	    Segment s = (Segment)enum.nextElement ();
	    Boundary b = s.getBoundary ();
	    if (!cc.d.db.isBoundary (b))
		cc.d.db.addBoundary (b);
	    cc.d.db.addSegment (b, s);
	}
	//	System.out.println ("adding nodes next");
	enum = nodes.elements ();
	while (enum.hasMoreElements ())
	{
	    Node n = (Node)enum.nextElement ();
	    cc.d.adjustNode (n);
	}
	//	System.out.println ("adding edges extracted from elements");
	enum = elems.elements ();
	while (enum.hasMoreElements ())
	{
	    Cell2D elem = (Cell2D)enum.nextElement ();
	    Edge[] edges = elem.getEdges ();
	    for (int i=0; i<edges.length; i++)
		cc.d.adjustEdge (edges[i]);
	}
	//	System.out.println ("adding elements and finish");
	enum = elems.elements ();
	while (enum.hasMoreElements ())
	{
	    Cell2D elem = (Cell2D)enum.nextElement ();
	    cc.d.addCell (elem);
	}
	//	System.out.println ("done");
	// the newly added items become selected
	Object[] info = (Object[])container.getInfo ();
	GridListener gl = parent.getDrawingArea ().getListener ();
	gl.clearSelection ();
	enum = container.getItemEnum ();
	while (enum.hasMoreElements ())
	    ((GridItem)enum.nextElement()).setMarker();
	// and the GUI is repaired
	gl.setSelectedItems (container.getItems());
	gl.getSelector().setHotspotMask ((boolean[])info[0]);
	gl.calcPerimeter (false);
	cc.gc.workingBoundary = (Boundary)info[1];
	cc.gc.gtk.updateBoundaryCombos ();
    }

    

    public static final int UNDO_ADDNODE       = 100000;
    public static final int UNDO_ADDTRI        = 100010;
    public static final int UNDO_ADDQUAD       = 100020;
    public static final int UNDO_ADDMACRO      = 100030;
    public static final int UNDO_ADDLINE       = 100040;
    public static final int UNDO_ADDCIRCLE     = 100050;
    public static final int UNDO_DELETE        = 100060;
    public static final int UNDO_TRANSLATE     = 100070;
    public static final int UNDO_ROTATE        = 100080;
    public static final int UNDO_SCALE         = 100090;
    public static final int UNDO_MIRROR        = 100100;
    public static final int UNDO_CUT           = 100110;
    public static final int UNDO_PASTE         = 100120;
    public static final int UNDO_PROPERTYNODE  = 100130;
    public static final int UNDO_PROPERTYBNODE = 100140;
    public static final int UNDO_PROPERTYLINE  = 100150;
    public static final int UNDO_PROPERTYCIRC  = 100160;




}




