package devisor2.grid.options;

import java.util.*;
import devisor2.foundation.base.*;
import devisor2.grid.GUI.help.*;
import devisor2.grid.GUI.dialogs.*;
import devisor2.grid.GUI.framework.MainFrame;
import devisor2.grid.backend.spellchecker.*;
import devisor2.grid.backend.undo.*;

/** 
 *  This class has three important roles in the DeViSoRGriD 2 application.
 *  <br><br>
 *  During startup, it is responsible for creating and loading the locale
 *  and RessourceBundle to provide access to button captions etc.
 *  It also instanciates ALL dialogs (which are implemented as static),
 *  therefore EVERY dialog the programmer ever adds to the application
 *  must be registered in this class's initDialogs method. As we implemented
 *  pluggable Look&Feel for the application, there has to be one central
 *  class where dialogs are stored so that a dynamic change of the look&feel
 *  affects the dialogs as well.
 *  <br><br>
 *  Secondly, this class offers access to the global variables the 
 *  application definitely needed. I know global variables and object-
 *  oriented programming don't match, but as we decided to design a very 
 *  modular and open software, it made things a lot easier to have global 
 *  access to the central options management and storage class Options and 
 *  to the locale class providing captions for all buttons etc.
 *  <br>
 *  The global variables are mostly single variables which can be accessed
 *  directly. 
 *  <br>
 *  Thirdly, the so-called MODE of the application is managed here. To 
 *  explain the term MODE: To make sure there is only one legal type of
 *  operation at any time, the application must always be in one and only one
 *  mode. The mode specifies what type of operation is invoked when the user
 *  performs clicks and drags on the center drawing area. 
 *  We made strict rules on mode changes resulting in performing certain 
 *  operations, documented in "ZAP BRANNAGAN's BIG BOOK OF OPERATIONS AND 
 *  RULES" to be found in the /manual/specs directory of your devisorgrid2 
 *  installation.
 *    <br><br>
 *  A last note on the dialog structure of the application: <br>
 *  All dialogs are implemented as static, allowing for minimum effort
 *  to display them. Here in this class, they are initialized, meaning the
 *  GUI is created. They still remain hidden though.
 *  Whenever wanted, they can be displayed using the corresponding
 *  <code>showDialog()<\code> method.
 * <br><br>
 *  Example of how to use the ControlCenter: <br>
 *  (1) Add a ControlCenter reference to the class you're in:<br>
 *      <code>private ControlCenter cc; <\code> <br>
 *  (2) In the constructor, set it to this class:<br>
 *      <code>cc = ControlCenter.getMyself ();<\code> <br>
 *  (3) Extract whatever you want from the Options reference op or
 *      the ResourceBundle reference rb. See documentation of
 *      Options or Locale_en_US for details about that.
 *   OR get the current selection type or boundary etc. respectively 

 *
 *  @author Dominik Goeddeke
 *  @see devisor2.options.Options
 *  @version 1.0 (fully tested)
 *
 */
public class ControlCenter
{
    /**
     *  the global Domain instance
     */
    public Domain d;

    /**
     *  the global precision scale factor
     */
    public int globalscale = 100000;
    public int globalscale2 = 100;
    public int globalscaledigits = 5;


    /**
     *  the global user space perimeter
     */
    public int[] perimeter = {-1*globalscale,-1*globalscale,1*globalscale,1*globalscale};


    /**
     *  the global Options instance
     */
    public Options op;

    /**
     *  the global DrawControl instance
     */
    public DrawControl dc;

    /**
     *  the global GridControl instance
     */
    public GridControl gc;

    /**
     *  the global Locale
     */
    public Locale locale;

    /**
     *  the global ResourceBundle
     */
    public ResourceBundle rb;

    /**
     *  the global help Frame reference
     */
    public HelpFrame help;

    /**
     *  the global faq Frame reference
     */
    public FAQFrame faq;


    /**
     *  the global read Frame reference
     */
    public ReadmeFrame readme;
    
    /**
     *  Flag to determine the copy mode:
     *  KEEPSTRUCTURE <=> new boundaries are created
     *  APPEND <=> cloned segments are appended to their boundary
     */
    public boolean copymode = APPEND;

    /**
     *  Flag whether NODES are selectable, default is FALSE
     */
    public boolean selectNodes = false;

    /**
     *  Flag whether ELEMENTS are selectable, default is FALSE
     */
    public boolean selectElements = false;

    /**
     *  Flag whether SEGMENTS are selectable, default is FALSE
     */
    public boolean selectSegments = false;

    /**
     *  Flag whether EDGES are selectable, default is FALSE
     */
    public boolean selectEdges = false;

    /**
     *  Flag to indicate the current ADD TYPE, use constants provided below in
     *  this class. Default is ADD_NOTHING.
     */
    public int addType = ADD_NOTHING;

    /**
     *  Flag to indicate the current ZOOM TYPE, please use constants provided
     *   in this class
     */
    public int zoomType = ZOOM_NONE;

    /**
     *  Flag to indicate the current MODE, use constants provided
     *  default is NO_MODE
     */
    public int mode = NO_MODE;

    /**
     *  Flag to indicate that direct input is active, default is false.
     */
    public boolean directMode = false;

    /**
     *  a self-reference (must be due to implementation details)
     */
    private static ControlCenter cc;

    /**
     *  a static self-deliverer for easy access from within the application.
     *  @return the global ControlCenter reference itself. 
     */
    public static ControlCenter getMyself ()
    {
	return cc;
    }

    /**
     *  reference to the dialog manager
     */
    public DialogManager dm;

    /**
     *  reference to the undo manager
     */
    public UndoManager um;

    /**
     *  the constructor initializes some neccessary stuff such as locale
     *  and resources.
     *  After constructor call, all global attributes are accessible
     *  via cc.attributename
     *  @param options - an Options instance created in the devisorgrid 
     *                   startup class
     */
    public ControlCenter (Options options)
    {
	cc = this;
	op = options;
	dc = new DrawControl (options);
	gc = new GridControl (null, this);
        d = new Domain ();
	Locale tmp = Locale.getDefault ();
	
	
	System.out.println ("Using locale for language "+tmp.getDisplayLanguage()+" and country scheme "+tmp.getDisplayCountry()+".");
	//	locale = new Locale (op.get(Options.other_locale_language),op.get(Options.other_locale_country));
	try
	{
	    locale = new Locale (tmp.getLanguage(), tmp.getCountry());
	    rb = ResourceBundle.getBundle (op.get(Options.other_ResourceBundleName),locale);
	}
	catch (Exception ex)
	{
	    System.out.println ("Could not find suitable locale, switching to default locale.");
	    locale = new Locale ("en", "US");
	    rb = ResourceBundle.getBundle (op.get(Options.other_ResourceBundleName),locale);
	}
    }

  

    /**
     *  @return the current mode as a string description with full submodes
     */
    public String getModeString ()
    {
	String s = wS("modedisplay_mode")+" ";
	switch (mode)
	    {
	    case ZOOM_MODE : s += wS("modedisplay_zooming")+" "; break;
	    case DRAW_MODE : s += wS("modedisplay_drawing")+" "; break;
	    case EDGESTATUS_MODE : s += wS("modedisplay_edgestatus")+" "; break;
	    case PARALLELBLOCK_MODE : s += wS("modedisplay_parallelblocks")+" "; break;
	    case PROPERTY_MODE : s += wS("modedisplay_propertyquery")+" "; break;
	    case RECTIFYELEMENT_MODE : s += wS("modedisplay_rectify")+" "; break;
	    case REFINE_MODE : s += wS("modedisplay_refine")+" "; break;
	    case MIRROR_MODE: s += wS("modedisplay_mirror")+" "; break;
	    default : s += wS("modedisplay_none")+" ";
	    }
	if (addType != ADD_NOTHING)
	{
	    s += wS("modedisplay_adding")+": ";
	    switch (addType)
	    {
		case ADD_NODE : s += wS("modedisplay_nodes")+" "; break;
		case ADD_LINE : s += wS("modedisplay_lines")+" "; break;
		case ADD_MULTILINE : s += wS("modedisplay_multilines")+" "; break;
		case ADD_CIRCLEP : s += wS("modedisplay_circlep")+" "; break;
		case ADD_CIRCLEM : s += wS("modedisplay_circlem")+" "; break;
		case ADD_CLIPBOARD : s += wS("modedisplay_clipboard")+" "; break;
		case ADD_TRI : s += wS("modedisplay_tris")+" "; break;
		case ADD_QUAD : s += wS("modedisplay_quads")+" "; break;
		case ADD_MACRO : s += wS("modedisplay_macros")+" "; break;
	    }
	}
	if (selectNodes || selectSegments || selectElements || selectEdges)
	{
	    s += wS("modedisplay_selecting")+" ";
	    if (selectNodes)
		s += wS("modedisplay_nodes")+" ";
	    if (selectElements)
		s += wS("modedisplay_elements")+" ";
	    if (selectSegments)
		s += wS("modedisplay_segments")+" ";
	    if (selectEdges)
		s += wS("modedisplay_edges")+" ";
	}
	if (copymode == APPEND)
	    s += " "+wS("modedisplay_copymode_append");
	if (copymode == KEEPSTRUCTURE)
	    s += " "+wS("modedisplay_copymode_keep");
	return s;
    }

    /**
     *  wrapper method for access to resource strings
     */
    public static final String wS(String key)
    {
       return (String)cc.rb.getObject(key);
    }
    
    /**
     *  constant for the copy mode keep boundary structure
     */
    public static final boolean KEEPSTRUCTURE = true;
    /**
     *  constant for the copy mode add new boundaries
     */
    public static final boolean APPEND = false;

    /**
     *  indicates that nothing is addable at the moment
     */
    public static final int ADD_NOTHING = 1000;

    /**
     *  indicates that NODES are addable at the moment
     */
    public static final int ADD_NODE = 1001;

    /**
     *  indicates that LINES are addable at the moment
     */
    public static final int ADD_LINE = 1002;

    /**
     *  indicates that MULTILINES are addable at the moment
     */
    public static final int ADD_MULTILINE = 1003;

    /**
     *  indicates that CIRCLEPS are addable at the moment
     */
    public static final int ADD_CIRCLEP = 1004;

    /**
     *  indicates that CIRCLEMS are addable at the moment
     */
    public static final int ADD_CIRCLEM = 1005;

    /**
     *  indicates that TRIS are addable at the moment
     */
    public static final int ADD_TRI = 1006;
 
    /**
     *  indicates that QUADS are addable at the moment
     */
    public static final int ADD_QUAD = 1007;

    /**
     *  indicates that MACROS are addable at the moment
     */
    public static final int ADD_MACRO = 1008;

    /**
     *  indicates that CLIPBOARD is addable at the moment
     */
    public static final int ADD_CLIPBOARD = 1009;

    /**
     *  indicates that no mode is the current mode
     */
    public static final int NO_MODE = 101;

    /**
     *  indicates that zooming is the current mode
     */
    public static final int ZOOM_MODE = 102;

    /**
     *  indicates that drawing is the current mode
     */
    public static final int DRAW_MODE = 103;

    /**
     *  indicates that setting edge status is the current mode
     */
    public static final int EDGESTATUS_MODE = 104;

    /**
     *  indicates that setting parallelblocks is the current mode
     */
    public static final int PARALLELBLOCK_MODE = 105;

    /**
     *  indicates that querying properties is the current mode
     */
    public static final int PROPERTY_MODE = 106;

    /**
     *  indicates that rectifying elements is the current mode
     */
    public static final int RECTIFYELEMENT_MODE = 107;
    /**
     *  indicates that refining is the current mode
     */
    public static final int REFINE_MODE = 108;
    /**
     *  indicates that mirroring is the current mode
     */
    public static final int MIRROR_MODE = 109;


    /**
     *  indicates that no zooming is available at the moment
     */
    public static final int ZOOM_NONE = 47;

    /**
     *  indicates that set zoom is available at the moment
     */
    public static final int ZOOM_SET = 48;

    /**
     *  indicates that zooming in is available at the moment
     */
    public static final int ZOOM_IN = 49;

    
}
