package devisor2.grid.options;

import java.util.*;
import java.io.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

/** 
 *  This class represents the central storage and management class for
 *  all Options the user can set throughout the whole application, and some
 *  additional settings for the GUI. <br>
 *  <br>
 *  I chose an instance of java.util.Properties to hold the options,
 *  which basically provides a container to store key-value-pairs, both
 *  of type String.
 *  As keys, only the ones declared as constants below in this class are 
 *  allowed. <br>
 *  <br>
 *  This class also provides access to the default options, which are
 *  hard-coded in the initdefaults() method. So every time other 
 *  developers intend to add further options to the application, all they
 *  have to do is: <br>
 *  (1) add a new unique key to the constants below (by convention, separate
 *      keys into option groups such as accelerators, view management, paths
 *      etc) <br>
 *  (2) add a new line to the initDefaults method using the key you just
 *      declared to provide a default initial value for the option <br>
 *  <br>
 *  Note on accerelators: the prefix "accelerator" of the keys is used to 
 *  determine which keys represent accelerators in the OptionsDialog's
 *  corresponding tab sheet. As accelerators are only assigned to functions
 *  available through the MainMenu, see there how to create a new
 *  menu item including mnemonic and accelerator. <br>
 *  <br>
 *  Note on draw options: For easier access, use the DrawControl instance in 
 *  the Control Center, which is updated automatically.<br>
 *  <br>
 *  Note on dialog positions: For the three dockable dialogs, the last
 *  position the user dragged the dialog to is stored, as well as the initial
 *  position. These screen coordinates are stored as integers representing
 *  the relative position of the dialog's upper left corner to the MainFrame's
 *  upper left corner. As the initial value depends on the MainFrame's size,
 *  it is set on startup and is therefore not hard-coded.<br>
 *  <br>
 *  This class is made globally accessible via the ControlCenter instance
 *  (see there for details), so all you have to do to extract an option
 *  at runtime is to call the "get (key)" method which will return the 
 *  requested option as String. <br>
 *  <br>
 *  Additional features are persistence, you can load and save your options
 *  as plain-text-key-value files, see the OptionsDialog class
 *  (or better its action listener OptionsActionListener) for details.
 *  To obtain further information about the file structure, browse the
 *  java language and API documentation, class Properties in package
 *  java.util, which you can browse online at the SUN web site. <br>
 *  
 *
 *  @author Dominik Goeddeke
 *  @see devisor2.GUI.base.dialog.OptionsDialog
 *  @see devisor2.GUI.foundation.MainMenu
 *  @version 0.9 (fully tested, but still further Options being added)
 *
 */


public class Options
{

    private Properties optiontable;
    private static Properties defaults;
    private String header = "DeViSoRGrid2 Options file, please do not modify";


    /**
     *  the Constructor just creates a new Options object and initializes the
     *  factory default settings
     */
    public Options ()
    {
	optiontable = new Properties ();
	initDefaults ();
    }

    /** 
     *  fills the defaults hashtable
     */
    private void initDefaults ()
    {
	defaults = new Properties ();

	defaults.setProperty (other_locale_country,"US");
	defaults.setProperty (other_locale_language,"en");
	defaults.setProperty (other_ResourceBundleName, "devisor2.grid.options.Locale");
	defaults.setProperty (other_undosteps, "20");
	defaults.setProperty (general_path, System.getProperty("user.dir"));
	defaults.setProperty (screenx, "1024");
	defaults.setProperty (screeny, "768");
	defaults.setProperty (metis_command, "/usr/local/metis/partnmesh");
	defaults.setProperty (general_devisorhome, System.getProperty("devisorgrid.home"));
	defaults.setProperty (general_trigen, System.getProperty("user.dir")+System.getProperty("file.separator")+"trigen2d");
	defaults.setProperty (general_featlib, "/home/people/featflow/");
	defaults.setProperty (other_lookandfeel, "javax.swing.plaf.metal.MetalLookAndFeel");
	// FOR SOME REASON, THE CONSTRUCTOR FOR THE URLTEXTAREA DOES
	// NOT ACCEPT THE ~ SYSTEM PROPERTY :-(
	defaults.setProperty (general_helpfile, "file://"+defaults.getProperty(general_devisorhome)+System.getProperty("file.separator")+"manual"+System.getProperty("file.separator")+"html"+System.getProperty("file.separator")+"DeViSoRGrid2_Manual.html");
	defaults.setProperty (general_faqfile, "file://"+defaults.getProperty(general_devisorhome)+System.getProperty("file.separator")+"manual"+System.getProperty("file.separator")+"html"+System.getProperty("file.separator")+"FAQ.html");
	// default draw options
	defaults.setProperty (draw_usercoords, "true");
	defaults.setProperty (draw_orientationmarkers, "true");
	defaults.setProperty (draw_drawboundary,"true");
	defaults.setProperty (draw_numberboundary,"true");
	defaults.setProperty (draw_drawnode,"true");
	defaults.setProperty (draw_numbernode,"true");
	defaults.setProperty (draw_drawedge,"true");
	defaults.setProperty (draw_numberedge,"true");
	defaults.setProperty (draw_drawelement,"true");
	defaults.setProperty (draw_numberelement,"true");
	defaults.setProperty (draw_crosselement,"false");
	defaults.setProperty (draw_fillelement,"false");
	defaults.setProperty (draw_drawgrid,"true");
	defaults.setProperty (draw_drawperimeter,"true");
	defaults.setProperty (draw_drawrect,"true");
	defaults.setProperty (click_tolerance, Integer.toString(5));
	defaults.setProperty (snap_enabled, "true");
	defaults.setProperty (snap_x, Integer.toString (10));
	defaults.setProperty (snap_y, Integer.toString (10));
	defaults.setProperty (epsilon, Integer.toString (40));
	// default colors
	defaults.setProperty (color_node, Integer.toString((Color.black).getRGB()));
	defaults.setProperty (color_edge, Integer.toString((Color.gray).getRGB()));
	defaults.setProperty (color_edgebase, Integer.toString((Color.black).getRGB()));
	defaults.setProperty (color_element, Integer.toString((Color.green).getRGB()));
	defaults.setProperty (color_boundarynumber, Integer.toString((Color.red).getRGB()));
	defaults.setProperty (color_nodenumber, Integer.toString((Color.red).getRGB()));
	defaults.setProperty (color_edgenumber, Integer.toString((Color.red).getRGB()));
	defaults.setProperty (color_elementnumber, Integer.toString((Color.black).getRGB()));
	defaults.setProperty (color_fill0, Integer.toString((Color.blue).getRGB()));
	defaults.setProperty (color_fill1, Integer.toString((Color.green).getRGB()));
	defaults.setProperty (color_fill2, Integer.toString((Color.cyan).getRGB()));
	defaults.setProperty (color_fill3, Integer.toString((Color.darkGray).getRGB()));
	defaults.setProperty (color_fill4, Integer.toString((Color.magenta).getRGB()));
	defaults.setProperty (color_fill5, Integer.toString((Color.orange).getRGB()));
	defaults.setProperty (color_fill6, Integer.toString((Color.pink).getRGB()));
	defaults.setProperty (color_fill7, Integer.toString((Color.red).getRGB()));
	defaults.setProperty (color_fill8, Integer.toString((Color.blue).getRGB()));
	defaults.setProperty (color_fill9, Integer.toString((Color.green).getRGB()));
	defaults.setProperty (color_fill10, Integer.toString((Color.darkGray).getRGB()));
	defaults.setProperty (color_fill11, Integer.toString((Color.cyan).getRGB()));
	defaults.setProperty (color_fill12, Integer.toString((Color.magenta).getRGB()));
	defaults.setProperty (color_fill13, Integer.toString((Color.orange).getRGB()));
	defaults.setProperty (color_fill14, Integer.toString((Color.pink).getRGB()));
	defaults.setProperty (color_fill15, Integer.toString((Color.red).getRGB()));
	defaults.setProperty (color_selection, Integer.toString((Color.blue).getRGB()));
	defaults.setProperty (color_rect, Integer.toString((Color.red).getRGB()));
	defaults.setProperty (color_grid, Integer.toString((Color.black).getRGB()));
	defaults.setProperty (color_background, Integer.toString((Color.white).getRGB()));
	defaults.setProperty (color_lasso, Integer.toString((Color.red).getRGB()));
	defaults.setProperty (color_boundary0, Integer.toString((Color.red).getRGB()));
	defaults.setProperty (color_boundary1, Integer.toString((Color.blue).getRGB()));
	defaults.setProperty (color_boundary2, Integer.toString((Color.yellow).getRGB()));
	defaults.setProperty (color_boundary3, Integer.toString((Color.cyan).getRGB()));
	defaults.setProperty (color_boundary4, Integer.toString((Color.magenta).getRGB()));
	defaults.setProperty (color_boundary5, Integer.toString((Color.red).getRGB()));
	defaults.setProperty (color_boundary6, Integer.toString((Color.blue).getRGB()));
	defaults.setProperty (color_boundary7, Integer.toString((Color.yellow).getRGB()));
	defaults.setProperty (color_boundary8, Integer.toString((Color.cyan).getRGB()));
	defaults.setProperty (color_boundary9, Integer.toString((Color.magenta).getRGB()));
	defaults.setProperty (color_boundary10, Integer.toString((Color.red).getRGB()));
	defaults.setProperty (color_boundary11, Integer.toString((Color.blue).getRGB()));
	defaults.setProperty (color_boundary12, Integer.toString((Color.yellow).getRGB()));
	defaults.setProperty (color_boundary13, Integer.toString((Color.cyan).getRGB()));
	defaults.setProperty (color_boundary14, Integer.toString((Color.magenta).getRGB()));
	defaults.setProperty (color_boundary15, Integer.toString((Color.red).getRGB()));

	// default fonts
	defaults.setProperty(font_numbering, "TimesRoman");
	defaults.setProperty(font_numbering_size, "12");
	defaults.setProperty(font_numbering_style, Integer.toString(Font.PLAIN));
	// default accelerator mappings
	defaults.setProperty (accelerator_new, "NONE");
	defaults.setProperty (accelerator_open, "F3");
	defaults.setProperty (accelerator_save, "CTRL+S");
	defaults.setProperty (accelerator_saveas_ucd, "NONE");
	defaults.setProperty (accelerator_saveas_feat, "NONE");
	defaults.setProperty (accelerator_saveas_feast, "NONE");
	defaults.setProperty (accelerator_merge, "NONE");
	defaults.setProperty (accelerator_print, "NONE");
	defaults.setProperty (accelerator_printls, "NONE");
	defaults.setProperty (accelerator_exit, "ALT+F4");
	defaults.setProperty (accelerator_undo, "CTRL+Z");
	defaults.setProperty (accelerator_redo, "NONE");
	defaults.setProperty (accelerator_cut, "CTRL+X");
	defaults.setProperty (accelerator_copy, "CTRL+C");
	defaults.setProperty (accelerator_paste, "CTRL+V");
	defaults.setProperty (accelerator_copymode, "NONE");
	defaults.setProperty (accelerator_delete, "CTRL-D");
	defaults.setProperty (accelerator_deleteworkingboundary, "NONE");
	defaults.setProperty (accelerator_scale, "NONE");
	defaults.setProperty (accelerator_rotate, "NONE");
	defaults.setProperty (accelerator_mirror, "NONE");
	defaults.setProperty (accelerator_selectboundary, "NONE");
	defaults.setProperty (accelerator_zoom_setzoom, "NONE");
	defaults.setProperty (accelerator_zoom_resetzoom, "NONE");
	defaults.setProperty (accelerator_zoom_zoomplus, "NONE");
	defaults.setProperty (accelerator_zoom_zoomminus, "NONE");
	defaults.setProperty (accelerator_selectnodes, "NONE");
	defaults.setProperty (accelerator_selectelements, "NONE");
	defaults.setProperty (accelerator_selectsegments, "NONE");
	defaults.setProperty (accelerator_selectedges, "NONE");
	defaults.setProperty (accelerator_viewproperties, "NONE");
	defaults.setProperty (accelerator_newboundary, "NONE");
	defaults.setProperty (accelerator_directinput, "NONE");
	defaults.setProperty (accelerator_newline, "NONE");
	defaults.setProperty (accelerator_newmultiline, "NONE");
	defaults.setProperty (accelerator_newcirclep, "NONE");
	defaults.setProperty (accelerator_newcirclem, "NONE");
	defaults.setProperty (accelerator_newanalytic, "NONE");
	defaults.setProperty (accelerator_newedge, "NONE");
	defaults.setProperty (accelerator_newtri, "NONE");
	defaults.setProperty (accelerator_newquad, "NONE");
	defaults.setProperty (accelerator_newmacro, "NONE");
	defaults.setProperty (accelerator_newnode, "NONE");
	defaults.setProperty (accelerator_adjustboundaries, "NONE");
	defaults.setProperty (accelerator_linkboundaries, "NONE");
	defaults.setProperty (accelerator_fromtri2prm, "NONE");
	defaults.setProperty (accelerator_swapboundaryorientation, "NONE");
	defaults.setProperty (accelerator_domainperimeter, "NONE");
	defaults.setProperty (accelerator_domainstatistics, "NONE");
	defaults.setProperty (accelerator_multipleboundarynodes, "NONE");
	defaults.setProperty (accelerator_edgestatus, "NONE");
	defaults.setProperty (accelerator_parallelblock, "NONE");
	defaults.setProperty (accelerator_loadbalancing, "NONE");
	defaults.setProperty (accelerator_rectifyelement, "NONE");
	defaults.setProperty (accelerator_refine, "NONE");
	defaults.setProperty (accelerator_spellchecker, "NONE");
	defaults.setProperty (accelerator_mergeboundaries, "NONE");
	defaults.setProperty (accelerator_generaloptions, "CTRL+O");
	defaults.setProperty (accelerator_drawoptions, "NONE");
	defaults.setProperty (accelerator_coloroptions, "NONE");
	defaults.setProperty (accelerator_acceleratoroptions, "NONE");
	defaults.setProperty (accelerator_miscoptions, "NONE");
	defaults.setProperty (accelerator_manual, "F1");
	defaults.setProperty (accelerator_faq, "NONE");
	defaults.setProperty (accelerator_readme, "NONE");
	defaults.setProperty (accelerator_about, "NONE");

    }

    /**
     *  saves the user settings to a file
     *
     *  @param filename the filename (full path) where to save to
     *  @throws java.io.IOException  - if an IOexception occurred during 
     *  writing
     *  @throws java.lang.ClassCastException - if the keys in the file could
     *   not be casted to Strings
     *  @throws java.io.FileNotFoundException  - if the file could not be 
     *   opened for writing
     */
    public void save (String filename)
	throws IOException, ClassCastException, FileNotFoundException
    {
	FileOutputStream outstream = new FileOutputStream (filename);
	try 
	    {
		optiontable.store (outstream,header);
	    }
	catch (IOException ex)
	    {
		outstream.close();
		throw ex;
	    }
	catch (ClassCastException ex)
	    {
		outstream.close();
		throw ex;
	    }
	outstream.close();
    }

    /**
     *  loads user settings from a file
     *
     *  @param filename the filename (full path) where to load from
     *  @throws java.io.IOException  - if an IOexception occurred during 
     *   reading
     *  @throws java.lang.ClassCastException - if the keys in the file 
     *   could not be casted to Strings
     *  @throws java.io.FileNotFoundException if the file could not be found
     */
    public void load (String filename)
	throws IOException, ClassCastException, FileNotFoundException
    {
	FileInputStream instream = new FileInputStream (filename);
	try 
	    {
		optiontable.load (instream);
	    }
	catch (IOException ex)
	    {
		instream.close();
		throw ex;
	    }
	catch (ClassCastException ex)
	    {
		instream.close();
		throw ex;
	    }
	instream.close();
    }


    /**
     *  returns an enumeration of all the keys in the options file
     *
     *  @returns an enumeration of all keys
     */
    public Enumeration keys ()
    {
	return optiontable.propertyNames ();
    }

    /**
     *  sets the config file header text to a specific user-defined value
     *
     *  @param header - a String containing the new header
     */
    public void setHeader (String header)
    {
	this.header = header;
    }

    /**
     *  returns the current header for config files
     *
     *  @return the current header as a String
     */
    public String getHeader ()
    {
	return header;
    }

    /**
     *  Resets all user-modified options to factory defaults 
     */ 
    public void resetToDefaults ()
    {
	Enumeration defaultsEnum = defaults.propertyNames ();
	while (defaultsEnum.hasMoreElements())
	    {
		String defaultkey = (String)defaultsEnum.nextElement();
		optiontable.setProperty (defaultkey,defaults.getProperty(defaultkey));
	    }
    }

    /**
     *  Returns the String value saved under the given key. If no such value
     *  is found, returns the corresponding default value.
     *  if no key can be found, an empty String is returned.
     *
     *  @param key - one of the constants declared in this class
     *  @return the corresponding value as a String
     */
    public String get (String key)
    {
	return optiontable.getProperty (key,"");
    }


    /**
     *  returns the default object saved under the given key
     *
     *  @param key - the key to the option you want the default for
     *  @return the default option for the given key
     */
    public String getDefault (String key)
    {
	return defaults.getProperty (key);
    }

    /**
     *  sets the value saved under the given key to the new Object
     *  @param key the key for the new value
     *  @param newObject the new value itself
     */
    public void set (String key, String newObject)
    {
	optiontable.setProperty (key,newObject);
    }


    /**
     *  This method just checks if the given accelerator is already in use
     *  in the application
     *
     *  @param accelerator - a String representation of an accelerator
     *  @return true if the accelerator is already in use, false otherwise
     */
    public boolean isAlreadyAccelerator (String accelerator)
    {
	// a little ugly code here, but it was late and i didn't feel
	// like writing something really good-looking
	// anyway, this is faster... :-)
	if (accelerator.equals("NONE")) return false;
	if (get (accelerator_new).equals(accelerator)) return true;
	if (get (accelerator_open).equals(accelerator)) return true;
	if (get (accelerator_save).equals(accelerator)) return true;
	if (get (accelerator_saveas_ucd).equals(accelerator)) return true;
	if (get (accelerator_saveas_feat).equals(accelerator)) return true;
	if (get (accelerator_saveas_feast).equals(accelerator)) return true;
	if (get (accelerator_merge).equals(accelerator)) return true;
	if (get (accelerator_print).equals(accelerator)) return true;
	if (get (accelerator_printls).equals(accelerator)) return true;
	if (get (accelerator_exit).equals(accelerator)) return true;
	if (get (accelerator_undo).equals(accelerator)) return true;
	if (get (accelerator_redo).equals(accelerator)) return true;
	if (get (accelerator_cut).equals(accelerator)) return true;
	if (get (accelerator_copy).equals(accelerator)) return true;
	if (get (accelerator_paste).equals(accelerator)) return true;
	if (get (accelerator_copymode).equals(accelerator)) return true;
	if (get (accelerator_delete).equals(accelerator)) return true;
	if (get (accelerator_deleteworkingboundary).equals(accelerator)) return true;
	if (get (accelerator_scale).equals(accelerator)) return true;
	if (get (accelerator_rotate).equals(accelerator)) return true;
	if (get (accelerator_mirror).equals(accelerator)) return true;
	if (get (accelerator_selectboundary).equals(accelerator)) return true;
	if (get (accelerator_zoom_setzoom).equals(accelerator)) return true;
	if (get (accelerator_zoom_resetzoom).equals(accelerator)) return true;
	if (get (accelerator_zoom_zoomplus).equals(accelerator)) return true;
	if (get (accelerator_zoom_zoomminus).equals(accelerator)) return true;
	if (get (accelerator_selectnodes).equals(accelerator)) return true;
	if (get (accelerator_selectelements).equals(accelerator)) return true;
	if (get (accelerator_selectsegments).equals(accelerator)) return true;
	if (get (accelerator_selectedges).equals(accelerator)) return true;
	if (get (accelerator_viewproperties).equals(accelerator)) return true;
	if (get (accelerator_newboundary).equals(accelerator)) return true;
	if (get (accelerator_directinput).equals(accelerator)) return true;
	if (get (accelerator_newline).equals(accelerator)) return true;
	if (get (accelerator_newmultiline).equals(accelerator)) return true;
	if (get (accelerator_newcirclep).equals(accelerator)) return true;
	if (get (accelerator_newcirclem).equals(accelerator)) return true;
	if (get (accelerator_newanalytic).equals(accelerator)) return true;
	if (get (accelerator_newedge).equals(accelerator)) return true;
	if (get (accelerator_newtri).equals(accelerator)) return true;
	if (get (accelerator_newquad).equals(accelerator)) return true;
	if (get (accelerator_newmacro).equals(accelerator)) return true;
	if (get (accelerator_newnode).equals(accelerator)) return true;
	if (get (accelerator_adjustboundaries).equals(accelerator)) return true;
	if (get (accelerator_linkboundaries).equals(accelerator)) return true;
	if (get (accelerator_fromtri2prm).equals(accelerator)) return true;
	if (get (accelerator_swapboundaryorientation).equals(accelerator)) return true;
	if (get (accelerator_domainperimeter).equals(accelerator)) return true;
	if (get (accelerator_domainstatistics).equals(accelerator)) return true;
	if (get (accelerator_multipleboundarynodes).equals(accelerator)) return true;
	if (get (accelerator_edgestatus).equals(accelerator)) return true;
	if (get (accelerator_parallelblock).equals(accelerator)) return true;
	if (get (accelerator_loadbalancing).equals(accelerator)) return true;
	if (get (accelerator_rectifyelement).equals(accelerator)) return true;
	if (get (accelerator_refine).equals(accelerator)) return true;
	if (get (accelerator_spellchecker).equals(accelerator)) return true;
	if (get (accelerator_mergeboundaries).equals(accelerator)) return true;
	if (get (accelerator_generaloptions).equals(accelerator)) return true;
	if (get (accelerator_drawoptions).equals(accelerator)) return true;
	if (get (accelerator_coloroptions).equals(accelerator)) return true;
	if (get (accelerator_acceleratoroptions).equals(accelerator)) return true;
	if (get (accelerator_miscoptions).equals(accelerator)) return true;
	if (get (accelerator_manual).equals(accelerator)) return true;
	if (get (accelerator_faq).equals(accelerator)) return true;
	if (get (accelerator_readme).equals(accelerator)) return true;
	if (get (accelerator_about).equals(accelerator)) return true;
	return false;
    }

    // general options

    /**
     *  key for the MainFrame's width
     */
    public static final String mainframe_width = "mainframe_width";
    /**
     *  key for the MainFrame's height
     */
    public static final String mainframe_height = "mainframe_height";
    /**
     *  key for the MainFrame's origin (horizontal)
     */
    public static final String mainframe_x = "mainframe_x";
    /**
     *  key for the MainFrame's origin (vertical)
     */
    public static final String mainframe_y = "mainframe_y";


    /**
     *  key for the metis mesher used for partioning
     */
    public static final String metis_command = "metis_command";
    /**
     *  key for the general path used for load-save-dialogs
     */
    public static final String general_path = "general_path";
    /**
     *  key for the width of the main frame
     */
    public static final String screenx = "screenx";
    /**
     *  key for the height of the main frame
     */
    public static final String screeny = "screeny";
    /**
     *  key for the DEVISORHOME path
     */
    public static final String general_devisorhome = "general_devisorhome";
    /**
     *  key for the helpfile path+name
     */
    public static final String general_helpfile = "general_helpfile";
    /**
     *  key for the faqfile path+name
     */
    public static final String general_faqfile = "general_faqfile";
    /**
     *  key for the trigen path
     */
    public static final String general_trigen = "general_trigen";
    /**
     *  key for the feat path
     */
    public static final String general_featlib = "general_featlib";


    // misc options
    /**
     *  key for the Look&Feel used
     */
    public static final String other_lookandfeel = "other_lookandfeel";
    /**
     *  key for the country code for the locale used
     */
    public static final String other_locale_country = "other_locale_country";
    /**
     *  key for the the language code for the locale used
     */
    public static final String other_locale_language = "other_locale_language";
    /**
     *  key for the name of the resource bundle backing the locale
     */
    public static final String other_ResourceBundleName = "other_ResourceBundlename";
    /**
     *  key for the maximum number of undoable steps
     */
    public static final String other_undosteps = "other_undosteps";


    // draw options

    /**
     *  key for flag if user coords are displayed
     */
    public static final String draw_usercoords = "draw_usercoords";
    /**
     *  key for flag if orientation markers are drawn
     */
    public static final String draw_orientationmarkers = "draw_orientationmarkers";
    /**
     *  key for flag if boundaries are painted
     */
    public static final String draw_drawboundary = "draw_drawboundary";
    /**
     *  key for flag if boundary numbers are painted
     */
    public static final String draw_numberboundary = "draw_numberboundary";
    /**
     *  key for flag if nodes are painted
     */
    public static final String draw_drawnode = "draw_drawnode";
    /**
     *  key for flag if node numbers are painted
     */
    public static final String draw_numbernode = "draw_numbernode";
    /**
     *  key for flag if edges are painted
     */
    public static final String draw_drawedge = "draw_drawedge";
    /**
     *  key for flag if edge numbers are painted
     */
    public static final String draw_numberedge = "draw_numberedge";
    /**
     *  key for flag if elements are painted
     */
    public static final String draw_drawelement = "draw_drawelement";
    /**
     *  key for flag if element numbers are painted
     */
    public static final String draw_numberelement = "draw_numberelement";
    /**
     *  key for flag if elements are crossed
     */
    public static final String draw_crosselement = "draw_crosselement";
    /**
     *  key for flag if elements are filled
     */
    public static final String draw_fillelement = "draw_fillelement";
    /**
     *  key for flag if the grid is painted
     */
    public static final String draw_drawgrid = "draw_drawgrid";
    /**
     *  key for flag if the perimeter is painted
     */
    public static final String draw_drawperimeter = "draw_drawperimeter";
    /**
     *  key for flag if rectangular elements are highlighted
     */
    public static final String draw_drawrect = "draw_drawrect";
    /**
     * key for the click tolerance
     */
    public static final String click_tolerance = "click_tolerance";
    /**
     *  key for snap enabling
     */
    public static final String snap_enabled = "snap_enabled";
    /**
     *  key for snap value in x direction
     */
    public static final String snap_x = "snap_x";
    /**
     *  key for snap value in y direction
     */
    public static final String snap_y = "snap_y";
    /**
     *  key for global epsilon
     */
    public static final String epsilon = "epsilon";

    /**
     *  key for the color nodes are painted with
     */
    public static final String color_node = "color_node";
    /**
     *  key for the base color edges are painted with
     */
    public static final String color_edgebase = "color_edgebase";
    /**
     *  key for the color edges are painted with
     */
    public static final String color_edge = "color_edge";
    /**
     *  key for the color elements are painted with
     */
    public static final String color_element = "color_element";
    /**
     *  key for the color boundarynumbers are painted with
     */
    public static final String color_boundarynumber = "color_boundarynumber";
    /**
     *  key for the color nodenumbers are painted with
     */
    public static final String color_nodenumber = "color_nodenumber";
    /**
     *  key for the color edgenumbers are painted with
     */
    public static final String color_edgenumber = "color_edgenumber";
    /**
     *  key for the color elementnumbers are painted with
     */
    public static final String color_elementnumber = "color_elementnumber";
    /**
     *  key for 1st fill palette color
     */
    public static final String color_fill0 = "color_fill0";
    /**
     *  key for 2nd fill palette color
     */
    public static final String color_fill1 = "color_fill1";
    /**
     *  key for 3rd fill palette color
     */
    public static final String color_fill2 = "color_fill2";
    /**
     *  key for 4th fill palette color
     */
    public static final String color_fill3 = "color_fill3";
    /**
     *  key for 5th fill palette color
     */
    public static final String color_fill4 = "color_fill4";
    /**
     *  key for 6th fill palette color
     */
    public static final String color_fill5 = "color_fill5";
    /**
     *  key for 7th fill palette color
     */
    public static final String color_fill6 = "color_fill6";
    /**
     *  key for 8th fill palette color
     */
    public static final String color_fill7 = "color_fill7";
    /**
     *  key for 9th fill palette color
     */
    public static final String color_fill8 = "color_fill8";
    /**
     *  key for 10th fill palette color
     */
    public static final String color_fill9 = "color_fill9";
    /**
     *  key for 11th fill palette color
     */
    public static final String color_fill10 = "color_fill10";
    /**
     *  key for 12th fill palette color
     */
    public static final String color_fill11 = "color_fill11";
    /**
     *  key for 13th fill palette color
     */
    public static final String color_fill12 = "color_fill12";
    /**
     *  key for 14th fill palette color
     */
    public static final String color_fill13 = "color_fill13";
    /**
     *  key for 15th fill palette color
     */
    public static final String color_fill14 = "color_fill14";
    /**
     *  key for 16th fill palette color
     */
    public static final String color_fill15 = "color_fill15";
    /**
     *  key for the highlight color for selections
     */
    public static final String color_selection = "color_selection";
    /**
     *  key for the highlight color for rectangular status
     */
    public static final String color_rect = "color_rect";
    /**
     *  key for the color of the grid
     */
    public static final String color_grid = "color_grid";
    /**
     *  key for the color of the background
     */
    public static final String color_background = "color_background";
    /**
     *  key for the color of the lasso
     */
    public static final String color_lasso = "color_lasso";
    /**
     *  key for the pallette color 0 for boundaries
     */
    public static final String color_boundary0 = "color_boundary0";
    /**
     *  key for the pallette color 1 for boundaries
     */
    public static final String color_boundary1 = "color_boundary1";
    /**
     *  key for the pallette color 2 for boundaries
     */
    public static final String color_boundary2 = "color_boundary2";
    /**
     *  key for the pallette color 3 for boundaries
     */
    public static final String color_boundary3 = "color_boundary3";
    /**
     *  key for the pallette color 4 for boundaries
     */
    public static final String color_boundary4 = "color_boundary4";
    /**
     *  key for the pallette color 5 for boundaries
     */
    public static final String color_boundary5 = "color_boundary5";
    /**
     *  key for the pallette color 6 for boundaries
     */
    public static final String color_boundary6 = "color_boundary6";
    /**
     *  key for the pallette color 7 for boundaries
     */
    public static final String color_boundary7 = "color_boundary7";
    /**
     *  key for the pallette color 8 for boundaries
     */
    public static final String color_boundary8 = "color_boundary8";
    /**
     *  key for the pallette color 9 for boundaries
     */
    public static final String color_boundary9 = "color_boundary9";
    /**
     *  key for the pallette color 10 for boundaries
     */
    public static final String color_boundary10 = "color_boundary10";
    /**
     *  key for the pallette color 11 for boundaries
     */
    public static final String color_boundary11 = "color_boundary11";
    /**
     *  key for the pallette color 12 for boundaries
     */
    public static final String color_boundary12 = "color_boundary12";
    /**
     *  key for the pallette color 13 for boundaries
     */
    public static final String color_boundary13 = "color_boundary13";
    /**
     *  key for the pallette color 14 for boundaries
     */
    public static final String color_boundary14 = "color_boundary14";
    /**
     *  key for the pallette color 15 for boundaries
     */
    public static final String color_boundary15 = "color_boundary15";


    // fonts
    /** font used for numbering */
    public static final String font_numbering = "font_numbering";
    /** size of that font */
    public static final String font_numbering_size = "font_numbering_size";
    /** style*/
    public static final String font_numbering_style = "font_numbering_style";

   


    // accelerators


    /**
     *  key for the accelerator assigned to NEW
     */
    public static final String accelerator_new = "accelerator_new";
    /**
     *  key for the accelerator assigned to OPEN
     */
    public static final String accelerator_open = "accelerator_open";
    /**
     *  key for the accelerator assigned to SAVE
     */
    public static final String accelerator_save = "accelerator_save";

    /**
     *  key for the accelerator assigned to SAVEAS_UCD
     */
    public static final String accelerator_saveas_ucd = "accelerator_saveas_ucd";
    /**
     *  key for the accelerator assigned to SAVEAS_FEAT
     */
    public static final String accelerator_saveas_feat = "accelerator_saveas_feat";
    /**
     *  key for the accelerator assigned to SAVEAS_FEAST
     */
    public static final String accelerator_saveas_feast = "accelerator_saveas_feast";
    /**
     *  key for the accelerator assigned to MERGE
     */
    public static final String accelerator_merge = "accelerator_merge";
    /**
     *  key for the accelerator assigned to PRINT
     */
    public static final String accelerator_print = "accelerator_print";
    /**
     *  key for the accelerator assigned to PRINT LANDSCAPE
     */
    public static final String accelerator_printls = "accelerator_printls";
    /**
     *  key for the accelerator assigned to EXIT
     */
    public static final String accelerator_exit = "accelerator_exit";
    /**
     *  key for the accelerator assigned to UNDO
     */
    public static final String accelerator_undo = "accelerator_undo";
    /**
     *  key for the accelerator assigned to REDO
     */
    public static final String accelerator_redo = "accelerator_redo";
    /**
     *  key for the accelerator assigned to CUT
     */
    public static final String accelerator_cut = "accelerator_cut";
    /**
     *  key for the accelerator assigned to COPY
     */
    public static final String accelerator_copy = "accelerator_copy";
    /**
     *  key for the accelerator assigned to PASTE
     */
    public static final String accelerator_paste = "accelerator_paste";
    /**
     *  key for the accelerator assigned to COPYMODE
     */
    public static final String accelerator_copymode = "accelerator_copymode";
    /**
     *  key for the accelerator assigned to DELETE
     */
    public static final String accelerator_delete = "accelerator_delete";
    /**
     *  key for the accelerator assigned to DELETEWB
     */
    public static final String accelerator_deleteworkingboundary = "accelerator_deleteworkingboundary";
    /**
     *  key for the accelerator assigned to SCALE
     */
    public static final String accelerator_scale = "accelerator_scale";
    /**
     *  key for the accelerator assigned to ROTATE
     */
    public static final String accelerator_rotate = "accelerator_rotate";
    /**
     *  key for the accelerator assigned to MIRROR
     */
    public static final String accelerator_mirror = "accelerator_mirror";
    /**
     *  key for the accelerator assigned to SELECTBOUNDARY
     */
    public static final String accelerator_selectboundary = "accelerator_selectboundary";
    /**
     *  key for the accelerator assigned to SETZOOM
     */
    public static final String accelerator_zoom_setzoom = "accelerator_zoom_setzoom";
    /**
     *  key for the accelerator assigned to RESETZOOM
     */
    public static final String accelerator_zoom_resetzoom = "accelerator_zoom_resetzoom";
    /**
     *  key for the accelerator assigned to ZOOMPLUS
     */
    public static final String accelerator_zoom_zoomplus = "accelerator_zoom_zoomplus";
    /**
     *  key for the accelerator assigned to ZOOMMINUS
     */
    public static final String accelerator_zoom_zoomminus = "accelerator_zoom_zoomminus";
    /**
     *  key for the accelerator assigned to SELECTNODES
     */
    public static final String accelerator_selectnodes = "accelerator_selectnodes";
    /**
     *  key for the accelerator assigned to SELECTELEMENTS
     */
    public static final String accelerator_selectelements = "accelerator_selectelements";
    /**
     *  key for the accelerator assigned to SELECTSEGMENTS
     */
    public static final String accelerator_selectsegments = "accelerator_selectsegments";
    /**
     *  key for the accelerator assigned to SELECTEDGES
     */
    public static final String accelerator_selectedges = "accelerator_selectedges";
  /**
     *  key for the accelerator assigned to VIEWPROPERTIES
     */
    public static final String accelerator_viewproperties = "accelerator_viewproperties";


    /**
     *  key for the accelerator assigned to NEWBOUNDARY
     */
    public static final String accelerator_newboundary = "accelerator_newboundary";
    /**
     *  key for the accelerator assigned to DIRECTINPUT
     */
    public static final String accelerator_directinput = "accelerator_directinput";
    /**
     *  key for the accelerator assigned to NEWLINE
     */
    public static final String accelerator_newline = "accelerator_newline";
  /**
     *  key for the accelerator assigned to NEWMULTILINE
     */
    public static final String accelerator_newmultiline = "accelerator_newmultiline";
  /**
     *  key for the accelerator assigned to NEWCIRCLEP
     */
    public static final String accelerator_newcirclep = "accelerator_newcirclep";
  /**
     *  key for the accelerator assigned to NEWCIRCLEM
     */
    public static final String accelerator_newcirclem = "accelerator_newcirclem";
  /**
     *  key for the accelerator assigned to NEWANALYTIC
     */
    public static final String accelerator_newanalytic = "accelerator_newanalytic";
    /**
     *  key for the accelerator assigned to NEWEDGE
     */
    public static final String accelerator_newedge = "accelerator_newedge";
    /**
     *  key for the accelerator assigned to NEWTRI
     */
    public static final String accelerator_newtri = "accelerator_newtri";
    /**
     *  key for the accelerator assigned to NEWQUAD
     */
    public static final String accelerator_newquad = "accelerator_newquad";
    /**
     *  key for the accelerator assigned to NEWMACRO
     */
    public static final String accelerator_newmacro = "accelerator_newmacro";
    /**
     *  key for the accelerator assigned to NEWNODE
     */
    public static final String accelerator_newnode = "accelerator_newnode";
    /**
     *  key for the accelerator assigned to ADJUSTBOUNDARIES
     */
    public static final String accelerator_adjustboundaries = "accelerator_adjustboundaries";
    /**
     *  key for the accelerator assigned to LINKBOUNDARIES
     */
    public static final String accelerator_linkboundaries = "accelerator_linkboundaries";

    /**
     *  key for the accelerator assigned to FROMTRI2PRM
     */
    public static final String accelerator_fromtri2prm = "accelerator_fromtri2prm";
    
    /**
     *  key for the accelerator assigned to SWAPBOUNDARYORIENTATION
     */
    public static final String accelerator_swapboundaryorientation = "accelerator_swapboundaryorientation";
    /**
     *  key for the accelerator assigned to DOMAINPERIMETER
     */
    public static final String accelerator_domainperimeter = "accelerator_domainperimeter";
    /**
     *  key for the accelerator assigned to DOMAINSTATISTICS
     */
    public static final String accelerator_domainstatistics = "accelerator_domainstatistics";
    /**
     *  key for the accelerator assigned to MULTIPLEBOUNDARYNODES
     */
    public static final String accelerator_multipleboundarynodes = "accelerator_multipleboundarynodes";
    /**
     *  key for the accelerator assigned to EDGESTATUS
     */
    public static final String accelerator_edgestatus = "accelerator_edgestatus";
    /**
     *  key for the accelerator assigned to PARALLELBLOCK
     */
    public static final String accelerator_parallelblock = "accelerator_parallelblock";
    /**
     *  key for the accelerator assigned to LOADBALANCING
     */
    public static final String accelerator_loadbalancing = "accelerator_loadbalancing";
    /**
     *  key for the accelerator assigned to RECTIFYELEMENT
     */
    public static final String accelerator_rectifyelement = "accelerator_rectifyelement";
    /**
     *  key for the accelerator assigned to REFINE
     */
    public static final String accelerator_refine = "accelerator_refine";
    /**
     *  key for the accelerator assigned to SPELLCHECKER
     */
    public static final String accelerator_spellchecker = "accelerator_spellchecker";
    /**
     *  key for the accelerator assigned to MERGEBOUNDARIES
     */
    public static final String accelerator_mergeboundaries = "accelerator_mergeboundaries";
    /**
     *  key for general options
     */
    public static final String accelerator_generaloptions = "accelerator_generaloptions";
    /**
     *  key for draw options
     */
    public static final String accelerator_drawoptions = "accelerator_drawoptions";
    /**
     *  key for color options
     */
    public static final String accelerator_coloroptions = "accelerator_coloroptions";
    /**
     *  key for accelerator options
     */
    public static final String accelerator_acceleratoroptions = "accelerator_acceleratoroptions";
    /**
     *  key for misc options
     */
    public static final String accelerator_miscoptions = "accelerator_miscoptions";
    /**
     *  key for the accelerator assigned to MANUAL
     */
    public static final String accelerator_manual = "accelerator_manual";
    /**
     *  key for the accelerator assigned to FAQ
     */
    public static final String accelerator_faq = "accelerator_faq";
    /**
     *  key for the accelerator assigned to README
     */
    public static final String accelerator_readme = "accelerator_readme";    
    /**
     *  key for the accelerator assigned to ABOUT
     */
    public static final String accelerator_about = "accelerator_about";


}




